Package: roxygen2
Title: In-Line Documentation for R
Version: 8.1.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Peter", "Danenberg", , "pcd@roxygen.org", role = c("aut", "cph")),
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "aut"),
    person("Manuel", "Eugster", role = c("aut", "cph")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Generate your Rd documentation, 'NAMESPACE' file, and
    collation field using specially formatted comments. Writing
    documentation in-line with code makes it easier to keep your
    documentation up-to-date as your requirements change. 'roxygen2' is
    inspired by the 'Doxygen' system for C++.
License: MIT + file LICENSE
URL: https://roxygen2.r-lib.org/, https://github.com/r-lib/roxygen2
BugReports: https://github.com/r-lib/roxygen2/issues
Depends: R (>= 4.1)
Imports: brew, cli (>= 3.3.0), commonmark, desc (>= 1.2.0), knitr,
        lifecycle, methods, pkgload (>= 1.5.2), R6 (>= 2.1.2), rdtools
        (>= 0.1.0), rlang (>= 1.1.0), utils, withr, xml2
Suggests: covr, R.methodsS3, S7, R.oo, rmarkdown (>= 2.16), testthat
        (>= 3.1.2), yaml
LinkingTo: cpp11
VignetteBuilder: knitr
Config/Needs/development: testthat
Config/Needs/website: tidyverse/tidytemplate
Config/roxygen2/load: installed
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0.9000
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
Language: en-GB
NeedsCompilation: yes
Packaged: 2026-07-31 13:20:14 UTC; hadleywickham
Author: Hadley Wickham [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Peter Danenberg [aut, cph],
  Gábor Csárdi [aut],
  Manuel Eugster [aut, cph],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-08-04 20:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-04 23:19:46 UTC; unix
Archs: roxygen2.so.dSYM

Package: testthat
Title: Unit Testing for R
Version: 3.2.1.9000
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person("R Core team", role = "ctb",
           comment = "Implementation of utils::recover()")
  )
Description: Software testing is important, but, in part because it is
    frustrating and boring, many of us avoid it. 'testthat' is a testing
    framework for R that is easy to learn and use, and integrates with
    your existing 'workflow'.
License: MIT + file LICENSE
URL: https://testthat.r-lib.org, https://github.com/r-lib/testthat
BugReports: https://github.com/r-lib/testthat/issues
Depends: 
    R (>= 3.6.0)
Imports:
    brio (>= 1.1.3),
    callr (>= 3.7.3),
    cli (>= 3.6.1),
    desc (>= 1.4.2),
    digest (>= 0.6.33),
    evaluate (>= 0.21),
    jsonlite (>= 1.8.7),
    lifecycle (>= 1.0.3),
    magrittr (>= 2.0.3),
    methods,
    pkgload (>= 1.3.2.1),
    praise (>= 1.0.0),
    processx (>= 3.8.2),
    ps (>= 1.7.5),
    R6 (>= 2.5.1),
    rlang (>= 1.1.1),
    utils,
    waldo (>= 0.5.1),
    withr (>= 2.5.0)
Suggests: 
    covr,
    curl (>= 0.9.5),
    diffviewer (>= 0.1.0),
    knitr,
    rmarkdown,
    rstudioapi,
    shiny,
    usethis,
    vctrs (>= 0.1.0),
    xml2
VignetteBuilder: 
    knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: watcher, parallel*
Encoding: UTF-8
Roxygen: list(markdown = TRUE, r6 = FALSE)
RoxygenNote: 7.2.3

Package: testCollateParse
Title: Tools to make developing R code easier
License: GPL-2
Description: A test to make sure collate field is parsed correctly when it has
    a file name per line and is indented (#790).  This package only has a
    DESCRIPTION file.
Author: Brodie Gaslam <brodie.gaslam@yahoo.com>
Maintainer: Brodie Gaslam <brodie.gaslam@yahoo.com>
Version: 0.1
Collate:
    'b.R'
    'c.R'

Package: roxygen2
Title: In-Line Documentation for R
Version: 7.3.2.9000
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Peter", "Danenberg", , "pcd@roxygen.org", role = c("aut", "cph")),
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "aut"),
    person("Manuel", "Eugster", role = c("aut", "cph")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Generate your Rd documentation, 'NAMESPACE' file, and
    collation field using specially formatted comments. Writing
    documentation in-line with code makes it easier to keep your
    documentation up-to-date as your requirements change. 'roxygen2' is
    inspired by the 'Doxygen' system for C++.
License: MIT + file LICENSE
URL: https://roxygen2.r-lib.org/, https://github.com/r-lib/roxygen2
BugReports: https://github.com/r-lib/roxygen2/issues
Depends: 
    R (>= 3.6)
Imports: 
    brew,
    cli (>= 3.3.0),
    commonmark,
    desc (>= 1.2.0),
    knitr,
    methods,
    pkgload (>= 1.0.2),
    purrr (>= 1.0.0),
    R6 (>= 2.1.2),
    rlang (>= 1.0.6),
    stringi,
    stringr (>= 1.0.0),
    utils,
    withr,
    xml2
Suggests: 
    covr,
    R.methodsS3,
    R.oo,
    rmarkdown (>= 2.16),
    testthat (>= 3.1.2),
    yaml
LinkingTo: 
    cpp11
VignetteBuilder: 
    knitr
Config/Needs/development: testthat
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
Language: en-GB
Roxygen: list(markdown = TRUE, load = "installed")
RoxygenNote: 7.3.2.9000

Package: testCollateMissing
Title: Tools to make developing R code easier
License: GPL-2
Description: 
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate:
    'b.r'

Package: testRbuildignore
Title: Tools to make developing R code easier
License: GPL-2
Description:
Author: Francois <francois.michonneau@gmail.com>
Maintainer: Francois <francois.michonneau@gmail.com>
Version: 0.1

Package: testRawNamespace
Title: Check that re-running on multi-line @rawNamespace directive is OK
License: GPL-2
Description: testRawNamespace.
Author: Hadley <hadley@rstudio.com>
Maintainer: Hadley <hadley@rstudio.com>
Encoding: UTF-8
Version: 0.1
RoxygenNote: 7.3.3.9000

Package: testImportFrom
Title: Check that merged @importFrom directives are idempotent
License: GPL-2
Description: testImportFrom.
Author: Hadley <hadley@rstudio.com>
Maintainer: Hadley <hadley@rstudio.com>
Encoding: UTF-8
Version: 0.1
RoxygenNote: 7.3.3.9000

Package: testNamespace
Title: Check that utf8 escapes are round tripped ok
License: GPL-2
Description:
Author: Hadley <hadley@rstudio.com>
Maintainer: Hadley <hadley@rstudio.com>
Encoding: UTF-8
Version: 0.1
RoxygenNote: 7.1.2.9000

Package: testNonASCII
Title: Test no change to Collate when there are no @includes
License: GPL-2
Description:
Author: Shrëktan <shrektan@126.com>
Maintainer: Shrëktan <shrektan@126.com>
Encoding: UTF-8
Version: 0.1

Package: testCollateNoIncludes
Title: Test no change to Collate when there are no @includes
License: GPL-2
Description:
Author: Geoff <geoff.lee99@gmail.com>
Maintainer: Geoff <geoff.lee99@gmail.com>
Version: 0.1
Collate: b.R a.R

# Generated by roxygen2: do not edit by hand

S3method(block_to_rd,default)
S3method(block_to_rd,roxy_block)
S3method(block_to_rd,roxy_block_r6class)
S3method(c,rd)
S3method(default_export,"NULL")
S3method(default_export,default)
S3method(default_export,rcclass)
S3method(default_export,s3method)
S3method(default_export,s4class)
S3method(default_export,s4generic)
S3method(default_export,s4method)
S3method(escape,"NULL")
S3method(escape,character)
S3method(escape,rd)
S3method(format,object)
S3method(format,rd_r6_class)
S3method(format,rd_r6_field)
S3method(format,rd_r6_fields)
S3method(format,rd_r6_inherited)
S3method(format,rd_r6_method)
S3method(format,rd_r6_methods)
S3method(format,rd_r6_super)
S3method(format,rd_section)
S3method(format,rd_section_alias)
S3method(format,rd_section_author)
S3method(format,rd_section_backref)
S3method(format,rd_section_concept)
S3method(format,rd_section_description)
S3method(format,rd_section_details)
S3method(format,rd_section_docType)
S3method(format,rd_section_encoding)
S3method(format,rd_section_examples)
S3method(format,rd_section_family)
S3method(format,rd_section_field)
S3method(format,rd_section_formals)
S3method(format,rd_section_format)
S3method(format,rd_section_inherit)
S3method(format,rd_section_inherit_dot_params)
S3method(format,rd_section_inherit_params_args)
S3method(format,rd_section_inherit_section)
S3method(format,rd_section_keyword)
S3method(format,rd_section_minidesc)
S3method(format,rd_section_name)
S3method(format,rd_section_note)
S3method(format,rd_section_package)
S3method(format,rd_section_param)
S3method(format,rd_section_prop)
S3method(format,rd_section_r6_class)
S3method(format,rd_section_rawRd)
S3method(format,rd_section_rcmethods)
S3method(format,rd_section_reexport)
S3method(format,rd_section_references)
S3method(format,rd_section_section)
S3method(format,rd_section_seealso)
S3method(format,rd_section_slot)
S3method(format,rd_section_source)
S3method(format,rd_section_title)
S3method(format,rd_section_usage)
S3method(format,rd_section_value)
S3method(format,roxy_tag)
S3method(merge,rd_section)
S3method(merge,rd_section_author)
S3method(merge,rd_section_inherit)
S3method(merge,rd_section_inherit_dot_params)
S3method(merge,rd_section_inherit_params_args)
S3method(merge,rd_section_inherit_section)
S3method(merge,rd_section_minidesc)
S3method(merge,rd_section_param)
S3method(merge,rd_section_prop)
S3method(merge,rd_section_r6_class)
S3method(merge,rd_section_reexport)
S3method(merge,rd_section_section)
S3method(merge,rd_section_seealso)
S3method(object_defaults,"function")
S3method(object_defaults,data)
S3method(object_defaults,default)
S3method(object_defaults,import)
S3method(object_defaults,package)
S3method(object_defaults,r6class)
S3method(object_defaults,r6method)
S3method(object_defaults,rcclass)
S3method(object_defaults,s3generic)
S3method(object_defaults,s3method)
S3method(object_defaults,s4class)
S3method(object_defaults,s4generic)
S3method(object_defaults,s4method)
S3method(object_defaults,s7class)
S3method(object_defaults,s7generic)
S3method(object_defaults,s7method)
S3method(object_defaults,value)
S3method(object_format,default)
S3method(object_name,"function")
S3method(object_name,default)
S3method(object_name,s3generic)
S3method(object_name,s3method)
S3method(object_name,s4generic)
S3method(object_name,s4method)
S3method(object_usage,"function")
S3method(object_usage,data)
S3method(object_usage,default)
S3method(object_usage,s3method)
S3method(object_usage,s4generic)
S3method(object_usage,s4method)
S3method(object_usage,s7class)
S3method(object_usage,s7generic)
S3method(object_usage,s7method)
S3method(object_usage,value)
S3method(print,object)
S3method(print,rd)
S3method(print,rd_section)
S3method(print,roxy_block)
S3method(print,roxy_tag)
S3method(roclet_clean,roclet_namespace)
S3method(roclet_clean,roclet_rd)
S3method(roclet_output,roclet_namespace)
S3method(roclet_output,roclet_rd)
S3method(roclet_output,roclet_vignette)
S3method(roclet_preprocess,default)
S3method(roclet_process,roclet_namespace)
S3method(roclet_process,roclet_rd)
S3method(roclet_process,roclet_vignette)
S3method(roxy_tag_ns,default)
S3method(roxy_tag_ns,roxy_tag_evalNamespace)
S3method(roxy_tag_ns,roxy_tag_export)
S3method(roxy_tag_ns,roxy_tag_exportClass)
S3method(roxy_tag_ns,roxy_tag_exportMethod)
S3method(roxy_tag_ns,roxy_tag_exportPattern)
S3method(roxy_tag_ns,roxy_tag_exportS3Method)
S3method(roxy_tag_ns,roxy_tag_import)
S3method(roxy_tag_ns,roxy_tag_importClassesFrom)
S3method(roxy_tag_ns,roxy_tag_importFrom)
S3method(roxy_tag_ns,roxy_tag_importMethodsFrom)
S3method(roxy_tag_ns,roxy_tag_rawNamespace)
S3method(roxy_tag_ns,roxy_tag_useDynLib)
S3method(roxy_tag_parse,default)
S3method(roxy_tag_parse,roxy_tag_R6method)
S3method(roxy_tag_parse,roxy_tag_aliases)
S3method(roxy_tag_parse,roxy_tag_author)
S3method(roxy_tag_parse,roxy_tag_backref)
S3method(roxy_tag_parse,roxy_tag_concept)
S3method(roxy_tag_parse,roxy_tag_describeIn)
S3method(roxy_tag_parse,roxy_tag_description)
S3method(roxy_tag_parse,roxy_tag_details)
S3method(roxy_tag_parse,roxy_tag_docType)
S3method(roxy_tag_parse,roxy_tag_encoding)
S3method(roxy_tag_parse,roxy_tag_eval)
S3method(roxy_tag_parse,roxy_tag_evalNamespace)
S3method(roxy_tag_parse,roxy_tag_evalRd)
S3method(roxy_tag_parse,roxy_tag_example)
S3method(roxy_tag_parse,roxy_tag_examples)
S3method(roxy_tag_parse,roxy_tag_examplesIf)
S3method(roxy_tag_parse,roxy_tag_export)
S3method(roxy_tag_parse,roxy_tag_exportClass)
S3method(roxy_tag_parse,roxy_tag_exportMethod)
S3method(roxy_tag_parse,roxy_tag_exportPattern)
S3method(roxy_tag_parse,roxy_tag_exportS3Method)
S3method(roxy_tag_parse,roxy_tag_family)
S3method(roxy_tag_parse,roxy_tag_field)
S3method(roxy_tag_parse,roxy_tag_format)
S3method(roxy_tag_parse,roxy_tag_import)
S3method(roxy_tag_parse,roxy_tag_importClassesFrom)
S3method(roxy_tag_parse,roxy_tag_importFrom)
S3method(roxy_tag_parse,roxy_tag_importMethodsFrom)
S3method(roxy_tag_parse,roxy_tag_include)
S3method(roxy_tag_parse,roxy_tag_includeRmd)
S3method(roxy_tag_parse,roxy_tag_inherit)
S3method(roxy_tag_parse,roxy_tag_inheritDotParams)
S3method(roxy_tag_parse,roxy_tag_inheritParams)
S3method(roxy_tag_parse,roxy_tag_inheritSection)
S3method(roxy_tag_parse,roxy_tag_keywords)
S3method(roxy_tag_parse,roxy_tag_md)
S3method(roxy_tag_parse,roxy_tag_method)
S3method(roxy_tag_parse,roxy_tag_name)
S3method(roxy_tag_parse,roxy_tag_noMd)
S3method(roxy_tag_parse,roxy_tag_noRd)
S3method(roxy_tag_parse,roxy_tag_note)
S3method(roxy_tag_parse,roxy_tag_order)
S3method(roxy_tag_parse,roxy_tag_param)
S3method(roxy_tag_parse,roxy_tag_prop)
S3method(roxy_tag_parse,roxy_tag_rawNamespace)
S3method(roxy_tag_parse,roxy_tag_rawRd)
S3method(roxy_tag_parse,roxy_tag_rdname)
S3method(roxy_tag_parse,roxy_tag_references)
S3method(roxy_tag_parse,roxy_tag_return)
S3method(roxy_tag_parse,roxy_tag_returns)
S3method(roxy_tag_parse,roxy_tag_section)
S3method(roxy_tag_parse,roxy_tag_seealso)
S3method(roxy_tag_parse,roxy_tag_slot)
S3method(roxy_tag_parse,roxy_tag_source)
S3method(roxy_tag_parse,roxy_tag_template)
S3method(roxy_tag_parse,roxy_tag_templateVar)
S3method(roxy_tag_parse,roxy_tag_title)
S3method(roxy_tag_parse,roxy_tag_usage)
S3method(roxy_tag_parse,roxy_tag_useDynLib)
S3method(roxy_tag_rd,default)
S3method(roxy_tag_rd,roxy_tag_.formals)
S3method(roxy_tag_rd,roxy_tag_.methods)
S3method(roxy_tag_rd,roxy_tag_.package)
S3method(roxy_tag_rd,roxy_tag_.reexport)
S3method(roxy_tag_rd,roxy_tag_author)
S3method(roxy_tag_rd,roxy_tag_backref)
S3method(roxy_tag_rd,roxy_tag_concept)
S3method(roxy_tag_rd,roxy_tag_description)
S3method(roxy_tag_rd,roxy_tag_details)
S3method(roxy_tag_rd,roxy_tag_docType)
S3method(roxy_tag_rd,roxy_tag_encoding)
S3method(roxy_tag_rd,roxy_tag_evalRd)
S3method(roxy_tag_rd,roxy_tag_example)
S3method(roxy_tag_rd,roxy_tag_examples)
S3method(roxy_tag_rd,roxy_tag_examplesIf)
S3method(roxy_tag_rd,roxy_tag_family)
S3method(roxy_tag_rd,roxy_tag_field)
S3method(roxy_tag_rd,roxy_tag_format)
S3method(roxy_tag_rd,roxy_tag_includeRmd)
S3method(roxy_tag_rd,roxy_tag_inherit)
S3method(roxy_tag_rd,roxy_tag_inheritDotParams)
S3method(roxy_tag_rd,roxy_tag_inheritParams)
S3method(roxy_tag_rd,roxy_tag_inheritSection)
S3method(roxy_tag_rd,roxy_tag_keywords)
S3method(roxy_tag_rd,roxy_tag_note)
S3method(roxy_tag_rd,roxy_tag_param)
S3method(roxy_tag_rd,roxy_tag_prop)
S3method(roxy_tag_rd,roxy_tag_rawRd)
S3method(roxy_tag_rd,roxy_tag_references)
S3method(roxy_tag_rd,roxy_tag_return)
S3method(roxy_tag_rd,roxy_tag_returns)
S3method(roxy_tag_rd,roxy_tag_section)
S3method(roxy_tag_rd,roxy_tag_seealso)
S3method(roxy_tag_rd,roxy_tag_slot)
S3method(roxy_tag_rd,roxy_tag_source)
S3method(roxy_tag_rd,roxy_tag_title)
S3method(roxy_tag_rd,roxy_tag_usage)
export(block_get_tag)
export(block_get_tag_value)
export(block_get_tags)
export(block_has_tags)
export(env_file)
export(env_package)
export(escape_examples)
export(is_s3_generic)
export(is_s3_method)
export(load_installed)
export(load_options)
export(load_pkgload)
export(load_source)
export(namespace_roclet)
export(needs_roxygenize)
export(object)
export(object_format)
export(parse_file)
export(parse_package)
export(parse_text)
export(rd_roclet)
export(rd_section)
export(roc_proc_text)
export(roclet)
export(roclet_clean)
export(roclet_find)
export(roclet_output)
export(roclet_preprocess)
export(roclet_process)
export(roclet_tags)
export(roxy_block)
export(roxy_meta_get)
export(roxy_tag)
export(roxy_tag_parse)
export(roxy_tag_rd)
export(roxy_tag_warning)
export(roxygenise)
export(roxygenize)
export(tag_code)
export(tag_examples)
export(tag_inherit)
export(tag_markdown)
export(tag_markdown_with_sections)
export(tag_name)
export(tag_name_description)
export(tag_toggle)
export(tag_two_part)
export(tag_value)
export(tag_words)
export(tag_words_line)
export(tags_list)
export(tags_metadata)
export(update_collate)
export(vignette_roclet)
export(warn_roxy_tag)
import(rlang)
importFrom(R6,R6Class)
importFrom(knitr,knit)
importFrom(knitr,opts_chunk)
importFrom(lifecycle,deprecated)
importFrom(stats,setNames)
importFrom(utils,URLdecode)
importFrom(utils,URLencode)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_children)
importFrom(xml2,xml_contents)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_name)
importFrom(xml2,xml_ns_strip)
importFrom(xml2,xml_text)
importFrom(xml2,xml_type)
useDynLib(roxygen2, .registration=TRUE)

# roxygen2 <a href="https://roxygen2.r-lib.org"><img src="man/figures/logo.png" align="right" height="138" alt="roxygen2 website" /></a>

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/roxygen2)](https://CRAN.R-project.org/package=roxygen2)
[![R-CMD-check](https://github.com/r-lib/roxygen2/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/roxygen2/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/r-lib/roxygen2/branch/main/graph/badge.svg)](https://app.codecov.io/gh/r-lib/roxygen2?branch=main)
<!-- badges: end -->

The premise of roxygen2 is simple: describe your functions in comments next to their definitions and roxygen2 will process your source code and comments to automatically generate `.Rd` files in `man/`, `NAMESPACE`, and, if needed, the `Collate` field in `DESCRIPTION`.

## Installation 

```R
# Install roxygen2 from CRAN
install.packages("roxygen2")

# Or the development version from GitHub:
# install.packages("pak")
pak::pak("r-lib/roxygen2")
```

## Usage

The premise of roxygen2 is simple: describe your functions in comments next to their definitions and roxygen2 will process your source code and comments to produce Rd files in the `man/` directory.  Here's a [simple example](https://stringr.tidyverse.org/reference/str_length.html) from the stringr package:

```R
#' The length of a string
#'
#' Technically this returns the number of "code points", in a string. One
#' code point usually corresponds to one character, but not always. For example,
#' an u with a umlaut might be represented as a single character or as the
#' combination a u and an umlaut.
#'
#' @inheritParams str_detect
#' @return A numeric vector giving number of characters (code points) in each
#'    element of the character vector. Missing string have missing length.
#' @seealso [stringi::stri_length()] which this function wraps.
#' @export
#' @examples
#' str_length(letters)
#' str_length(NA)
#' str_length(factor("abc"))
#' str_length(c("i", "like", "programming", NA))
str_length <- function(string) {
}
```

When you `roxygenise()` (or `devtools::document()`) your package these comments will be automatically transformed to the `.Rd` that R uses to generate the documentation you see when you type `?str_length`. 

## Learn more

To get started, first read `vignette("roxygen2")`. Then read more about the specific package component that you want to generate:

* `vignette("rd-functions")` describes how to document your functions with roxygen2.

* `vignette("rd-S3")`, `vignette("rd-S4")`, `vignette("rd-R6")`, and `vignette("rd-S7")` discuss documenting the various OOP systems.

* `vignette("rd-datasets")` and `vignette("rd-packages")` cover documenting datasets and the package itself.

* `vignette("rd-formatting")` gives the details of roxygen2's rmarkdown support.

* `vignette("reuse")` demonstrates the tools available to reuse documentation in multiple places.

* `vignette("namespace")` describes how to generate a `NAMESPACE` file, how namespacing works in R, and how you can use roxygen2 to be specific about what your package needs and supplies.

* For the `Collate` field in the `DESCRIPTION`, see `?update_collate()`.

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(comment = "#>", collapse = TRUE)

## -----------------------------------------------------------------------------
# #' Add two numbers together
# #'
# #' @param x,y A pair of numbers.
# #' @returns A number.
# #' @export
# #' @examples
# #' 1 + 2
# add <- function(x, y) {
#   x + y
# }

## -----------------------------------------------------------------------------
#' Sum of vector elements
#'
#' `sum` returns the sum of all the values present in its arguments.
#'
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.
sum <- function(..., na.rm = TRUE) {}

## -----------------------------------------------------------------------------
# #' Detect the presence/absence of a match
# #'
# #' `str_detect()` returns a logical vector with `TRUE` for each element of
# #' `string` that matches `pattern` and `FALSE` otherwise. It's equivalent to
# #' `grepl(pattern, string)`.

## -----------------------------------------------------------------------------
# #' View strings and matches
# #'
# #' @description
# #' `str_view()` is used to print the underlying representation of a string and
# #' to see how a `pattern` matches.
# #'
# #' Matches are surrounded by `<>` and unusual whitespace (i.e. all whitespace
# #' apart from `" "` and `"\n"`) are surrounded by `{}` and escaped.

## -----------------------------------------------------------------------------
#' Sum of vector elements
#'
#' @description
#' `sum` returns the sum of all the values present in its arguments.
#'
#' @details
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.

## -----------------------------------------------------------------------------
# #' @param pattern Pattern to look for.
# #'
# #'   The default interpretation is a regular expression, as described in
# #'   `vignette("regular-expressions")`. Use [regex()] for finer control of the
# #'   matching behaviour.
# #'
# #' @param string Input vector. Either a character vector, or something
# #'   coercible to one.

## -----------------------------------------------------------------------------
# #' @param na.rm Remove missing values? If `FALSE` (the default), the result
# #'   will be `NA` if any element of `string` is `NA`.

## -----------------------------------------------------------------------------
# #' @param side Side on which to remove whitespace: `"left"`, `"right"`, or
# #'   `"both"` (the default).

## -----------------------------------------------------------------------------
# #' @param whitespace_only A boolean.
# #'   * `TRUE` (the default): wrapping will only occur at whitespace.
# #'   * `FALSE`: can break on any non-word character (e.g. `/`, `-`).

## -----------------------------------------------------------------------------
# #' @returns A logical vector the same length as `string`.

## -----------------------------------------------------------------------------
# #' @returns
# #' An object of the same type as `.data`. The output has the following
# #' properties:
# #'
# #' * Rows are a subset of the input, but appear in the same order.
# #' * Columns are not modified.
# #' * The number of groups may be reduced (if `.preserve` is not `TRUE`).
# #' * Data frame attributes are preserved.

## -----------------------------------------------------------------------------
# #' @examples
# #' fruit <- c("apple", "banana", "pear", "pineapple")
# #' str_detect(fruit, "a")
# #' str_detect(fruit, "^a")
# #' str_detect(fruit, "a$")
# #'
# #' # Also vectorised over pattern
# #' str_detect("aecfg", letters)

## -----------------------------------------------------------------------------
# #' @examples
# #' # Row sizes must be compatible when column-binding
# #' try(bind_cols(tibble(x = 1:3), tibble(y = 1:2)))

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
# #' Size of an object
# #'
# #' @description
# #' `size()` is an S7 generic that determines the size of an object,
# #' with methods available for the following classes:
# #'
# #' `r doclisting::methods_list("size")`
# #'
# #' @param x An object.
# #' @param ... Not used.
# #' @returns A single number.
# #' @export
# size <- new_generic("size", "x")

## -----------------------------------------------------------------------------
# #' Size of an object
# #'
# #' @description
# #' `size()` determines the size of an object.
# #'
# #' # Methods
# #' `size()` is an S7 generic with methods available for the following
# #' classes:
# #'
# #' `r doclisting::methods_list("size")`
# #'
# #' @param x An object.
# #' @param ... Not used.
# #' @returns A single number.
# #' @export
# size <- new_generic("size", "x")

## -----------------------------------------------------------------------------
# #' @rdname size
# method(size, class_any) <- function(x, ...) {
#   length(x)
# }

## -----------------------------------------------------------------------------
# #' A range
# #'
# #' Create a range represented by a numeric `start` and `end`. The start must
# #' always be less than the end.
# #'
# #' @param start Start of range.
# #' @param end End of range.
# #' @prop length Length of the range (read-only).
# #' @returns An `Range` S7 object.
# #' @export
# Range <- new_class(
#   "Range",
#   properties = list(
#     start = class_numeric,
#     end = class_numeric,
#     length = new_property(getter = function(self) self@end - self@start),
#     validator = function(self) {
#       if (self@start > self@end) {
#         "start must be less than or equal to end"
#       }
#     }
#   )
# )

## -----------------------------------------------------------------------------
# #' Size of a range
# #'
# #' The size of a range is its [size()], i.e. its length.
# #'
# #' @param x A `Range` object.
# #' @param ... Not used.
# #' @returns A single number.
# method(size, Range) <- function(x, ...) {
#   x@length
# }

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
# #' Prices of over 50,000 round cut diamonds
# #'
# #' A dataset containing the prices and other attributes of almost 54,000
# #'  diamonds. The variables are as follows:
# #'
# #' @format A data frame with 53940 rows and 10 variables:
# #' \describe{
# #'   \item{price}{price in US dollars ($326--$18,823)}
# #'   \item{carat}{weight of the diamond (0.2--5.01)}
# #'   \item{cut}{quality of the cut (Fair, Good, Very Good, Premium, Ideal)}
# #'   \item{color}{diamond colour, from D (best) to J (worst)}
# #'   \item{clarity}{a measurement of how clear the diamond is (I1 (worst), SI2,
# #'     SI1, VS2, VS1, VVS2, VVS1, IF (best))}
# #'   \item{x}{length in mm (0--10.74)}
# #'   \item{y}{width in mm (0--58.9)}
# #'   \item{z}{depth in mm (0--31.8)}
# #'   \item{depth}{total depth percentage = z / mean(x, y) = 2 * z / (x + y) (43--79)}
# #'   \item{table}{width of top of diamond relative to widest point (43--95)}
# #' }
# #'
# #' @source {ggplot2} tidyverse R package.
# "diamonds"

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
# #' R6 Class Representing a Person
# #'
# #' A person has a name and a hair color.
# #' @export
# Person <- R6::R6Class(
#   "Person",
#   public = list(
#     # ... fields and methods ...
#   )
# )

## -----------------------------------------------------------------------------
# Person <- R6::R6Class(
#   "Person",
#   public = list(
#     #' @field name First or full name of the person.
#     name = NULL,
# 
#     #' @field birthdate Date of birth, as a [Date].
#     birthdate = NULL
#   ),
#   active = list(
#     #' @field age Age in years, computed from `birthdate` (read-only).
#     age = function() {
#       as.numeric(difftime(Sys.Date(), self$birthdate, units = "days")) / 365.25
#     }
#   )
# )

## -----------------------------------------------------------------------------
# Person <- R6::R6Class(
#   "Person",
#   public = list(
#     #' @description
#     #' Create a new person object.
#     #' @param name Name.
#     #' @param hair Hair color.
#     #' @returns A new `Person` object.
#     initialize = function(name = NA, hair = NA) {
#       self$name <- name
#       self$hair <- hair
#       self$greet()
#     },
# 
#     #' @description
#     #' Change hair color.
#     #' @param val New hair color.
#     set_hair = function(val) {
#       self$hair <- val
#     },
# 
#     #' @description
#     #' Say hi.
#     greet = function() {
#       cat(paste0("Hello, my name is ", self$name, ".\n"))
#     }
#   )
# )

## -----------------------------------------------------------------------------
# #' @description
# #' Say goodbye.
# Person$set("public", "goodbye", function() {
#   cat(paste0("Goodbye from ", self$name, ".\n"))
# })

#' Class A
#'
#' @description
#' Class A description.
#'
#' @details
#' Class A details
#'
#' @param fogbehindme This is not show up in the output at all.
#' @param Z zzzzzzz

A <- R6::R6Class(
  "A",
  public = list(
    #' @field field1 A field 1.
    field1 = NULL,
    #' @description A method 1.
    #' @examples
    #' ## Example for meth1
    meth1 = function(Z) { },
    #' @description Method 2 description.
    #' @details Method 2 details.
    #' @param Z Overriding Z argument for meth2.
    #' @param ... Rest.
    #' @examples
    #' ## Example for meth2
    meth2 = function(Z = 10, ...) { },
    #' @field field2 A field 2.
    field2 = "foobar",
    #' @details Method 3 details.
    #' @param duplicate This one is
    #' @param duplicate Twice
    #' @return twice.
    #' @return really?
    meth3 = function(duplicate, missing) { },
    #' @field field3 A field 3.
    field3 = "baz"
  ),
  active = list(
    #' @field active1 A binding 1.
    active1 = function(x) { },
    #' @field active2 A binding 2.
    active2 = function(x) { },
    #' @field active3 A binding 2.
    active3 = function(x) { }
  )
)

#' Class B
#'
#' @description
#' Class B Description.
#'
#' @details
#' Class B details.

B <- R6::R6Class(
  "B",
  inherit = A,
  public = list(
    #' @field field1 B field 1.
    field1 = NULL,
    #' @field field4 B field 4.
    field4 = NULL,
    #' @description B method 1.
    #' @param Z Still zzzzzzzz.
    meth1 = function(Z) { },
    #' @description A method 4.
    meth4 = function() { }
  ),
  active = list(
    #' @field active1 B binding 1.
    active1 = function(x) { },
    #' @field active4 B binding 4.
    active4 = function(x) { },
    #' @field active5 B binding 5.
    active5 = function(x) { }
  )
)

#' Class C
#'
#' @description
#' Class C Description.
#'
#' @details
#' Classs C details.
#'
#' @field nosuchfield This will warn.

C <- R6::R6Class(
  "C",
  inherit = B,
  cloneable = FALSE,
  public = list(
    field2 = NULL,
    #' @description C method 2.
    #' @param Z zzzzz
    #' @param ... etc
    meth2 = function(Z = 10, ...) { },
    #' @field field5 C field 5.
    field5 = "foobar",
    #' @description C method 5.
    meth5 = function() { },
    undocumented_field = NULL,
    undocumented_method = function() { },
    #' @field duplicatefield Multiple.
    #' @field duplicatefield times.
    duplicatefield = NULL
  ),
  active = list(
    #' @field active2 C binding 2.
    active2 = function(x) { },
    #' @field active4 C binding 4.
    active4 = function(x) { },
    #' @field active6 C binding 6.
    active6 = function(x) { },
    undocumented_binding = function(x) { },
    #' @field duplicate_binding Double.
    #' @field duplicate_binding Double double.
    duplicate_binding = function(x) { }
  )
)

#' @description
#' This roclet is the workhorse of `roxygen`, producing the `.Rd` files that
#' R uses to document that functions, datasets, packages, classes, and
#' other objects.
#'
#' See `vignette("rd")` for details.

#' @export
roxy_tag_parse.roxy_tag_family <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_family <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_family <- function(x, ...) {
  NULL
}

# -------------------------------------------------------------------------

topics_process_family_prefix <- function(family) {

  default <- paste0("Other ", family, ": ")

  # check for meta (use default prefix when unset)
  meta <- roxy_meta_get("rd_family_title")
  if (is.null(meta))
    return(default)

  # validate meta structure
  valid <- is.character(meta) || is.list(meta)
  if (!valid) {
    cli::cli_abort("{.code rd_family_title} is set, but is not a named list / vector")
  }

  # extract element
  prefix <- meta[[family]]
  if (is.null(prefix))
    return(default)

  prefix

}

topics_process_family <- function(topics, env) {
  family_index <- invert(topics$simple_values("family"))
  aliases <- topics$simple_values("alias")

  for (topic_name in names(topics$topics)) {
    topic <- topics$get(topic_name)
    families <- topic$get_value("family")

    for (family in families) {
      related <- family_index[[family]]
      topic$add(rd_section("concept", family))

      others <- setdiff(related, topic_name)
      if (length(others) < 1)
        next

      other_aliases <- aliases[others]
      other_aliases_order <- map_chr(other_aliases, function(x) escape(x[1]))

      by_file <- map_chr(other_aliases[order_c(other_aliases_order)], function(x) {
        obj <- find_object(x[1], env)
        suffix <- if (is.function(obj$value)) "()" else ""
        paste0("\\code{\\link{", escape(x[1]), "}", suffix, "}")
      })
      links <- paste(by_file, collapse = ",\n")
      seealso <- topics_process_family_prefix(family)

      topic$add(rd_section("seealso", paste0(seealso, "\n", links)))
    }
  }

  invisible()
}

##' Title
##'
##' Description
##'
##' Details
##'
##' @param x,y,z Descriptions for x, y, z
##' @param a Description of a
##' @param b
##'   Description of b
##' @section Important:
##' Don't run with scissors!
##' @export
NULL

##' Title
##'
##' Description
##'
##' Details
##'
##' @param x,y,z Descriptions for x, y, z
##' @param a Description of a
##' @param b
##'   Description of b
##' @section Important:
##' Don't run with scissors!
##' @export
NULL

#' Find the Rd file of a topic
#'
#' @param pkg Package to search in, or `NA` if no package was specified.
#'   If the same as the dev package, then we treat it as `NA`.
#' @param topic Topic to search for. This is the escaped, so it is `"\%\%"` and
#'   not `"%%"`.
#' @param tag The roxy tag object that contains the link. We use this for
#'   better warnings, that include the file name and line number (of the tag).
#' @return String. File name to link to.
#'
#' @details
#' If `pkg` is `NA` or the package being documented, we'll just leave the
#' topic alone.
#'
#' If `pkg` is not `NA` and not the package being documented (the _dev_
#' package), then we need to be able to find the Rd file. If we can't, that's
#' a warning and the link is left untouched. This typically happens when the
#' linked package is not installed or cannot be loaded.
#'
#' @noRd

find_topic_filename <- function(pkg, topic, tag = NULL) {
  if (is.na(pkg) || identical(roxy_meta_get("current_package"), pkg)) {
    topic
  } else {
    try_find_topic_in_package(pkg, topic, tag)
  }
}

#' Find a help topic in a package
#'
#' This is used by both `find_topic_filename()` and
#' `format.rd_section_reexport()` that creates the re-exports page. The error
#' messages are different for the two, so errors are not handled here.
#'
#' @param pkg Package name. This cannot be `NA`.
#' @inheritParams find_topic_filename
#' @return File name if the topic was found, `NA` if the package could be
#'   searched, but the topic was not found. Errors if the package cannot be
#'   searched. (Because it is not installed or cannot be loaded, etc.)
#'
#' @noRd

find_topic_in_package <- function(pkg, topic) {
  # This is needed because we have the escaped text here, and parse_Rd will
  # un-escape it properly.
  on.exit(close(con), add = TRUE)
  con <- textConnection(topic)
  raw_topic <- str_trim(tools::parse_Rd(con)[[1]][1])
  basename(utils::help((raw_topic), (pkg))[1])
}

try_find_topic_in_package <- function(pkg, topic, tag) {
  path <- tryCatch(
    find_topic_in_package(pkg, topic),
    error = function(err) {
      warn_roxy_tag(tag, "refers to unavailable topic {pkg}::{topic}", parent = err)
      topic
    }
  )

  if (is.na(path)) {
    warn_roxy_tag(tag, "refers to unavailable topic {pkg}::{topic}")
    topic
  } else {
    path
  }
}

resolve_qualified_link <- function(topic) {
  if (is_namespaced(topic)) {
    target <- str_split_fixed(topic, "::", n = 2)
    file <- find_topic_in_package(target[1], target[2])
    paste0(target[1], ":", file)
  } else {
    paste0("=", topic)
  }
}

test_that("generates informative warnings", {
  expect_snapshot({
    tag <- roxy_test_tag()
    find_topic_filename("11papaya", "foo", tag)
    find_topic_filename("stringr", "foofofofoo", tag)
  })
})

#' The length of a string (in characters).
#'
#' @param string input character vector
#' @return numeric vector giving number of characters in each element of the
#'   character vector.  Missing strings have missing length.
#' @seealso \code{\link{nchar}} which this function wraps
#' @export
#' @examples
#' str_length(letters)
#' str_length(c("i", "like", "programming", NA))

#' Construct an `rd_section` object
#'
#' An `rd_section` represents an Rd command that can appear at the top-level
#' of an Rd document, like `\name{}`, `\title{}`, `\description{}`, or
#' `\section{}`.
#'
#' @section Methods:
#' If provide your own `rd_section` type, you'll also need to define a
#' `format.rd_section_{type}` method that returns formatted Rd output. You
#' may also need to provide a `merge.rd_section_{type}` method if two
#' sections can not be combined with `rd_section(x$type, c(x$value, y$value))`.
#' See `vignette("extending")` for more details.
#'
#' @param type Section type. Stored in `type` field, and in class
#'   `rd_section_{type}`. To avoid namespace clashes between different
#'   extensions, this should include the package name.
#' @param value Section data. Only used by `format()` and `merge()` methods.
#' @export
#' @keywords internal
rd_section <- function(type, value) {
  if (is.null(value) || identical(value, "NULL")) {
    # NULL is special sentinel value that suppresses output of that field
    return()
  }

  structure(
    list(
      type = type,
      value = value
    ),
    class = c(paste0("rd_section_", type), "rd_section")
  )
}

#' @export
print.rd_section <- function(x, ...) {
  cat(format(x), "\n")
}

#' @export
format.rd_section <- function(x, ...) {
  cli::cli_abort("`format.{class(x)[[1]]}` method not found")
}

#' @export
merge.rd_section <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  rd_section(x$type, c(x$value, y$value))
}

format_rd <- function(x, ..., sort = TRUE) {
  # One rd macro for each value
  x$value <- unique(x$value)
  if (sort) {
    x$value <- sort_c(x$value)
  }

  map_chr(x$value, rd_macro, field = x$type)
}

format_first <- function(x, ...) {
  # Only use the first value
  rd_macro(x$type, x$value[1])
}

format_collapse <- function(x, ..., indent = 0, exdent = 0) {
  # Collapse all into a single string
  value <- paste0(x$value, collapse = "\n\n")
  rd_macro(x$type, value, space = TRUE)
}

rd_section_description <- function(name, dt, dd) {
  if (length(dt) == 0) return("")

  items <- paste0("\\item{\\code{", dt, "}}{", dd, "}", collapse = "\n\n")
  paste0("\\section{", name, "}{\n\n",
    "\\describe{\n",
    items,
    "\n}}\n"
  )
}

#' @export
roxy_tag_parse.roxy_tag_param <- function(x) {
  tag_two_part(x, "an argument name", "a description")
}

#' @export
roxy_tag_rd.roxy_tag_param <- function(x, base_path, env) {
  value <- setNames(x$val$description, x$val$name)
  rd_section(x$tag, value)
}

#' @export
merge.rd_section_param <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  # When parameters appear in both x and y, keep values from y
  # This happens for example when inherit_dot_params adds a "..." param after
  # inherit_params has done the same.
  to_add <- setdiff(names(x$value), names(y$value))
  rd_section(x$type, c(x$value[to_add], y$value))
}

#' @export
format.rd_section_param <- function(x, ...) {
  names <- names(x$value)

  # add space to multiple arguments so they can wrap
  names <- gsub(",", ", ", names)
  items <- paste0("\\item{", names, "}{", x$value, "}", collapse = "\n\n")

  rd_macro("arguments", items, space = TRUE)
}

# Other helpers -----------------------------------------------------------

# Postprocessing to reset ordering of parameter documentation
topics_fix_params_order <- function(topics) {
  for (topic in topics$topics) {
    # Compute correct ordering of parameter documentation
    # Check what's needed...
    needed <- topic$get_value("formals")

    # (Workaround for dupes that can occur but perhaps shouldn't,
    #  cf. https://github.com/r-lib/roxygen2/commit/83d125dce50a072534988787d49ffe206d19b232#commitcomment-6742169)
    needed <- unique(needed)

    # ...and what's documented (here we look only at the first parameter
    # in a multi-parameter documentation)
    documented <- get_documented_params(topic, only_first = TRUE)

    # We operate on indexes to make sure that no documentation is lost during
    # the reordering
    documented_indexes <- seq_along(documented)

    # We compute the indexes in the current documentation in the required order
    # and append everything that's missing in the order found
    required_order <- match(needed, documented)
    required_order <- required_order[!is.na(required_order)]
    required_order <- c(required_order, setdiff(documented_indexes, required_order))

    # Overwrite all param fields to fix order
    param <- topic$get_value("param")[required_order]
    topic$add(rd_section("param", param), overwrite = TRUE)
  }

  invisible()
}

#' @export
roxy_tag_parse.roxy_tag_aliases <- function(x) tag_words(x, min = 1)
#' @export
format.rd_section_alias <- function(x, ...) {
  x$value <- gsub("%", "\\%", x$value, fixed = TRUE)
  format_rd(x, ..., sort = FALSE)
}

#' @export
roxy_tag_parse.roxy_tag_name <- function(x) tag_value(x)
#' @export
format.rd_section_name <- function(x, ...) {
  x$value <- gsub("%", "\\%", x$value, fixed = TRUE)
  format_first(x, ...)
}

topic_add_name_aliases <- function(topic, block, name) {
  tags <- block_get_tags(block, "aliases")

  if (length(tags) == 0) {
    aliases <- character()
  } else {
    aliases <- unlist(map(tags, \(x) x[["val"]]))
  }

  if (any(aliases == "NULL")) {
    # Don't add default aliases
    aliases <- setdiff(aliases, "NULL")
  } else {
    aliases <- c(name, block$object$alias, aliases)
  }
  aliases <- unique(aliases)

  topic$add(rd_section("name", name))
  topic$add(rd_section("alias", aliases))
}

#' A `RoxyTopic` is an ordered collection of unique rd_sections
#'
#' @description
#' A `RoxyTopic` object corresponds to a generated `.Rd` file.
#'
#' @param type Section type, a character scalar.
#' @param overwrite Whether to overwrite an existing section. If `FALSE`
#' then the two sections will be merged.
#'
#' @keywords internal

RoxyTopic <- R6::R6Class(
  "RoxyTopic",
  public = list(
    #' @field sections Named list of sections. Each item must be an
    #' [rd_section()] object.
    sections = list(),

    #' @field filename Path to the `.Rd` file to generate.
    filename = "",

    #' @description Format the `.Rd` file. It considers the sections in
    #' particular order, even though Rd tools will reorder them again.
    #'
    #' @param ... Passed to the `format()` methods of the [rd_section()]
    #' objects, the sections.
    #' @return Character string.

    format = function(...) {
      # This has to happen here to get a better order when combining topics
      order <- c(
        "backref",
        "docType",
        "encoding",
        "name",
        "alias",
        "title",
        "format",
        "source",
        "usage",
        "param",
        "value",
        "description",
        "details",
        "minidesc",
        "field",
        "slot",
        "rcmethods",
        "note",
        "section",
        "examples",
        "references",
        "seealso",
        "author",
        "concept",
        "keyword",
        "rawRd"
      )
      sections <- move_names_to_front(self$sections, order)

      formatted <- lapply(sections, format, ...)
      paste0(
        made_by("%"),
        paste0(unlist(formatted), collapse = "\n")
      )
    },

    #' @description Check if an `.Rd` file is valid
    #' @return Logical flag, `TRUE` for valid `.Rd` files

    is_valid = function() {
      # Needs both title and name sections to generate valid Rd
      all(self$has_section(c("title", "name")))
    },

    #' @description Check if an `.Rd` file has a certain section.
    #' @return Logical flag.

    has_section = function(type) {
      type %in% names(self$sections)
    },

    #' @description Query a section.
    #' @return The [rd_section] object representing the section, or `NULL`
    #' if the topic has no such section.

    get_section = function(type) {
      self$sections[[type]]
    },

    #' @description Query the value of a section. This is the value of
    #' the [rd_section] object.
    #' @return Value.

    get_value = function(type) {
      self$get_section(type)$value
    },

    #' @description Get the Rd code of a section.
    #' @return Character vector, one element per line.

    get_rd = function(type) {
      format(self$get_section(type))
    },

    #' @description Get the value of the `name` section. This is the name
    #' of the Rd topic.
    #' @return Character scalar.

    get_name = function() {
      self$get_value("name")[[1]]
    },

    #' @description Query the topics this topic inherits `type` from.
    #' @return A character vector of topic names.

    inherits_from = function(type) {
      if (!self$has_section("inherit")) {
        return(character())
      }

      inherit <- self$get_value("inherit")

      inherits_field <- map_lgl(inherit$fields, \(x) type %in% x)
      sources <- inherit$source[inherits_field]

      if ("NULL" %in% sources) {
        return(character())
      }

      sources
    },

    #' @description Query the topics this topic inherits sections from.
    #' @return A character vector of topic names.

    inherits_section_from = function() {
      if (!self$has_section("inherit_section")) {
        return(character())
      }

      self$get_value("inherit_section")$source
    },

    #' @description Add one or more sections to the topic.
    #' @param x Section(s) to add. It may be
    #' another `RoxyTopic` object, all of its sections will be added;
    #' or an [rd_section] object;
    #' or a list of [rd_section] objects to add.
    #' @param block Name of block to use in error messages.

    add = function(x, block = "???", overwrite = FALSE) {
      if (inherits(x, "RoxyTopic")) {
        self$add(x$sections, block, overwrite = overwrite)
      } else if (inherits(x, "rd_section")) {
        self$add_section(x, block, overwrite = overwrite)
      } else if (is.list(x)) {
        for (section in x) {
          self$add_section(section, block, overwrite = overwrite)
        }
      } else if (is.null(x)) {
        # skip
      } else {
        cli::cli_abort(
          "Don't know how to add object of type {.cls {class(x)[1]}}.",
          .internal = TRUE
        )
      }
      invisible()
    },

    #' @description Add a section.
    #' @details
    #' Ensures that each type of name (as given by its name), only appears
    #' once in `self$sections`. This method if for internal use only.
    #' @param section [rd_section] object to add.
    #' @param block Name of block to use in error messages.

    add_section = function(section, block = "???", overwrite = FALSE) {
      if (is.null(section)) {
        return()
      }
      type <- section$type
      if (self$has_section(type) && !overwrite) {
        section <- merge(self$get_section(type), section, block = block)
      }

      self$sections[[type]] <- section
      invisible()
    }
  )
)

move_names_to_front <- function(x, to_front) {
  nms <- names(x)
  x[union(intersect(to_front, nms), nms)]
}

test_that("adding tags merges with existing", {
  rd <- RoxyTopic$new()

  rd$add(rd_section("x", 1))
  rd$add(rd_section("x", 2))

  expect_equal(rd$get_value("x"), c(1, 2))
})

test_that("unless overwrite = TRUE", {
  rd <- RoxyTopic$new()

  rd$add(rd_section("x", 1))
  rd$add(rd_section("x", 2), overwrite = TRUE)

  expect_equal(rd$get_value("x"), 2)
})

test_that("can add a complete file", {
  rd1 <- RoxyTopic$new()
  rd2 <- RoxyTopic$new()

  rd1$add(rd_section("x", 1))
  rd2$add(rd_section("x", 2))
  rd2$add(rd1)

  expect_equal(rd2$get_value("x"), c(2, 1))
})

test_that("@describeIn suggests @rdname", {
  block <- "
    #' @describeIn foo
    NULL
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("@describeIn generic destination captures s3 method source", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    f <- function(x) UseMethod('f')

    #' @describeIn f Method for a
    #'
    f.a <- function(x) 1
  ")[[1]]
  expect_equal(out$get_value("minidesc")$extends, "generic")
  expect_equal(out$get_value("minidesc")$generic, "f")
  expect_equal(out$get_value("minidesc")$class, "a")
})

test_that("@describeIn generic destination captures s4 method source", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    setGeneric('f', function(x) standardGeneric('f'))

    #' @describeIn f Method for a
    setMethod(f, signature('a'), function(x) 1)
  ")[[1]]
  out$get_value("minidesc")

  expect_equal(out$get_value("minidesc")$extends, "generic")
  expect_equal(out$get_value("minidesc")$generic, "f")
  expect_equal(out$get_value("minidesc")$class, "a")
})

test_that("@describeIn constructor destination captures s3 method source", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    boo <- function() structure(list(), class = 'boo')

    #' @describeIn boo mean method
    #'
    mean.boo <- function(x) 1
    ")[[1]]

  expect_equal(out$get_value("minidesc")$extends, "class")
  expect_equal(out$get_value("minidesc")$generic, "mean")
  expect_equal(out$get_value("minidesc")$class, "boo")
})

test_that("@describeIn constructor destination captures s4 method source", {
  out <- roc_proc_text(rd_roclet(), "
    setGeneric('mean')

    #' Title
    setClass('a')

    #' @describeIn a mean method
    setMethod('mean', 'a', function(x) 1)
    ")[[1]]
  out$get_value("minidesc")
  expect_equal(out$get_value("minidesc")$extends, "class")
  expect_equal(out$get_value("minidesc")$generic, "mean")
  expect_equal(out$get_value("minidesc")$class, "a")
})

test_that("@describeIn function destination captures function source", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    f <- function(x) 1

    #' @describeIn f A
    f2 <- function(x) 1
    ")[[1]]

  expect_equal(out$get_value("minidesc")$name, "f2()")
  expect_equal(out$get_value("minidesc")$extends, "")
  expect_equal(out$get_value("minidesc")$generic, "")
  expect_equal(out$get_value("minidesc")$class, "")
})

test_that("@describeIn class captures function name with data", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #' @name f
    NULL

    #' @describeIn f A
    f2 <- function(x) 1
    ")[[1]]

  expect_equal(out$get_value("minidesc")$name, "f2()")
})

test_that("@describeIn class captures function description", {
  out <- roc_proc_text(rd_roclet(), "
  #' Title
  f <- function(x) 1

  #' @describeIn f A
  f2 <- function(x) 1
  ")[[1]]

  expect_equal(out$get_value("minidesc")$desc, "A")
})

test_that("Multiple @describeIn functions combined into one", {
  out <- roc_proc_text(rd_roclet(), "
    #' Power
    #' @param x base
    #' @param exp exponent
    power <- function(x, exp) x ^ exp

    #' @describeIn power Square a number
    square <- function(x) power(x, 2)

    #' @describeIn power Cube a number
    cube <- function(x) power(x, 3)
    "
  )[[1]]

  expect_snapshot(out$get_section("minidesc"))
})

test_that("multiple methods and others are combined into a generic", {
  out <- roc_proc_text(rd_roclet(), "
    #' Zap generic
    #'
    #' @param x Object to zap.
    zap <- function(x) UseMethod('zap')

    #' @describeIn zap method
    zap.numeric <- function(x) {}

    #' @describeIn zap method
    zap.character <- function(x) {}

    #' @describeIn zap function (method for different generic)
    print.qux <- function(x) {}

    #' @describeIn zap function
    zap_helper <- function(x) {}
    "
  )[[1]]

  expect_snapshot(out$get_section("minidesc"))
})

test_that("multiple methods and others are combined into a class constructor", {
  out <- roc_proc_text(rd_roclet(), "
    #' Class constructor
    #'
    #' @param x x
    foo <- function(x) {}

    #' @describeIn foo method
    print.foo <- function(x) {}

    #' @describeIn foo method
    format.foo <- function(x) {}

    #' @describeIn foo function (method for different class)
    format.bar <- function(x) {}

    #' @describeIn foo function
    is_foo <- function(x) {}
  ")[[1]]
  expect_snapshot(out$get_section("minidesc"))

  # more complicated with disambiguated class name "pkg_class"
  out <- roc_proc_text(rd_roclet(), "
    #' Class constructor with disambiguated class
    #'
    #' @param x x
    baz <- function(x) {}

    #' @describeIn baz method
    print.roxygen2_baz <- function(x) {}

    #' @describeIn baz function (method for another class)
    format.quuz_baz <- function(x) {}"
  )[[1]]
  expect_snapshot(out$get_section("minidesc"))
})

test_that("infix and replacement names get nice label", {
  out <- roc_proc_text(rd_roclet(), "
    #' foo
    foo <- 100

    #' @describeIn foo infix
    `%foo%` <- function(x, y) foo(x, y)

    #' @describeIn foo replacement for foo
    `foo<-` <- function(x, value) 10

    ")[[1]]
  expect_snapshot(out$get_section("minidesc"))
})

test_that("s4 methods get nice label", {
  withr::defer(removeClass("foo1"))
  out <- roc_proc_text(rd_roclet(), "
      #' Class
      #'
      setClass('foo1', slots = c(id = 'character'), where = .GlobalEnv)

      #' @describeIn foo1 generic
      setGeneric('m_id', function(x) standardGeneric('m_id'))

      #' @describeIn foo1 function
      setMethod('m_id', 'foo1', function(x) x@id)
    ")[[1]]
  expect_snapshot(out$get_section("minidesc"))

  withr::defer(removeClass("foo2"))
  withr::defer(removeClass("foo3"))
  out <- roc_proc_text(rd_roclet(), "
    setClass('foo2', where = .GlobalEnv)
    setClass('foo3', where = .GlobalEnv)

    #' bar1
    setGeneric('bar1', function(x, y) standardGeneric('bar1'))

    #' @describeIn bar1 method1
    setMethod('bar1', c('foo2', 'foo3'), function(x, y) 1)
    #' @describeIn bar1 method2
    setMethod('bar1', c('foo3', 'foo2'), function(x, y) 1)
  ")[[1]]
  expect_snapshot(out$get_section("minidesc"))
})

test_that("complains about bad usage", {
  block <- "
    #' bar
    bar <- 100

    #' @name bar
    #' @describeIn foo shortcut for foo
    NULL
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))

  block <- "
    #' bar
    bar <- 100

    #' @name bar
    #' @describeIn foo shortcut for foo
    foo <- 10
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))

  block <- "
    #' bar
    bar <- 100

    #' @rdname bar
    #' @describeIn foo shortcut for foo
    foo <- 10
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

# code --------------------------------------------------------------------
# see test-markdown-code.R for evaluated code

test_that("backticks are converted to \\code & \\verb", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some \\code{code} included. \\verb{More code.}
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("code blocks work", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @description
    #' Before
    #' ```
    #' x %in% 1:10
    #' ```
    #' After
    #' @md
    foo <- function() {}")[[1]]
  expect_snapshot_output(cat(out1$get_value("description")))

  # And check that extra empty paragraphs don't affect the output
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @description
    #' Before
    #'
    #' ```
    #' x %in% 1:10
    #' ```
    #'
    #' After
    #' @md
    foo <- function() {}")[[1]]
  expect_equal(out1$get_value("description"), out2$get_value("description"))
})

test_that("code block with language creates HTML tag", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @description
    #' Before
    #' ```r
    #' x %in% 1:10
    #' ```
    #' After
    #' @md
    foo <- function() {}")[[1]]
  expect_snapshot_output(cat(out1$get_value("description")))
})

test_that("inline code escapes %", {
  expect_equal(markdown("`5%`"), "\\verb{5\\%}")
  expect_equal(markdown("`'5%'`"), "\\code{'5\\%'}")
  expect_equal(markdown("`%*%`"), "\\code{\\%*\\%}")
})

test_that("inline verbatim escapes Rd special chars", {
  expect_equal(markdown("`{`"), "\\verb{\\{}")
  expect_equal(markdown("`}`"), "\\verb{\\}}")
  expect_equal(markdown("`\\`"), "\\verb{\\\\}")
})

test_that("special operators get \\code{}, not \\verb{}", {
  expect_equal(markdown("`if`"), "\\code{if}")
})

test_that("inline code works with < and >", {
  out <- roc_proc_text(rd_roclet(), "
    #' `SELECT <name> FROM <table>`
    #' @md
    f <- function() 1
  ")[[1]]

  expect_equal(out$get_value("title"), "\\verb{SELECT <name> FROM <table>}")
})


# lists -------------------------------------------------------------------

test_that("itemized lists work", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' * itemized
    #' * list
    #'
    #' And then another one:
    #' * item 1
    #' * item 2
    #' * item 3
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' \\itemize{
    #' \\item itemized
    #' \\item list
    #' }
    #'
    #' And then another one:
    #' \\itemize{
    #' \\item item 1
    #' \\item item 2
    #' \\item item 3
    #' }
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("numbered lists work", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' 1. numbered
    #' 2. list
    #'
    #' And then another one:
    #' 1. item 1
    #' 1. item 2
    #' 1. item 3
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' \\enumerate{
    #' \\item numbered
    #' \\item list
    #' }
    #'
    #' And then another one:
    #' \\enumerate{
    #' \\item item 1
    #' \\item item 2
    #' \\item item 3
    #' }
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("nested lists are OK", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' 1. numbered
    #'    * itemized
    #'    * sublist
    #' 2. list
    #'
    #' And then another one:
    #' * item 1
    #' * item 2
    #'    * sublist
    #'    * within
    #' * item 3
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some
    #' \\enumerate{
    #' \\item numbered
    #'   \\itemize{
    #'      \\item itemized
    #'      \\item sublist
    #'   }
    #' \\item list
    #' }
    #'
    #' And then another one:
    #' \\itemize{
    #' \\item item 1
    #' \\item item 2
    #'    \\itemize{
    #'      \\item sublist
    #'      \\item within
    #'    }
    #' \\item item 3
    #' }
    #' @md
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})


# html --------------------------------------------------------------------

test_that("can insert block and inline html", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' <p>This is a paragraph</p>
    #' <p>This is another paragraph</p>
    #' @md
    foo <- function() {}")[[1]]
  expect_snapshot(out$get_section("description"))

  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' This is a paragraph containing a manually inserted image
    #' before-<img src='foo.png'>-after
    #' @md
    foo <- function() {}")[[1]]
  expect_snapshot(out$get_section("description"))
})

# tables ------------------------------------------------------------------


test_that("can convert table to Rd", {
  txt <- "
    | x   | y   |
    | --- | --- |
    | 1   | 2   |

    | x   | y   |
    | :-: | --: |
    | 1   | 2   |

    | x     | y         |
    | ----- | --------- |
    | 1 _2_ | 3 *4* `5` |
  "
  txt <- gsub("\n    ", "\n", txt)
  tables <- strsplit(txt, "\n\n")[[1]]

  expect_snapshot({
    for (table in tables) {
      cat_line(table)
      cat_line(markdown(table))
      cat_line()
    }
  })
})

# inline formatting -------------------------------------------------------

test_that("emphasis works", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some _emphasis_ included. _More emph._
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some \\emph{emphasis} included. \\emph{More emph.}
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("strong (bold) text works", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some **bold** included. **More bold.**
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some \\strong{bold} included. \\strong{More bold.}
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("markdown links are converted", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [http://acme.com]() for details.
    #' And here is a named link: [igraph](http://igraph.org).
    #' Here is another kind of link: <https://log.r-hub.io>.
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\url{http://acme.com} for details.
    #' And here is a named link: \\href{http://igraph.org}{igraph}.
    #' Here is another kind of link: \\url{https://log.r-hub.io}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("images are recognized", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description
    #'
    #' Details with a plot: ![](example.jpg \"Plot title\")
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description
    #'
    #' Details with a plot: \\figure{example.jpg}{Plot title}
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("markdown is parsed in all fields where it is supported", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' @title Title **with bold**
    #'
    #' @description Description **with bold**
    #'
    #' @details Details **with bold**
    #'
    #' @references References **with bold**
    #'
    #' @note Note **with bold**
    #'
    #' @seealso See also **with bold**
    #'
    #' @return Return **with bold**
    #'
    #' @author Author **with bold**
    #'
    #' @section Foobar:
    #' With some **bold text**.
    #'
    #' @format Format **with bold**
    #'
    #' @source Source **with bold**
    #'
    #' @param param Param **with bold**
    #'
    #' @slot slot Slot **with bold**
    #'
    #' @field field Field **with bold**
    #'
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' @title Title \\strong{with bold}
    #'
    #' @description Description \\strong{with bold}
    #'
    #' @details Details \\strong{with bold}
    #'
    #' @references References \\strong{with bold}
    #'
    #' @note Note \\strong{with bold}
    #'
    #' @seealso See also \\strong{with bold}
    #'
    #' @return Return \\strong{with bold}
    #'
    #' @author Author \\strong{with bold}
    #'
    #' @section Foobar:
    #' With some \\strong{bold text}.
    #'
    #' @format Format \\strong{with bold}
    #'
    #' @source Source \\strong{with bold}
    #'
    #' @param param Param \\strong{with bold}
    #'
    #' @slot slot Slot \\strong{with bold}
    #'
    #' @field field Field \\strong{with bold}
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})


test_that("markdown emphasis is ok", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some *keywords* included.
    #' So far so good. \\preformatted{ *these are not
    #'   emphasised*. Or are they?
    #' }
    #' @md
    foo <- function() {}")[[1]]
  desc1 <- "Description with some \\emph{keywords} included.
So far so good. \\preformatted{ *these are not
  emphasised*. Or are they?
}"
  expect_equal(out1$get_section("description")[[2]], desc1)
})

test_that("% is automatically escaped", {
  expect_equal(markdown("5%"), "5\\%")
})

test_that("Escaping is kept", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description. It has \\rd \\commands.
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description. It has \\rd \\commands.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("Do not pick up `` in arguments \\item #519", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description.
    #'
    #' @param `_arg1` should not be code. But `this should`.
    #' @param `_arg2` should not be code, either. `But this.`
    #'
    #' @md
    foo <- function(`_arg1`, `_arg2`) {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description.
    #'
    #' @param `_arg1` should not be code. But \\verb{this should}.
    #' @param `_arg2` should not be code, either. \\verb{But this.}
    #'
    foo <- function(`_arg1`, `_arg2`) {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("unhandled markdown generates warning", {
  text <- "
    #' Title
    #'
    #' > block quotes do not work,
    #' > sadly
    #'
    #' Blabla
    #' @md
    #' @name x
    NULL
  "
  expect_snapshot_warning(roc_proc_text(rd_roclet(), text))
})

test_that("level 1 heading in markdown generates warning in some tags", {
  text <- "
    #' Title
    #'
    #' @seealso this and that
    #' # This is not good
    #'
    #' Blabla
    #' @md
    #' @name x
    NULL
  "
  expect_snapshot_warning(roc_proc_text(rd_roclet(), text))
})

test_that("level >2 markdown headings work in @description", {
  text <- "
    #' Title
    #'
    #' @description
    #' ## This is good
    #' yes
    #'
    #' @details
    #' Blabla
    #' @md
    #' @name x
    NULL
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  expect_equal_strings(
    out$get_value("description"),
    "\\subsection{This is good}{\n\nyes\n}"
  )
})

test_that("level >2 markdown headings work in @details", {
  text <- "
    #' Title
    #'
    #' Description.
    #'
    #' @details
    #' ## Heading 2
    #' ### Heading 3
    #' Text.
    #' @md
    #' @name x
    NULL
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  expect_equal_strings(
    out$get_value("details"),
    "\\subsection{Heading 2}{\n\\subsection{Heading 3}{\n\nText.\n}\n\n}"
  )
})

test_that("level >2 markdown headings work in @return", {
  text <- "
    #' Title
    #'
    #' Description.
    #'
    #' @return Even this
    #' ## Can have a subsection.
    #' Yes.
    #' @md
    #' @name x
    NULL
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  expect_equal_strings(
    out$get_value("value"),
    "Even this\n\\subsection{Can have a subsection.}{\n\nYes.\n}"
  )
})

test_that("level 1 heading in @details", {
  text1 <- "
    #' Title
    #'
    #' Description.
    #'
    #' @details
    #' Leading text goes into details.
    #' # This is its own section
    #' ## Can have a subsection
    #' Yes.
    #' # Another section
    #' With text.
    #' @md
    #' @name x
    NULL
  "
  out1 <- roc_proc_text(rd_roclet(), text1)[[1]]
  text2 <- "
    #' Title
    #'
    #' Description.
    #' @details
    #' Leading text goes into details.
    #' @section This is its own section:\\subsection{Can have a subsection}{
    #'
    #' Yes.
    #' }
    #'
    #' @section Another section:With text.
    #' @name x
    NULL
  "
  out2 <- roc_proc_text(rd_roclet(), text2)[[1]]

  # make sure sections are in the same order
  expect_equal(sort(names(out1$sections)), sort(names(out2$sections)))
  out2$sections <- out2$sections[names(out1$sections)]

  expect_equivalent_rd(out1, out2)
})

test_that("headings and empty sections", {
  text1 <- "
    #' Title
    #'
    #' Description.
    #'
    #' @details
    #' # This is its own section
    #' With text.
    #' @md
    #' @name x
    NULL
  "
  out1 <- roc_proc_text(rd_roclet(), text1)[[1]]
  expect_false("details" %in% names(out1$fields))
})

test_that("markdown() on empty input", {
  expect_identical(markdown(""), "")
  expect_identical(markdown("  "), "")
  expect_identical(markdown("\n"), "")
})

test_that("markup in headings", {
  text1 <- "
    #' Title
    #'
    #' Description.
    #'
    #' @details
    #' Leading text goes into details.
    #' # Section with `code`
    #' ## Subsection with **strong**
    #' Yes.
    #' @md
    #' @name x
    NULL
  "
  out1 <- roc_proc_text(rd_roclet(), text1)[[1]]
  expect_equal(
    out1$get_value("section"),
    list(
      title = "Section with \\code{code}",
      content = paste(
        sep = "\n",
        "\\subsection{Subsection with \\strong{strong}}{",
        "",
        "Yes.",
        "}"
      )
    )
  )
})

test_that("alternative knitr engines", {
  expect_snapshot(
    print(out1 <- roc_proc_text(rd_roclet(), "
      #' Title
      #'
      #' Description.
      #'
      #' ```{verbatim}
      #' #| file = testthat::test_path(\"example.Rmd\")
      #' ```
      #' @md
      #' @name x
      NULL
    "))
  )
})

test_that("can override default options", {
  local_roxy_meta_set("knitr_chunk_options", list(comment = "###"))

  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' ```{r}
    #' 1+1
    #' ```
    #' @md
    foo <- function() { }
  ")[[1]]
  expect_match(out$get_section("description")$value, "###", fixed = TRUE)
})

test_that("image formats work", {
  expect_snapshot(
    roc_proc_text(rd_roclet(), "
      #' Title
      #'
      #' ![](example.svg \"Plot title 1\")
      #' ![](example.pdf \"Plot title 2\")
      #' ![](example.PNG \"Plot title 3\")
      #' @md
      foo <- function() { }
    ")[[1]]
  )
})

test_that("@example loads from specified files", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #'
    #' @example Rd-example-1.R
    #' @example Rd-example-2.R
    NULL"
  )[[1]]

  examples <- out$get_value("examples")
  expect_match(examples, "example <- 'example1'", all = FALSE, fixed = TRUE)
  expect_match(examples, "example <- 'example2'", all = FALSE, fixed = TRUE)
})

test_that("@example captures examples (#470)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #' @examples
    #' TRUE
    #' @examples
    #' FALSE
    NULL"
  )[[1]]

  examples <- out$get_value("examples")
  expect_equal(examples, rd(c("TRUE", "FALSE")))
})

test_that("@examples and @example interleave", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #' @example Rd-example-1.R
    #' @examples a <- 2
    #' @example Rd-example-2.R
    NULL"
  )[[1]]

  expect_snapshot_output(out$get_section("examples"))
})

test_that("@example does not introduce extra empty lines", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #' @example Rd-example-3.R
    NULL"
  )[[1]]

  expect_equal(re_count(out$get_value("examples"), "\n"), 1L)
})

test_that("@example gives warning if used instead of @examples", {
  block <- "
    #' @name a
    #' @title a
    #' @example
    #' a <- 1
    #' a + b
    NULL
  "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), block)[[1]])
  expect_null(out$get_value("examples"))
})

test_that("warns if path doesn't exist", {
  block <- "
    #' @name a
    #' @title a
    #' @example this-path-doesnt-exist.R
    NULL
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("@examplesIf", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #' @examplesIf foo::bar()
    #' maybe-run-this-code
    #' @examplesIf foobar()
    #' and-this
    #' and-that
    NULL"
  )[[1]]

  expect_snapshot_output(out$get_section("examples"))
})

test_that("@examplesIf warns about unparseable condition", {
  block <- "
    #' @name a
    #' @title a
    #' @examplesIf 1 +
    #' maybe-run-this-code
    NULL
  "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), block))
  expect_equal(out[[1]]$get_section("examples"), NULL)
})

test_that("@examplesIf warns on empty body (#1695)", {
  block <- "
    #' @name a
    #' @title a
    #' @examplesIf interactive()
    NULL
  "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), block))
  expect_equal(out[[1]]$get_section("examples"), NULL)
})

test_that("strings in R comments don't affect brace matching (#1492)", {
  block <- "
    #' @name a
    #' @title a
    #' @examples
    #' # {greeting}'
    NULL"

  expect_silent(out <- roc_proc_text(rd_roclet(), block)[[1]])
  expect_equal(out$get_value("examples"), rd("# {greeting}'"))
})

test_that("braces don't need to match inside of raw strings (#1492)", {
  block <- "
    #' @name a
    #' @title a
    #' @examples
    #' r'( '{{ )'
    NULL
  "
  expect_silent(out <- roc_proc_text(rd_roclet(), block)[[1]])
  expect_equal(out$get_value("examples"), rd("r'( '{{ )'"))
})

test_that("% in @examples escaped before matching braces test (#213)", {
  block <- "
    #' @name a
    #' @title a
    #' @examples
    #' {a %% b}
    NULL
  "
  out <- roc_proc_text(rd_roclet(), block)[[1]]
  expect_equal(out$get_value("examples"), rd("{a \\%\\% b}"))
})

# strip_rd_example_tags -----------------------------------------------------

test_that("strip_rd_example_tags unwraps \\dontrun and \\donttest", {
  expect_equal(strip_rd_example_tags("\\dontrun{\n1 + 1\n}"), "1 + 1")
  expect_equal(strip_rd_example_tags("\\donttest{\n1 + 1\n}"), "1 + 1")
})

test_that("strip_rd_example_tags removes \\dontshow entirely", {
  expect_equal(strip_rd_example_tags("\\dontshow{\n1 + 1\n}"), "")
})

test_that("strip_rd_example_tags preserves surrounding code", {
  expect_equal(
    strip_rd_example_tags("x <- 1\n\\dontrun{\n1 + 1\n}\ny <- 2"),
    "x <- 1\n1 + 1\ny <- 2"
  )
})

test_that("strip_rd_example_tags handles multibyte characters", {
  expect_equal(
    strip_rd_example_tags("x <- 'café'\n\\dontrun{\ny <- 'naïve'\n}"),
    "x <- 'café'\ny <- 'naïve'"
  )
})

test_that("strip_rd_example_tags handles nested tags", {
  expect_equal(
    strip_rd_example_tags("\\dontrun{\n\\dontshow{setup}\n1 + 1\n}"),
    "1 + 1"
  )
})

test_that("strip_rd_example_tags handles examplesIf wrappers", {
  input <- paste0(
    "\\dontshow{if (TRUE) withAutoprint(\\{ # examplesIf}\n",
    "1 + 1\n",
    "\\dontshow{\\}) # examplesIf}"
  )
  expect_equal(strip_rd_example_tags(input), "1 + 1")
})

test_that("strip_rd_example_tags is no-op with no tags", {
  expect_equal(strip_rd_example_tags("job$run()"), "job$run()")
})

# escapes ------------------------------------------------------------------

test_that("only % escaped in @examples", {
  expect_equal(escape_examples("x %*% y"), rd("x \\%*\\% y"))
  expect_equal(escape_examples("# \\x"), rd("# \\x"))
  expect_equal(escape_examples("'34.00\\'"), rd("'34.00\\'"))
})

test_that("multi-line macros in @example", {
  # https://github.com/r-lib/roxygen2/issues/974
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #'
    #' @example Rd-example-4.txt
    NULL"
  )[[1]]

  expect_equal(
    format(out$get_section("examples")),
    "\\examples{\n\\dontrun{\n1 + 1\n}\n}"
  )
})

test_that("@prop creates Additional properties section", {
  text <- "
    #' Important class.
    #'
    #' @prop a prop a
    #' @prop b prop b
    a <- function() {}
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  value <- out$get_value("prop")
  expect_equal(value$name, c("a", "b"))
  expect_equal(value$description, c("prop a", "prop b"))

  expect_snapshot(out$get_section("prop"))
})

test_that("@prop class@name groups by class", {
  text <- "
    #' Classes.
    #'
    #' @prop Parent@x prop x
    #' @prop Child@y prop y
    a <- function() {}
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  value <- out$get_value("prop")
  expect_equal(value$class, c("Parent", "Child"))
  expect_equal(value$name, c("x", "y"))

  expect_snapshot(out$get_section("prop"))
})

test_that("@prop with mismatched braces warns and doesn't crash", {
  text <- "
    #' A class.
    #'
    #' @prop a prop a
    #' }
    a <- function() {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), text))
})

test_that("@prop class@name warns on invalid spec", {
  text <- "
    #' A class.
    #'
    #' @prop @x prop x
    a <- function() {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), text))
})

test_that("@prop without class doesn't use subsections", {
  text <- "
    #' A class.
    #'
    #' @prop a prop a
    #' @prop b prop b
    MyClass <- function() {}
  "
  out <- roc_proc_text(rd_roclet(), text)[[1]]
  rd <- format(out$get_section("prop"))
  expect_no_match(rd, "subsection")
  expect_match(rd, "Additional properties")
})

test_that("markdown file can be included", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("List:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' @title Title
    #' @details List:
    #' \\itemize{
    #' \\item item1
    #' \\item item2
    #' }
    #'
    #' Inline \\code{code} and \\emph{emphasis}.
    #' @name foobar
    NULL")[[1]]
  # make sure sections are in the same order
  expect_equal(sort(names(out1$sections)), sort(names(out2$sections)))
  out2$sections <- out2$sections[names(out1$sections)]
  expect_equivalent_rd(out1, out2)
})

test_that("markdown with headers", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "# Header 1",
    "",
    "## Header 2",
    "",
    "Text",
    "",
    "## Header 22",
    "",
    "# Header 11",
    "",
    "Text again")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- "Text at the front"
  exp_secs <- list(
    title = c("Header 1", "Header 11"),
    content = c(
      "\\subsection{Header 2}{\n\nText\n}\n\n\\subsection{Header 22}{\n}",
      "Text again"
    )
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
  expect_equal_strings(out1$get_value("section"), exp_secs)
})

test_that("subsection within details", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "",
    "## Subsection in details",
    "",
    "Some subsection text")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\subsection{Subsection in details}{ Some subsection text }"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("links to functions", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "This is a link: [roxygenize()].",
    "Another one: [stringr::str_length()]"
  )
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "This is a link: \\code{\\link[=roxygenize]{roxygenize()}}.",
    "Another one:\n\\code{\\link[stringr:str_length]{stringr::str_length()}}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("links to functions, with anchors", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "This is a link: [roxygenize()].",
    "Another one: [stringr::str_length()]",
    "",
    "[roxygenize()]: https://roxygen2.r-lib.org/reference/roxygenize.html",
    "[stringr::str_length()]: https://stringr.tidyverse.org/reference/str_length.html"
  )
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "This is a link: \\code{\\link[=roxygenize]{roxygenize()}}.",
    "Another one:\n\\code{\\link[stringr:str_length]{stringr::str_length()}}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("empty Rmd", {
  tmp <- tempfile()
  on.exit(unlink(tmp), add = TRUE)
  tag <- roxy_tag("includeRmd", tmp)

  cat("", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))

  cat("  ", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))

  cat("\n", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))
})

test_that("inline html", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "",
    "## Subsection in details",
    "",
    "Some subsection text with <span>inline html</span>.")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\subsection{Subsection in details}{",
    "Some subsection text with ",
    "\\if{html}{\\out{<span>}}inline html\\if{html}{\\out{</span>}}.\n}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("html block", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "<a id=\"test\"></a>",
    "",
    "Text")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\if{html}{\\out{<a id=\"test\">}}\\if{html}{\\out{</a>}}",
    "Text"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("include as another section", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("List:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @includeRmd %s description
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' @title Title
    #' @description List:
    #' \\itemize{
    #' \\item item1
    #' \\item item2
    #' }
    #'
    #' Inline \\code{code} and \\emph{emphasis}.
    #' @name foobar
    NULL")[[1]]
  # make sure sections are in the same order
  expect_equal(sort(names(out1$sections)), sort(names(out2$sections)))
  out2$sections <- out2$sections[names(out1$sections)]
  expect_equivalent_rd(out1, out2)
})

test_that("order of sections is correct", {
  skip_if_not(rmarkdown::pandoc_available())

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("# Rmd\n\nList:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @description desc
    #' @details details
    #' @includeRmd %s
    #' @section After:
    #' This is after.
    #' @section After2:
    #' This is even more after.
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  expect_match(format(out1), "Rmd.*After.*After2")
})

test_that("invalid syntax generates useful warning", {
  block <- "
    #' A
    #' @field
    #' @slot
    a <- function() {}
  "

  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("@fields creates a new section and lists fields", {
  out <- roc_proc_text(rd_roclet(), "
    #' Important class.
    #'
    #' @field a field a
    #'
    #' @field b field b
    #'
    setRefClass('test')
    ")[[1]]
  expect_equal(out$get_value("field"), c(a = "field a", b = "field b"))
})

test_that("@slot creates a new section and lists slots", {
  out <- roc_proc_text(rd_roclet(), "
      #' Important class.
      #'
      #' @slot a slot a
      #' @slot b slot b
      setClass('test')
    ")[[1]]
  expect_equal(out$get_value("slot"), c(a = "slot a", b = "slot b"))
})

test_that("generic keys produce expected output", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @title a
    #' @note test
    #' @author test
    #' @seealso test
    #' @references test
    #' @name a
    NULL"
  )[[1]]
  expect_equal(out$get_value("references"), "test")
  expect_equal(out$get_value("note"), "test")
  expect_equal(out$get_value("seealso"), "test")
  expect_equal(out$get_value("author"), "test")
})


test_that("author duplicated get removed", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @title a
    #' @author A
    NULL

    #' @name b
    #' @rdname a
    #' @author A
    NULL

    #' @name c
    #' @rdname a
    #' @author B
    NULL"
  )[[1]]
  expect_equal(out$get_value("author"), c("A", "B"))
})

test_that("@format overrides defaults", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #'
    #' @format abc
    #'
    x <- list(a = 1, b = 2)"
  )[[1]]

  expect_equal(out$get_value("format"), "abc")
})

test_that("@format NULL suppresses default usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #'
    #' @format NULL
    x <- list(a = 1, b = 2)"
  )[[1]]

  expect_equal(out$get_value("format"), NULL)
})

test_that("@format not escaped", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @format %
    x <- list(a = 1, b = 2)"
  )[[1]]

  expect_equal(out$get_value("format"), "%")
  expect_equal(out$get_rd("format"), "\\format{\n%\n}")
})


test_that("@title warns about multiple paragraphs", {
  block <- "
    #' @title Paragraph 1
    #'
    #' Paragraph 2
    x <- list(a = 1, b = 2)
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("integration test", {
  path <- local_package_copy(test_path("testR6"))
  suppressMessages(roxygenise(path))
  withr::defer(pkgload::unload("testR6"))

  rd_files <- sort(dir(file.path(path, "man"), full.names = TRUE))
  for (rd_file in rd_files) {
    expect_snapshot(cat(read_lines(rd_file), sep = "\n"))
    expect_silent(chk <- tools::checkRd(rd_file))
    expect_equal(length(chk), 0L)
  }
})

test_that("multiple R6 classes in one topic (@rdname) produce valid Rd", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @name shared
    NULL

    #' @rdname shared
    A <- R6::R6Class('A',
      public = list(
        #' @field x Field x.
        x = NULL,
        #' @description Method a.
        meth_a = function() {}
      )
    )

    #' @rdname shared
    B <- R6::R6Class('B',
      public = list(
        #' @field y Field y.
        y = NULL,
        #' @description Method b.
        meth_b = function() {}
      )
    )
    "
  )[[1]]

  rd <- out$get_rd("rawRd")
  expect_match(rd, "\\item{\\code{x}}{Field x.}", fixed = TRUE)
  expect_match(rd, "\\item{\\code{y}}{Field y.}", fixed = TRUE)
  expect_match(rd, "A$meth_a", fixed = TRUE)
  expect_match(rd, "B$meth_b", fixed = TRUE)
})

test_that("markdown is off by default", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some `code` included. `More code.`"
  )
})

test_that("turning on/off markdown globally", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some `code` included. `More code.`"
  )

  old <- roxy_meta_set("markdown", TRUE)
  on.exit(roxy_meta_set("markdown", old))
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some \\code{code} included. \\verb{More code.}"
  )
})

test_that("turning on/off markdown locally", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    #' @noMd
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some `code` included. `More code.`"
  )

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    #' @md
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some \\code{code} included. \\verb{More code.}"
  )

  old <- roxy_meta_set("markdown", TRUE)
  on.exit(roxy_meta_set("markdown", old))
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    #' @noMd
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some `code` included. `More code.`"
  )

  ## on / on
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description with some `code` included. `More code.`
    #' @md
    foo <- function() {}")[[1]]
  expect_equal(
    out1$get_value("description"),
    "Description with some \\code{code} included. \\verb{More code.}"
  )

})

test_that("warning for both @md and @noMd", {
  block <- "
    #' Title
    #'
    #' `code`
    #' @md
    #' @noMd
    foo <- function() {}
  "

  expect_snapshot_warning(out1 <- roc_proc_text(rd_roclet(), block))
  expect_equal(out1[[1]]$get_value("description"), "`code`")

  # No translation even if markdown on generally
  local_markdown()
  suppressWarnings(out2 <- roc_proc_text(rd_roclet(), block))
  expect_equal(out2[[1]]$get_value("description"), "`code`")
})

test_that("can find template from name", {
  base <- test_path("templates/")

  expect_equal(
    template_find(base, "UCase"),
    file.path(base, "man-roxygen", "UCase.R")
  )

  # On case-insentive file systems, will find upper case version first
  expect_equal(
    tolower(template_find(base, "lcase")),
    tolower(file.path(base, "man-roxygen", "lcase.r"))
  )

  expect_equal(
    template_find(base, "new-path"),
    file.path(base, "man" , "roxygen", "templates", "new-path.R")
  )

  expect_error(
    template_find(base, "missing"),
    "Can't find template"
  )
})

test_that("templates gives useful error if not found", {
  block <- "
    #' @template doesn't-exist
    x <- 10
  "
  expect_snapshot(roc_proc_text(rd_roclet(), block), error = TRUE)
})

test_that("templates replace variables with their values", {
  out <- roc_proc_text(rd_roclet(), "
    #' @template values
    #' @templateVar x a
    #' @templateVar y b
    #' @templateVar z c
    x <- 10")[[1]]

  expect_equal(out$get_value("title"), "a")
  expect_equal(out$get_value("param"), c(b = "c"))
})

test_that("@usage overrides default", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @usage a(a=2)
    a <- function(a=1) {}"
  )[[1]]
  expect_equal(out$get_value("usage"), rd("a(a=2)"))
})

test_that("@usage overrides default for @docType data", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    #'
    #' @name abc
    #' @docType data
    #' @usage data(abc)
    NULL"
  )[[1]]

  expect_equal(out$get_value("usage"), rd("data(abc)"))
})

test_that("@usage NULL suppresses default usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @usage NULL
    a <- function(a=1) {}"
  )[[1]]

  expect_equal(out$get_value("usage"), NULL)
})

test_that("quoted topics have usage statements", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    \"f\" <- function(a = 1, b = 2, c = a + b) {}"
  )[[1]]

  expect_equal(out$get_value("usage"), rd("f(a = 1, b = 2, c = a + b)"))
  expect_equal(
    out$get_rd("usage"),
    r"(\usage{
f(a = 1, b = 2, c = a + b)
})"
  )
})

# Escaping --------------------------------------------------------------------

test_that("usage escaping preserved when combined", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    foo <- function(x = '%') x

    #' @rdname foo
    bar <- function(y = '%') y
  "
  )[[1]]

  expect_s3_class(out$get_value("usage"), "rd")
})

test_that("default usage not double escaped", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Regular
    mean.foo <- function(x) 'foo'
  "
  )[[1]]

  expect_equal(
    out$get_rd("usage"),
    r"(\usage{
\method{mean}{foo}(x)
})"
  )
})

test_that("% and \\ are escaped in usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    a <- function(a='%\\\\') {}"
  )[[1]]
  expect_equal(out$get_value("usage"), escape('a(a = "%\\\\")'))
  expect_equal(
    out$get_rd("usage"),
    r"[\usage{
a(a = "\%\\\\")
}]"
  )
})

test_that("% and \\ not escaped in manual usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    #' @usage %\\
    a <- function(a) {}
  "
  )[[1]]
  expect_equal(out$get_value("usage"), rd('%\\'))
  expect_equal(
    out$get_rd("usage"),
    r"(\usage{
%\
})"
  )
})

test_that("Special vars removed in rc methods usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Class Blob
    ABCD <- setRefClass('ABC', methods = list(
      draw = function(x = 1) {
        \"2\"
        x
      })
    )
  "
  )[[1]]

  expect_equal(out$get_value("rcmethods"), list("draw(x = 1)" = "2"))
})

# object_usage ------------------------------------------------------------

test_that("usage captured from formals", {
  expect_equal(
    call_to_usage(f <- function() {}),
    "f()"
  )
  expect_equal(
    call_to_usage(f <- function(a = 1) {}),
    "f(a = 1)"
  )
})

test_that("argument containing function is generates correct usage", {
  expect_equal(
    call_to_usage(f <- function(a = function(x) 1) {}),
    "f(a = function(x) 1)"
  )
})

test_that("backticks retained when needed", {
  expect_equal(
    call_to_usage(f <- function(`_a`) {}),
    "f(`_a`)"
  )

  expect_equal(
    call_to_usage(`-f` <- function(x) {}),
    "`-f`(x)"
  )
})

test_that("% escaped when not in infix function", {
  expect_equal(
    call_to_usage(`%foo%bar` <- function(x, table) {}),
    r"(`\%foo\%bar`(x, table))"
  )
  expect_equal(
    call_to_usage(`%foo%bar<-` <- function(x, value) {}),
    r"(`\%foo\%bar`(x) <- value)"
  )
})

test_that("default usage formats data correctly", {
  expect_equal(
    call_to_usage(hello <- 1),
    "hello"
  )
})

test_that("default usage formats replacement functions correctly", {
  expect_equal(
    call_to_usage(`f<-` <- function(x, value) {}),
    "f(x) <- value"
  )
  expect_equal(
    call_to_usage(`f<-` <- function(x, y, value) {}),
    "f(x, y) <- value"
  )
})

test_that("default usage formats infix functions correctly", {
  expect_equal(call_to_usage("%.%" <- function(a, b) {}), r"(a \%.\% b)")
  expect_equal(call_to_usage(":" <- function(a, b) {}), "a:b")
  expect_equal(call_to_usage("+" <- function(a, b) {}), "a + b")

  # even if it contains <-
  expect_equal(call_to_usage("%<-%" <- function(a, b) {}), r"(a \%<-\% b)")

  # defaults are ignored
  expect_equal(call_to_usage(":" <- function(a = 1, b = 2) {}), "a:b")
})

test_that("default usage formats S3 methods correctly", {
  expect_equal(
    call_to_usage(mean.foo <- function(x) {}),
    r"(\method{mean}{foo}(x))"
  )
  expect_equal(
    call_to_usage(mean.function <- function(x) {}),
    r"(\method{mean}{`function`}(x))"
  )
  expect_equal(
    call_to_usage("+.foo" <- function(x, b) {}),
    r"(\method{+}{foo}(x, b))"
  )
  expect_equal(
    call_to_usage("%%.foo" <- function(x, b) {}),
    r"(\method{\%\%}{foo}(x, b))"
  )
  expect_equal(
    call_to_usage("[<-.foo" <- function(x, value) {}),
    r"(\method{[}{foo}(x) <- value)"
  )
})

test_that("S4 classes have no default usage", {
  expect_equal(
    call_to_usage({
      setClass("Foo")
    }),
    character()
  )
})

test_that("default usage correct for S4 generics", {
  expect_equal(
    call_to_usage({
      setGeneric("foo", function(x, y) {})
    }),
    "foo(x, y)"
  )
})

test_that("default usage correct for S4 methods", {
  expect_equal(
    call_to_usage({
      setClass("Foo")
      setMethod("sum", "Foo", function(x, ..., na.rm = FALSE) {})
    }),
    r"(\S4method{sum}{Foo}(x, ..., na.rm = FALSE))"
  )

  expect_equal(
    call_to_usage({
      setClass("Foo")
      setMethod("+", "Foo", function(e1, e2) "foo")
    }),
    r"(\S4method{+}{Foo,ANY}(e1, e2))"
  )

  expect_equal(
    call_to_usage({
      setClass("Foo")
      setMethod("[<-", "Foo", function(x, i, j, ..., value) "foo")
    }),
    r"(\S4method{[}{Foo}(x, i, j, ...) <- value)"
  )

  expect_equal(
    call_to_usage({
      setGeneric("%&&%", function(x, y) standardGeneric("%&&%"))
      setMethod("%&&%", signature("logical", "logical"), function(x, y) {})
    }),
    r"(\S4method{\%&&\%}{logical,logical}(x, y))"
  )
})

test_that("default usage correct for S4 coercion methods", {
  expect_equal(
    call_to_usage({
      setClass("Foo", representation(x = "numeric"))
      setAs("numeric", "Foo", function(from) new("Foo", x = from))
    }),
    r"(\S4method{coerce}{numeric,Foo}(from, to = "Foo", strict = TRUE))"
  )
})

test_that("default usage correct for S4 methods with different args to generic", {
  expect_equal(
    call_to_usage({
      setGeneric("testfun", function(x, ...) standardGeneric("testfun"))
      setMethod("testfun", "matrix", function(x, add = FALSE, ...) {
        x - 1
      })
    }),
    r"(\S4method{testfun}{matrix}(x, add = FALSE, ...))"
  )
})

test_that("non-syntactic S4 class names are not escaped in usage", {
  expect_equal(
    call_to_usage({
      setGeneric("rhs", function(x) standardGeneric("rhs"))
      setMethod("rhs", "<-", function(x) x[[3]])
    }),
    r"(\S4method{rhs}{<-}(x))"
  )
})


# Wrapping --------------------------------------------------------------------

test_that("new wrapping style doesn't change unexpectedly", {
  expect_snapshot_output({
    cat(
      call_to_usage({
        f <- function(
          a = '                                    a',
          b = '                                    b',
          c = '                                    c',
          d = '                                    d'
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        f <- function(
          a = c(
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef'
          )
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        mean.reallyratherquitelongclassname <-
          function(
            reallyreatherquitelongargument = 'reallyratherquitelongvalue_____________________'
          ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        `long_replacement_fun<-` <- function(
          x,
          a = 'aaaaaaaaaaaaaaaa',
          b = 'aaaaaaaaaaaaaaaa',
          c = 'aaaaaaaaaaaaaaaa',
          value
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        function_name <- function(
          x,
          y,
          xy = "abcdef",
          xyz = c(
            `word word word word` = "abcdef",
            `word word word` = "abcdef",
            `word word word` = "abcdef",
            `word word word` = "abcdef"
          )
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        function_name <- function(
          f = function(x) {
            1
            2
          }
        ) {}
      }),
      "\n\n"
    )
  })
})

test_that("old wrapping style doesn't change unexpectedly", {
  local_roxy_meta_set("old_usage", TRUE)

  expect_snapshot_output({
    cat(
      call_to_usage({
        f <- function(
          a = '                                    a',
          b = '                                    b',
          c = '                                    c',
          d = '                                    d'
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        f <- function(
          a = c(
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef',
            'abcdef'
          )
        ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        mean.reallyratherquitelongclassname <-
          function(
            reallyreatherquitelongargument = 'reallyratherquitelongvalue_____________________'
          ) {}
      }),
      "\n\n"
    )

    cat(
      call_to_usage({
        `long_replacement_fun<-` <- function(
          x,
          a = 'aaaaaaaaaaaaaaaa',
          b = 'aaaaaaaaaaaaaaaa',
          c = 'aaaaaaaaaaaaaaaa',
          value
        ) {}
      }),
      "\n\n"
    )

    # breaking works after escapes (#265)
    cat(
      call_to_usage({
        f <- function(
          xxxxxxxxxxxxxxxxxx1,
          xxxxxxxxxxxxxxxxxx2,
          xxxxxxxxxxxxxxxxxx3,
          x = "\"'",
          xxxxxxxxxxxxxxxxxx4,
          xxxxxxxxxxxxxxxxxx5,
          xxxxxxxxxxxxxxxxxx6,
          xxxxxxxxxxxxxxxxxx7
        ) {}
      }),
      "\n\n"
    )
  })
})

test_that("S7 class usage is constructor", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      Dog <- S7::new_class(
        "Dog",
        properties = list(
          name = S7::class_character,
          age = S7::class_numeric
        )
      )
    }),
    "Dog(name = character(0), age = integer(0))"
  )
})

test_that("S7 generic usage is function signature", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      speak <- S7::new_generic("speak", "x")
    }),
    "speak(x, ...)"
  )
})

test_that("S7 method usage includes comment", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      Dog <- S7::new_class("Dog")
      speak <- S7::new_generic("speak", "x")
      S7::method(speak, Dog) <- function(x) "Woof"
    }),
    "## S7 method for class <Dog>\nspeak(x)"
  )
})

test_that("S7 multi-dispatch method usage includes all classes", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      Dog <- S7::new_class("Dog")
      Cat <- S7::new_class("Cat")
      greet <- S7::new_generic("greet", c("x", "y"))
      S7::method(greet, list(Dog, Cat)) <- function(x, y) "hi"
    }),
    "## S7 method for classes <Dog>, <Cat>\ngreet(x, y)"
  )
})

test_that("S7 union method usage shows member classes", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      Dog <- S7::new_class("Dog")
      Cat <- S7::new_class("Cat")
      Pet <- S7::new_union(Dog, Cat)
      speak <- S7::new_generic("speak", "x")
      S7::method(speak, Pet) <- function(x) "hi"
    }),
    "## S7 method for class <Dog>/<Cat>\nspeak(x)"
  )
})

test_that("S7 bracket methods use subsetting syntax", {
  skip_unless_r(">= 4.3.0")
  expect_equal(
    call_to_usage({
      Vec <- S7::new_class("Vec")
      `[` <- S7::new_generic("[", "x")
      S7::method(`[`, Vec) <- function(x, i, ...) x
    }),
    "## S7 method for class <Vec>\nx[i, ...]"
  )
  expect_equal(
    call_to_usage({
      Vec <- S7::new_class("Vec")
      `[<-` <- S7::new_generic("[<-", "x")
      S7::method(`[<-`, Vec) <- function(x, i, ..., value) x
    }),
    "## S7 method for class <Vec>\nx[i, ...] <- value"
  )
  expect_equal(
    call_to_usage({
      Vec <- S7::new_class("Vec")
      `[[` <- S7::new_generic("[[", "x")
      S7::method(`[[`, Vec) <- function(x, i) x
    }),
    "## S7 method for class <Vec>\nx[[i]]"
  )
  expect_equal(
    call_to_usage({
      Vec <- S7::new_class("Vec")
      `[[<-` <- S7::new_generic("[[<-", "x")
      S7::method(`[[<-`, Vec) <- function(x, i, value) x
    }),
    "## S7 method for class <Vec>\nx[[i]] <- value"
  )
})

test_that("preserves non-breaking-space", {
  expect_equal(
    call_to_usage(f <- function(a = "\u{A0}") {}),
    'f(a = "\u{A0}")'
  )
})

test_that("long families are wrapped", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @family Long family name
    #'
    long_function_name_________________________1 <- function() {}

    #' Title
    #' @family Long family name
    long_function_name_________________________2 <- function() {}

    #' Title
    #' @family Long family name
    #'
    long_function_name_________________________3 <- function() {}

    #' Title
    #' @family Long family name
    long_function_name_________________________4 <- function() {}
  "
  )[[1]]

  seealso <- out$get_value("seealso")
  expect_true(grepl("^Other Long family name:", seealso))
  expect_equal(re_count(seealso, "\n"), 3)
})

test_that("special names escaped in family tag", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @family Long family name
    f <- function() {}

    #' Title
    #' @family Long family name
    '%+%' <- function(a, b) {}
  "
  )[[1]]

  seealso <- out$get_value("seealso")
  expect_true(grepl("^Other Long family name:", seealso))
  expect_match(seealso, "\\\\%\\+\\\\%")
})

test_that("family links to name only, not all aliases", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @aliases f2 f3
    #'
    #' @family many aliases
    f <- function() {}

    #' Title
    #' @aliases g2 g3
    #'
    #' @family many aliases
    g <- function() {}
  "
  )[[1]]

  seealso <- out$get_value("seealso")
  expect_true(grepl("^Other many aliases:", seealso))
  expect_match(seealso, r"(\code{\link)", fixed = TRUE)
})

test_that("families listed in same order as input", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo
    #' @family a
    foo <- function() {}

    #' foo
    #' @family b
    #' @family a
    bar <- function() {}

    #' foo
    #' @family b
    baz <- function() {}
  "
  )[[2]]

  seealso <- out$get_value("seealso")
  expect_match(seealso[1], "^Other b")
  expect_match(seealso[2], "^Other a")
})

test_that("only functions get () suffix", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo
    #' @family a
    foo <- function() {}

    #' bar
    #' @family a
    bar <- 1:10
  "
  )

  expect_equal(
    out[[1]]$get_value("seealso"),
    "Other a:\n\\code{\\link{bar}}"
  )
  expect_equal(
    out[[2]]$get_value("seealso"),
    "Other a:\n\\code{\\link[=foo]{foo()}}"
  )
})

test_that("family also included in concepts", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo
    #' @family a
    foo <- function() {}
  "
  )[[1]]

  expect_equal(out$get_value("concept"), "a")
})

test_that("custom family prefixes can get colon if needed", {
  seealso <- function() {
    out <- roc_proc_text(
      rd_roclet(),
      "
      #' foo
      #' @family a
      foo <- function() {}

      #' bar
      #' @family a
      bar <- function() {}
    "
    )[[1]]

    out$get_value("seealso")
  }

  local_roxy_meta_set("rd_family_title", list(a = "Custom prefix"))
  expect_match(seealso(), "^Custom prefix:")

  local_roxy_meta_set("rd_family_title", list(a = "Custom prefix:"))
  expect_match(seealso(), "^Custom prefix:[^:]")
})

test_that("custom family prefixes can include Markdown", {
  local_roxy_meta_set(
    "rd_family_title",
    list(a = "Custom ***strongly emphasized*** prefix")
  )
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo
    #' @family a
    foo <- function() {}

    #' bar
    #' @family a
    bar <- function() {}
  "
  )[[1]]

  expect_match(
    out$get_value("seealso"),
    "^Custom \\\\emph\\{\\\\strong\\{strongly emphasized\\}} prefix:"
  )
})

test_that("@family with @rdname doesn't produce duplicate seealso (#1530)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #' @family a
    foo <- function() {}

    #' @rdname foo
    #' @family a
    bar <- function() {}

    #' baz
    #' @family a
    baz <- function() {}
  "
  )

  seealso <- out[["foo.Rd"]]$get_value("seealso")
  expect_length(seealso, 1)
  expect_match(seealso, "^Other a:")
})

test_that("careful ordering", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo1
    #' @family a
    foo1 <- function() {}

    #' foo2
    #' @family a
    foo2 <- function() {}

    #' Foo3
    #' @family a
    Foo3 <- function() {}

    #' foo
    #' @family a
    foo <- function() {}
  "
  )

  expect_snapshot({
    out
  })
})

test_that("markdown file can be included", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("List:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' @title Title
    #' @details List:
    #' \\itemize{
    #' \\item item1
    #' \\item item2
    #' }
    #'
    #' Inline \\code{code} and \\emph{emphasis}.
    #' @name foobar
    NULL")[[1]]
  # make sure sections are in the same order
  expect_equal(sort(names(out1$sections)), sort(names(out2$sections)))
  out2$sections <- out2$sections[names(out1$sections)]
  expect_equivalent_rd(out1, out2)
})

test_that("markdown with headers", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "# Header 1",
    "",
    "## Header 2",
    "",
    "Text",
    "",
    "## Header 22",
    "",
    "# Header 11",
    "",
    "Text again")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- "Text at the front"
  exp_secs <- list(
    title = c("Header 1", "Header 11"),
    content = c(
      "\\subsection{Header 2}{\n\nText\n}\n\n\\subsection{Header 22}{\n}",
      "Text again"
    )
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
  expect_equal_strings(out1$get_value("section"), exp_secs)
})

test_that("subsection within details", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "",
    "## Subsection in details",
    "",
    "Some subsection text")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\subsection{Subsection in details}{ Some subsection text }"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("links to functions", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "This is a link: [roxygenize()].",
    "Another one: [stringr::str_length()]"
  )
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "This is a link: \\code{\\link[=roxygenize]{roxygenize()}}.",
    "Another one:\n\\code{\\link[stringr:str_length]{stringr::str_length()}}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("links to functions, with anchors", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "This is a link: [roxygenize()].",
    "Another one: [stringr::str_length()]",
    "",
    "[roxygenize()]: https://roxygen2.r-lib.org/reference/roxygenize.html",
    "[stringr::str_length()]: https://stringr.tidyverse.org/reference/str_length.html"
  )
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "This is a link: \\code{\\link[=roxygenize]{roxygenize()}}.",
    "Another one:\n\\code{\\link[stringr:str_length]{stringr::str_length()}}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("empty Rmd", {
  tmp <- tempfile()
  on.exit(unlink(tmp), add = TRUE)
  tag <- roxy_test_tag()

  cat("", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))

  cat("  ", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))

  cat("\n", sep = "", file = tmp)
  expect_equal(rmd_eval_rd(tmp, tag), structure("", names = ""))
})

test_that("inline html", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "",
    "## Subsection in details",
    "",
    "Some subsection text with <span class='x'>inline html</span>.")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\subsection{Subsection in details}{",
    "Some subsection text with ",
    "\\if{html}{\\out{<span class=\"x\">}}inline html\\if{html}{\\out{</span>}}.\n}"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("html block", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat(sep = "\n", file = tmp,
    "Text at the front",
    "",
    "<a id=\"test\"></a>",
    "",
    "Text")
  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  exp_details <- paste0(
    "Text at the front",
    "\\if{html}{\\out{<a id=\"test\">}}\\if{html}{\\out{</a>}}",
    "Text"
  )
  expect_equal_strings(out1$get_value("details"), exp_details)
})

test_that("include as another section", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("List:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @includeRmd %s description
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' @title Title
    #' @description List:
    #' \\itemize{
    #' \\item item1
    #' \\item item2
    #' }
    #'
    #' Inline \\code{code} and \\emph{emphasis}.
    #' @name foobar
    NULL")[[1]]
  # make sure sections are in the same order
  expect_equal(sort(names(out1$sections)), sort(names(out2$sections)))
  out2$sections <- out2$sections[names(out1$sections)]
  expect_equivalent_rd(out1, out2)
})

test_that("order of sections is correct", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp, recursive = TRUE), add = TRUE)
  cat("# Rmd\n\nList:\n\n* item1\n* item2\n\nInline `code` and _emphasis_.\n",
      file = tmp)
  rox <- sprintf("
    #' Title
    #' @description desc
    #' @details details
    #' @includeRmd %s
    #' @section After:
    #' This is after.
    #' @section After2:
    #' This is even more after.
    #' @name foobar
    NULL", tmp)
  out1 <- roc_proc_text(rd_roclet(), rox)[[1]]
  expect_match(format(out1), "Rmd.*After.*After2")
})

test_that("useful warnings", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  text <- "
    #' Title
    #' @includeRmd path
    #' @name foobar
    NULL"
  expect_snapshot_warning(roc_proc_text(rd_roclet(), text))

  path <- withr::local_tempfile(fileext = ".Rmd", lines = c(
    "```{r}",
    "stop('Error')",
    "```"
  ))
  path <- normalizePath(path)

  text <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", path
  )
  expect_snapshot(
    . <- roc_proc_text(rd_roclet(), text),
    transform = function(x) gsub(path, "<temp-path.Rmd>", x, fixed =TRUE)
  )
})

test_that("sets width", {
  skip_if_not(rmarkdown::pandoc_available("2.17"))

  local_options(width = 123)
  temp_rd <- withr::local_tempfile(lines = "`r getOption('width')`")

  rox <- sprintf("
    #' Title
    #' @includeRmd %s
    #' @name foobar
    NULL", temp_rd)
  out <- roc_proc_text(rd_roclet(), rox)[[1]]
  expect_equal(out$get_value("details"), "80")
})

test_that("one line per concept", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @title a
    #' @name a
    #' @concept test1
    #' @concept test2
    NULL"
  )[[1]]

  expect_equal(out$get_value("concept"), c("test1", "test2"))
  expect_equal(out$get_rd("concept"), c("\\concept{test1}", "\\concept{test2}"))
})

test_that("simple keys produce expected output", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @title a
    #' @encoding test
    #' @name a
    NULL"
  )[[1]]
  expect_equal(out$get_value("encoding"), "test")
})

test_that("keywords split into pieces", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @keywords a b
    #' @title a
    #' @name a
    NULL"
  )[[1]]

  expect_equal(out$get_value("keyword"), c("a", "b"))
})

# tokenise_block ----------------------------------------------------------

test_that("parses into tag and value", {
  x <- tokenise_block("#' @xyz abc", file = "", offset = 0)
  expect_equal(length(x), 1)

  expect_equal(x[[1]]$tag, "xyz")
  expect_equal(x[[1]]$raw, "abc")
})

test_that("description block gets empty tag", {
  x <- tokenise_block("#' abc", file = "", offset = 0L)
  expect_equal(length(x), 1)

  expect_equal(x[[1]]$tag, "")
  expect_equal(x[[1]]$raw, "abc")
})

test_that("multi line tags collapsed into one", {
  x <- tokenise_block(c(
    "#' @tag abc",
    "#'   def"),
    file = "", offset = 0L)
  expect_equal(length(x), 1)
  expect_equal(x[[1]]$raw, "abc\n  def")
})

test_that("description block gets empty tag when followed by tag", {
  x <- tokenise_block(c(
    "#' abc",
    "#' @xyz abc"),
    file = "", offset = 0L)
  expect_equal(length(x), 2)

  expect_equal(x[[1]]$tag, "")
  expect_equal(x[[1]]$raw, "abc")

  expect_equal(x[[2]]$tag, "xyz")
  expect_equal(x[[2]]$raw, "abc")
})

test_that("leading whitespace is ignored", {
  ref <- tokenise_block("#' abc", file = "", offset = 0L)

  expect_equal(tokenise_block("   #' abc", file = "", offset = 0L), ref)
})

test_that("need one or more #", {
  ref <- tokenise_block("#' abc", file = "", offset = 0L)

  expect_equal(tokenise_block("##' abc", file = "", offset = 0L), ref)
  expect_equal(tokenise_block("###' abc", file = "", offset = 0L), ref)
})

test_that("@@ becomes @", {
  expect_equal(tokenise_block("#' @tag @@", file = "", offset = 0L)[[1]]$raw, "@")
})

# Inline comments ---------------------------------------------------------

test_that("Inline comments are supported", {
  out <- roc_proc_text(rd_roclet(), "
    #' Description
    a <- function(x) {
      #' @param x an integer
      stopifnot(is.integer(x))
    }")[[1]]
  expect_equal(out$get_value("param"), c(x = "an integer"))
})

test_that("Inline comments just before the closing brace are allowed", {
  out <- roc_proc_text(rd_roclet(), "
    #' Description
    a <- function(x) {
      #' @param x an integer
      stopifnot(is.integer(x))

      #' @seealso somewhere
    }")[[1]]
  expect_equal(out$get_value("seealso"), "somewhere")
})

test_that("Inline comments do not extend past the closing brace", {
  out <- roc_proc_text(rd_roclet(), "
    #' Description
    a <- function(x) {
      #' @param x an integer
      stopifnot(is.integer(x))
    }; #' @seealso somewhere")[[1]]
  expect_null(out$get_value("seealso"))
})

test_that("Line numbers are ok", {

  check_line_nums <- function(block, lines) {
    for (t in names(lines)) {
      expect_equal(block_get_tag(block, t)$line, lines[[t]], info = t)
    }
  }

  text <-
    "#' @title
     #' Foo
     #'
     #' @description
     #' Description
     #'
     #' @details
     #' Details
     #'
     #' @param x xyz
     #' @export
     NULL"
  block <- parse_text(text)[[1]]
  ls <- c(title = 1, description = 4, details = 7, param = 10, export = 11)
  check_line_nums(block, ls)

  text <-
    "#' @title Foo
     #'
     #' @description Description
     #'
     #' @details Details
     #'
     #' @param x xy
     #' z
     #'
     #' @export
     NULL"
  block <- parse_text(text)[[1]]
  ls <- c(title = 1, description = 3, details = 5, param = 7, export = 10)
  check_line_nums(block, ls)

  text <-
    "#' @title Foo
     #'
     #' @description Description
     #'
     #' @details Details
     # not - a - roxy - comment
     #' @param x xy
     #' z
     quote(neither - is -
     #' @export
     this)"
  block <- parse_text(text)[[1]]
  ls <- c(title = 1, description = 3, details = 5, param = 7, export = 10)
  check_line_nums(block, ls)

  text <-
    "# 1
     # 2
     #' foo
     #'
     #' Description
     # 6
     # 7
     #' @param x xyz
     NULL"
  block <- parse_text(text)[[1]]
  ls <- c(title = 3, description = 5, param = 8)
  check_line_nums(block, ls)
})

test_that("exporting a call to :: produces re-exports documentation", {
  block <- "
    #' @export
    testthat::auto_test
  "
  out <- roc_proc_text(rd_roclet(), block)[[1]]

  expect_equal(
    out$get_section("reexport"),
    rd_section_reexport("testthat", "auto_test", "auto_test")
  )
  expect_equal(out$get_value("title"), "Objects exported from other packages")
  expect_equal(out$get_value("keyword"), "internal")
  expect_snapshot_output(cat(format(out)))

  # And generates correct namespace definitions
  out <- roc_proc_text(namespace_roclet(), block)
  expect_equal(out, c("export(auto_test)", "importFrom(testthat,auto_test)"))
})

test_that("multiple re-exports are combined", {
  out <- roc_proc_text(rd_roclet(), "
    #' @export
    testthat::expect_lt

    #' @export
    testthat::expect_gt
    ")[[1]]

  expect_equal(
    out$get_section("reexport"),
    rd_section_reexport(
      c("testthat", "testthat"),
      c("expect_lt", "expect_gt"),
      c("comparison-expectations", "comparison-expectations")
    )
  )
})

test_that("description generated correctly", {
  roc <- rd_roclet()
  out <- roc_proc_text(rd_roclet(), "
    #' @importFrom magrittr %>%
    #' @export
    magrittr::`%>%`
    ")[[1]]

  expect_null(out$get_section("description"))
})

test_that("Source reference is included as comment", {
  out <- roc_proc_text(rd_roclet(), "
    #' @name a
    #' @title a
    #' @docType package
    NULL")[[1]]

  expect_match(out$get_rd("backref"), "^% Please edit documentation in ")
})

test_that("Explicit @backref is included as comment", {
  out <- roc_proc_text(rd_roclet(), "
    #' @name a
    #' @title a
    #' @backref back/ref.file
    #' @backref root.file
    #' @docType package
    NULL")[[1]]

  expect_equal(out$get_value("backref"), c("back/ref.file", "root.file"))
})

test_that("r6_method_from_row extracts all components", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Say hello.
        #' @details Be polite.
        #' @param who Name.
        #' @return A greeting.
        #' @examples c$greet('world')
        greet = function(who) paste('hi', who)
      )
    )"
  docs <- r6_doc(text)
  method <- docs$methods$self[[1]]

  expect_equal(method$name, "greet")
  expect_equal(method$class, "C")
  expect_equal(method$description, "Say hello.")
  expect_equal(method$details, "Be polite.")
  expect_equal(method$params, list(list(name = "who", description = "Name.")))
  expect_equal(method$return, "A greeting.")
  expect_equal(method$examples, "c$greet('world')")
})

test_that("@returns works as method-level tag (#1148)", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Run.
        #' @returns The result.
        run = function() 1
      )
    )"
  docs <- r6_doc(text)
  expect_equal(docs$methods$self[[1]]$return, "The result.")
})

test_that("warns about multiple @return(s) tags", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Run.
        #' @return First.
        #' @returns Second.
        run = function() 1
      )
    )"
  expect_snapshot(docs <- r6_doc(text))
  expect_equal(docs$methods$self[[1]]$return, "First.")
})

test_that("class-level @param x,y inherited by methods (#1600)", {
  text <- "
    #' Class
    #' @param x,y Numbers.
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Method.
        meth = function(x, y) {}
      )
    )"
  expect_silent(docs <- r6_doc(text))
  expect_equal(
    docs$methods$self[[1]]$params,
    list(list(name = "x, y", description = "Numbers."))
  )
})

test_that("warns about undocumented params", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Run.
        run = function(x, y) NULL
      )
    )"
  expect_snapshot(docs <- r6_doc(text))
})

test_that("warns about duplicated params", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Run.
        #' @param x First.
        #' @param x Second.
        run = function(x) NULL
      )
    )"
  expect_snapshot(docs <- r6_doc(text))
})

test_that("initialize() inherits params from @field", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @field x A field.
        x = NULL,
        #' @field y Another field.
        y = NULL,
        #' @description Create.
        initialize = function(x, y) {
          self$x <- x
          self$y <- y
        }
      )
    )"
  docs <- r6_doc(text)
  init <- docs$methods$self[[1]]
  expect_equal(
    init$params,
    list(
      list(name = "x", description = "A field."),
      list(name = "y", description = "Another field.")
    )
  )
})

test_that("initialize() @param takes precedence over @field", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @field x A field.
        x = NULL,
        #' @description Create.
        #' @param x Custom description.
        initialize = function(x) {
          self$x <- x
        }
      )
    )"
  docs <- r6_doc(text)
  init <- docs$methods$self[[1]]
  expect_equal(
    init$params,
    list(list(name = "x", description = "Custom description."))
  )
})

test_that("format.rd_r6_method produces method subsection", {
  method <- rd_r6_method(
    name = "greet",
    class = "Person",
    formals = as.pairlist(alist(who = , how = "nicely")),
    description = "Say hello.",
    params = list(
      list(name = "who", description = "Name to greet."),
      list(name = "how", description = "Greeting style.")
    )
  )
  expect_snapshot(cat(format(method), sep = "\n"))
})

test_that("format.rd_r6_method renames initialize to new", {
  method <- rd_r6_method(
    name = "initialize",
    class = "Foo",
    formals = NULL,
    description = "Create object."
  )
  expect_snapshot(cat(format(method), sep = "\n"))
})

test_that("format.rd_r6_method includes optional sections", {
  method <- rd_r6_method(
    name = "run",
    class = "Job",
    formals = NULL,
    description = "Run the job.",
    details = "Some details.",
    return = "The result.",
    examples = "job$run()"
  )
  expect_snapshot(cat(format(method), sep = "\n"))
})

test_that("format.rd_r6_method strips \\dontrun etc from examples (#1072)", {
  method <- rd_r6_method(
    name = "run",
    class = "Job",
    formals = NULL,
    description = "Run the job.",
    examples = "
      \\dontrun{donrun()}
      \\donttest{donttest()}
      \\dontshow{dontshow()}
    "
  )
  expect_snapshot(cat(format(method), sep = "\n"))
})

test_that("format.rd_r6_method omits empty optional sections", {
  method <- rd_r6_method(
    name = "run",
    class = "Job",
    formals = NULL,
    description = "Run."
  )
  expect_snapshot(cat(format(method), sep = "\n"))
})

# Superclass param inheritance -----------------------------------------------

test_that("method inherits params from superclass method", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @description Do something.
        #' @param x An x value.
        #' @param y A y value.
        run = function(x, y) NULL
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Do something else.
        run = function(x, y) NULL
      )
    )"
  docs <- r6_doc(text)
  method <- docs$methods$self[[1]]
  expect_equal(
    method$params,
    list(
      list(name = "x", description = "An x value."),
      list(name = "y", description = "A y value.")
    )
  )
})

test_that("method inherits only missing params from superclass", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @description Do something.
        #' @param x An x value.
        #' @param y A y value.
        run = function(x, y) NULL
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Do something else.
        #' @param x Overridden x.
        run = function(x, y) NULL
      )
    )"
  docs <- r6_doc(text)
  method <- docs$methods$self[[1]]
  expect_equal(
    method$params,
    list(
      list(name = "x", description = "Overridden x."),
      list(name = "y", description = "A y value.")
    )
  )
})

test_that("method inherits class-level params from superclass", {
  text <- "
    #' Parent
    #' @param x An x value.
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @description Do something.
        run = function(x) NULL
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Do something else.
        run = function(x) NULL
      )
    )"
  docs <- r6_doc(text)
  method <- docs$methods$self[[1]]
  expect_equal(
    method$params,
    list(list(name = "x", description = "An x value."))
  )
})

test_that("initialize() inherits params from superclass initialize()", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @description Create.
        #' @param x An x value.
        #' @param y A y value.
        initialize = function(x, y) NULL
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Create child.
        #' @param z A z value.
        initialize = function(x, y, z) NULL
      )
    )"
  docs <- r6_doc(text)
  init <- docs$methods$self[[1]]
  expect_equal(
    init$params,
    list(
      list(name = "x", description = "An x value."),
      list(name = "y", description = "A y value."),
      list(name = "z", description = "A z value.")
    )
  )
})

test_that("child initialize() inherits params from superclass @field", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @field x An x value.
        x = NULL,
        #' @field y A y value.
        y = NULL,
        #' @description Create.
        initialize = function(x, y) {
          self$x <- x
          self$y <- y
        }
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Create child.
        #' @param z A z value.
        initialize = function(x, y, z) {
          super$initialize(x, y)
        }
      )
    )"
  docs <- r6_doc(text)
  init <- docs$methods$self[[1]]
  expect_equal(
    init$params,
    list(
      list(name = "x", description = "An x value."),
      list(name = "y", description = "A y value."),
      list(name = "z", description = "A z value.")
    )
  )
})

test_that("method inherits params through multiple levels", {
  text <- "
    #' Grandparent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @description Do something.
        #' @param x An x value.
        run = function(x) NULL
      )
    )

    #' Parent
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        #' @description Do something.
        #' @param x An x value.
        run = function(x) NULL
      )
    )

    #' Child
    C <- R6::R6Class('C', cloneable = FALSE,
      inherit = B,
      public = list(
        #' @description Do something else.
        run = function(x) NULL
      )
    )"
  docs <- r6_doc(text)
  method <- docs$methods$self[[1]]
  expect_equal(
    method$params,
    list(list(name = "x", description = "An x value."))
  )
})

# Construction tests -------------------------------------------------------

test_that("can construct empty class", {
  text <- "
    #' Class
    C <- R6::R6Class(cloneable = FALSE)"
  docs <- r6_doc(text)

  expect_s3_class(docs, "rd_r6_class")
  expect_equal(docs$methods, rd_r6_methods("C"))

  expect_equal(docs$fields$fields, list())
  expect_equal(docs$active_bindings$fields, list())
})

test_that("class description is not duplicated", {
  text <- "
    #' Title
    #'
    #' Description
    foo <- R6::R6Class(
      public = list(
        #' @description foo
        foo = function() {}
      )
    )
  "

  out <- roc_proc_text(rd_roclet(), text)[[1]]
  expect_equal(out$get_value("description"), "Description")
})

test_that("title-only class has single description", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    foo <- R6::R6Class(cloneable = FALSE)
  "
  )[[1]]
  expect_equal(out$get_value("description"), "Title")

  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @title Title
    foo <- R6::R6Class(cloneable = FALSE)
  "
  )[[1]]
  expect_equal(out$get_value("description"), "Title")
})

test_that("class with only active bindings doesn't error (#1610)", {
  text <- "
    #' Class
    C <- R6::R6Class('C',
      active = list(
        #' @field x A value.
        x = function(val) val
      ),
      cloneable = FALSE
    )"
  docs <- r6_doc(text)
  expect_equal(docs$methods, rd_r6_methods("C"))
})

test_that("warns about unmatched components ", {
  text <- "
    #' Class
    #' @field field1 Yep.
    C <- R6::R6Class(
      public = list(
        #' @description Dangling.
        field1 = NULL
      )
    )"
  expect_snapshot(docs <- r6_doc(text))
})

# Format tests -------------------------------------------------------------

test_that("format.rd_r6_class with fields", {
  docs <- rd_r6_class(
    class = "Foo",
    fields = rd_r6_fields(list(
      rd_r6_field("x", "A number."),
      rd_r6_field("y", "A string.")
    ))
  )
  expect_snapshot(cat(format(docs), sep = "\n"))
})

test_that("format.rd_r6_class with active bindings", {
  docs <- rd_r6_class(
    class = "Foo",
    active_bindings = rd_r6_fields(
      list(rd_r6_field("val", "A value.")),
      type = "active"
    )
  )
  expect_snapshot(cat(format(docs), sep = "\n"))
})

test_that("format.rd_r6_class with no inherited methods", {
  text <- "
    C1 <- R6::R6Class('C1', cloneable = FALSE)

    #' @title Title
    #' @description Description.
    C2 <- R6::R6Class('C2',
      inherit = C1,
      cloneable = FALSE,
      public = list(
        #' @description method1
        meth1 = function() 1
      )
    )"
  docs <- r6_doc(text)
  expect_snapshot(cat(format(docs), sep = "\n"))
})

test_that("format.rd_r6_class with inherited methods", {
  text <- "
    A <- R6::R6Class('A',
      public = list(
        shared = function() 1,
        only_a = function() 2
      )
    )

    #' Class B.
    B <- R6::R6Class('B',
      inherit = A,
      public = list(
        #' @description Method from B.
        shared = function() 3
      )
    )"
  docs <- r6_doc(text)
  expect_snapshot(cat(format(docs), sep = "\n"))
})

test_that("format.rd_r6_class with markdown sections", {
  local_roxy_meta_set("markdown", TRUE)
  text <- "
    #' @title Title
    #' @description Description.
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Method description.
        #'
        #' # Description section
        #'
        #' Description section body.
        #' @details # Details section
        #'
        #' Details section body.
        meth = function() { }
      )
    )"
  docs <- r6_doc(text)
  expect_snapshot(cat(format(docs), sep = "\n"))
})

test_that("can eval inline code", {
  out1 <- roc_proc_text(rd_roclet(), "

    #' @title Title `r 1 + 1`
    #' @description Description `r 2 + 2`
    #' @md
    foo <- function() NULL

  ")[[1]]
  expect_equal(out1$get_value("title"), "Title 2")
  expect_equal(out1$get_value("description"), "Description 4")
})

test_that("can eval fenced code", {
  out1 <- roc_proc_text(rd_roclet(), "

    #' @title Title
    #' @details Details
    #' ```{r lorem}
    #' 1+1
    #' ```
    #' @md
    foo <- function() NULL

  ")[[1]]
  expect_match(out1$get_value("details"), "2")
})

test_that("use same env within, but not across blocks", {
  example <- "
    #' Title `r baz <- 420` `r baz`
    #'
    #' Description `r exists('baz', inherits = FALSE)`
    #' @md
    bar <- function() NULL

    #' Title
    #'
    #' Description `r exists('baz', inherits = FALSE)`
    #' @md
    zap <- function() NULL
  "
  out1 <- roc_proc_text(rd_roclet(), example)[[1]]
  out2 <- roc_proc_text(rd_roclet(), example)[[2]]
  expect_equal(out1$get_value("title"), "Title  420")
  expect_equal(out1$get_value("description"), "Description TRUE")
  expect_equal(out2$get_value("description"), "Description FALSE")
})

test_that("appropriate knit print method for fenced and inline is applied", {
  rlang::local_bindings(
    knit_print.foo = function(x, inline = FALSE, ...) {
      knitr::asis_output(ifelse(inline, "inline", "fenced"))
    },
    .env = globalenv()
  )
  out1 <- roc_proc_text(rd_roclet(), "
    #' @title Title `r structure('default', class = 'foo')`
    #'
    #' @details Details
    #'
    #' ```{r}
    #' structure('default', class = 'foo')
    #' ```
    #'
    #' @md
    #' @name bar
    NULL
  ")
  expect_match(out1$bar.Rd$get_value("details"), "fenced", fixed = TRUE)
  expect_match(out1$bar.Rd$get_value("title"), "inline", fixed = TRUE)
})

test_that("can create markdown markup", {
  expect_identical(
    markdown("Description `r paste0('_', 'keyword', '_')`"),
    "Description \\emph{keyword}"
  )
})

test_that("can create markdown markup piecewise", {
  expect_identical(
    markdown(
      "Description [`r paste0('https://url')`](`r paste0('link text')`)"
    ),
    "Description \\link{https://url}(link text)"
  )
})

test_that("can create escaped markdown markup", {
  # this workaround is recommended by @yihui
  # "proper" escaping for inline knitr tracked in https://github.com/yihui/knitr/issues/1704
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #' Description `r paste0('\\x60', 'bar', '\\x60')`
    #' @md
    foo <- function() NULL
  ")[[1]]
  expect_match(out1$get_value("title"), "\\code{bar}", fixed = TRUE)
})

test_that("NULL creates no text", {
  expect_identical(
    markdown("Description --`r NULL`--"),
    "Description ----"
  )
})


test_that("multi-line inline code gives useful warning", {
  block <- "
    #' Title
    #'
    #' `r 1 +
    #' 1`
    #' @md
    foo <- function() {}
  "

  expect_snapshot(
    out <- roc_proc_text(rd_roclet(), block)[[1]]
  )
  expect_equal(out$get_value("description"), "\\verb{r 1 + 1}")
})

test_that("inline code gives useful warning", {
  block <- "
    #' Title
    #'
    #' `r 1 + `
    #' @md
    foo <- function() {}
  "

  expect_snapshot(
    out <- roc_proc_text(rd_roclet(), block)[[1]]
  )
  expect_equal(out$get_value("description"), "\\verb{r 1 + }")
})

test_that("interleaving fences and inline code", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @details Details `r x <- 10; x`
    #'
    #' ```{r}
    #' y <- x + 10
    #' y
    #' ```
    #'
    #' @md
    #' @name dummy
    NULL")[[1]]

  expect_snapshot(cat(out1$get_value("details")))
})

test_that("preserves white space", {
    out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @details
    #'
    #' ```{r}
    #' a <- 1
    #'
    #' b <- 2
    #' ```
    #'
    #' ```{r}
    #' c <- 3
    #' ```
    #'
    #' @md
    #' @name dummy
    NULL")[[1]]

  expect_snapshot(cat(out1$get_value("details")))
})

test_that("fence options are used", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @details Details
    #'
    #' ```{r eval = FALSE}
    #' this - would - fail - to - eval
    #' ```
    #'
    #' @md
    #' @name dummy
    NULL")[[1]]

  details <- out1$get_value("details")
  expect_false(grepl("Error", details))
})

test_that("dynamic code in fragile tags still runs", {
  out <- markdown("foo \\out{`r 1+1`} bar")
  expect_equal(out, "foo \\out{2} bar")
})

test_that("fragile tags in dynamic code are left alone", {
  out <- markdown("foo `r substr('\\\\out{xxx}', 2, 4)` bar")
  expect_equal(out, "foo out bar")
})

test_that("fragile tags in generated code", {
  out <- markdown("foo `r '\\\\out{*1*}'` bar")
  expect_equal(out, "foo \\out{*1*} bar")

  expect_silent(out2 <- markdown("foo `r '\\\\out{<span></span>}'` bar"))
  expect_equal(out2, "foo \\out{<span></span>} bar")
})

test_that("workaround for cmark sourcepos bug (#1353) works", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' line1
    #'    pre `r \"1\"` 2 `r 1+2` post
    #'
    #' no workaround needed `r 'here'`
    #' @md
    foo <- function() {}
  ")[[1]]

  expect_equal(out$get_section("description")$value, "line1\npre 1 2 3 post")
  expect_equal(out$get_section("details")$value, "no workaround needed here")
})


test_that("doesn't generate NA language", {
  out <- roc_proc_text(rd_roclet(), "
  #' Title
  #'
  #' ```
  #' r <- 1:10
  #' ```
  #' @md
  foo <- function() {}")[[1]]
  expect_false(grepl("NA", out$get_section("description")$value))
})

test_that("@param documents arguments", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param a first
    #' @param z last
    a <- function(a=1, z=2) {}
  "
  )[[1]]

  expect_equal(out$get_value("param"), c(a = "first", z = "last"))
})

test_that("backtick-quoted @param names are parsed correctly (#1696)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param `arg 1` first
    #' @param `arg 2` last
    a <- function(`arg 1` = 1, `arg 2` = 2) {}
  "
  )[[1]]

  expect_equal(
    out$get_value("param"),
    c("arg 1" = "first", "arg 2" = "last")
  )
})

test_that("grouped args get spaces", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param a,z Two arguments
    a <- function(a=1, z=2) {}
  "
  )[[1]]
  expect_match(out$get_rd("param"), "a, z")
})

test_that("empty @param generates warning", {
  block <- "
    #' A
    #' @param
    #'
    a <- function() {}
  "

  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("indented @param bullet list is not nested (#1102)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #' @md
    #' @param foo
    #'   * A
    #'   * B
    #'   * C
    a <- function(foo) {}
  "
  )[[1]]

  expect_match(out$get_rd("param"), "\\item A", fixed = TRUE)
  expect_match(out$get_rd("param"), "\\item B", fixed = TRUE)
  expect_match(out$get_rd("param"), "\\item C", fixed = TRUE)
  expect_no_match(out$get_rd("param"), "itemize\\{\n\\\\item B")
})

test_that("data objects don't get params", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' x
    #' @rdname xy
    x <- 'x'
  "
  )[[1]]
  expect_equal(out$get_value("param"), NULL)
})

test_that("arguments ordered by usage", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #'
    #' @param y Y
    #' @param x X
    #' @rdname rd
    a <- function(x, y) {}
  "
  )[[1]]

  expect_named(out$get_value("param"), c("x", "y"))
})

test_that("multiple arguments ordered by first", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #'
    #' @param y Y
    #' @param x,z X,Z
    #' @param w W
    b <- function(x, y, z, w) {}
  "
  )[[1]]

  expect_named(out$get_value("param"), c("x,z", "y", "w"))
})

test_that("can control processing order with @order", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @rdname bar
    #' @details 2
    #' @order 2
    foo <- function() {}

    #' @rdname bar
    #' @details 1
    #' @order 1
    bar <- function() {}

    #' @title Title
    #' @name bar
    NULL
  "
  )[[1]]

  expect_equal(out$get_value("details"), c("1", "2"))
})

test_that("can extract superclasses", {
  text <- "
    C1 <- R6::R6Class('C1', cloneable = FALSE)

    #' Class
    C2 <- R6::R6Class('C2',
      inherit = C1,
      public = list(
        #' @description method1
        meth1 = function() 1
      )
    )"
  docs <- r6_doc(text)

  expect_equal(
    docs$superclasses,
    rd_r6_super(
      "C2",
      package = "R_GlobalEnv",
      classname = "C1",
      has_topic = FALSE
    )
  )
})

test_that("format.rd_r6_super with one superclass", {
  supers <- rd_r6_super(
    "Child",
    package = "mypkg",
    classname = "Parent",
    has_topic = FALSE
  )
  expect_snapshot(cat(format(supers), sep = "\n"))
})

test_that("format.rd_r6_super omits pkg:: for same-package classes", {
  local_roxy_meta_set("current_package", "mypkg")
  supers <- rd_r6_super(
    "Child",
    package = c("mypkg", "otherpkg"),
    classname = c("Parent", "GrandParent"),
    has_topic = c(FALSE, FALSE)
  )
  expect_snapshot(cat(format(supers), sep = "\n"))
})

test_that("format.rd_r6_super returns nothing when empty", {
  expect_null(format(rd_r6_super("Foo")))
})

test_that("warn about unknown tags", {
  block <- "
    #' @unknown
    foo <- function() {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("incomplete rd in prequel or tag raises issue", {
  block <- "
    #' Title {
    #' @aliases title{
    x <- 1
  "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), block))
  expect_equal(out[[1]]$get_value("title"), "")
  expect_equal(out[[1]]$get_value("alias"), "x")
})

test_that("rawRd inserted unchanged", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @rawRd #this is a comment
    #' @name a
    #' @title a
    NULL"
  )[[1]]

  lines <- strsplit(format(out), "\n")[[1]]
  expect_equal(lines[[9]], "#this is a comment")
})

test_that("evalRd inserted unchanged", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    z <- 10
    #' @evalRd as.character(z * 2)
    #' @name a
    #' @title a
    NULL"
  )[[1]]

  expect_equal(out$get_value("rawRd"), "20")
})

test_that("error-ful evalRd generates warning", {
  expect_snapshot({
    expect_parse_failure(roxy_tag_eval(roxy_test_tag(val = 1)))
    expect_parse_failure(roxy_tag_eval(roxy_test_tag(val = NA_character_)))
    expect_parse_failure(roxy_tag_eval(roxy_test_tag(
      val = quote(stop('Uhoh'))
    )))
  })
})

test_that("can extract inherited methods", {
  text <- "
    A <- R6::R6Class('A',
      public = list(
        shared = function() 1,
        only_a = function() 2
      )
    )

    #' Class B.
    B <- R6::R6Class('B',
      inherit = A,
      public = list(
        #' @description Method from B.
        shared = function() 3
      )
    )"
  docs <- r6_doc(text)

  expect_s3_class(docs$methods$inherited, "rd_r6_inherited")
  expect_equal(docs$methods$inherited$name, "only_a")
})

test_that("no inherited methods when none exist", {
  text <- "
    C1 <- R6::R6Class('C1', cloneable = FALSE)

    #' Class
    C2 <- R6::R6Class('C2',
      inherit = C1,
      public = list(
        #' @description method1
        meth1 = function() 1
      )
    )"
  docs <- r6_doc(text)
  expect_equal(docs$methods$inherited, rd_r6_inherited())
})

test_that("format.rd_r6_inherited renders method list", {
  inherited <- rd_r6_inherited(
    package = c("pkg", "pkg"),
    classname = c("A", "A"),
    name = c("foo", "bar")
  )

  expect_snapshot(cat(format(inherited), sep = "\n"))
})

test_that("format.rd_r6_inherited returns nothing when empty", {
  expect_null(format(rd_r6_inherited()))
})

# name --------------------------------------------------------------------

test_that("name captured from assignment", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    a <- function() {}
  "
  )[[1]]

  expect_equal(out$get_value("name"), "a")
  expect_equal(out$get_value("alias"), "a")
  expect_equal(out$get_value("title"), "Title.")
})

test_that("name also captured from assignment by =", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    a = function() {}
  "
  )[[1]]

  expect_equal(out$get_value("name"), "a")
  expect_equal(out$get_value("alias"), "a")
  expect_equal(out$get_value("title"), "Title.")
})


test_that("`$` not to be parsed as assignee in foo$bar(a = 1)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' foo object
    foo <- list(bar = function(a) a)
    foo$bar(a = 1)
  "
  )[[1]]

  expect_equal(out$get_value("name"), "foo")
})

test_that("names escaped, not quoted", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    '%a%' <- function(x, y) x + y
  "
  )[[1]]
  expect_equal(out$get_rd("name"), "\\name{\\%a\\%}")
})

test_that("quoted names captured from assignment", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    \"myfunction\" <- function(...) {}
  "
  )[[1]]

  expect_equal(out$get_value("name"), "myfunction")
  expect_equal(out$get_value("alias"), "myfunction")

  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    `myfunction` <- function(...) {}
  "
  )[[1]]
  expect_equal(out$get_value("name"), "myfunction")
  expect_equal(out$get_value("alias"), "myfunction")

  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title.
    \"my function\" <- function(...) {}
  "
  )[[1]]

  expect_equal(out$get_value("name"), "my function")
  expect_equal(out$get_value("alias"), "my function")
})

test_that("@name overides default", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @name b
    a <- function() {}
  "
  )[[1]]

  expect_equal(out$get_value("name"), "b")
  expect_setequal(out$get_value("alias"), c("a", "b"))
})


# alias -------------------------------------------------------------------

test_that("aliases split into pieces", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @aliases a b
    #' @title a
    #' @name a
    NULL"
  )[[1]]

  expect_match(out$get_value("alias"), "a", all = FALSE, fixed = TRUE)
  expect_match(out$get_value("alias"), "b", all = FALSE, fixed = TRUE)
})

test_that("aliases escaped, not quoted", {
  out1 <- roc_proc_text(
    rd_roclet(),
    "
    #' @name %a%
    #' @aliases a
    #' @title a
    NULL"
  )[[1]]
  expect_equal(out1$get_rd("alias"), c("\\alias{\\%a\\%}", "\\alias{a}"))

  out2 <- roc_proc_text(
    rd_roclet(),
    "
    #' @name a
    #' @aliases %a%
    #' @title a
    NULL"
  )[[1]]
  expect_equal(out2$get_rd("alias"), c("\\alias{a}", "\\alias{\\%a\\%}"))
})

test_that("can use NULL to suppress default aliases", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @aliases NULL
    #' @title a
    #' @name a
    NULL"
  )[[1]]

  expect_equal(out$get_value("alias"), character())
})

test_that("aliases get deduplicated", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @aliases a b a
    #' @title a
    #' @name a
    NULL"
  )[[1]]

  expect_equal(out$get_rd("alias"), c("\\alias{a}", "\\alias{b}"))
})

test_that("aliases get deduplicated with defaults suppressed", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' @aliases NULL b c b
    #' @title a
    #' @name a
    NULL"
  )[[1]]

  expect_equal(out$get_rd("alias"), c("\\alias{b}", "\\alias{c}"))
})

test_that("refclass with assignment gets both aliases", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    B3 <- setRefClass('B3')
  "
  )[[1]]

  expect_equal(out$get_value("alias"), c("B3-class", "B3"))
})


test_that("refclass gets -class alias", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    setRefClass('B2')
  "
  )[[1]]

  expect_equal(out$get_value("alias"), "B2-class")
})

test_that("colons inside code in the title are ignored", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @md
    #' @section Similar to `base::split()`: Content.
    foo <- function(x) x
  "
  )[[1]]

  expect_equal(
    out$get_section("section"),
    rd_section_section("Similar to \\code{base::split()}", " Content.")
  )
})

test_that("warn if forgotten colon", {
  block <- "
    #' Foo
    #'
    #' @section Haz dox
    #' Here.
    #' There
    foo <- function(x = '%') x
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("@section-s with identical titles are merged", {
  block <- "
    #' Foo
    #'
    #' @section Haz dox: Here.
    #'
    #' @section TL: DR.
    foo <- function(x = '%') x

    #' @rdname foo
    #' @section RT: FM.
    #' @section Haz dox:
    #'   Got news.
    bar <- function(y = '%') y
  "
  out <- roc_proc_text(rd_roclet(), block)[[1]]

  expect_equal(
    out$get_section("section"),
    rd_section_section(
      c("Haz dox", "TL", "RT"),
      c(" Here.\n\n\n  Got news.", " DR.", " FM.")
    )
  )
})

test_that("@section-s with different titles are kept as they are", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @section A: 1
    #' @section B: 2
    foo <- function(x) x

    #' @rdname foo
    #' @section C: 3
    bar <- function(x) x
  "
  )[[1]]

  expect_equal(
    out$get_section("section"),
    rd_section_section(LETTERS[1:3], c(" 1", " 2", " 3"))
  )
})

# Output ------------------------------------------------------------------

# A simple object to represent rd escaped text.
rd <- function(x) {
  structure(x, class = "rd")
}

#' @export
c.rd <- function(...) {
  rd(NextMethod())
}

#' @export
print.rd <- function(x, ...) {
  out <- paste0("<rd> ", x, collapse = "\n")
  cat(out)
}

escape <- function(x) UseMethod("escape")
#' @export
escape.NULL <- function(x) NULL
#' @export
escape.rd <- function(x) x
#' @export
escape.character <- function(x) {
  # wrap_usage uses \u{A0}, the unicode non-breaking space, which
  # is not necessarily valid in windows locales. useBytes is a quick
  # hack to fix the problem.
  x1 <- gsub(r"(\)", r"(\\)", x, fixed = TRUE, useBytes = TRUE)
  x2 <- gsub("%", r"(\%)", x1, fixed = TRUE, useBytes = TRUE)

  rd(x2)
}

# Works like paste, but automatically escapes all input variables,
# but not literal strings
build_rd <- function(..., collapse = NULL, sep = "") {
  args <- dots(...)
  env <- parent.frame()

  escaped <- lapply(args, function(arg) {
    if (is.character(arg)) {
      return(arg)
    }

    escape(eval(arg, env))
  })

  string <- do.call("paste", c(escaped, list(collapse = collapse, sep = sep)))
  rd(string)
}

dots <- function(...) {
  eval(substitute(alist(...)))
}

# Translate a field and values into an Rd macro.
# Multiple values get their own braces.
rd_macro <- function(field, ..., space = FALSE) {
  if (space) {
    values <- paste0("\n", paste0(..., collapse = "\n"), "\n")
  } else {
    values <- trimws(c(...))
  }

  paste0("\\", field, paste0("{", values, "}", collapse = ""))
}


# Input -------------------------------------------------------------------

get_tags <- function(rd, tag) {
  Filter(\(x) identical(attr(x, "Rd_tag"), tag), rd)
}

# helpers -----------------------------------------------------------------

parse_rd <- function(x) {
  con <- textConnection(x)
  on.exit(close(con), add = TRUE)

  tryCatch(
    tools::parse_Rd(con, fragment = TRUE, encoding = "UTF-8"),
    warning = function(cnd) NULL
  )
}

# Generated in .onLoad()
as_character_rd <- NULL
make_as_character_rd <- function() {
  # "as.character.Rd" appears to a few commands in TWOARGS
  # this code hacks the body of the function to add it
  fn <- internal_f("tools", "as.character.Rd")

  body <- body(fn)
  idx <- detect_index(
    body,
    \(x) is_call(x, "<-", 2) && is_symbol(x[[2]], "TWOARG")
  )
  if (idx == 0) {
    return(fn)
  }

  body[[idx]][[3]] <- call_modify(
    body[[idx]][[3]],
    "\\href",
    "\\ifelse",
    "\\if"
  )
  body(fn) <- body
  fn
}

has_topic <- function(topic, package) {
  tryCatch(
    {
      out <- exec("help", topic, package, .env = global_env())
      inherits(out, "dev_topic") ||
        (inherits(out, "help_files_with_topic") && length(out) == 1)
    },
    error = function(c) FALSE
  )
}

test_that("has_topic() works as you'd expect", {
  expect_equal(has_topic("abbreviate", "base"), TRUE)
  expect_equal(has_topic("abbreviateXXXX", "base"), FALSE)

  expect_equal(has_topic("foo", "PACKAGEDOESNOTEXIST"), FALSE)
})

made_by_roxygen <- function(path) {
  if (!file.exists(path)) {
    return(TRUE)
  }

  first <- read_lines(path, n = 1)
  check_made_by(first)
}

check_made_by <- function(first) {
  if (length(first) == 0L) {
    return(FALSE)
  }
  grepl("^. Generated by roxygen2", first)
}

made_by <- function(comment) {
  # This text is used by IDE to display a special warning. DO NOT CHANGE
  # without consulting the IDE team
  paste0(comment, " Generated by roxygen2: do not edit by hand\n")
}

test_that("determine if file is autogenerated", {
  expect_true(made_by_roxygen("made-by-roxygen/with-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/without-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/empty.Rd"))
  expect_true(check_made_by(made_by("#")))
  expect_false(check_made_by(character()))
})

#' Class A
#'
#' @description
#' Class A description.
#'
#' @details
#' Class A details
#'
#' @param Z zzzzzzz
#' @example example.R

A <- R6::R6Class(
  "A",
  public = list(
    #' @field field1 A field 1.
    field1 = NULL,
    #' @description A method 1.
    #' @examples
    #' ## Example for meth1
    meth1 = function(Z) {},
    #' @description Method 2 description.
    #' @details Method 2 details.
    #' @param Z Overriding Z argument for meth2.
    #' @param ... Rest.
    #' @return A value.
    #' @examples
    #' ## Example for meth2
    #' ## Second line
    #' ## Third line
    meth2 = function(Z = 10, ...) {},
    #' @field field2 A field 2.
    field2 = "foobar",
    #' @description Method 3.
    #' @param x An argument.
    meth3 = function(x) {},
    #' @field field3 A field 3.
    field3 = "baz"
  ),
  active = list(
    #' @field active1 A binding 1.
    active1 = function(x) {},
    #' @field active2 A binding 2.
    active2 = function(x) {},
    #' @field active3 A binding 3.
    active3 = function(x) {}
  )
)

#' Class B
#'
#' @description
#' Class B Description.
#'
#' @details
#' Class B details.

B <- R6::R6Class(
  "B",
  inherit = A,
  public = list(
    #' @field field1 B field 1.
    field1 = NULL,
    #' @field field4 B field 4.
    field4 = NULL,
    #' @description B method 1.
    #' @param Z Still zzzzzzzz.
    meth1 = function(Z) {},
    #' @description B method 4.
    meth4 = function() {}
  ),
  active = list(
    #' @field active1 B binding 1.
    active1 = function(x) {},
    #' @field active4 B binding 4.
    active4 = function(x) {},
    #' @field active5 B binding 5.
    active5 = function(x) {}
  )
)

#' Class C
#'
#' @description
#' Class C Description.
#'
#' @details
#' Class C details.

C <- R6::R6Class(
  "C",
  inherit = B,
  cloneable = FALSE,
  public = list(
    #' @field field2 C field 2.
    field2 = NULL,
    #' @description C method 2.
    #' @param Z zzzzz
    #' @param ... etc
    meth2 = function(Z = 10, ...) {},
    #' @field field5 C field 5.
    field5 = "foobar",
    #' @description C method 5.
    meth5 = function() {}
  ),
  active = list(
    #' @field active2 C binding 2.
    active2 = function(x) {},
    #' @field active4 C binding 4.
    active4 = function(x) {},
    #' @field active6 C binding 6.
    active6 = function(x) {}
  )
)

package_files <- function(path = ".") {
  all <- normalizePath(r_files(path))

  collate <- desc::desc_get_collate(file = file.path(path, "DESCRIPTION"))

  collate <- normalizePath(file.path(path, "R", collate))

  rfiles <- c(collate, setdiff(all, collate))
  ignore_files(rfiles, path)
}

r_files <- function(path) {
  sort_c(dir(file.path(path, "R"), "\\.[Rr]$", full.names = TRUE))
}

ignore_files <- function(rfiles, path) {
  rbuildignore <- file.path(path, ".Rbuildignore")
  if (!file.exists(rbuildignore)) {
    return(rfiles)
  }

  # Strip leading directory and slashes
  rfiles_relative <- sub(
    normalizePath(path, winslash = "/"),
    "",
    normalizePath(rfiles, winslash = "/"),
    fixed = TRUE
  )
  rfiles_relative <- sub("^[/]*", "", rfiles_relative)

  # Remove any files that match any perl-compatible regexp
  patterns <- read_lines(rbuildignore)
  patterns <- patterns[patterns != ""]
  if (length(patterns) == 0L) {
    return(rfiles)
  }
  matches <- lapply(patterns, grepl, rfiles_relative, perl = TRUE)
  matches <- Reduce("|", matches)
  rfiles[!matches]
}

test_that("respects order in Collate (#790)", {
  expect_equal(
    package_files('testCollateParse'),
    normalizePath(c("testCollateParse/R/b.r", "testCollateParse/R/c.r"))
  )
})

test_that("roxygen ignores files with matching pattern in .Rbuildignore", {
  local_package_copy(test_path("testRbuildignore"))

  expect_equal(basename(package_files()), c("a.R", "ignore_me.R"))

  write_lines("^R/ignore_me.R$", ".Rbuildignore")
  expect_equal(basename(package_files()), "a.R")

  # Continues to work even if empty lines or missing files
  write_lines(c("^R/ignore_me.R$", "", ".nonexistentfile"), ".Rbuildignore")
  expect_equal(basename(package_files()), "a.R")
})

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(comment = "#>", collapse = TRUE)

## -----------------------------------------------------------------------------
#' Sum of vector elements
#'
#' `sum` returns the sum of all the values present in its arguments.
#'
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.
sum <- function(..., na.rm = TRUE) {}

## -----------------------------------------------------------------------------
#' Sum of vector elements
#' 
#' @description
#' `sum` returns the sum of all the values present in its arguments.
#'
#' @details
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.

#' Blocks
#'
#' @description
#' A `roxy_block` represents a single roxygen2 block.
#'
#' The `block_*` functions provide a few helpers for common operations:
#' * `block_has_tags(blocks, tags)`: does `block` contain any of these `tags`?
#' * `block_get_tags(block, tags)`: get all instances of `tags`
#' * `block_get_tag(block, tag)`: get single tag. Returns `NULL` if 0,
#'    throws warning if more than 1.
#' * `block_get_tag_value(block, tag)`: gets `val` field from single tag.
#'
#' @param tags A list of [roxy_tag]s.
#' @param file,line Location of the `call` (i.e. the line after the last
#'   line of the block).
#' @param call Expression associated with block.
#' @param object Optionally, the object associated with the block, found
#'   by inspecting/evaluating `call`.
#' @param block A `roxy_block` to manipulate.
#' @param tag A single tag name.
#' @export
#' @keywords internal
#' @examples
#' # The easiest way to see the structure of a roxy_block is to create one
#' # using parse_text:
#' text <- "
#'   #' This is a title
#'   #'
#'   #' @param x,y A number
#'   #' @export
#'   f <- function(x, y) x + y
#' "
#'
#' # parse_text() returns a list of blocks, so I extract the first
#' block <- parse_text(text)[[1]]
#' block
roxy_block <- function(tags,
                       file,
                       line,
                       call,
                       object = NULL) {
  stopifnot(is.list(tags))
  stopifnot(is.character(file), length(file) == 1)
  stopifnot(is.integer(line), length(line) == 1)

  structure(
    list(
      tags = tags,
      file = file,
      line = line,
      call = call,
      object = object
    ),
    class = "roxy_block"
  )
}

is_roxy_block <- function(x) inherits(x, "roxy_block")

#' @export
print.roxy_block <- function(x, ...) {
  call <- deparse(x$call, nlines = 2)
  if (length(call) == 2) {
    call <- paste0(call[[1]], " ...")
  }
  obj <- format(x$object)

  cat_line("<roxy_block> [", basename(x$file), ":", x$line, "]")
  cat_line("  $tag")
  cat_line("    ", map_chr(x$tags, format, file = x$file))
  cat_line("  $call   ", call)
  cat_line("  $object ", obj[[1]])
  cat_line("  ", obj[-1])
}

block_create <- function(call, srcref, tokens = c()) {
  if (is_empty(tokens)) {
    return(NULL)
  }

  tags <- parse_tags(tokens)
  if (length(tags) == 0) {
    return(NULL)
  }

  roxy_block(
    tags = tags,
    file = attr(srcref, "srcfile")$filename,
    line = srcref[[1]],
    call = call
  )
}

block_set_env <- function(block, env) {
  block <- block_evaluate(block, env)
  block <- block_find_object(block, env)
  block
}

block_evaluate <- function(block, env) {

  tags <- block_get_tags(block, "eval")
  if (length(tags) == 0) {
    return(block)
  }

  # Evaluate
  results <- lapply(tags, roxy_tag_eval, env = env)
  results <- lapply(results, function(x) {
    if (is.null(x)) {
      character()
    } else {
      paste0("#' ", x)
    }
  })

  # Tokenise and parse
  tokens <- lapply(results, tokenise_block,
    file = block$file,
    offset = block$line
  )
  tags <- lapply(tokens, parse_tags)

  # Interpolate results back into original locations
  block_replace_tags(block, "eval", tags)
}

block_find_object <- function(block, env) {
  stopifnot(is_roxy_block(block))

  object <- object_from_call(
    call = block$call,
    env = env,
    block = block,
    file = block$file
  )
  block$object <- object

  class(block) <- unique(c(
    paste0("roxy_block_", class(object)),
    class(block)
  ))

  # Add in defaults generated from the object
  defaults <- object_defaults(object, block)
  defaults <- c(defaults, list(roxy_generated_tag(block, "backref", block$file)))

  default_tags <- map_chr(defaults, "tag")
  defaults <- defaults[!default_tags %in% block_tags(block)]

  block$tags <- c(block$tags, defaults)
  block
}

# block accessors ---------------------------------------------------------

block_tags <- function(block) {
  map_chr(block$tags, "tag")
}

#' @export
#' @rdname roxy_block
block_has_tags <- function(block, tags) {
  any(block_tags(block) %in% tags)
}

#' @export
#' @rdname roxy_block
block_get_tags <- function(block, tags) {
  block$tags[block_tags(block) %in% tags]
}

#' @export
#' @rdname roxy_block
block_get_tag <- function(block, tag) {
  matches <- which(block_tags(block) %in% tag)
  n <- length(matches)
  if (n == 0) {
    NULL
  } else if (n == 1) {
    block$tags[[matches]]
  } else {
    warn_roxy_block(block, "Block must contain only one @{tag}")
    block$tags[[matches[[1]]]]
  }
}

#' @export
#' @rdname roxy_block
block_get_tag_value <- function(block, tag) {
  block_get_tag(block, tag)$val
}

block_replace_tags <- function(block, tags, values) {
  indx <- which(block_tags(block) %in% tags)
  stopifnot(length(indx) == length(values))

  tags <- lapply(block$tags, list)
  tags[indx] <- values

  block$tags <- compact(unlist(tags, recursive = FALSE))
  block
}

# parsing -----------------------------------------------------------------

parse_tags <- function(tokens) {
  # Set up evaluation environment for markdown
  pkgenv <- roxy_meta_get("env") %||% baseenv()
  evalenv <- new.env(parent = pkgenv)
  local_roxy_meta_set("evalenv", evalenv)

  markdown_activate(tokens)

  tokens <- parse_description(tokens)
  tokens <- map(tokens, roxy_tag_parse)
  compact(tokens)
}

#' @export
roxy_tag_parse.roxy_tag_eval <- function(x) {
  tag_code(x)
}

#' @export
roxy_tag_parse.roxy_tag_include <- function(x) {
  tag_value(x)
}

parse_description <- function(tags) {
  if (length(tags) == 0) {
    return(tags)
  }

  tag_names <- vapply(tags, `[[`, "tag", FUN.VALUE = character(1))
  if (tag_names[1] != "") {
    return(tags)
  }

  intro <- tags[[1]]
  intro$val <- str_trim(intro$raw)
  if (intro$val == "") {
    return(tags[-1])
  }

  tags <- tags[-1]
  tag_names <- tag_names[-1]

  paragraphs <- str_split(intro$val, fixed('\n\n'))[[1]]
  lines <- str_count(paragraphs, "\n") + rep(2, length(paragraphs))
  offsets <- c(0, cumsum(lines))

  # 1st paragraph = title (unless has @title)
  if ("title" %in% tag_names) {
    title <- NULL
  } else if (length(paragraphs) > 0) {
    title <- roxy_tag("title", paragraphs[1], NULL, intro$file, intro$line + offsets[[1]])
    paragraphs <- paragraphs[-1]
    offsets <- offsets[-1]
  } else {
    title <- roxy_tag("title", "", NULL, intro$file, intro$line)
  }

  # 2nd paragraph = description (unless has @description)
  if ("description" %in% tag_names || length(paragraphs) == 0) {
    description <- NULL
  } else if (length(paragraphs) > 0) {
    description <- roxy_tag("description", paragraphs[1], NULL, intro$file, intro$line + offsets[[1]])
    paragraphs <- paragraphs[-1]
    offsets <- offsets[-1]
  }

  # Every thing else = details, combined with @details
  if (length(paragraphs) > 0) {
    details_para <- paste(paragraphs, collapse = "\n\n")

    # Find explicit @details tags
    didx <- which(tag_names == "details")
    if (length(didx) > 0) {
      explicit_details <- map_chr(tags[didx], "raw")
      tags <- tags[-didx]
      details_para <- paste(c(details_para, explicit_details), collapse = "\n\n")
    }

    details <- roxy_tag("details", details_para, NULL, intro$file, intro$line + offsets[[1]])
  } else {
    details <- NULL
  }

  c(compact(list(title, description, details)), tags)
}

#' Load roxygen2 options
#'
#' @description
#' Options can be stored in `DESCRIPTION` using `Config/roxygen2/` fields,
#' or in `man/roxygen/meta.R`. Call `roxy_meta_get()` to access current
#' option values from within tag and roclet methods.
#'
#' Options in `man/roxygen/meta.R` override those present in `DESCRIPTION`.
#'
#' @section Possible options:
#'
#' * `roclets` `<character>`: giving names of [roclets][roclet] to run. See
#'    [roclet_find()] for details.
#'
#' * `packages` `<character>`: packages to load that implement new tags.
#'
#' * `load` `<string>`: how to load R code. See [load] for details.
#'
#' * `old_usage` `<flag>`: use old style usage formatting?
#'
#' * `markdown` `<flag>`: translate markdown syntax to Rd?
#'
#' * `r6` `<flag>`: document R6 classes?
#'
#' * `current_package` `<string>` (read only): name of package being documented.
#'
#' * `rd_family_title` `<list>`: overrides for `@family` titles. See the
#'    _rd-functions_ vignette for details: `vignette("rd-functions")`
#'
#' * `knitr_chunk_options` `<list>`: default chunk options used for knitr.
#'
#' * `restrict_image_formats` `<flag>`: if `TRUE` then PDF images are only
#'   included in the PDF manual, and SVG images are only included in the HTML
#'   manual. (This only applies to images supplied via markdown.)
#'
#' @section How to set:
#' Either set in `DESCRIPTION` using `Config/roxygen2/` fields:
#'
#' ```
#' Config/roxygen2/markdown: TRUE
#' Config/roxygen2/load: installed
#' ```
#'
#' Or if you need more complex options (like `rd_family_title` or
#' `knitr_chunk_options`), put them in `man/roxygen/meta.R`:
#'
#' ```
#' list(
#'   rd_family_title = list(models = "Model functions"),
#'   knitr_chunk_options = list(fig.width = 7)
#' )
#' ```
#'
#' @param base_path Path to package.
#' @export
#' @family extending
load_options <- function(base_path = ".") {
  old <- load_options_roxygen(base_path)
  config <- load_options_config(base_path)
  desc <- utils::modifyList(old, config)
  meta <- load_options_meta(base_path)
  opts <- utils::modifyList(desc, meta)

  defaults <- list(
    roclets = c("collate", "namespace", "rd"),
    packages = character(),
    load = "pkgload",
    old_usage = FALSE,
    markdown = FALSE,
    r6 = TRUE,
    current_package = NA_character_,
    current_package_dir = NA_character_,
    rd_family_title = list(),
    knitr_chunk_options = NULL,
    restrict_image_formats = TRUE,
    lazy_data = FALSE
  )

  unknown_opts <- setdiff(names(opts), names(defaults))
  if (length(unknown_opts) > 0) {
    warn(paste0(
      "Unknown Roxygen options ",
      paste(unknown_opts, collapse = ", "),
      ".\n",
      "Supported options: ",
      paste(names(defaults), collapse = ", ")
    ))
  }

  utils::modifyList(defaults, opts)
}

load_options_roxygen <- function(base_path = ".") {
  desc_path <- file.path(base_path, "DESCRIPTION")
  dcf <- read.dcf(desc_path, fields = c("Roxygen", "Package", "LazyData"))
  desc_opts <- dcf[[1, 1]]

  if (is.na(desc_opts)) {
    opts <- list()
  } else {
    opts <- eval(parse(text = desc_opts), child_env(baseenv()))
  }

  opts$current_package <- dcf[[1, 2]]
  opts$current_package_dir <- normalizePath(base_path)
  opts$lazy_data <- tolower(dcf[[1, 3]]) %in% c("true", "yes")
  opts
}

config_fields <- c(
  "markdown",
  "load",
  "old_usage",
  "r6",
  "restrict_image_formats",
  "packages",
  "roclets"
)

# Fields that should be parsed as a single TRUE/FALSE value
config_flags <- c(
  "markdown",
  "old_usage",
  "r6",
  "restrict_image_formats"
)

load_options_config <- function(base_path = ".") {
  fields <- paste0("Config/roxygen2/", config_fields)
  values <- desc::desc_get(fields, file = base_path)
  names(values) <- config_fields

  values <- values[!is.na(values)]
  Map(parse_config_value, values, names(values))
}

parse_config_value <- function(x, field = "") {
  values <- scan(
    text = x,
    what = "character",
    sep = ",",
    strip.white = TRUE,
    quiet = TRUE
  )

  if (field %in% config_flags) {
    flag <- as.logical(toupper(values))
    if (length(flag) != 1 || is.na(flag)) {
      cli::cli_abort(
        "{.field Config/roxygen2/{field}} must be {.val TRUE} or {.val FALSE}, not {.val {x}}.",
        call = NULL
      )
    }
    return(flag)
  }

  utils::type.convert(values, as.is = TRUE)
}

load_options_meta <- function(base_path = ".", path = "man/roxygen/meta.R") {
  # Only look for .R for consistency with style advice
  meta_path <- file.path(base_path, path)

  if (!file.exists(meta_path)) {
    return(list())
  }

  value <- tryCatch(
    source(meta_path, local = child_env(baseenv()))$value,
    error = function(err) {
      warn("Failed to source `man/roxygen/meta.R`")
      list()
    }
  )

  if (!is.list(value)) {
    warn("`man/roxygen/meta.R` must yield a named list")
    return(list())
  }

  value
}

# Global binding management -----------------------------------------------

roxy_meta <- new_environment()

#' @export
#' @rdname load_options
#' @param key Key of the options, e.g. `"packages"`.
#' @param default Default value.
roxy_meta_get <- function(key = NULL, default = NULL) {
  env_get(roxy_meta, key, default = default)
}

roxy_meta_set <- function(key, value = NULL) {
  env_poke(roxy_meta, key, value)
}

roxy_meta_clear <- function() {
  env_unbind(roxy_meta, env_names(roxy_meta))
}

roxy_meta_load <- function(base_path = getwd()) {
  roxy_meta_clear()
  env_bind(roxy_meta, !!!load_options(base_path))
}

local_roxy_meta_set <- function(key, value, envir = caller_env()) {
  old_value <- roxy_meta_set(key, value)
  withr::defer(roxy_meta_set(key, old_value), envir = envir)
}

test_that("can load options from old Roxygen field", {
  opts <- load_options(test_path("test-options"))
  expect_equal(opts$old_usage, TRUE) # from DESCRIPTION
  expect_equal(opts$markdown, TRUE) # from meta.R
})

test_that("can load options from Config/roxygen2/ fields", {
  opts <- load_options(test_path("test-options-config"))
  expect_equal(opts$old_usage, TRUE)
  expect_equal(opts$packages, c("mypkg1", "mypkg2"))
})

test_that("Config/roxygen2/ fields take priority over Roxygen field", {
  opts <- load_options(test_path("test-options-both"))
  expect_equal(opts$old_usage, TRUE) # from Roxygen field

  expect_equal(opts$markdown, FALSE) # Config overrides Roxygen
})

test_that("parse_config_value handles scalars and vectors", {
  expect_equal(parse_config_value("TRUE"), TRUE)
  expect_equal(parse_config_value("FALSE"), FALSE)

  expect_equal(parse_config_value("installed"), "installed")
  expect_equal(parse_config_value("a, b, c"), c("a", "b", "c"))
})

test_that("warns on invalid meta.R files", {
  expect_warning(
    load_options_meta(test_path("test-options"), "meta-error.R"),
    "Failed to source"
  )

  expect_warning(
    load_options_meta(test_path("test-options"), "meta-character.R"),
    "yield a named list"
  )
})

test_that("proper link references are added", {
  cases <- list(
    c("foo [func()] bar",           "[func()]: R:func()"),
    c("foo [obj] bar",              "[obj]: R:obj"),
    c("foo [text][func()] bar",     "[func()]: R:func()"),
    c("foo [text][obj] bar",        "[obj]: R:obj"),
    c("foo [pkg::func()] bar",       "[pkg::func()]: R:pkg::func()"),
    c("foo [pkg::obj] bar",          "[pkg::obj]: R:pkg::obj"),
    c("foo [text][pkg::func()] bar", "[pkg::func()]: R:pkg::func()"),
    c("foo [text][pkg::obj] bar",    "[pkg::obj]: R:pkg::obj"),
    c("foo [linktos4-class] bar",    "[linktos4-class]: R:linktos4-class"),
    c("foo [pkg::s4-class] bar",     "[pkg::s4-class]: R:pkg::s4-class")
  )

  for (i in seq_along(cases)) {
    expect_match(
      add_linkrefs_to_md(cases[[i]][1]),
      cases[[i]][2],
      fixed = TRUE
    )
  }
})

test_that("can not have [ inside of link", {
  expect_equal(
    markdown("`[[`. [subset()]"),
    "\\code{[[}. \\code{\\link[=subset]{subset()}}"
  )
})

test_that("can escape [ to avoid spurious links", {
  expect_equal(
    markdown("\\[test\\]"),
    "[test]"
  )

  expect_equal(
    markdown("\\[ [test] \\]"),
    "[ \\link{test} ]"
  )
})

test_that("\\Sexpr with options not converted to links", {
  expect_equal(
     markdown("\\Sexpr[results=rd]{runif(1)}"),
     "\\Sexpr[results=rd]{runif(1)}"
   )
})

test_that("% in links are escaped", {
  expect_equal(markdown("[x][%%]"), "\\link[=\\%\\%]{x}")
  expect_equal(markdown("[%][x]"), "\\link[=x]{\\%}")
  expect_equal(markdown("[%%]"), "\\link{\\%\\%}")
  expect_equal(markdown("[base::%%]"), "\\link[base:Arithmetic]{base::\\%\\%}")
})

test_that("{ and } in links are escaped (#1259)", {
  expect_equal(markdown("[`foo({ bar })`][x]"), "\\code{\\link[=x]{foo(\\{ bar \\})}}")
  expect_equal(markdown("[`{{`][x]"), "\\code{\\link[=x]{\\{\\{}}")

  # Non code parts are not escaped (invalid Rd)
  expect_equal(markdown("[foo({ bar })][x]"), "\\link[=x]{foo({ bar })}")
})

test_that("non-text nodes in links fails", {
  tag <- roxy_tag("title", NULL, NULL, file = "foo.R", line = 10)

  expect_snapshot({
    markdown("[`foo` bar][x]", tag = tag)
    markdown("[__baz__][x]", tag = tag)
  })
})

test_that("commonmark picks up the various link references", {
  cases <- list(
    list("foo [func()] bar",            c("R:func()", "func()")),
    list("foo [obj] bar",               c("R:obj", "obj")),
    list("foo [text][func()] bar",      c("R:func()", "text")),
    list("foo [text][obj] bar",         c("R:obj", "text")),
    list("foo [pkg::func()] bar",       c("R:pkg::func()", "pkg::func()")),
    list("foo [pkg::obj] bar",          c("R:pkg::obj", "pkg::obj")),
    list("foo [text][pkg::func()] bar", c("R:pkg::func()", "text")),
    list("foo [text][pkg::obj] bar",    c("R:pkg::obj", "text")),
    list("foo [linktos4-cl] bar",       c("R:linktos4-cl", "linktos4-cl")),
    list("foo [pkg::s4-cl] bar",        c("R:pkg::s4-cl", "pkg::s4-cl"))
  )

  for (i in seq_along(cases)) {
    x <- commonmark::markdown_xml(add_linkrefs_to_md(cases[[i]][[1]]))
    xdoc <- xml2::xml_ns_strip(xml2::read_xml(x))
    link <- xml2::xml_find_first(xdoc, "//link")
    expect_equal(xml2::xml_attr(link, "destination"), cases[[i]][[2]][1])
    text <- xml2::xml_find_first(link, "./text")
    expect_equal(xml2::xml_text(text), cases[[i]][[2]][2], info = i)
  }
})

test_that("short and sweet links work", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [function()].
    #' And also [object].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\code{\\link[=function]{function()}}.
    #' And also \\link{object}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  expect_snapshot(
    out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' See [11pkg::function()], [11pkg::object].
    #' @md
    foo <- function() {}")[[1]]
  )
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' See \\code{\\link[11pkg:function]{11pkg::function()}}, \\link[11pkg:object]{11pkg::object}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [name][dest].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\link[=dest]{name}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  block <- "
    #' Title
    #'
    #' Description, see [name words][stringr::bar111].
    #' @md
    foo <- function() {}
  "
  expect_snapshot(out1 <- roc_proc_text(rd_roclet(), block)[[1]])

  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\link[stringr:bar111]{name words}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [terms][terms.object].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\link[=terms.object]{terms}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [abc][abc-class].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\link[=abc-class]{abc}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title.
    #'
    #' In another package: [and this one][desc::desc].
    #' [name words][desc::desc].
    #'
    #' @md
    #' @name markdown-test
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title.
    #'
    #' In another package: \\link[desc:desc]{and this one}.
    #' \\link[desc:desc]{name words}.
    #'
    #' @name markdown-test
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("a weird markdown link bug is fixed", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Dummy page to test roxygen's markdown formatting
    #'
    #' Links are very tricky, so I'll put in some links here:
    #' Link to a function: [roxygenize()].
    #' Link to an object: [roxygenize] (we just treat it like an object here).
    #'
    #' Link to another package, function: [desc::desc()].
    #' Link to another package, non-function: [desc::desc].
    #'
    #' Link with link text: [this great function][roxygenize()],
    #' [`roxygenize`][roxygenize()], or [that great function][roxygenize].
    #'
    #' In another package: [and this one][desc::desc].
    #'
    #' @md
    #' @name markdown-test
    #' @keywords internal
    NULL")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Dummy page to test roxygen's markdown formatting
    #'
    #' Links are very tricky, so I'll put in some links here:
    #' Link to a function: \\code{\\link[=roxygenize]{roxygenize()}}.
    #' Link to an object: \\link{roxygenize} (we just treat it like an object here).
    #'
    #' Link to another package, function: \\code{\\link[desc:desc]{desc::desc()}}.
    #' Link to another package, non-function: \\link[desc:desc]{desc::desc}.
    #'
    #' Link with link text: \\link[=roxygenize]{this great function},
    #' \\code{\\link[=roxygenize]{roxygenize}}, or \\link[=roxygenize]{that great function}.
    #'
    #' In another package: \\link[desc:desc]{and this one}.
    #'
    #' @name markdown-test
    #' @keywords internal
    NULL")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("another markdown link bug is fixed", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [escape_rd_for_md()].
    #'
    #' And also [object].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\code{\\link[=escape_rd_for_md]{escape_rd_for_md()}}.
    #'
    #' And also \\link{object}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("markdown code as link text is rendered as code", {

  suppressWarnings(out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [`name`][dest],
    #' [`function`][function()],
    #' [`filter`][stats::filter()],
    #' [`bar`][pkg::bar],
    #' [`terms`][terms.object],
    #' [`abc`][abc-class].
    #' @md
    foo <- function() {}")[[1]])
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\code{\\link[=dest]{name}},
    #' \\code{\\link[=function]{function}},
    #' \\code{\\link[stats:filter]{filter}},
    #' \\code{\\link[pkg:bar]{bar}},
    #' \\code{\\link[=terms.object]{terms}},
    #' \\code{\\link[=abc-class]{abc}}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("non-code link in backticks works", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [`foobar`].
    #' Also [`this_too`].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\code{\\link{foobar}}.
    #' Also \\code{\\link{this_too}}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("[] is not picked up in code", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @param connect_args `[named list]`\\cr Connection arguments
    #' Description, see `[foobar]`.
    #' Also `[this_too]`.
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @param connect_args \\verb{[named list]}\\cr Connection arguments
    #' Description, see \\verb{[foobar]}.
    #' Also \\verb{[this_too]}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("[]() links are still fine", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [some thing](http://www.someurl.com).
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\href{http://www.someurl.com}{some thing}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Link
    #' text [broken
    #' across lines](http://www.someurl.com) preserve
    #' whitespace, even when
    #' [broken across
    #' several
    #' lines](http://www.someurl.com),
    #' or with varying
    #' [amounts \
    #'   of  \
    #' interspersed   \
    #'   whitespace](http://www.someurl.com).
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Link
    #' text \\href{http://www.someurl.com}{broken across lines} preserve
    #' whitespace, even when
    #' \\href{http://www.someurl.com}{broken across several lines},
    #' or with varying
    #' \\href{http://www.someurl.com}{amounts of interspersed whitespace}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

})

test_that("links to S4 classes are OK", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [linktos4-class] as well.
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\linkS4class{linktos4} as well.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

  suppressWarnings(out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [pkg::linktos4-class] as well.
    #' @md
    foo <- function() {}")[[1]])
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\link[pkg:linktos4-class]{pkg::linktos4} as well.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

})

test_that("linebreak in 'text' of [text][foo] turns into single space", {

  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Link
    #' text [broken
    #' across lines][fcn] preserve
    #' whitespace, even when
    #' [broken across
    #' several
    #' lines][fcn],
    #' or with varying
    #' [amounts \
    #'   of  \
    #' interspersed   \
    #'   whitespace][fcn].
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Link
    #' text \\link[=fcn]{broken across lines} preserve
    #' whitespace, even when
    #' \\link[=fcn]{broken across several lines},
    #' or with varying
    #' \\link[=fcn]{amounts of interspersed whitespace}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)

})

test_that("markup in link text", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see [`code link text`][func].
    #' And also [`code as well`](https://external.com).
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description, see \\code{\\link[=func]{code link text}}.
    #' And also \\href{https://external.com}{\\verb{code as well}}.
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("linking to self is unqualified", {
  local_roxy_meta_set("current_package", "myself")
  rd <- markdown("foo [myself::fun()] and [myself::obj] bar")
  expect_equal(
    rd,
    "foo \\code{\\link[=fun]{fun()}} and \\link{obj} bar"
  )
})

test_that("percents are escaped in link targets", {
  out1 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' [link % text](https://foo.bar/link%20target)
    #' @md
    foo <- function() {}")[[1]]
  out2 <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' \\href{https://foo.bar/link%20target}{link % text}
    #' @md
    foo <- function() {}")[[1]]
  expect_equivalent_rd(out1, out2)
})

test_that("don't resolve if current_package not set", {
  expect_equal(find_package("cli_abort"), NA_character_)
})

test_that("topics in current package don't need qualification", {
  local_roxy_meta_set("current_package", "cli")
  expect_equal(find_package("cli_abort"), NA_character_)
})

test_that("imported functions qualified with package name", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_equal(find_package("cli_abort"), "cli")
})

test_that("base functions don't need qualification", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_equal(find_package("mean"), NA_character_)
})

test_that("base functions re-exported by deps don't need qualification", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_equal(find_package("is.null"), NA_character_)
})

test_that("useful warning if no topic found", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_snapshot(. <- find_package("doesntexist"))
})

test_that("re-exported topics are identified", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_equal(. <- find_package("process"), "processx")
})

test_that("gives useful warning if same name in multiple packages", {
  skip_on_cran() # in case pkgload/rlang changes this
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  expect_snapshot(. <- find_package("pkg_env"))
})


test_that("topic found in multiple base packages doesn't warn", {
  local_roxy_meta_set("current_package", "testMdLinks")
  local_roxy_meta_set("current_package_dir", test_path("testMdLinks"))

  # plot is in both base and graphics
  expect_no_message(expect_equal(find_package("plot"), NA_character_))
})

# Manage a list of topics, indexed by file name.
# Adding a topic with an existing file name merges it with the existing topic
RoxyTopics <- R6::R6Class(
  "RoxyTopics",
  public = list(
    topics = list(),

    add = function(topic, block) {
      if (is.null(topic)) {
        return()
      }
      stopifnot(inherits(topic, "RoxyTopic"))

      filename <- topic$filename
      if (filename %in% names(self$topics)) {
        self$topics[[filename]]$add(topic, block)
      } else {
        self$topics[[filename]] <- topic
      }

      invisible()
    },

    # Drop any topics that don't have a title
    drop_invalid = function() {
      for (topic in names(self$topics)) {
        if (!self$topics[[topic]]$is_valid()) {
          warn_roxy_topic(topic, "Skipping; no name and/or title")
          self$topics[[topic]] <- NULL
        }
      }

      invisible()
    },

    get = function(filename) {
      self$topics[[filename]]
    },

    # Given a topic name, find its file name.
    find_filename = function(name) {
      for (i in seq_along(self$topics)) {
        if (name %in% self$topics[[i]]$get_value("name")) {
          return(names(self$topics)[[i]])
        }
      }
      NA_character_
    },

    # Topologically sort the topics.
    #
    # @param deps A function. Is passed RoxyTopic, and should return a character
    #   vector of topic names
    topo_order = function(dependencies) {
      topo <- TopoSort$new()

      for (i in seq_along(self$topics)) {
        name <- names(self$topics)[[i]]
        topo$add(name)

        dep_topics <- dependencies(self$topics[[i]])
        for (dep_topic in dep_topics) {
          dep_rd <- self$find_filename(dep_topic)
          if (!is.na(dep_rd)) {
            topo$add_ancestor(name, dep_rd)
          }
        }
      }

      topo$sort()
    },

    # Call fun in topological order defined by dep.
    topo_apply = function(dep, fun, ...) {
      topics_topo <- self$topo_order(dep)
      for (topic_name in topics_topo) {
        topic <- self$get(topic_name)
        fun(topic, self, ...)
      }

      invisible()
    },

    apply = function(fun, ...) {
      for (topic in self$topics) {
        fun(topic, self, ...)
      }
      invisible()
    },

    # Extract values for simple fields
    simple_values = function(field) {
      fields <- lapply(self$topics, \(rd) rd$get_section(field))
      lapply(compact(fields), "[[", "value")
    }
  )
)

test_that("useful error if no DESCRIPTION", {
  path <- local_package_copy(test_path("no-desc"), set_version = FALSE)

  expect_snapshot(
    roxygen_setup(path),
    error = TRUE,
    transform = function(x) gsub(path, "<path>", x)
  )
})

test_that("informs about initial setup", {
  path <- local_package_copy(test_path("empty"), set_version = FALSE)

  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))
})

test_that("warns about non UTF-8 encoding", {
  path <- local_package_copy(test_path("empty"))
  desc::desc_set(
    file = path,
    Encoding = "latin1",
    "Config/roxygen2/version" = "8.0.0"
  )

  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))
})

test_that("warns if roxygen version is too new", {
  path <- local_package_copy(test_path("empty"))
  desc::desc_set(file = path, "Config/roxygen2/version" = "10.0.0")

  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))
})

test_that("informs about major changes in 7.0.0", {
  path <- local_package_copy(test_path("empty"), set_version = FALSE)
  desc::desc_set(file = path, RoxygenNote = "5.0.0")

  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))
})

test_that("removes old RoxygenNote field", {
  path <- local_package_copy(test_path("empty"), set_version = FALSE)
  desc::desc_set(file = path, RoxygenNote = "7.0.0")

  suppressMessages(roxygen_setup(path, cur_version = "8.0.0"))

  desc <- desc::desc(file = path)
  expect_false(desc$has_fields("RoxygenNote"))
  expect_equal(desc$get("Config/roxygen2/version")[[1]], "8.0.0")
})

test_that("removes old RoxygenNote field even if already migrated (#1876)", {
  path <- local_package_copy(test_path("empty"))
  desc::desc_set(
    file = path,
    "Config/roxygen2/version" = "8.0.0",
    RoxygenNote = "7.3.3"
  )

  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))

  desc <- desc::desc(file = path)
  expect_false(desc$has_fields("RoxygenNote"))
  expect_equal(desc$get("Config/roxygen2/version")[[1]], "8.0.0")
})

test_that("doesn't touch DESCRIPTION if already up to date", {
  path <- local_package_copy(test_path("empty"))
  desc::desc_set(file = path, "Config/roxygen2/version" = "8.0.0")

  desc_path <- file.path(path, "DESCRIPTION")
  before <- file.mtime(desc_path)
  expect_snapshot(roxygen_setup(path, cur_version = "8.0.0"))
  expect_equal(file.mtime(desc_path), before)
})

#' @export
roxy_tag_parse.roxy_tag_describeIn <- function(x) {
  if (!is.na(x$raw) && !grepl("[[:space:]]+", x$raw)) {
    warn_roxy_tag(
      x,
      c(
        "requires a name and description",
        i = "Did you want @rdname instead?"
      )
    )
    NULL
  } else {
    tag_two_part(x, "a topic name", "a description", multiline = "always")
  }
}

topic_add_describe_in <- function(topic, block, env) {
  tag <- block_get_tag(block, "describeIn")
  if (is.null(tag)) {
    return()
  }

  if (is.null(block$object)) {
    warn_roxy_tag(tag, "must be used with an object")
    return()
  }
  if (block_has_tags(block, "name")) {
    warn_roxy_tag(tag, "can not be used with @name")
    return()
  }
  if (block_has_tags(block, "rdname")) {
    warn_roxy_tag(tag, "can not be used with @rdname")
    return()
  }
  if (is.null(object_name(block$object))) {
    warn_roxy_tag(tag, "not supported with this object type")
    return()
  }

  dest <- find_object(tag$val$name, env)
  metadata <- build_minidesc_metadata(block$object, dest)

  topic$add(rd_section_minidesc(
    name = object_name(block$object),
    desc = tag$val$description,
    extends = metadata$extends,
    generic = metadata$generic,
    class = metadata$class
  ))
  dest$topic
}

# Field -------------------------------------------------------------------

#' Record data for minidescription sections from `@describeIn`
#'
#' @param name name of the source function.
#' @param desc description passed to `@describeIn`.
#' @param extends how the source function extends the destination function:
#' - `"generic"` if the source extends a (S3 or S4) generic in the destination,
#' - `"class"` if the source extends an informal S3 or formal S4 constructor
#'    in the destination.
#'    For S3, there is always only *one* class.
#'    For S4, the methods' signature is used instead, to cover multiple dispatch.
#' - `""` (default) otherwise.
#' @param generic,class name of the generic and class that is being extended by
#' the method, otherwise empty string (`""`).
#' @return a dataframe with one row for each `@describeIn`, wrapped inside
#' `rd_section()`
#' @noRd
rd_section_minidesc <- function(
  name,
  desc,
  extends = c("", "generic", "class"),
  generic = "",
  class = ""
) {
  check_string(name)
  check_character(desc)
  rlang::arg_match(extends)
  check_string(generic)
  check_string(class)

  data <- data.frame(
    name = name,
    desc = desc,
    extends = extends,
    generic = generic,
    class = class
  )
  rd_section("minidesc", data)
}

#' @export
merge.rd_section_minidesc <- function(x, y, ..., block) {
  stopifnot(identical(class(x), class(y)))

  rd_section("minidesc", rbind(x$value, y$value))
}

# Rd Output -------------------------------------------------------------------

#' @export
format.rd_section_minidesc <- function(x, ...) {
  order <- intersect(c("generic", "class", ""), unique(x$value$extends))
  by <- factor(x$value$extends, levels = order)
  subsections <- split(x$value, by)
  body <- map2_chr(subsections, names(subsections), format_section)

  paste0(body, collapse = "\n")
}

format_section <- function(df, type) {
  title <- switch(
    type,
    class = "Methods (by generic)",
    generic = "Methods (by class)",
    "Functions"
  )

  bullets <- paste0("\\code{", df$name, "}: ", df$desc, "\n")
  body <- paste0(
    "\\itemize{\n",
    paste0("\\item ", bullets, "\n", collapse = ""),
    "}"
  )

  paste0("\\section{", title, "}{\n", body, "}")
}

# Helpers -----------------------------------------------------------------

# Imperfect:
# * will fail with S3 methods that need manual disambiguation (rare)
# * can't use if @name overridden, but then you could just the use alias
find_object <- function(name, env) {
  if (methods::isClass(name, where = env)) {
    object(methods::getClass(name, where = env), NULL, "s4class")
  } else if (exists(name, envir = env)) {
    object_from_name(name, env, NULL)
  } else {
    object(NULL, name, "data")
  }
}

#' Build metadata for how to present `@describeIn` tag
#' @return list of character scalars named `extends`, `generic`, `class`.
#' See rd_section_minidesc() for details.
#' @noRd
build_minidesc_metadata <- function(src, dest) {
  src_type <- class(src)[1]
  dest_type <- class(dest)[1]
  dest_name <- as.character(dest$topic)

  if (src_type == "s3method") {
    generic <- attr(src$value, "s3method")[1]
    class <- attr(src$value, "s3method")[2]
    if (dest_type == "s3generic" && generic == dest_name) {
      # src method fits dest generic
      extends <- "generic"
    } else if (fits_constructor(dest_name, src)) {
      # src method fits informal dest constructor (heuristically)
      extends <- "class"
    } else {
      extends <- ""
    }
  } else if (src_type == "s4method") {
    generic <- as.character(src$value@generic)
    class <- sig2class(src$value@defined)
    if (dest_type == "s4generic") {
      # TODO must test whether src method fits dest generic
      extends <- "generic"
    } else if (dest_type == "s4class") {
      extends <- "class"
      # TODO must test whether src method fits dest constructor
    } else {
      extends <- ""
    }
  } else {
    generic <- ""
    class <- ""
    extends <- ""
  }
  list(extends = extends, generic = generic, class = class)
}

# Turn S4 signature into a string
sig2class <- function(sig) {
  if (length(sig) == 1) {
    as.character(sig)
  } else {
    paste0(names(sig), " = ", sig, collapse = ",")
  }
}

# Is destination is probably constructor for src?
fits_constructor <- function(dest_name, src) {
  src_class <- attr(src$value, "s3method")[2]

  # simple case where class name is the same as the constructor name
  if (src_class == dest_name) {
    return(TRUE)
  }

  # more complex case where class name = package name + constructor name
  evalenv <- roxy_meta_get("env") %||% parent.frame() # needed for tests
  pkg_name <- utils::packageName(evalenv) %||% ""

  src_class == paste0(pkg_name, "_", dest_name)
}


object_name <- function(x) {
  UseMethod("object_name")
}
#' @export
object_name.default <- function(x) {
  x$alias
}
#' @export
object_name.function <- function(x) {
  object_name_fun(x$alias, x)
}
#' @export
object_name.s3generic <- object_name.function
#' @export
object_name.s3method <- function(x) {
  method <- attr(x$value, "s3method")
  as.character(function_usage(method[[1]], list(as.name(method[[2]]))))
  #
  #   name <- paste(, collapse = ".")
  #   object_name_fun(name, x)
}
#' @export
object_name.s4generic <- function(x) {
  object_name_fun(x$value@generic, x)
}
#' @export
object_name.s4method <- function(x) {
  classes <- lapply(x$value@defined, as.name)
  if (length(classes) == 1) {
    names(classes) <- NULL
  }
  as.character(function_usage(x$value@generic, classes))
}

object_name_fun <- function(name, x, format_name = identity) {
  if (is_replacement_fun(name) || is_infix_fun(name)) {
    args <- formals(x$value)
  } else {
    args <- NULL
  }

  as.character(function_usage(name, args, format_name))
}

#' Expand embedded inline code
#'
#' @description
#' For example this becomes two: `r 1+1`.
#' Variables can be set and then reused, within the same
#' tag: `r x <- 100; NULL`
#' The value of `x` is `r x`.
#'
#' We have access to the internal functions of the package, e.g.
#' since this is _roxygen2_, we can refer to the internal `markdown`
#' function, and this is `TRUE`: `r is.function(markdown)`.
#'
#' To insert the name of the current package: `r packageName()`.
#'
#' The `iris` data set has `r ncol(iris)` columns:
#' `r paste0("\x60\x60", colnames(iris), "\x60\x60", collapse = ", ")`.
#'
#' ```{r}
#' # Code block demo
#' x + 1
#' ```
#'
#' Chunk options:
#'
#' ```{r results = "hold"}
#' names(mtcars)
#' nrow(mtcars)
#' ```
#'
#' Plots:
#'
#' ```{r test-figure}
#' plot(1:10)
#' ```
#'
#' Alternative knitr engines:
#'
#' ```{verbatim}
#' #| file = "tests/testthat/example.Rmd"
#' ```
#'
#' Also see `vignette("rd-formatting")`.
#'
#' @param text Input text.
#' @return
#' Text with R code expanded.
#' A character vector of the same length as the input `text`.
#'
#' @keywords internal
markdown_evaluate <- function(text) {
  text <- paste(text, collapse = "\n")
  mdxml <- xml_ns_strip(md_to_mdxml(text, sourcepos = TRUE))
  code_nodes <- xml_find_all(mdxml, ".//code | .//code_block")
  rcode_nodes <- keep(code_nodes, is_markdown_code_node)
  if (length(rcode_nodes) == 0) {
    return(text)
  }
  rcode_pos <- parse_md_pos(map_chr(rcode_nodes, xml_attr, "sourcepos"))
  rcode_pos <- work_around_cmark_sourcepos_bug(text, rcode_pos)
  out <- eval_code_nodes(rcode_nodes)
  re_set_all_pos(text, rcode_pos, out, rcode_nodes)
}

# Work around commonmark sourcepos bug for inline R code
# https://github.com/r-lib/roxygen2/issues/1353
work_around_cmark_sourcepos_bug <- function(text, rcode_pos) {
  if (Sys.getenv("ROXYGEN2_NO_SOURCEPOS_WORKAROUND", "") != "") {
    return(rcode_pos)
  }

  lines <- strsplit(text, "\n", fixed = TRUE)[[1]]

  for (l in seq_len(nrow(rcode_pos))) {
    # Do not try to fix multi-line code, we error for that (below)
    if (rcode_pos$start_line[l] != rcode_pos$end_line[l]) {
      next
    }
    line <- lines[rcode_pos$start_line[l]]
    start <- rcode_pos$start_column[l]

    # Maybe correct? At some point this will be fixed upstream, hopefully.
    if (substr(line, start - 1, start + 1) == "`r ") {
      next
    }

    # Maybe indented and we can shift it?
    # It is possible that the shift that we try accidentally matches
    # "`r ", but it seems to be extremely unlikely. An example is this:
    # #'       ``1`r `` `r 22*10`
    # (seven spaces after the #', so an indent of six spaces. If we shift
    # the real "`r " left by six characters, there happens to be another
    # "`r " there.

    m <- regexpr("^[ ]+", line)
    indent <- attr(m, "match.length")
    if (
      m > 0L &&
        substr(line, start - 1 + indent, start + 1 + indent) == "`r "
    ) {
      rcode_pos$start_column[l] <- rcode_pos$start_column[l] + indent
      rcode_pos$end_column[l] <- rcode_pos$end_column[l] + indent
    }
  }

  rcode_pos
}

is_markdown_code_node <- function(x) {
  info <- xml_attr(x, "info")
  substr(xml_text(x), 1, 2) == "r " ||
    (!is.na(info) && grepl("^[{][a-zA-z]+[}, ]", info))
}

parse_md_pos <- function(text) {
  nums <- map(strsplit(text, "[:-]"), as.integer)
  data.frame(
    start_line = map_int(nums, \(x) x[[1]]),
    start_column = map_int(nums, \(x) x[[2]]),
    end_line = map_int(nums, \(x) x[[3]]),
    end_column = map_int(nums, \(x) x[[4]])
  )
}

eval_code_nodes <- function(nodes) {
  evalenv <- roxy_meta_get("evalenv")
  # This should only happen in our test cases
  if (is.null(evalenv)) {
    evalenv <- new.env(parent = baseenv())
  }

  map_chr(nodes, eval_code_node, env = evalenv)
}

eval_code_node <- function(node, env) {
  if (xml_name(node) == "code") {
    # write knitr markup for inline code
    text <- paste0("`", xml_text(node), "`")
  } else {
    lang <- xml_attr(node, "info")
    # write knitr markup for fenced code
    text <- paste0("```", if (!is.na(lang)) lang, "\n", xml_text(node), "```\n")
  }

  chunk_opts <- utils::modifyList(
    knitr_chunk_defaults(),
    as.list(roxy_meta_get("knitr_chunk_options", NULL))
  )

  roxy_knit(text, env, chunk_opts)
}

knitr_chunk_defaults <- function() {
  list(
    error = FALSE,
    fig.path = "man/figures/",
    fig.process = basename,
    comment = "#>",
    collapse = TRUE
  )
}

re_set_all_pos <- function(text, pos, value, nodes) {
  # Cmark has a bug when reporting source positions for multi-line
  # code tags, and it does not count the indenting space in the
  # continuation lines: https://github.com/commonmark/cmark/issues/296
  types <- xml_name(nodes)
  if (any(types == "code" & pos$start_line != pos$end_line)) {
    cli::cli_abort("multi-line `r ` markup is not supported", call = NULL)
  }

  # Need to split the string, because of the potential multi-line
  # code tags, and then also recode the positions
  lens <- nchar(strsplit(text, "\n", fixed = TRUE)[[1]])
  shifts <- c(0, cumsum(lens + 1L))
  shifts <- shifts[-length(shifts)]
  start <- shifts[pos$start_line] + pos$start_column
  end <- shifts[pos$end_line] + pos$end_column

  # Create intervals for the parts we keep
  keep_start <- c(1, end + 2L)
  keep_end <- c(start - 2L, nchar(text))

  # Now piece them together
  out <- paste0(
    substring(text, keep_start, keep_end),
    c(value, ""),
    collapse = ""
  )
  attributes(out) <- attributes(text)
  out
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

escapeExamples <- function(x) {
    .Call(`_roxygen2_escapeExamples`, x)
}

findEndOfTag <- function(string, is_code = FALSE) {
    .Call(`_roxygen2_findEndOfTag`, string, is_code)
}

rdComplete <- function(string, is_code = FALSE) {
    .Call(`_roxygen2_rdComplete`, string, is_code)
}

leadingSpaces <- function(lines) {
    .Call(`_roxygen2_leadingSpaces`, lines)
}

tokenise_block <- function(lines, file = "", offset = 0L) {
    .Call(`_roxygen2_tokenise_block`, lines, file, offset)
}

find_includes <- function(path) {
    .Call(`_roxygen2_find_includes`, path)
}

wrapUsage <- function(string, width = 80L, indent = 2L) {
    .Call(`_roxygen2_wrapUsage`, string, width, indent)
}

# rdComplete ---------------------------------------------------------------

test_that("braces must balance", {
  expect_true(rdComplete("{}", is_code = FALSE))
  expect_true(rdComplete("{{}}", is_code = FALSE))

  expect_false(rdComplete("{", is_code = FALSE))
  expect_false(rdComplete("}", is_code = FALSE))
})

test_that("can't end with escape", {
  expect_false(rdComplete("\\", is_code = FALSE))
})

test_that("escaped brackets are ignored", {
  expect_true(rdComplete("\\{", is_code = FALSE))
  expect_true(rdComplete("\\}", is_code = FALSE))
  expect_false(rdComplete("{\\}", is_code = FALSE))
})

test_that("brackets in comments are ignored", {
  expect_true(rdComplete("% {", is_code = FALSE))
  expect_true(rdComplete("% }", is_code = FALSE))
})

test_that("R comments don't close latex-like tags", {
  expect_true(rdComplete(r"(A comment \code{#}.)", is_code = FALSE))
})

test_that("newline ends comment", {
  expect_false(rdComplete("%\n{", is_code = FALSE))
})

test_that("escape disables comment", {
  expect_false(rdComplete(r"(\%{)", is_code = FALSE))
})

test_that("strings must be closed in code", {
  expect_false(rdComplete("'", is_code = TRUE))
  expect_false(rdComplete('"', is_code = TRUE))
})

test_that("strings respect escapes", {
  expect_false(rdComplete("'\\'", is_code = TRUE)) # '\'
  expect_true(rdComplete("'\\''", is_code = TRUE)) # '\''
})

test_that("braces in strings don't need to match in code", {
  expect_true(rdComplete("'{{'", is_code = TRUE))
})

test_that("raw strings are handled in code", {
  expect_true(rdComplete('r"(text)"', is_code = TRUE))
  expect_true(rdComplete('r"{text}"', is_code = TRUE))
  expect_true(rdComplete('r"["text]"', is_code = TRUE))
  expect_true(rdComplete('R"(text)"', is_code = TRUE))
  expect_true(rdComplete('r"--(text)--"', is_code = TRUE))
  expect_true(rdComplete("r'(text)'", is_code = TRUE))
})

test_that("incomplete raw strings are detected in code", {
  expect_false(rdComplete('r"', is_code = TRUE))
  expect_false(rdComplete('r"(', is_code = TRUE))

  expect_false(rdComplete('r"(text', is_code = TRUE))
  expect_false(rdComplete('r"(text)', is_code = TRUE))
  expect_false(rdComplete('r"(text)\'', is_code = TRUE))

  expect_false(rdComplete('r"--(text)--', is_code = TRUE))
  expect_false(rdComplete('r"--(text)-"', is_code = TRUE))
  expect_false(rdComplete('r"--(text)-', is_code = TRUE))
  expect_false(rdComplete('r"--(text)"', is_code = TRUE))
})

test_that("braces in raw strings don't need to match in code", {
  expect_true(rdComplete('r"({)"', is_code = TRUE))
})

test_that("strings in code comments don't need to be closed", {
  expect_true(rdComplete("# '", is_code = TRUE))
})

test_that("braces in code must match", {
  expect_false(rdComplete("# {", is_code = TRUE))
  expect_true(rdComplete("# {}", is_code = TRUE))
  expect_false(rdComplete("# {} {", is_code = TRUE))

  # Allow user to close a brace in a comment (needed by `@examplesIf`)
  expect_true(rdComplete("{ # }", is_code = TRUE))
})

# findEndOfTag -------------------------------------------------------------

test_that("handles simple cases", {
  expect_equal(findEndOfTag("{abc}", is_code = FALSE, start = 0L), 4)
  expect_equal(findEndOfTag("{abc} rest", is_code = FALSE, start = 0L), 4)
  expect_equal(findEndOfTag("{a}{b} rest", is_code = FALSE, start = 0L), 5)
  expect_equal(findEndOfTag("{a{b}c} rest", is_code = FALSE, start = 0L), 6)
})

test_that("returns -1 for incomplete input", {
  expect_equal(findEndOfTag("{abc", is_code = FALSE, start = 0L), -1)
})

# Pass 1: within-class resolution ---------------------------------------------
# Resolves params from method @param -> class-level @param -> @field.
# See r6_resolve_params() for the core logic.

topic_add_r6_methods <- function(rd, block, env, base_path) {
  docs <- r6_class_from_block(block, env)
  block <- r6_fix_intro(block)

  # Add class-level tags (skip tags stamped for a specific method)
  for (tag in block$tags) {
    if (r6_tag_type(tag, block) == "class") {
      rd$add(roxy_tag_rd(tag, env = env, base_path = base_path))
    }
  }

  # Store unresolved R6 docs; pass 2 will inherit and format. The value is a
  # named list keyed by classname so that multiple R6 classes sharing a topic
  # (via @rdname) can coexist and be looked up individually.
  classname <- block$object$value$classname %||% ""
  rd$add(rd_section("r6_class", set_names(list(docs), classname)))
}

#' @export
format.rd_section_r6_class <- function(x, ...) NULL

#' @export
merge.rd_section_r6_class <- function(x, y, ...) {
  rd_section("r6_class", c(x$value, y$value))
}

# When an R6 class has inline @description tags for methods, parse_description()
# parser puts the class description into @details instead of @description.
# This function detects that case and promotes the class-level @details back
# to @description.
r6_fix_intro <- function(block) {
  types <- map_chr(block$tags, \(t) r6_tag_type(t, block))
  tags <- map_chr(block$tags, \(t) t$tag)

  has_class_desc <- any(tags == "description" & types == "class")
  has_class_details <- any(tags == "details" & types == "class")
  has_method_desc <- any(tags == "description" & types == "method")

  if (!has_class_desc && has_class_details && has_method_desc) {
    # Promote the first class-level @details to @description
    for (i in seq_along(block$tags)) {
      if (tags[[i]] == "details" && types[[i]] == "class") {
        block$tags[[i]]$tag <- "description"
        break
      }
    }
  }

  block
}

# Classify an R6 block tag:
# - "class": top-level Rd (e.g. @title, @description before class body)
# - "method": inline tag associated with a method
# - "other": @field/@param tags consumed by field/param extraction
r6_tag_type <- function(tag, block) {
  if (!is.null(tag$r6method)) {
    return("method")
  }

  inline <- !is.na(tag$line) && tag$line >= block$line
  method_tags <- c(
    "description",
    "details",
    "return",
    "returns",
    "examples",
    "noRd"
  )

  if (tag$tag %in% c("field", "name", "title")) {
    "other"
  } else if (tag$tag %in% method_tags && inline) {
    "method"
  } else if (tag$tag == "param") {
    # class-level @param are used as defaults for all methods
    if (inline) "method" else "other"
  } else {
    "class"
  }
}

tag_is <- function(tag, name) {
  tag$tag == name
}

tag_names <- function(tag) {
  trimws(strsplit(tag$val$name, ",")[[1]])
}

tag_has_name <- function(tag, names) {
  any(tag_names(tag) %in% names)
}

# Pass 2: R6 inheritance resolution -----------------------------------------

# Resolve R6 superclass inheritance after all blocks have been converted
# to topics. Mirrors topics_process_inherit() but for R6 field/param docs.
topics_process_r6_inherit <- function(topics) {
  r6_deps <- function(topic) {
    docs_list <- topic$get_value("r6_class")
    unlist(lapply(docs_list, \(d) d$superclasses$classname))
  }

  topics$topo_apply(r6_deps, r6_resolve_topic)
}

r6_resolve_topic <- function(topic, topics) {
  docs_list <- topic$get_value("r6_class")
  if (is.null(docs_list)) {
    return()
  }

  topic_name <- topic$get_name()

  resolved <- lapply(docs_list, function(docs) {
    r6_resolve_class(docs, topic_name, topics)
  })

  # Update stored docs (now resolved, so child classes can read them)
  topic$sections$r6_class <- rd_section("r6_class", resolved)

  # Format and inject into topic. Each class produces its own block of Rd.
  rd_lines <- unlist(lapply(resolved, format))
  topic$add(rd_section("rawRd", paste(rd_lines, collapse = "\n")))

  # Add combined examples across all methods of all classes in the topic
  all_methods <- list(
    self = unlist(lapply(resolved, \(d) d$methods$self), recursive = FALSE)
  )
  ex_lines <- r6_all_examples(all_methods)
  if (length(ex_lines) > 0) {
    ex_txt <- paste0(ex_lines, collapse = "\n")
    topic$add(rd_section("examples", ex_txt), overwrite = FALSE)
  }
}

r6_resolve_class <- function(docs, topic_name, topics) {
  # Collect resolved parent docs from already-processed topics
  parent_docs <- list()
  for (classname in docs$superclasses$classname) {
    parent_file <- topics$find_filename(classname)
    if (is.na(parent_file)) {
      next
    }
    parent_topic <- topics$get(parent_file)
    if (is.null(parent_topic)) {
      next
    }
    parent_r6 <- parent_topic$get_value("r6_class")
    # Parent topic may contain multiple R6 classes; pick the one by name
    parent_docs[[classname]] <- parent_r6[[classname]]
  }

  # Inherit fields and active bindings
  docs$fields <- r6_resolve_fields(docs$fields, parent_docs, topic_name)
  docs$active_bindings <- r6_resolve_fields(
    docs$active_bindings,
    parent_docs,
    topic_name
  )

  # Inherit method params
  docs$methods$self <- lapply(docs$methods$self, function(method) {
    r6_resolve_method_params(method, parent_docs, topic_name)
  })

  docs
}

# Field inheritance ----------------------------------------------------------

r6_resolve_fields <- function(fields_obj, parent_docs, topic_name) {
  label <- if (fields_obj$type == "field") "field" else "active binding"
  section <- if (fields_obj$type == "field") "fields" else "active_bindings"

  docd <- r6_field_names(fields_obj$fields)
  expected <- fields_obj$expected

  miss <- setdiff(expected, docd)
  inherited <- r6_find_super_fields(miss, parent_docs, section)
  fields_obj$fields <- c(fields_obj$fields, inherited)
  docd <- c(docd, r6_field_names(inherited))

  miss <- setdiff(expected, docd)
  if (length(miss) > 0) {
    warn_roxy_topic(topic_name, "Undocumented R6 {label}{?s}: {miss}")
  }

  fields_obj
}

r6_find_super_fields <- function(missing, parent_docs, section) {
  if (length(missing) == 0 || length(parent_docs) == 0) {
    return(list())
  }

  result <- list()
  for (super_doc in parent_docs) {
    if (is.null(super_doc)) {
      next
    }

    for (field in super_doc[[section]]$fields) {
      if (field$name %in% missing) {
        result <- c(result, list(field))
        missing <- setdiff(missing, field$name)
      }
    }
    if (length(missing) == 0) break
  }

  result
}

# Param inheritance ----------------------------------------------------------

# Inherit params from parent classes (child -> parent -> grandparent).
# Within-class param resolution (method -> class-level -> @field) is handled
# earlier by r6_resolve_params().
r6_resolve_method_params <- function(method, parent_docs, topic_name) {
  fnames <- names(method$formals)
  if (length(fnames) == 0) {
    return(method)
  }

  miss <- setdiff(fnames, r6_param_names(method$params))
  if (length(miss) > 0) {
    inherited <- r6_find_super_params(method$name, miss, parent_docs)
    method$params <- c(method$params, inherited)

    # Re-order according to formals
    firstnames <- map_chr(
      strsplit(map_chr(method$params, \(x) x$name), ","),
      \(x) trimws(x[[1]])
    )
    method$params <- method$params[order(match(firstnames, fnames))]
  }

  # Only now, after two rounds of resolution can we warn about missing params.
  miss <- setdiff(fnames, r6_param_names(method$params))
  for (m in miss) {
    warn_roxy_topic(
      topic_name,
      c(
        "Must use one @param for each argument",
        x = "{method$name}({m}) is not documented"
      )
    )
  }

  method
}

r6_find_super_params <- function(method_name, missing, parent_docs) {
  if (length(parent_docs) == 0) {
    return(list())
  }

  for (super_doc in parent_docs) {
    if (is.null(super_doc)) {
      next
    }

    super_method <- Find(
      \(m) m$name == method_name,
      super_doc$methods$self
    )
    if (is.null(super_method)) {
      next
    }

    result <- list()
    for (param in super_method$params) {
      param_names <- trimws(unlist(strsplit(param$name, ",")))
      if (any(param_names %in% missing)) {
        result <- c(result, list(param))
        missing <- setdiff(missing, param_names)
      }
    }

    if (length(result) > 0) return(result)
  }

  list()
}

expect_equivalent_rd <- function(out1, out2) {
  out1$sections$backref <- NULL
  out2$sections$backref <- NULL
  expect_equal(out1, out2)
}

expect_equal_strings <- function(s1, s2, ignore_ws = TRUE) {
  if (ignore_ws) {
    s1 <- gsub("\\s", "", s1, perl = TRUE)
    s2 <- gsub("\\s", "", s2, perl = TRUE)
  }
  expect_equal(s1, s2)
}

expect_parse_failure <- function(code) {
  (expect_condition(expect_null(code)))
}

r6_doc <- function(text, env = new.env(parent = globalenv()), n = NULL) {
  eval(parse(text = text, keep.source = TRUE), envir = env)
  blocks <- merge_external_r6methods(parse_text(text, env = env))

  # Pass 1: extract local docs (no inheritance)
  r6_blocks <- Filter(
    function(b) inherits(b, "roxy_block_r6class"),
    blocks
  )
  all_docs <- lapply(r6_blocks, r6_class_from_block, env = env)
  names(all_docs) <- map_chr(
    r6_blocks,
    \(b) b$object$value$classname %||% b$object$alias %||% ""
  )

  # Pass 2: resolve inheritance in dependency order
  target <- n %||% length(r6_blocks)
  parent_docs <- list()
  for (i in seq_along(all_docs)) {
    docs <- all_docs[[i]]
    topic_name <- names(all_docs)[[i]]
    docs$fields <- r6_resolve_fields(docs$fields, parent_docs, topic_name)
    docs$active_bindings <- r6_resolve_fields(
      docs$active_bindings,
      parent_docs,
      topic_name
    )
    docs$methods$self <- lapply(docs$methods$self, function(method) {
      r6_resolve_method_params(method, parent_docs, topic_name)
    })
    all_docs[[i]] <- docs
    parent_docs[[topic_name]] <- docs
  }

  all_docs[[target]]
}

local_package_copy <- function(path, env = caller_env(), set_version = TRUE) {
  temp_path <- withr::local_tempdir(.local_envir = env)

  file.copy(path, temp_path, recursive = TRUE)
  pkg_path <- dir(temp_path, full.names = TRUE)[[1]]

  if (set_version) {
    desc::desc_set(
      file = pkg_path,
      RoxygenNote = as.character(packageVersion("roxygen2"))
    )
  }

  normalizePath(pkg_path, winslash = "/")
}

#' @export
roxy_tag_rd.roxy_tag_.methods <- function(x, base_path, env) {
  desc <- lapply(x$val, function(x) docstring(x$value@.Data))
  usage <- map_chr(x$val, function(x) {
    function_usage(x$value@name, formals(x$value@.Data))
  })

  has_docs <- !map_lgl(desc, is.null)
  desc <- desc[has_docs]
  usage <- usage[has_docs]

  rd_section("rcmethods", setNames(desc, usage))
}
#' @export
format.rd_section_rcmethods <- function(x, ...) {
  rd_section_description("Methods", names(x$value), x$value)
}

# Extract all methods from an RC definition, returning a list of "objects".
rc_methods <- function(obj) {
  stopifnot(methods::is(obj, "refClassRepresentation"))

  parents <- obj@refSuperClasses
  parent_methods <- unlist(lapply(parents, function(x) {
    methods::getRefClass(x)$methods()
  }))
  method_names <- sort_c(setdiff(ls(envir = obj@refMethods), parent_methods))
  methods <- mget(method_names, envir = obj@refMethods)

  lapply(methods, object, alias = NULL, type = "rcmethod")
}

add_rc_metadata <- function(val, name, class) {
  class(val) <- c("rcmethod", "function")
  attr(val, "rcclass") <- class
  attr(val, "rcmethod") <- name

  val
}

get_method <- function(obj, method_name) {
  eval(call("$", quote(obj), as.name(method_name)))
}

# Modified from methods:::.refMethodDoc - a function has a doc string
# if it's a call to {, with more than 1 element, and the first element is
# a character vector.
docstring <- function(f) {
  stopifnot(is.function(f))
  if (is.primitive(f)) return(NULL)

  b <- body(f)
  if (length(b) <= 2 || !identical(b[[1]], quote(`{`))) return(NULL)

  first <- b[[2]]
  if (!is.character(first)) return(NULL)
  if (first == "") return(NULL)

  trim_docstring(first)
}

# Implementation converted from
# http://www.python.org/dev/peps/pep-0257/#handling-docstring-indentation
trim_docstring <- function(docstring) {
  if (docstring == "") return("")

  # Convert tabs to spaces (using four spaces for tabs)
  # and split into a vector of lines:
  lines <- strsplit(gsub("\t", "    ", docstring), "\n")[[1]]
  if (length(lines) == 1) return(strip(lines))

  # Determine minimum indentation (first line doesn't count):
  indent <- min(leadingSpaces(lines[-1]))

  # Remove indentation (first line is special):
  trimmed <- c(
    strip(lines[1]),
    substr(lines[-1], indent + 1, 1000L)
  )

  # Return a single string:
  string <- paste0(trimmed, collapse = "\n")

  # Strip off trailing and leading blank lines:
  gsub("^\n+|\n+$", "", string)
}

strip <- function(x) gsub("^ +| + $", "", x)

# Docstrings -------------------------------------------------------------------

test_that("base functions don't have docstrings", {
  expect_equal(docstring(`[`), NULL)
  expect_equal(docstring(mean), NULL)
})

test_that("function return string doesn't have docstring", {
  expect_equal(docstring(function() "a"), NULL)
  expect_equal(docstring(function() {"a"}), NULL)
})

test_that("first string in function is docstring", {
  expect_equal(docstring(function() {"a"; 1}), "a")
})

test_that("trim_docstring handles indentation correctly", {
  expect_equal(trim_docstring("a\n  b\n  c"), "a\nb\nc")
  expect_equal(trim_docstring("a\nb\nc"), "a\nb\nc")
  expect_equal(trim_docstring("a\n  b\n   c"), "a\nb\n c")
  expect_equal(trim_docstring("   a\n  b\n   c"), "a\nb\n c")
})

# Method documentation ---------------------------------------------------------

env <- pkg_env()

A1 <- setRefClass("A1", methods = list(
  f = function() {
    "This function has a docstring"
    1
  },
  g = function() {
    "This function doesn't"
  }
), where = env)

B1 <- setRefClass("B1", contains = "A1", methods = list(
  f1 = function() {
    "This function has a docstring"
    1
  },
  g1 = function() {
    "This function doesn't"
  }
), where = env)

classA <- getClass("A1", where = env)
classB <- getClass("B1", where = env)

test_that("rc_methods only lists methods belong to class (not parents)", {
  expect_equal(length(rc_methods(classA)), 2)
  expect_equal(length(rc_methods(classB)), 2)
})

test_that("RC methods included included in own section", {
  out <- roc_proc_text(rd_roclet(), "
    #' Class ABC
    setRefClass('ABC', methods = list(
      f = function() {
      'This function has a docstring'
      1
      }
    ))
  ")[[1]]

  methods <- out$get_value("rcmethods")
  expect_equal(names(methods), "f()")
  expect_match(methods[[1]], "This function has a docstring")
})

removeClass("B1", where = env)
removeClass("A1", where = env)

#' Roclet: make `NAMESPACE`
#'
#' @description
#' This [roclet] automates the production of a `NAMESPACE` file, which controls
#' the functions imported and exported by your package, as described in
#' [Writing R extensions](https://cran.r-project.org/doc/manuals/r-release/R-exts.html).
#' It is run by default by [roxygenize()].
#'
#' The `NAMESPACE` is generated in two passes: the first generates only
#' import directives (because this can be computed without evaluating package
#' code), and the second generates everything (after the package has been
#' loaded).
#'
#' See `vignette("namespace")` for details.
#'
#' @export
#' @seealso [tags-namespace] for tags that generate `NAMESPACE` directives.
#' @examples
#' # The most common namespace tag is @@export, which declares that a function
#' # is part of the external interface of your package
#' #' @export
#' foofy <- function(x, y, z) {
#' }
#' # This results in the following line in `NAMESPACE`:
#' # export(foofy)
#'
#' # You'll also often find global imports living in a file called
#' # R/{package}-package.R.
#' #' @importFrom magrittr %>%
#' #' @import rlang
#' NULL
#'
#' # This results in the following lines in `NAMESPACE`:
#' # importFrom(magrittr,"%>%")
#' # import(rlang)
namespace_roclet <- function() {
  roclet("namespace")
}

#' @export
roclet_process.roclet_namespace <- function(x, blocks, env, base_path) {
  warn_missing_s3_exports(blocks, env)

  blocks_to_ns(blocks, env)
}

#' @export
roclet_output.roclet_namespace <- function(x, results, base_path, ...) {
  NAMESPACE <- file.path(base_path, "NAMESPACE")
  results <- c(made_by("#"), results)

  # Always check for roxygen2 header before overwriting NAMESPACE (#436),
  # even when running for the first time
  write_if_different(NAMESPACE, results, check = TRUE)

  NAMESPACE
}

#' @export
roclet_clean.roclet_namespace <- function(x, base_path) {
  NAMESPACE <- file.path(base_path, "NAMESPACE")
  if (made_by_roxygen(NAMESPACE)) {
    unlink(NAMESPACE)
  }
}

# NAMESPACE updates -------------------------------------------------------

import_directives <- c(
  "import",
  "importFrom",
  "importClassesFrom",
  "importMethodsFrom",
  "useDynLib"
)

update_namespace_imports <- function(base_path) {
  NAMESPACE <- file.path(base_path, "NAMESPACE")
  if (!made_by_roxygen(NAMESPACE) || !file.exists(NAMESPACE)) {
    return(invisible())
  }

  directives <- c(
    namespace_imports(base_path),
    as.list(namespace_exports(NAMESPACE))
  )
  results <- c(made_by("#"), ns_format(directives))
  write_if_different(NAMESPACE, results, check = TRUE)

  invisible()
}

# Here we hand roll parsing and tokenisation from roxygen2 primitives so
# we can filter tags that we know don't require package code. Returns the
# unformatted directives (see `block_directives()`) so the caller can merge
# them with the existing exports in `ns_format()`.
namespace_imports <- function(base_path = ".") {
  paths <- package_files(base_path)
  parsed <- lapply(paths, parse, keep.source = TRUE)
  srcrefs <- lapply(parsed, utils::getSrcref)
  # Ensure parse warnings only fire once during the main parse_package() pass
  suppressMessages(
    blocks <- unlist(
      lapply(srcrefs, namespace_imports_blocks),
      recursive = FALSE
    )
  )

  block_directives(blocks, emptyenv())
}

namespace_imports_blocks <- function(srcref) {
  comment_refs <- comments(srcref)
  tokens <- lapply(comment_refs, tokenise_ref)

  import_tags <- c(import_directives, "rawNamespace")
  tokens_filtered <- lapply(tokens, function(tokens) {
    tokens[map_lgl(tokens, \(x) x$tag %in% import_tags)]
  })

  compact(lapply(tokens_filtered, function(tokens) {
    block_create(
      call = NULL,
      srcref = srcref(srcfile("NAMESPACE"), rep(1, 4)),
      tokens = tokens
    )
  }))
}

# NB: this is designed as the conjugate of namespace_imports(), so also
#   includes @rawNamespace entries which may/may not also include import directives.
namespace_exports <- function(path) {
  parsed <- as.list(parse(path, keep.source = TRUE))

  is_import_directive <- function(x) is_call(x, import_directives)
  export_lines <- attr(parsed, "srcref")[!map_lgl(parsed, is_import_directive)]
  # Each multiline directives are a single element so they're sorted correctly
  unlist(lapply(export_lines, \(x) paste(as.character(x), collapse = "\n")))
}

# NAMESPACE generation ----------------------------------------------------

blocks_to_ns <- function(blocks, env) {
  ns_format(block_directives(blocks, env))
}

block_directives <- function(blocks, env) {
  directives <- map(blocks, function(block) {
    map(block$tags, roxy_tag_ns, block = block, env = env)
  })
  compact(list_c(directives))
}

# `roxy_tag_ns()` returns either a rendered directive (a character vector) or,
# for `@importFrom`, a structured `import_from()` so that all the imports from a
# package can be merged into a single directive here. This dodges a
# `loadNamespace()` performance issue that scales with the number of
# `importFrom()` directives.
ns_format <- function(directives) {
  is_import <- map_lgl(directives, \(x) inherits(x, "import_from"))

  text <- unique(as.character(unlist(
    directives[!is_import],
    use.names = FALSE
  )))
  import_from <- merge_import_from(directives[is_import])

  lines <- c(text, import_from)
  lines[order_c(lines)]
}

import_from <- function(package, funs) {
  structure(list(package = package, funs = funs), class = "import_from")
}

# Merge the `import_from()` directives by package into one `importFrom()` each.
merge_import_from <- function(imports) {
  packages <- map_chr(imports, \(x) x$package)
  funs <- split(
    lapply(imports, \(x) x$funs),
    factor(packages, levels = sort_c(unique(packages)))
  )
  map_chr(names(funs), function(package) {
    funs <- unlist(funs[[package]], use.names = FALSE)
    funs <- sort_c(unique(auto_quote(funs)))
    format_import_from(package, funs)
  })
}

format_import_from <- function(package, funs) {
  package <- auto_quote(package)
  if (length(funs) == 1) {
    sprintf("importFrom(%s,%s)", package, funs)
  } else {
    fun_lines <- paste0("  ", funs, collapse = ",\n")
    paste0("importFrom(", package, ",\n", fun_lines, "\n)")
  }
}

# Namespace tag methods ---------------------------------------------------

roxy_tag_ns <- function(x, block, env) {
  UseMethod("roxy_tag_ns")
}

#' @export
roxy_tag_ns.default <- function(x, block, env) {}

#' @export
roxy_tag_parse.roxy_tag_evalNamespace <- function(x) {
  tag_code(x)
}
#' @export
roxy_tag_ns.roxy_tag_evalNamespace <- function(x, block, env) {
  roxy_tag_eval(x, env)
}

#' @export
roxy_tag_parse.roxy_tag_export <- function(x) {
  tag_words(x)
}
#' @export
roxy_tag_ns.roxy_tag_export <- function(x, block, env) {
  if (identical(x$val, "")) {
    # FIXME: check for empty exports (i.e. no name)
    default_export(block$object, block)
  } else {
    export(x$val)
  }
}

#' @export
roxy_tag_parse.roxy_tag_exportClass <- function(x) {
  tag_words(x, 1)
}
#' @export
roxy_tag_ns.roxy_tag_exportClass <- function(x, block, env) {
  export_class(x$val)
}

#' @export
roxy_tag_parse.roxy_tag_exportMethod <- function(x) {
  tag_words(x, min = 1)
}
#' @export
roxy_tag_ns.roxy_tag_exportMethod <- function(x, block, env) {
  export_s4_method(x$val)
}

#' @export
roxy_tag_parse.roxy_tag_exportPattern <- function(x) {
  tag_words(x, min = 1)
}
#' @export
roxy_tag_ns.roxy_tag_exportPattern <- function(x, block, env) {
  one_per_line("exportPattern", x$val)
}

#' @export
roxy_tag_parse.roxy_tag_exportS3Method <- function(x) {
  tag_words(x, min = 0, max = 2)
}
#' @export
roxy_tag_ns.roxy_tag_exportS3Method <- function(x, block, env) {
  obj <- block$object

  if (identical(x$val, "NULL")) {
    return()
  }

  if (identical(x$val, "")) {
    if (!inherits(obj, "s3method")) {
      warn_roxy_tag(x, "must be used with a known S3 method")
      return()
    }

    method <- attr(obj$value, "s3method")
  } else if (length(x$val) == 1) {
    if (!inherits(obj, "function") && !inherits(obj, "s3method")) {
      warn_roxy_tag(x, "must be used with a function")
      return()
    }

    if (!grepl("::", x$val, fixed = TRUE)) {
      warn_roxy_tag(x, "must have form package::generic")
      return()
    }

    generic <- re_split_half(x$val, "::")
    generic_re <- paste0("^", generic[[2]], "\\.")

    if (!grepl(generic_re, obj$alias)) {
      warn_roxy_tag(
        x,
        "generic ({.str {generic[[2]]}}) doesn't match function ({.str {obj$alias}})",
      )
      return()
    }

    class <- sub(generic_re, "", obj$alias)
    method <- c(x$val, class)
  } else {
    method <- x$val
  }

  export_s3_method(method)
}

#' @export
roxy_tag_parse.roxy_tag_import <- function(x) {
  tag_words(x, min = 1)
}
#' @export
roxy_tag_ns.roxy_tag_import <- function(x, block, env) {
  ns_verbatim("import", x$val) %||%
    one_per_line_ignore_current("import", x$val)
}

#' @export
roxy_tag_parse.roxy_tag_importClassesFrom <- function(x) {
  tag_words(x, min = 2, multiline = "indent")
}
#' @export
roxy_tag_ns.roxy_tag_importClassesFrom <- function(x, block, env) {
  repeat_first_ignore_current("importClassesFrom", x$val)
}

#' @export
roxy_tag_parse.roxy_tag_importFrom <- function(x) {
  tag_words(x, min = 2, multiline = "indent")
}
#' @export
roxy_tag_ns.roxy_tag_importFrom <- function(x, block, env) {
  pkg <- x$val[1L]
  if (identical(pkg, peek_roxygen_pkg())) {
    return(NULL)
  }

  funs <- x$val[-1L]
  if (requireNamespace(pkg, quietly = TRUE)) {
    # be sure to match '%>%', `%>%`, "%>%" all to %>% given by getNamespaceExports, #1570
    unknown_idx <- !strip_quotes(funs) %in% getNamespaceExports(pkg)
    if (any(unknown_idx)) {
      warn_roxy_tag(
        x,
        "Excluding unknown {cli::qty(sum(unknown_idx))} export{?s} from {.package {pkg}}: {.code {funs[unknown_idx]}}"
      )
      funs <- funs[!unknown_idx]
    }
    if (length(funs) == 0) {
      return(NULL)
    }
  }

  import_from(pkg, funs)
}

#' @export
roxy_tag_parse.roxy_tag_importMethodsFrom <- function(x) {
  tag_words(x, min = 2, multiline = "indent")
}
#' @export
roxy_tag_ns.roxy_tag_importMethodsFrom <- function(x, block, env) {
  repeat_first_ignore_current("importMethodsFrom", x$val)
}

#' @export
roxy_tag_parse.roxy_tag_rawNamespace <- function(x) {
  tag_code(x)
}
#' @export
roxy_tag_ns.roxy_tag_rawNamespace <- function(x, block, env) {
  x$raw
}

#' @export
roxy_tag_parse.roxy_tag_useDynLib <- function(x) {
  tag_words(x, min = 1)
}
#' @export
roxy_tag_ns.roxy_tag_useDynLib <- function(x, block, env) {
  if (length(x$val) == 1) {
    return(paste0("useDynLib(", auto_quote(x$val), ")"))
  }

  ns_verbatim("useDynLib", x$val) %||%
    repeat_first("useDynLib", x$val)
}

# Default export methods --------------------------------------------------

default_export <- function(x, block) {
  UseMethod("default_export")
}
#' @export
default_export.s4class <- function(x, block) {
  c(
    if (!is.null(block$object$alias)) export(block$object$alias),
    export_class(x$value@className)
  )
}
#' @export
default_export.s4generic <- function(x, block) export(x$value@generic)
#' @export
default_export.s4method <- function(x, block) export_s4_method(x$value@generic)
#' @export
default_export.s3method <- function(x, block) {
  export_s3_method(auto_quote(attr(x$value, "s3method")))
}
#' @export
default_export.rcclass <- function(x, block) export_class(x$value@className)
#' @export
default_export.default <- function(x, block) export(x$alias)
#' @export
default_export.NULL <- function(x, block) {
  export(block_get_tag_value(block, "name"))
}

# Helpers -----------------------------------------------------------------

export <- function(x) one_per_line("export", x)
export_class <- function(x) one_per_line("exportClasses", x)
export_s4_method <- function(x) one_per_line("exportMethods", x)
export_s3_method <- function(x) {
  args <- paste0(x, collapse = ",")
  paste0("S3method(", args, ")")
}

# Escape hatch: if a directive contains a comma, insert it verbatim rather than
# quoting each word.
ns_verbatim <- function(name, x) {
  if (!any(grepl(",", x))) {
    return(NULL)
  }
  paste0(name, "(", paste0(x, collapse = " "), ")")
}

one_per_line <- function(name, x) {
  if (length(x)) {
    paste0(name, "(", auto_quote(x), ")")
  } else {
    NULL
  }
}
repeat_first <- function(name, x) {
  paste0(name, "(", auto_quote(x[1]), ",", auto_quote(x[-1]), ")")
}

one_per_line_ignore_current <- function(name, x) {
  current <- peek_roxygen_pkg()

  # Ignore any occurrence of `current` inside `x`
  if (is_string(current)) {
    x <- x[x != current]
  }

  one_per_line(name, x)
}
repeat_first_ignore_current <- function(name, x) {
  current <- peek_roxygen_pkg()

  # Ignore the whole command if "first" is `current`
  if (is_string(current) && length(x) && x[[1]] == current) {
    NULL
  } else {
    repeat_first(name, x)
  }
}

# missing s3 exports ------------------------------------------------------

warn_missing_s3_exports <- function(blocks, env) {
  objs <- as.list(env)
  funs <- Filter(is.function, objs)
  methods <- funs[map_lgl(names(funs), is_s3_method, env = env)]

  s3blocks <- blocks[map_lgl(
    blocks,
    block_has_tags,
    c("export", "exportS3Method")
  )]
  s3objects <- map(s3blocks, \(block) block$object$value)
  s3functions <- Filter(is.function, s3objects)

  undocumented <- methods[!methods %in% s3functions]
  srcrefs <- map(undocumented, attr, "srcref")

  map2(undocumented, names(undocumented), function(fun, name) {
    warn_roxy_function(
      fun,
      "S3 method {.arg {name}} needs @export or @exportS3Method tag"
    )
  })
}

test_that("end-to-end NAMESPACE generation works", {
  path <- local_package_copy(test_path("testNamespace"))

  suppressMessages(roxygenise(path))
  withr::defer(pkgload::unload("testNamespace"))

  ns <- read_lines(file.path(path, "NAMESPACE"))
  expect_equal(ns, c(
    "# Generated by roxygen2: do not edit by hand",
    "",
    "export(f)",
    "export(g)"
  ))
})

# @export -----------------------------------------------------------------

test_that("export quote object name appropriate", {
  out <- roc_proc_text(namespace_roclet(), "#' @export\na <- function(){}")
  expect_equal(out, 'export(a)')

  out <- roc_proc_text(namespace_roclet(), "#' @export\n`+` <- function(){}")
  expect_equal(out, 'export("+")')

  out <- roc_proc_text(namespace_roclet(), "#' @export\n`\\`` <- function(){}")
  expect_equal(out, 'export("`")')
})

test_that("export parameter overrides default", {
  out <- roc_proc_text(namespace_roclet(), "#' @export b\na <- function(){}")
  expect_equal(out, 'export(b)')
})

test_that("multiple export parameters generate multiple exports", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @export a b
    a <- function(){}")
  expect_equal(out, c('export(a)', 'export(b)'))
})

test_that("export trimmed before line test", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @export
    #'
    a <- function(){}")
  expect_equal(out, 'export(a)')
})


test_that("export detects S4 class", {
  out <- roc_proc_text(namespace_roclet(), "#' @export\nsetClass('a')")
  expect_equal(out, 'exportClasses(a)')
})

test_that("exports constructor function if created", {
  out <- roc_proc_text(namespace_roclet(), "#' @export\na <- setClass('a')")
  expect_equal(out, c('export(a)', 'exportClasses(a)'))
})

test_that("export detects S4 generic", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @export
    setGeneric('foo', function(x) standardGeneric('foo'))
  ")
  expect_equal(out, 'export(foo)')
})


test_that("export detects S3 method", {
  out <- roc_proc_text(namespace_roclet(), "#' @export\nmean.foo <- function(x) 'foo'")
  expect_equal(out, 'S3method(mean,foo)')
})

test_that("export handles non-syntactic names", {
  out <- roc_proc_text(namespace_roclet(),  "
    #' @export
    `mean.foo-bar` <- function(x) 'foo'
  ")
  expect_equal(out, "S3method(mean,\"foo-bar\")")

  out <- roc_proc_text(namespace_roclet(),  "
    `foo-bar` <- function(x) UseMethod('foo-bar')
    #' @export
    `foo-bar.integer` <- function(x) 'foo'
  ")
  expect_equal(out, "S3method(\"foo-bar\",integer)")
})

test_that("@exportS3method generates fully automatically", {
  out <- roc_proc_text(namespace_roclet(),"
    #' @exportS3Method
    mean.foo <- function(x) 'foo'
  ")
  expect_equal(out, "S3method(mean,foo)")

  block <- "
    #' @exportS3Method
    f <- function(x) 'foo'
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))
})

test_that("@exportS3methd can create literal directive", {
  out <- roc_proc_text(namespace_roclet(),
    "#' @exportS3Method base::mean foo
    NULL
  ")
  expect_equal(out, "S3method(base::mean,foo)")
})

test_that("@exportS3method can extract class from generic", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @exportS3Method pkg::foo
    foo.bar <- function(x) 'foo'
  ")
  expect_equal(out, "S3method(pkg::foo,bar)")

  block <- "
    #' @exportS3Method pkg_foo
    foo.bar <- function(x) 'foo'
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))

  block <- "
    #' @exportS3Method pkg::foo
    foo1.bar <- 10
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))

  block <- "
    #' @exportS3Method pkg::foo
    foo1.bar <- function(x) 'foo'
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))
})

test_that("exportClass overrides default class name", {
  out <- roc_proc_text(namespace_roclet(), "#' @exportClass b\nsetClass('a')")
  expect_equal(out, 'exportClasses(b)')
})

test_that("export detects method name", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @export\n
    setMethod('max', 'a', function(x, ...) x[1])")
  expect_equal(out, 'exportMethods(max)')
})

test_that("export method escapes if needed", {
  out <- roc_proc_text(namespace_roclet(), "
    setGeneric('x<-', function(x, value) standardGeneric('x<-'))
    #' @export\n
    setMethod('x<-', 'a', function(x, value) value)")
  expect_equal(out, 'exportMethods("x<-")')
})

test_that("export uses name if no object present", {
  out <- roc_proc_text(namespace_roclet(), "
    #' Title
    #'
    #' @export
    #' @name x
    NULL
  ")
  expect_equal(out, 'export(x)')
})

test_that("default export uses exportClass for RC objects", {
  out <- roc_proc_text(namespace_roclet(), "
    #' Title
    #'
    #' @export
    x <- setRefClass('X')
  ")
  expect_equal(out, 'exportClasses(X)')
})

test_that("exportMethod overrides default method name", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @exportMethod c
    setMethod('max', 'a', function(x, ...) x[1])")
  expect_equal(out, 'exportMethods(c)')
})

test_that("other namespace tags produce correct output", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @exportPattern test
    #' @import test
    #' @importFrom test test1 test2
    #' @importClassesFrom test test1 test2
    #' @importMethodsFrom test test1 test2
    NULL")

  expect_equal(sort(out), sort(c(
    "exportPattern(test)",
    "import(test)",
    "importFrom(test,test1)",
    "importFrom(test,test2)",
    "importClassesFrom(test,test1)",
    "importClassesFrom(test,test2)",
    "importMethodsFrom(test,test1)",
    "importMethodsFrom(test,test2)"
  )))
})

test_that("import directives for current package are ignored", {
  withr::local_envvar(c("ROXYGEN_PKG" = "ignored"))

  out <- roc_proc_text(namespace_roclet(), "
    #' @import ignored
    #' @import test ignored test2
    #' @importFrom ignored test1 test2
    #' @importClassesFrom ignored test1 test2
    #' @importMethodsFrom ignored test1 test2
    NULL")

  expect_equal(sort(out), sort(c(
    "import(test)",
    "import(test2)"
  )))
})

test_that("poorly formed importFrom throws error", {
  block <- "
    #' @importFrom test
    NULL
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))
})

test_that("multiline importFrom parsed correctly", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @importFrom test test1
    #'   test2
    NULL
  ")
  expect_equal(sort(out), sort(c(
    "importFrom(test,test1)",
    "importFrom(test,test2)"
  )))
})

test_that("useDynLib imports only selected functions", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @useDynLib test
    #' @useDynLib test a
    #' @useDynLib test a b
    NULL")

    expect_equal(sort(out), sort(
      c("useDynLib(test)", "useDynLib(test,a)", "useDynLib(test,b)")))
})

test_that("useDynLib doesn't quote if comma present", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @useDynLib test, .registration = TRUE
    NULL")

  expect_equal(sort(out), "useDynLib(test, .registration = TRUE)")
})

test_that("empty NAMESPACE generates zero-length vector", {
  base_path <- test_path("empty")

  env <- pkgload::load_all(base_path, quiet = TRUE)$env
  withr::defer(pkgload::unload("empty"))

  blocks <- parse_package(base_path, env = env)

  results <- roclet_process(namespace_roclet(), blocks, env = env, base_path)
  expect_equal(results, character())
})

test_that("can regenerate NAMESPACE even if its broken", {
  path <- local_package_copy(test_path("broken-namespace"))
  expect_snapshot(update_namespace_imports(path))

  expect_equal(
    read_lines(file.path(path, "NAMESPACE")),
    c(
      "# Generated by roxygen2: do not edit by hand",
      "",
      "importFrom(stats,median)"
    )
  )
})

# Raw ---------------------------------------------------------------------

test_that("rawNamespace must be valid code", {
  block <- "
    #' @rawNamespace a +
    NULL
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))
})

test_that("rawNamespace inserted unchanged", {
  out <- roc_proc_text(namespace_roclet(), "
    #' @name a
    #' @rawNamespace xyz
    #'   abc
    NULL")

  expect_equal(out, "xyz\n  abc")
})

test_that("rawNamespace does not break idempotency", {
  test_pkg <- test_path("testRawNamespace")
  NAMESPACE <- file.path(test_pkg, "NAMESPACE")

  lines_orig <- read_lines(NAMESPACE)

  expect_no_error(roxygenize(test_pkg, namespace_roclet()))

  # contents unchanged
  expect_equal(read_lines(NAMESPACE), lines_orig)
})

# @evalNamespace ----------------------------------------------------------

test_that("evalNamespace warns for bad code", {
  block <- "
    #' @evalNamespace a +
    #' @name a
    #' @title a
    NULL
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))

  block <- "
    #' @evalNamespace stop('Uhoh')
    #' @name a
    #' @title a
    NULL
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))

  block <- "
    #' @evalNamespace 1
    #' @name a
    #' @title a
    NULL
  "
  expect_snapshot(. <- roc_proc_text(namespace_roclet(), block))
})

test_that("evalNamespace code is inserted when its value is a string", {
  out1 <- roc_proc_text(namespace_roclet(), "
    nms <- paste(letters[1:3], collapse = ',')
    #' @evalNamespace sprintf('export(%s)', nms)
    #' @name a
    #' @title a
    NULL")
  out2 <- roc_proc_text(namespace_roclet(), "
    nms <- paste(letters[1:3], collapse = ',')
    #' @evalNamespace sprintf('export(%s)',
    #'                        nms)
    #' @name a
    #' @title a
    NULL")

  expect_equal(out1, "export(a,b,c)")
  expect_equal(out2, "export(a,b,c)")
})

test_that("evalNamspace can yield a vector", {
  out <- roc_proc_text(namespace_roclet(), "
    nms <- letters[1:2]
    #' @evalNamespace paste0('export(', nms, ')')
    #' @name a
    #' @title a
    NULL")

  expect_equal(out, c("export(a)", "export(b)"))
})


# helpers -----------------------------------------------------------------

test_that("auto_quote behaves as needed", {
  expect_equal(auto_quote("x"), "x")
  expect_equal(auto_quote("if"), '"if"') # quotes non-syntactic
  expect_equal(auto_quote("'if'"), "'if'") # unless already quoted
})

test_that("can extract non-imports from namespace preserving source", {
  path <- withr::local_tempfile(lines = c(
    "export(x)",
    "import(y)"
  ))
  expect_equal(namespace_exports(path), "export(x)")

  path <- withr::local_tempfile(lines = "export(x, y, z)")
  expect_equal(namespace_exports(path), "export(x, y, z)")

  lines <- c(
    "if (TRUE) {",
    "  import(x, y, z)",
    "}",
    "import(a)",
    "export(b)"
  )
  path <- withr::local_tempfile(lines = lines)
  expect_equal(
    namespace_exports(path),
    c(
      paste(lines[1:3], collapse = "\n"),
      lines[5L]
    )
  )
})

test_that("invalid imports generate correct declarations", {
  # No matched functions --> no output
  block <- "
    #' @importFrom utils InvalidUtilsFunction
    NULL
  "
  expect_message(out <- roc_proc_text(namespace_roclet(), block))
  expect_equal(out, character())

  # Matched functions --> only drop unmatched functions
  block <- "
    #' @importFrom utils head InvalidUtilsFunction
    NULL
  "
  expect_message(out <- roc_proc_text(namespace_roclet(), block))
  expect_equal(out, "importFrom(utils,head)")
})

test_that("invalid imports generate helpful message", {
  block <- "
    #' @importFrom utils head InvalidUtilsFunction1
    NULL
  "
  expect_snapshot(out <- roc_proc_text(namespace_roclet(), block))

  block <- "
    #' @importFrom utils head InvalidUtilsFunction1 InvalidUtilsFunction2
    NULL
  "
  expect_snapshot(out <- roc_proc_text(namespace_roclet(), block))
})

test_that("nothing we can do if package isn't installed", {
  block <- "
    #' @importFrom AnUnknownUnavailablePackage Unchecked
    NULL
  "
  expect_no_message(out <- roc_proc_text(namespace_roclet(), block))
  expect_equal(out, "importFrom(AnUnknownUnavailablePackage,Unchecked)")
})

test_that("non-syntactic imports can use multiple quoting forms", {
  lines <- c(
    "#' @importFrom stringr %>%",
    "#' @importFrom stringr `%>%`",
    "#' @importFrom stringr '%>%'",
    "#' @importFrom stringr \"%>%\"",
    "NULL"
  )

  import <- expect_no_warning(roc_proc_text(namespace_roclet(), lines))
  expect_equal(import, c(
    "importFrom(stringr,\"%>%\")",
    "importFrom(stringr,'%>%')",
    "importFrom(stringr,`%>%`)"
  ))
})

# warn_missing_s3_exports -------------------------------------------------

test_that("warns if S3 method not documented", {
  # Need to manually transform since the srcref is coming from the function;
  # roc_proc_text() uses fake srcrefs for the blocks themselves
  fix_srcref <- function(x) gsub("file[a-z0-9]+", "<text>", x)

  block <- "
      foo <- function(x) UseMethod('foo')
      foo.numeric <- function(x) 1

      mean.myclass <- function(x) 2
    "
  expect_snapshot(
    . <- roc_proc_text(namespace_roclet(), block),
    transform = fix_srcref
  )

  # Works even if method contains {
  block <- "
    foo <- function(x) UseMethod('foo')
    `foo.{` <- function(x) 1
  "
  expect_snapshot(
    . <- roc_proc_text(namespace_roclet(), block),
    transform = fix_srcref
  )
})

test_that("can suppress the warning", {
  block <- "
    #' @exportS3Method NULL
    mean.myclass <- function(x) 1
  "
  expect_silent(out <- roc_proc_text(namespace_roclet(), block))
  expect_equal(out, character())
})


test_that("doesn't warn for potential false postives", {
  roc <- namespace_roclet()
  expect_no_warning({
    roc_proc_text(roc, "foo.numeric <- function(x) 1")
    roc_proc_text(roc, "is.numeric <- function(x) 1")
    roc_proc_text(roc, "as.numeric <- function(x) 1")
  })
})

# object ------------------------------------------------------------------

test_that("has thoughtful print method", {
  text <-
    "#' This is a title
    #'
    #' @param x,y A number
    #' @export
    f <- function(x, y) x + y
  "
  block <- parse_text(text)[[1]]
  expect_snapshot_output(block)
})

# description block -------------------------------------------------------

test_that("title and description taken from first line if only one", {
  out <- roc_proc_text(rd_roclet(), "
    #' title
    #' @name a
    NULL")[[1]]
  expect_equal(out$get_value("description"), "title")
  expect_equal(out$get_value("title"), "title")
})

test_that("description taken from multiple titles if merged", {
  out <- roc_proc_text(rd_roclet(), "
    #' T1
    #' @name a
    NULL

    #' T2
    #' @name a
    NULL
    ")[[1]]
  expect_equal(out$get_value("title"), c("T1", "T2"))
  expect_equal(out$get_value("description"), c("T1", "T2"))
})

test_that("title, description and details extracted correctly", {
  out <- roc_proc_text(rd_roclet(), "
    #' title
    #'
    #' description
    #'
    #' details
    #' @name a
    NULL")[[1]]
  expect_equal(out$get_value("description"), "description")
  expect_equal(out$get_value("details"), "details")
})

test_that("title taken from first paragraph", {
  out <- roc_proc_text(rd_roclet(), "
    #' Description with sentence.
    #'
    #' That continueth.
    #' @name a
    NULL")[[1]]
  expect_equal(out$get_value("title"), "Description with sentence.")
  expect_equal(out$get_value("description"), "That continueth.")
})

test_that("@title overrides default title", {
  out <- roc_proc_text(rd_roclet(), "
    #' Would be title
    #' @title Overridden title
    #' @name a
    NULL")[[1]]
  expect_equal(out$get_value("title"), "Overridden title")
  expect_equal(out$get_value("description"), "Would be title")
})

test_that("docs parsed correctly if no blank text", {
  out <- roc_proc_text(rd_roclet(), "
    #' @title My title
    #' @description My description
    #' @param x value
    a <- function(x) {}")[[1]]

  expect_equal(out$get_value("title"), "My title")
  expect_equal(out$get_value("description"), "My description")
})

test_that("question mark ends sentence", {
  out <- roc_proc_text(rd_roclet(), "
    #' Is a number odd?
    is.odd <- function(a) {}")[[1]]
  expect_equal(out$get_value("title"), "Is a number odd?")

})

test_that("no ending punctuation does not produce ellipsis", {
  out <- roc_proc_text(rd_roclet(), "
    #' Whether a number is odd
    is.odd <- function(a) {}")[[1]]
  expect_equal(out$get_value("title"), "Whether a number is odd")
})

test_that("details are merged if needed", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' Description
    #'
    #' Details1
    #'
    #' Details2
    #'
    #' @details Details3
    #'
    #' Details4
    foo <- function(x) {}")[[1]]

  expect_equal(
    out$get_value("details"),
    "Details1\n\nDetails2\n\nDetails3\n\nDetails4"
  )
})

test_that("whitespace is not detected as details", {
  expect_silent(
    out <- roc_proc_text(
      rd_roclet(), "
        #' Title
        #'
        #'
        #' Description
        #'
        #'
        #'
        foo <- function(x) {}"
    )[[1]]
  )

  expect_null(out$get_value("details"))
})


test_that("@description and @details are merged", {
  out <- roc_proc_text(rd_roclet(), "
    #' Foo
    #'
    #' This.
    #'
    #' OBTW.
    foo <- function(x = '%') x

    #' @rdname foo
    #' @description And that.
    #' @details ORLY?
    bar <- function(y = '%') y
  ")[[1]]

  expect_equal(out$get_value("description"), c("This.", "And that."))
  expect_equal(out$get_value("details"), c("OBTW.", "ORLY?"))
})

test_that("empty description block is silently removed", {
  expect_warning(
    roc_proc_text(rd_roclet(), "
      #'
      #'
      f <- function() {}
      "
    ),
    NA
  )
})

test_that("description block preserves whitespace", {
  out <- parse_text("
    #' Title
    #'
    #' Line 1
    #'   Line 2
    #'
    #' Line 1
    #'   Line 2
    f <- function() {}
    "
  )[[1]]

  expect_equal(block_get_tag_value(out, "description"), "Line 1\n  Line 2")
  expect_equal(block_get_tag_value(out, "details"), "Line 1\n  Line 2")
})


test_that("line numbers offset correctly", {
  out <- parse_text(
    "#' Title
    #'
    #' Line 3
    #' Line 4
    #' Line 5
    #'
    #' Line 7
    #' Line 8
    f <- function() {}
    "
  )[[1]]

  expect_equal(out$tags[[1]]$line, 1)
  expect_equal(out$tags[[2]]$line, 3)
  expect_equal(out$tags[[3]]$line, 7)
})

test_that("even with explicit title/description", {
  out <- parse_text(
    "#' Line 1
    #' Line 2
    #' Line 3
    #'
    #' Line 5
    #' Line 6
    #' @title This is a title
    f <- function() {}
    "
  )[[1]]

  expect_equal(out$tags[[1]]$line, 1)
  expect_equal(out$tags[[2]]$line, 5)
  expect_equal(out$tags[[3]]$line, 7)
})

# evaluate ----------------------------------------------------------------

test_that("evaluation occurs during parsing", {
  out <- roc_proc_text(rd_roclet(), "
    foo <- function() c('@name a', '@title a')
    #' @eval foo()
    NULL")[[1]]

  expect_equal(out$get_value("title"), "a")
  expect_equal(out$filename, "a.Rd")
})

test_that("errors are propagated", {
  expect_snapshot_warning(
    roc_proc_text(rd_roclet(), "
      foo <- function() stop('Uhoh')
      #' Title
      #' @name foo
      #' @eval foo()
      NULL"
    )
  )
})

test_that("must return non-NA string", {
  expect_snapshot_warning(
    roc_proc_text(rd_roclet(), "
      foo <- function() NA
      #' @eval foo()
      NULL"
    )
  )

  expect_snapshot_warning(
    roc_proc_text(rd_roclet(), "
      foo <- function() NA_character_
      #' @eval foo()
      NULL"
    )
  )
})

test_that("also works with namespace roclet", {
  out <- roc_proc_text(namespace_roclet(), "
    foo <- function() '@export a'
    #' @eval foo()
    #' @name a
    #' @title a
    NULL")

  expect_equal(out, "export(a)")
})


# other -------------------------------------------------------------------


test_that("warns about duplicate tags", {
  expect_snapshot_warning({
    roc_proc_text(rd_roclet(), "
      #' Foo
      #' @rdname foo
      #' @rdname bar
      foo <- function() {}
    ")
  })
})

test_that("@docType data automatically adds sensible defaults", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title.
    #'
    #' @docType data
    a <- data.frame(a = 1:10)
  ")[[1]]

  expect_equal(out$get_value("usage"), rd("a"))
  expect_equal(out$get_value("keyword"), "datasets")
  expect_false(is.null(out$get_value("format")))
})

test_that("@docType data automatically added to data objects", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title.
    a <- data.frame(a = 1:10)
  ")[[1]]

  expect_equal(out$get_value("docType"), "data")
})

test_that("@docType data automatically added to data objects created elsewhere", {
  out <- roc_proc_text(rd_roclet(), "
    a <- data.frame(a = 1:10)
    #' Title.
    'a'
  ")[[1]]

  expect_equal(out$get_value("docType"), "data")
  expect_equal(out$get_value("usage"), rd("a"))
  expect_equal(out$get_value("keyword"), "datasets")
})

# Reference classes ----------------------------------------------------------

test_that("@docType class automatically added to reference class objects", {
  out <- roc_proc_text(rd_roclet(), "
    #' Title.
    #'
    a <- setRefClass('a')")[[1]]

  expect_equal(out$get_value("docType"), "class")
})


# imports -----------------------------------------------------------------

test_that("only generates re-exports if no @name or @rdname", {
  block <- "
    #' @export
    stats::median
  "
  out <- roc_proc_text(rd_roclet(), block)[[1]]
  expect_equal(out$get_value("name"), "reexports")
  expect_equal(out$get_value("keyword"), "internal")

  block <- "
    #' Title
    #' @name stats-imports
    #' @export
    stats::median

    #' @rdname stats-imports
    stats::acf
  "
  out <- roc_proc_text(rd_roclet(), block)[[1]]
  expect_equal(out$get_value("name")[[1]], "stats-imports")
  expect_equal(out$get_value("alias"), c("stats-imports", "median", "acf"))
  expect_equal(out$get_value("keyword"), NULL)
})

test_that("imports are automatically imported", {
  block <- "
    #' @export
    stats::median
  "
  out <- roc_proc_text(namespace_roclet(), block)
  expect_equal(out, c("export(median)", "importFrom(stats,median)"))

  block <- "
    #' @export
    #' @name foo
    stats::median
  "
  out <- roc_proc_text(namespace_roclet(), block)
  expect_equal(out, c("export(median)", "importFrom(stats,median)"))
})

# packages -----------------------------------------------------------------

test_that("can create package documentation", {
  path <- local_package_copy(test_path("empty"))
  desc::desc_set(
    file = path,
    Package = "roxygendevtest",
    Title = "Package Title",
    Description = "Package description."
  )

  block <- "
    #' @details Details.
  '_PACKAGE'
  "
  withr::with_dir(
    path,
    blocks <- parse_text(block, env = new.env())
  )
  out <- roclet_process(rd_roclet(), blocks, env = new.env(), base_path = ".")[[1]]

  expect_equal(out$get_value("name"), "roxygendevtest-package")
  expect_equal(out$get_value("alias"), c("roxygendevtest", "roxygendevtest-package"))
  expect_equal(out$get_value("title"), "roxygendevtest: Package Title")
  expect_equal(out$get_value("description"), "Package description.")
  expect_equal(out$get_value("docType"), "package")
  expect_equal(out$get_value("details"), "Details.")
})

#' @useDynLib roxygen2, .registration=TRUE
#' @keywords internal
#' @import rlang
"_PACKAGE"

## usethis namespace: start
#' @importFrom knitr knit
#' @importFrom knitr opts_chunk
#' @importFrom lifecycle deprecated
#' @importFrom R6 R6Class
#' @importFrom stats setNames
#' @importFrom utils head
#' @importFrom utils tail
#' @importFrom utils URLdecode
#' @importFrom utils URLencode
#' @importFrom xml2 xml_attr
#' @importFrom xml2 xml_children
#' @importFrom xml2 xml_contents
#' @importFrom xml2 xml_find_all
#' @importFrom xml2 xml_name
#' @importFrom xml2 xml_ns_strip
#' @importFrom xml2 xml_text
#' @importFrom xml2 xml_type
## usethis namespace: end
NULL

# Tags --------------------------------------------------------------------

#' @export
roxy_tag_parse.roxy_tag_inherit <- function(x) tag_inherit(x)
#' @export
roxy_tag_rd.roxy_tag_inherit <- function(x, base_path, env) {
  rd_section_inherit(x$val$source, list(x$val$fields))
}

#' @export
roxy_tag_parse.roxy_tag_inheritParams <- function(x) tag_value(x)
#' @export
roxy_tag_rd.roxy_tag_inheritParams <- function(x, base_path, env) {
  rd_section_inherit(x$val, list("params"))
}

#' @export
roxy_tag_parse.roxy_tag_inheritDotParams <- function(x) {
  tag_two_part(x, "source", "args", required = FALSE, markdown = FALSE)
}
#' @export
roxy_tag_rd.roxy_tag_inheritDotParams <- function(x, base_path, env) {
  rd_section_inherit_dot_params(x$val$source, x$val$args)
}

#' @export
roxy_tag_parse.roxy_tag_inheritSection <- function(x) tag_name_description(x)
#' @export
roxy_tag_rd.roxy_tag_inheritSection <- function(x, base_path, env) {
  rd_section_inherit_section(x$val$name, x$val$description)
}


# Fields ------------------------------------------------------------------

# For each unique source, list which fields it inherits from
rd_section_inherit <- function(source, fields) {
  stopifnot(is.character(source), is.list(fields))
  stopifnot(!anyDuplicated(source))
  stopifnot(length(source) == length(fields))

  rd_section("inherit", list(source = source, fields = fields))
}

#' @export
merge.rd_section_inherit <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))

  dedup <- collapse(
    c(x$value$source, y$value$source),
    c(x$value$fields, y$value$fields),
    function(x) Reduce(union, x)
  )

  rd_section("inherit", list(source = dedup$key, fields = dedup$value))
}

#' @export
format.rd_section_inherit <- function(x, ...) NULL

rd_section_inherit_section <- function(source, title) {
  stopifnot(is.character(source), is.character(title))
  stopifnot(length(source) == length(title))

  rd_section("inherit_section", list(source = source, title = title))
}

#' @export
format.rd_section_inherit_section <- function(x, ...) NULL

#' @export
merge.rd_section_inherit_section <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  rd_section_inherit_section(c(x$value$source, y$value$source), c(x$value$title, y$value$title))
}

rd_section_inherit_dot_params <- function(source, args) {
  stopifnot(is.character(source), is.character(args))
  stopifnot(length(source) == length(args))

  rd_section("inherit_dot_params", list(source = source, args = args))
}

#' @export
format.rd_section_inherit_dot_params <- function(x, ...) NULL

#' @export
merge.rd_section_inherit_dot_params <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  rd_section_inherit_dot_params(c(x$value$source, y$value$source), c(x$value$args, y$value$args))
}


# Process inheritance -----------------------------------------------------

topics_process_inherit <- function(topics, env) {
  inherits <- function(type) {
    function(x) x$inherits_from(type)
  }

  topics$topo_apply(inherits("return"), inherit_field,
    roxy_name = "return", rd_name = "value")
  topics$topo_apply(inherits("title"), inherit_field, "title")
  topics$topo_apply(inherits("description"), inherit_field, "description")
  topics$topo_apply(inherits("details"), inherit_field, "details")
  topics$topo_apply(inherits("seealso"), inherit_field, "seealso")
  topics$topo_apply(inherits("references"), inherit_field, "references")
  topics$topo_apply(inherits("examples"), inherit_field, "examples")
  topics$topo_apply(inherits("author"), inherit_field, "author")
  topics$topo_apply(inherits("source"), inherit_field, "source")
  topics$topo_apply(inherits("note"), inherit_field, "note")

  # First inherit individual sections, then all sections.
  topics$topo_apply(function(x) x$inherits_section_from(), inherit_section)
  topics$topo_apply(inherits("sections"), inherit_sections)

  topics$topo_apply(inherits("params"), inherit_params)
  # Can't inherit ... into ... so can do in any order
  topics$apply(inherit_dot_params, env = env)

  invisible()
}

# Inherit parameters -----------------------------------------------------------

inherit_params <- function(topic, topics) {
  inheritors <- topic$inherits_from("params")
  if (length(inheritors) == 0) {
    return()
  }

  documented <- get_documented_params(topic)
  needed <- topic$get_value("formals")
  missing <- setdiff(needed, documented)
  if (length(missing) == 0) {
    cli::cli_warn(
      c(
        "@inheritParams failed in topic {.str {topic$get_name()}}.",
        x = "All parameters are already documented; none remain to be inherited."
      ),
      call = NULL
    )
    return()
  }

  # Work through inherited params seeing if any match the parameters
  # we're missing
  for (inheritor in inheritors) {
    inherited_params <- find_params(inheritor, topics, source = topic$get_name())

    for (param in inherited_params) {
      match <- match_param(param$name, missing)
      if (!is.null(match)) {
        param_val <- setNames(param$value, paste(match, collapse = ","))
        topic$add(rd_section("param", param_val))
        missing <- setdiff(missing, match)
      }
    }
    if (length(missing) == 0) break
  }

}

# Ignore . prefix since it's sometimes necessary to add because a
# function uses ...

# Match parameters ignoring dots
match_param <- function(from, to) {
  flip_dot <- function(x) {
    has_dot <- grepl("^\\.", x)
    ifelse(has_dot, gsub("^\\.", "", x), paste0(".", x))
  }

  to_std <- c(to, flip_dot(to))
  if (!all(from %in% to_std)) {
    return(NULL)
  }

  union(
    setdiff(to[match(from, to)], NA),
    setdiff(to[match(from, flip_dot(to))], NA)
  )
}

inherit_dot_params <- function(topic, topics, env) {
  inheritors <- topic$get_value("inherit_dot_params")
  if (is.null(inheritors))
    return()

  # Need to find formals for each source
  funs <- lapply(inheritors$source, function(x) eval(parse(text = x), envir = env))
  args <- map2(funs, inheritors$args, select_args_text, topic = topic)

  # Then pull out the ones we need
  docs <- lapply(inheritors$source, find_params, topics = topics)
  arg_matches <- function(args, docs) {
    match <- map_lgl(docs, function(x) all(x$name %in% args))
    matched <- docs[match]
    setNames(
      lapply(matched, "[[", "value"),
      map_chr(matched, function(x) paste(x$name, collapse = ","))
    )
  }
  docs_selected <- unlist(map2(args, docs, arg_matches))

  # Only document params under "..." that aren't otherwise documented
  documented <- get_documented_params(topic)
  non_documented_params <- setdiff(names(docs_selected), documented)
  docs_selected <- docs_selected[non_documented_params]

  # Build the Rd
  # (1) Link to function(s) that was inherited from
  src <- inheritors$source
  dest <- map_chr(src, resolve_qualified_link)
  from <- paste0("\\code{\\link[", dest, "]{", src, "}}", collapse = ", ")

  # (2) Show each inherited argument
  arg_names <- paste0("\\code{", names(docs_selected), "}")
  args <- paste0("    \\item{", arg_names, "}{", docs_selected, "}", collapse = "\n")

  rd <- paste0(
    "\n",
    "  Arguments passed on to ", from, "\n",
    "  \\describe{\n",
    args, "\n",
    "  }"
  )
  topic$add(rd_section("param", c("..." = rd)))
}


get_documented_params <- function(topic, only_first = FALSE) {
  documented <- names(topic$get_value("param"))
  if (length(documented) > 0) {
    documented <- strsplit(documented, ",")
    if (only_first)
      documented <- map_chr(documented, 1)
    else
      documented <- unlist(documented)
  }

  documented[documented == "\\dots"] <- "..."
  documented
}

find_params <- function(name, topics, source) {
  topic <- get_rd(name, topics, source = source)
  if (is.null(topic)) {
    return()
  }

  params <- topic_params(topic)
  if (is.null(params))
    return()

  param_names <- str_trim(names(params))
  param_names[param_names == "\\dots"] <- "..."

  Map(list,
    name = strsplit(param_names, ",\\s*"),
    value = unlist(params)
  )
}

topic_params <- function(x) {
  if (inherits(x, "Rd")) {
    arguments <- get_tags(x, "\\arguments")
    if (length(arguments) != 1) {
      return(list())
    }
    items <- get_tags(arguments[[1]], "\\item")

    values <- map_chr(items, function(y) rd2text(y[[2]], attr(x, "package")))
    params <- map_chr(items, function(y) rd2text(y[[1]], attr(x, "package")))

    setNames(values, params)
  } else {
    x$get_value("param")
  }
}

# Inherit sections --------------------------------------------------------

inherit_sections <- function(topic, topics) {
  current_secs <- topic$get_value("section")$title

  for (inheritor in topic$inherits_from("sections")) {
    inheritor <- get_rd(inheritor, topics, source = topic$get_name())
    if (is.null(inheritor)) {
      return()
    }

    sections <- find_sections(inheritor)
    needed <- !(sections$title %in% current_secs)
    if (!any(needed))
      next

    topic$add(
      rd_section_section(sections$title[needed], sections$content[needed])
    )
  }
}

inherit_section <- function(topic, topics) {
  sections <- topic$get_value("inherit_section")
  sources <- sections$source
  titles <- sections$title

  for (i in seq_along(sources)) {
    inheritor <- get_rd(sources[[i]], topics, source = topic$get_name())
    if (is.null(inheritor)) {
      return()
    }

    new_section <- find_sections(inheritor)
    selected <- new_section$title %in% titles[[i]]

    if (sum(selected) != 1) {
      cli::cli_warn(
        c(
          "@inheritSection failed in topic {.str {topic$get_name()}}.",
          x = "Can't find section {.str {titles[[i]]}} in topic {sources[[i]]}."
        ),
        call = NULL
      )
      return()
    }

    topic$add(
      rd_section_section(new_section$title[selected], new_section$content[selected])
    )
  }
}

find_sections <- function(topic) {
  if (inherits(topic, "Rd")) {
    tag <- get_tags(topic, "\\section")

    titles <- map_chr(map(tag, 1), rd2text, package = attr(topic, "package"))
    contents <- map_chr(map(tag, 2), rd2text, package = attr(topic, "package"))

    list(title = titles, content = contents)
  } else {
    topic$get_value("section")
  }
}


# Inherit from single field ----------------------------------------------------

inherit_field <- function(topic, topics, rd_name, roxy_name = rd_name) {
  # Already has the field, so don't need to inherit
  if (topic$has_section(rd_name))
    return()

  # Otherwise, try each try function listed in inherits
  for (inherit_from in topic$inherits_from(roxy_name)) {
    inherit_topic <- get_rd(inherit_from, topics, source = topic$get_name())
    if (is.null(inherit_topic)) {
      next
    }

    inheritee <- find_field(inherit_topic, rd_name)
    if (is.null(inheritee))
      next

    topic$add(rd_section(rd_name, inheritee))
    return()
  }
}

find_field <- function(topic, field_name) {
  if (inherits(topic, "Rd")) {
    tag <- get_tags(topic, paste0("\\", field_name))
    if (length(tag) == 0)
      return()

    value <- tag[[1]]
    attr(value, "Rd_tag") <- NULL

    str_trim(rd2text(value, attr(topic, "package")))
  } else {
    topic$get_value(field_name)
  }
}

rd2text <- function(x, package) {
  x <- tweak_links(x, package)
  chr <- as_character_rd(structure(x, class = "Rd"), deparse = TRUE)
  paste(chr, collapse = "")
}

# Convert relative to absolute links
tweak_links <- function(x, package) {
  tag <- attr(x, "Rd_tag")

  if (is.list(x)) {
    if (!is.null(tag) && tag == "\\link") {
      opt <- attr(x, "Rd_option")
      if (is.null(opt)) {
        if (has_topic(x[[1]], package)) {
          attr(x, "Rd_option") <- structure(package, Rd_tag = "TEXT")
        }
      } else if (is_string(opt) && substr(opt, 1, 1) == "=") {
        topic <- substr(opt, 2, nchar(opt))

        if (has_topic(topic, package)) {
          attr(x, "Rd_option") <- structure(paste0(package, ":", topic), Rd_tag = "TEXT")
        }
      }
    } else if (length(x) > 0) {
      x[] <- map(x, tweak_links, package = package)
    }
  }

  x
}


# Find info in Rd or topic ------------------------------------------------

get_rd <- function(name, topics, source) {
  if (is_namespaced(name)) {
    # External package
    parsed <- parse_expr(name)
    pkg <- as.character(parsed[[2]])
    fun <- as.character(parsed[[3]])

    get_rd_from_help(pkg, fun, source)
  } else {
    # Current package
    rd_name <- topics$find_filename(name)
    if (identical(rd_name, NA_character_)) {
      cli::cli_warn(
        c(
          "@inherits failed in topic {.str {source}}.",
          x = "Can't find topic {.str {name}}."
        ),
        call = NULL
      )
    }
    topics$get(rd_name)
  }
}

get_rd_from_help <- function(package, alias, source) {
  if (!is_installed(package)) {
    cli::cli_warn(
      c(
        "@inherits failed in topic {.str {source}}.",
        x = "Package {package} is not installed."
      ),
      call = NULL
    )
    return()
  }

  help <- utils::help((alias), (package))
  if (length(help) == 0) {
    cli::cli_warn(
      c(
        "@inherits failed in topic {.str {source}}.",
        x = "Can't find topic {package}::{alias}."
      ),
      call = NULL
    )
    return()
  }

  out <- internal_f("utils", ".getHelpFile")(help)
  attr(out, "package") <- package
  out
}

# Rd parsing --------------------------------------------------------------

test_that("can round-trip Rd", {
  rd <- tools::parse_Rd(test_path("escapes.Rd"))

  field <- find_field(rd, "description")
  lines <- strsplit(field, "\n")[[1]]
  expect_equal(
    lines,
    c(
      "% Comment", # Latex comments shouldn't be escaped
      "\\code{\\\\}" # Backslashes in code should be
    )
  )
})

test_that("\\links are transformed", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #'
    #' @inheritParams digest::sha1
    wrapper <- function(algo) {}"
  )[[1]]

  # \\link{} should include [digest]
  expect_snapshot_output(out$get_section("param"))
})

test_that("markdown doesn't get get extra parens", {
  expect_equal(rd2text(parse_rd("\\href{a}{b}")), "\\href{a}{b}\n")
  expect_equal(rd2text(parse_rd("\\ifelse{a}{b}{c}")), "\\ifelse{a}{b}{c}\n")
  expect_equal(rd2text(parse_rd("\\if{a}{b}")), "\\if{a}{b}\n")
})

test_that("relative links converted to absolute", {
  link_to_base <- function(x) {
    rd2text(parse_rd(x), package = "base")
  }

  expect_equal(
    link_to_base("\\link{abbreviate}"),
    "\\link[base]{abbreviate}\n"
  )
  expect_equal(
    link_to_base("\\link[=abbreviate]{abbr}"),
    "\\link[base:abbreviate]{abbr}\n"
  )

  # Doesn't affect links that already have
  expect_equal(
    link_to_base("\\link[foo]{abbreviate}"),
    "\\link[foo]{abbreviate}\n"
  )
  expect_equal(
    link_to_base("\\link[foo::abbreviate]{abbr}"),
    "\\link[foo::abbreviate]{abbr}\n"
  )

  # linkS4class converted to absolute link (#1634)
  link_to_methods <- function(x) {
    rd2text(parse_rd(x), package = "methods")
  }
  expect_equal(
    link_to_methods("\\linkS4class{genericFunction}"),
    "\\link[methods:genericFunction-class]{genericFunction}\n"
  )
})

# tag parsing -------------------------------------------------------------

test_that("invalid syntax gives useful warning", {
  block <- "
    #' @inheritDotParams
    #' @inheritSection
    NULL
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("warns on unknown inherit type", {
  text <- "
    #' @inherit fun blah
    NULL
  "
  expect_snapshot(parse_text(text))
})

test_that("no options gives default values", {
  block <- parse_text(
    "
    #' @inherit fun
    NULL
  "
  )[[1]]

  expect_equal(block_get_tag_value(block, "inherit")$fields, inherit_components)
})

test_that("some options overrides defaults", {
  block <- parse_text(
    "
    #' @inherit fun return
    NULL
  "
  )[[1]]

  expect_equal(block_get_tag_value(block, "inherit")$fields, "return")
})


# Inherit return values ---------------------------------------------------

test_that("can inherit return values from roxygen topic", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @return ABC
    a <- function(x) {}

    #' B
    #'
    #' @inherit a
    b <- function(y) {}
  "
  )[[2]]

  expect_equal(out$get_value("value"), "ABC")
})


test_that("takes value from first with return", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A1
    #' @return A
    a1 <- function(x) {}

    #' A2
    a2 <- function() {}

    #' B
    #' @return B
    b <- function(x) {}

    #' C
    #' @inherit a2
    #' @inherit b
    #' @inherit a1
    c <- function(y) {}
  "
  )[[3]]

  expect_equal(out$get_value("value"), "B")
})

test_that("can inherit return value from external function", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A1
    #' @inherit base::mean
    a1 <- function(x) {}
  "
  )[[1]]

  expect_match(out$get_value("value"), "before the mean is computed.$")
  expect_match(out$get_value("value"), "^If \\\\code")
})


# Inherit seealso ---------------------------------------------------------

test_that("can inherit return values from roxygen topic", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @seealso ABC
    a <- function(x) {}

    #' B
    #'
    #' @inherit a
    b <- function(y) {}
  "
  )[[2]]

  expect_equal(out$get_value("seealso"), "ABC")
})

# Inherit description and details -----------------------------------------

test_that("can inherit description from roxygen topic", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' B
    #'
    #' @return ABC
    a <- function(x) {}

    #' @title C
    #' @inherit a description
    b <- function(y) {}
  "
  )[[2]]

  expect_equal(out$get_value("description"), "B")
})

test_that("inherits description if omitted", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' B
    #'
    #' @return ABC
    a <- function(x) {}

    #' C
    #' @inherit a description
    b <- function(y) {}
  "
  )[[2]]

  expect_equal(out$get_value("description"), "B")
})

test_that("can inherit details from roxygen topic", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' B
    #'
    #' C
    #'
    #' @return ABC
    a <- function(x) {}

    #' D
    #'
    #' E
    #'
    #' @inherit a details
    b <- function(y) {}
  "
  )[[2]]

  expect_equal(out$get_value("description"), "E")
  expect_equal(out$get_value("details"), "C")
})


# Inherit sections --------------------------------------------------------

test_that("inherits missing sections", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #' @section A:1
    #' @section B:1
    a <- function(x) {}

    #' D
    #'
    #' @section A:2
    #' @inherit a sections
    b <- function(y) {}
  "
  )[[2]]

  section <- out$get_value("section")
  expect_equal(section$title, c("A", "B"))
  expect_equal(section$content, c("2", "1"))
})

test_that("can inherit single section", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #' @section A:1
    #' @section B:1
    a <- function(x) {}

    #' D
    #'
    #' @inheritSection a B
    b <- function(y) {}
  "
  )[[2]]

  section <- out$get_value("section")
  expect_equal(section$title, "B")
  expect_equal(section$content, "1")
})


test_that("warns if can't find section", {
  code <- "
    #' a
    a <- function(x) {}

    #' b
    #'
    #' @inheritSection a A
    b <- function(y) {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), code))
})

# Inherit parameters ------------------------------------------------------

test_that("match_params can ignore . prefix", {
  expect_equal(match_param("a", c("x", "y", "z")), NULL)
  expect_equal(match_param("x", c("x", "y", "z")), "x")
  expect_equal(match_param(".x", c("x", "y", "z")), "x")
  expect_equal(match_param("x", c(".x", ".y", ".z")), ".x")
  expect_equal(match_param(".x", c(".x", ".y", ".z")), ".x")
  expect_equal(match_param(c(".x", "y"), c(".x", ".y", ".z")), c(".x", ".y"))
  expect_equal(match_param(c(".x", "x"), c("x", ".x")), c(".x", "x"))
  expect_equal(match_param(c(".x", "x"), "x"), "x")
  expect_equal(match_param("x", c(".x", "x")), c("x", ".x"))
})

test_that("multiple @inheritParam tags gathers all params", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    a <- function(x) {}

    #' B
    #'
    #' @param y Y
    b <- function(y) {}

    #' C
    #'
    #' @inheritParams a
    #' @inheritParams b
    c <- function(x, y) {}
    "
  )

  params <- out[["c.Rd"]]$get_value("param")
  expect_equal(length(params), 2)

  expect_equal(params[["x"]], "X")
  expect_equal(params[["y"]], "Y")
})

test_that("multiple @inheritParam tags gathers all params", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    a <- function(x) {}

    #' B
    #'
    #' @param .y Y
    b <- function(.y) {}

    #' C
    #'
    #' @inheritParams a
    #' @inheritParams b
    c <- function(.x, y) {}
    "
  )[[3]]
  expect_equal(out$get_value("param"), c(.x = "X", y = "Y"))
})

test_that("@inheritParam preserves mixed names", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #' @param .x,x X
    a <- function(x, .x) {}

    #' B
    #' @inheritParams a
    b <- function(x, .x) {}
  "
  )[[2]]

  expect_equal(out$get_value("param"), c(".x,x" = "X"))
})

test_that("can inherit from same arg twice", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    a <- function(x) {}

    #' B
    #'
    #' @inheritParams a
    b <- function(x) {}

    #' C
    #'
    #' @inheritParams a
    #' @rdname b
    c <- function(.x) {}
    "
  )[[2]]
  expect_equal(out$get_value("param"), c("x,.x" = "X"))
})

test_that("@inheritParams can inherit from inherited params", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' C
    #'
    #' @inheritParams b
    c <- function(x) {}

    #' B
    #'
    #' @inheritParams a
    b <- function(x) {}

    #' A.
    #'
    #' @param x X
    a <- function(x) {}
    "
  )

  expect_equal(out[["c.Rd"]]$get_value("param"), c(x = "X"))
})

test_that("multiple @inheritParam inherits from existing topics", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' My mean
    #'
    #' @inheritParams base::mean
    mymean <- function(x, trim) {}"
  )[[1]]
  params <- out$get_value("param")
  expect_equal(length(params), 2)
  expect_equal(sort(names(params)), c("trim", "x"))
})


test_that("@inheritParam can inherit multivariable arguments", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param x,y X and Y
    A <- function(x, y) {}

    #' B
    #'
    #' @inheritParams A
    B <- function(x, y) {}"
  )[[2]]
  expect_equal(out$get_value("param"), c("x,y" = "X and Y"))

  # Even when the names only match without .
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param x,y X and Y
    A <- function(x, y) {}

    #' B
    #'
    #' @inheritParams A
    B <- function(.x, .y) {}"
  )[[2]]
  expect_equal(out$get_value("param"), c(".x,.y" = "X and Y"))
})

test_that("@inheritParam only inherits exact multiparam matches", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param x,y X and Y
    A <- function(x, y) {}

    #' B
    #'
    #' @inheritParams A
    B <- function(x) {}"
  )[[2]]
  expect_equal(out$get_value("param"), NULL)
})

test_that("@inheritParams inherits params documented with dots (#1718)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A
    #' @param a A variable.
    #' @param b,\\dots Parameters to pass.
    A <- function(a, b, ...) {}

    #' B
    #' @inheritParams A
    B <- function(a, b, ...) {}
    "
  )
  expect_equal(
    out[[2]]$get_value("param"),
    c("a" = "A variable.", "b,..." = "Parameters to pass.")
  )
})

test_that("@inheritParam understands compound docs", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Title
    #'
    #' @param x x
    #' @param y x
    x <- function(x, y) {}

    #' Title
    #'
    #' @inheritParams x
    #' @param y y
    y <- function(x, y) {}"
  )[[2]]
  params <- out$get_value("param")
  expect_equal(params, c(x = "x", y = "y"))
})

test_that("warned if no params need documentation", {
  code <- "
    #' Title
    #'
    #' @param x x
    #' @param y x
    #' @inheritParams foo
    x <- function(x, y) {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), code))
})

test_that("argument order, also for incomplete documentation", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param y Y
    #' @param x X
    a <- function(x, y) {}

    #' B
    #'
    #' @param y Y
    b <- function(x, y) {}

    #' C
    #'
    #' @param x X
    c <- function(x, y) {}

    #' D
    #'
    #' @inheritParams b
    #' @param z Z
    d <- function(x, y, z) {}

    #' E
    #'
    #' @inheritParams c
    #' @param y Y
    e <- function(x, y, z) {}
  "
  )

  expect_equal(out[["a.Rd"]]$get_value("param"), c(x = "X", y = "Y"))
  expect_equal(out[["b.Rd"]]$get_value("param"), c(y = "Y"))
  expect_equal(out[["c.Rd"]]$get_value("param"), c(x = "X"))
  expect_equal(out[["d.Rd"]]$get_value("param"), c(y = "Y", z = "Z"))
  expect_equal(out[["e.Rd"]]$get_value("param"), c(x = "X", y = "Y"))
})

test_that("argument order with @inheritParam", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    a <- function(x, y) {}

    #' B1
    #'
    #' @param y B
    #' @inheritParams a
    b1 <- function(x, y) {}

    #' B2
    #'
    #' @inheritParams a
    #' @param y B
    b2 <- function(x, y) {}

    #' C1
    #'
    #' @param x C
    #' @inheritParams a
    c1 <- function(x, y) {}

    #' C2
    #'
    #' @inheritParams a
    #' @param x C
    c2<- function(x, y) {}
    "
  )

  expect_equal(out[["b1.Rd"]]$get_value("param"), c(x = "X", y = "B"))
  expect_equal(out[["b2.Rd"]]$get_value("param"), c(x = "X", y = "B"))
  expect_equal(out[["c1.Rd"]]$get_value("param"), c(x = "C", y = "Y"))
  expect_equal(out[["c2.Rd"]]$get_value("param"), c(x = "C", y = "Y"))
})


test_that("inherit params ... named \\dots", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @param x x
    #' @param \\dots foo
    foo <- function(x, ...) {}

    #' Bar
    #'
    #' @inheritParams foo
    #' @param \\dots bar
    bar <- function(x=1, ...) {}
  "
  )[[2]]

  expect_equal(
    out$get_value("param"),
    c(x = "x", "\\dots" = "bar")
  )
})

# inheritParams argument filtering ----------------------------------------

test_that("@inheritParams can include specific args", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    #' @param z Z
    a <- function(x, y, z) {}

    #' B
    #'
    #' @inheritParams a x z
    b <- function(x, y, z) {}
    "
  )[[2]]

  params <- out$get_value("param")
  expect_equal(params, c(x = "X", z = "Z"))
})

test_that("@inheritParams can exclude specific args", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    #' @param z Z
    a <- function(x, y, z) {}

    #' B
    #'
    #' @inheritParams a -y
    b <- function(x, y, z) {}
    "
  )[[2]]

  params <- out$get_value("param")
  expect_equal(params, c(x = "X", z = "Z"))
})

test_that("@inheritParams filtering works with external packages", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' My mean
    #'
    #' @inheritParams base::mean -na.rm
    mymean <- function(x, trim, na.rm) {}"
  )[[1]]

  params <- out$get_value("param")
  expect_true("trim" %in% names(params))
  expect_false("na.rm" %in% names(params))
})

test_that("multiple @inheritParams can each filter args (#1879)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    a <- function(x, y) {}

    #' B.
    #'
    #' @param z Z
    #' @param w W
    b <- function(z, w) {}

    #' C
    #'
    #' @inheritParams a x
    #' @inheritParams b -w
    c <- function(x, y, z, w) {}
    "
  )[[3]]

  params <- out$get_value("param")
  expect_equal(params, c(x = "X", z = "Z"))
})

test_that("@inheritParams filters are unioned for a repeated source", {
  filter <- function(...) {
    tags <- paste0("    #' @inheritParams ", c(...), collapse = "\n")
    out <- roc_proc_text(
      rd_roclet(),
      paste0(
        "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    #' @param z Z
    a <- function(x, y, z) {}

    #' B
    #'
",
        tags,
        "
    b <- function(x, y, z) {}
    "
      )
    )[[2]]
    out$get_value("param")
  }

  expect_equal(filter("a x", "a z"), c(x = "X", z = "Z"))
  expect_equal(filter("a x", "a -y"), c(x = "X", z = "Z"))
  # Each tag selects independently, so the order of tags doesn't matter
  expect_equal(filter("a -y", "a x"), c(x = "X", z = "Z"))
  expect_equal(filter("a x", "a -x"), c(x = "X", y = "Y", z = "Z"))
  # An unfiltered tag selects everything
  expect_equal(filter("a x", "a"), c(x = "X", y = "Y", z = "Z"))
  expect_equal(filter("a", "a x"), c(x = "X", y = "Y", z = "Z"))
})

test_that("@inheritParams without args still works", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' A.
    #'
    #' @param x X
    #' @param y Y
    a <- function(x, y) {}

    #' B
    #'
    #' @inheritParams a
    b <- function(x, y) {}
    "
  )[[2]]

  params <- out$get_value("param")
  expect_equal(params, c(x = "X", y = "Y"))
})

# inheritDotParams --------------------------------------------------------

test_that("can inherit all from single function", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @param x x
    #' @param y y
    foo <- function(x, y) {}

    #' Bar
    #'
    #' @inheritDotParams foo
    bar <- function(...) {}
  "
  )[[2]]

  expect_snapshot_output(test_path("test-rd-inherit-dots.txt"))
})

test_that("does not produce multiple ... args", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @inheritParams bar
    #' @inheritDotParams baz
    foo <- function(x, ...) {}

    #' Bar
    #'
    #' @param x x
    #' @param ... dots
    bar <- function(x, ...) {}

    #' Baz
    #'
    #' @param y y
    #' @param z z
    baz <- function(y, z) {}
  "
  )[[1]]

  expect_snapshot_output(test_path("test-rd-inherit-dots-inherit.txt"))
})

test_that("can inherit dots from several functions", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @param x x
    #' @param y y1
    foo <- function(x, y) {}

    #' Bar
    #'
    #' @param y y2
    #' @param z z
    bar <- function(z) {}

    #' Foobar
    #'
    #' @inheritDotParams foo
    #' @inheritDotParams bar
    foobar <- function(...) {}
  "
  )[[3]]

  expect_snapshot_output(out$get_section("param"))
})

test_that("inheritDotParams does not add already-documented params", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Wrapper around original
    #'
    #' @inherit original
    #' @inheritDotParams original
    #' @param y some more specific description
    #' @export
    wrapper <- function(x = 'some_value', y = 'some other value', ...) {
      original(x = x, y = y, ...)
    }

    #' Original function
    #'
    #' @param x x description
    #' @param y y description
    #' @param z z description
    #' @export
    original <- function(x, y, z, ...) {}
  "
  )[[1]]

  params <- out$get_value("param")
  dot_param <- params[["..."]]
  expect_named(params, c("x", "y", "..."))
  expect_false(grepl("item{x}{x description}", dot_param, fixed = TRUE))
  expect_false(grepl("item{y}{y description}", dot_param, fixed = TRUE))
  expect_match(dot_param, "item{\\code{z}}{z description}", fixed = TRUE)
})

test_that("useful error for bad inherits", {
  text <- "
    #' Foo
    #'
    #' @param x x
    #' @param y y
    foo <- function(x, y) {}

    #' Bar
    #'
    #' @inheritDotParams foo -z
    bar <- function(...) {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), text))
})

test_that("warns when no params to inherit (#1671)", {
  text <- "
    #' Foo
    #'
    #' @param ... not used
    foo <- function(...) {}

    #' Bar
    #'
    #' @param y y
    #' @inheritDotParams foo
    bar <- function(y, ...) {}
  "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), text))
  expect_false("..." %in% names(out[["bar.Rd"]]$get_value("param")))
})

test_that("inheritDotParams matches when doc uses dotted name but formal doesn't (#1826)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' @param x x
    #' @param .y,y doc for y
    foo <- function(x, y) {}

    #' Bar
    #'
    #' @inheritDotParams foo x y
    bar <- function(...) {}
  "
  )[[2]]

  dot_param <- out$get_value("param")[["..."]]
  expect_match(dot_param, "item{\\code{y}}", fixed = TRUE)
})

test_that("inheritDotParams uses all documented params, not just formals (#1840)", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' My mean
    #'
    #' @inheritDotParams base::mean
    mymean <- function(x, ...) {}
  "
  )[[1]]

  dot_param <- out$get_value("param")[["..."]]
  expect_match(dot_param, "\\code{trim}", fixed = TRUE)
  expect_match(dot_param, "\\code{na.rm}", fixed = TRUE)
})

test_that("inheritDotParams warns when source not found (#1602)", {
  text <- "
    #' Test
    #' @inheritDotParams format
    test = function(...) {}
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), text))
})

# inherit everything ------------------------------------------------------

test_that("can inherit all from single function", {
  out <- roc_proc_text(
    rd_roclet(),
    "
    #' Foo
    #'
    #' Description
    #'
    #' Details
    #'
    #' @param x x
    #' @param y y
    #' @author Hadley
    #' @source my mind
    #' @note my note
    #' @format my format
    #' @examples
    #' x <- 1
    foo <- function(x, y) {}

    #' @inherit foo
    bar <- function(x, y) {}
  "
  )[[2]]

  expect_named(out$get_value("param"), c("x", "y"))
  expect_equal(out$get_value("title"), "Foo")
  expect_equal(out$get_value("description"), "Description")
  expect_equal(out$get_value("details"), "Details")
  expect_equal(out$get_value("examples"), rd("x <- 1"))
  expect_equal(out$get_value("author"), "Hadley")
  expect_equal(out$get_value("source"), "my mind")
  expect_equal(out$get_value("format"), "my format")
  expect_equal(out$get_value("note"), "my note")
})


# get_rd() -----------------------------------------------------------------

test_that("useful warnings if can't find topics", {
  expect_snapshot({
    get_rd("notinstalled::pkg", source = "source")
    get_rd("base::doesntexist", source = "source")
    get_rd("doesntexist", RoxyTopics$new(), source = "source")
  })
})

test_that("can find section in existing docs", {
  out <- find_sections(get_rd("base::attach"))
  expect_equal(out$title, "Good practice")
})

# Without sections --------------------------------------------------------

#' @export
roxy_tag_parse.roxy_tag_author <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_author <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_author <- function(x, ...) {
  format_collapse(x, ...)
}
#' @export
merge.rd_section_author <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  # Remove duplicated authors, e.g. when using @rdname
  rd_section(x$type, unique(c(x$value, y$value)))
}


#' @export
roxy_tag_parse.roxy_tag_format <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_format <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_format <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_note <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_note <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_note <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_references <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_references <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_references <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_return <- function(x) tag_markdown(x)
#' @export
roxy_tag_parse.roxy_tag_returns <- roxy_tag_parse.roxy_tag_return
#' @export
roxy_tag_rd.roxy_tag_return <- function(x, base_path, env) {
  rd_section("value", x$val)
}
#' @export
roxy_tag_rd.roxy_tag_returns <- roxy_tag_rd.roxy_tag_return
#' @export
format.rd_section_value <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_seealso <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_seealso <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_seealso <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_source <- function(x) tag_markdown(x)
#' @export
roxy_tag_rd.roxy_tag_source <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_source <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_title <- function(x) {

  if (str_count(x$raw, "\n\n") >= 1) {
    warn_roxy_tag(x, "must be a single paragraph")
  }

  tag_markdown(x)
}
#' @export
roxy_tag_rd.roxy_tag_title <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_title <- function(x, ...) {
  format_first(x, ...)
}

# With sections -----------------------------------------------------------

#' @export
roxy_tag_parse.roxy_tag_description <- function(x) {
  tag_markdown_with_sections(x)
}
#' @export
roxy_tag_rd.roxy_tag_description <- function(x, base_path, env) {
  rd_section_markdown(x$tag, x$val)
}
#' @export
format.rd_section_description <- function(x, ...) {
  format_collapse(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_details <- function(x) {
  tag_markdown_with_sections(x)
}
#' @export
roxy_tag_rd.roxy_tag_details <- function(x, base_path, env) {
  rd_section_markdown(x$tag, x$val)
}
#' @export
format.rd_section_details <- function(x, ...) {
  format_collapse(x, ...)
}

rd_section_markdown <- function(name, value) {
  # Any additional components are sections
  if (length(value) > 1) {
    titles <- names(value)
    value <- unname(value)

    name <- c(name, rep("section", length(value) - 1))
    value <- c(
      list(value[[1]]),
      map2(titles[-1], value[-1], ~ list(title = .x, content = .y))
    )

    if (value[[1]] == "") {
      name <- name[-1]
      value <- value[-1]
    }
  }

  map2(name, value, rd_section)
}

#' Load package code
#'
#' @description
#' roxygen2 is a dynamic documentation system, which means it works with the
#' objects inside your package, not just the source code used to create them.
#' These functions offer various ways of loading your package to suit various
#' constraints:
#'
#' * `load_pkgload()` uses `pkgload::load_all()` to simulate package loading
#'   as closely as we know how. It offers high fidelity handling of code that
#'   uses S4, but requires that the package be compiled.
#'
#' * `load_source()` simulates package loading by attaching packages listed in
#'   `Depends` and `Imports`, then sources all files in the `R/` directory.
#'   This was the default strategy used in roxygen2 6.0.0 and earlier;
#'   it's primary advantage is that it does not need compilation.
#'
#' * `load_installed()` uses the installed version of the package. Use this
#'   strategy if you have installed a development version of the package
#'   already. This is the highest fidelity strategy, but requires work
#'   outside of roxygen2.
#'
#' You can change the default strategy for your function with roxygen2 `load`
#' option. Override the default off `pkgload` to use the `source` or
#' `installed` strategies:
#'
#' ```
#' Roxygen: list(load = "source")
#' ```
#' @name load
#' @param path Path to source package
NULL

#' @rdname load
#' @export
load_pkgload <- function(path) {
  pkgload::load_all(path, helpers = FALSE, attach_testthat = FALSE)$env
}

#' @rdname load
#' @export
load_installed <- function(path) {
  package <- desc::desc_get_field("Package", file = path)
  asNamespace(package)
}

#' @rdname load
#' @export
load_source <- function(path) {
  # Create environment
  env <- new.env(parent = globalenv())
  methods::setPackageName("roxygen_devtest", env)

  # Attach dependencies
  deps <- desc::desc_get_deps(path)
  pkgs <- deps$package[
    deps$type %in% c("Depends", "Imports") & deps$package != "R"
  ]
  lapply(pkgs, require, character.only = TRUE)

  # Source files
  lapply(package_files(path), sys_source, envir = env)

  env
}

sys_source <- function(file, envir = baseenv()) {
  exprs <- parse(text = read_lines(file))
  for (expr in exprs) {
    eval(expr, envir = envir)
  }
  invisible()
}

# Helpers -----------------------------------------------------------------

find_load_strategy <- function(x, option = roxy_meta_get("load", "pkgload")) {
  if (is.function(x)) {
    return(x)
  }

  if (is.null(x)) {
    x <- option
    if (!is.character(x) || length(x) != 1) {
      cli::cli_abort("roxygen2 {.code load} option must be a string")
    }
  } else {
    if (!is.character(x) || length(x) != 1) {
      cli::cli_abort("{.code load_code} must be a string or function")
    }
  }

  switch(x,
    pkgload = load_pkgload,
    source = load_source,
    installed = load_installed,
    cli::cli_abort("Unknown value of {.code load} option")
  )
}

test_that("load_installed retrieves installed package", {
  skip_if_not(file.exists(test_path("../../DESCRIPTION")))
  env <- load_installed(test_path("../.."))
  expect_identical(env, asNamespace("roxygen2"))
})

test_that("can load simple package with load_pkgload()", {
  suppressMessages(env <- load_pkgload(test_path("testRbuildignore")))
  expect_equal(env_get(env, "a"), 1)
})

test_that("can load simple package with load_source()", {
  env <- load_source(test_path("testRbuildignore"))
  expect_equal(env_get(env, "a"), 1)
})

# find_load_strategy ------------------------------------------------------

test_that("function returned as is", {
  expect_equal(find_load_strategy(load_installed), load_installed)
})

test_that("look up string", {
  expect_equal(find_load_strategy("installed"), load_installed)
  expect_equal(find_load_strategy("pkgload"), load_pkgload)
  expect_equal(find_load_strategy("source"), load_source)
  expect_snapshot(find_load_strategy("blahblahb"), error = TRUE)
})

test_that("NULL uses option", {
  expect_equal(find_load_strategy(NULL, "installed"), load_installed)
})

test_that("informative errors for bad inputs", {
  expect_snapshot(
    {
      find_load_strategy(1)
      find_load_strategy(NULL, list())
    },
    error = TRUE
  )
})

#' Determine if a function is an S3 generic or S3 method
#'
#' @description
#' `is_s3_generic` compares name to `.knownS3Generics` and
#' `.S3PrimitiveGenerics`, then looks at the function body to see if it
#' calls [UseMethod()].
#'
#' `is_s3_method` builds names of all possible generics for that function
#' and then checks if any of them actually is a generic.
#'
#' @param name Name of function.
#' @param env Base environment in which to look for function definition.
#' @keywords internal
#' @export
is_s3_generic <- function(name, env = parent.frame()) {
  if (name == "") {
    return(FALSE)
  }
  if (!exists(name, envir = env, mode = "function")) {
    return(FALSE)
  }

  f <- get(name, envir = env, mode = "function")
  if (inherits(f, "groupGenericFunction")) {
    return(TRUE)
  }

  ns_name <- tryCatch(getNamespaceName(environment(f)), error = function(e) "")
  if (identical(unname(.knownS3Generics[name]), ns_name)) {
    return(TRUE)
  }

  if (is.primitive(f)) {
    known_generics <- c(
      names(.knownS3Generics),
      internal_f("tools", ".get_internal_S3_generics")()
    )
    return(name %in% known_generics)
  }

  calls_use_method(body(f))
}

calls_use_method <- function(x) {
  # Base cases
  if (missing(x)) {
    return(FALSE)
  }
  if (!is.call(x)) {
    return(FALSE)
  }

  if (identical(x[[1]], quote(UseMethod))) {
    return(TRUE)
  }
  if (length(x) == 1) {
    return(FALSE)
  }
  # Recursive case: arguments to call
  for (arg in as.list(x[-1])) {
    if (calls_use_method(arg)) return(TRUE)
  }

  FALSE
}

#' @rdname is_s3_generic
#' @export
is_s3_method <- function(name, env = parent.frame()) {
  !is.null(find_generic(name, env))
}

is.s3method <- function(x) inherits(x, "s3method")
is.s3generic <- function(x) inherits(x, "s3generic")
is.s3 <- function(x) inherits(x, c("s3method", "s3generic"))

find_generic <- function(name, env = parent.frame()) {
  # If it's an S4 generic, it's definitely not an S3 method
  if (methods::isGeneric(name, where = env)) {
    return(NULL)
  }

  # special case all.equal() methods to avoid treating as all() methods (#1587)
  if (startsWith(name, "all.equal.") && is_s3_generic("all.equal", env)) {
    method <- substr(name, nchar("all.equal.") + 1, nchar(name))
    return(c("all.equal", method))
  }

  pieces <- strsplit(name, ".", fixed = TRUE)[[1]]
  n <- length(pieces)

  # No . in name, so can't be method
  if (n == 1) {
    return(NULL)
  }

  for (i in seq_len(n - 1)) {
    generic <- paste0(pieces[seq_len(i)], collapse = ".")
    class <- paste0(pieces[(i + 1):n], collapse = ".")

    if (is_s3_generic(generic, env)) return(c(generic, class))
  }
  NULL
}

s3_method <- function(f, method) {
  check_function(f)
  check_character(method)

  class(f) <- c("s3method", "function")
  attr(f, "s3method") <- method

  f
}

s3_method_info <- function(x) {
  stopifnot(is.s3(x))
  attr(x, "s3method")
}

test_that("primitive generics detected", {
  expect_true(is_s3_generic("["))
  expect_true(is_s3_method("[.data.frame"))

  expect_true(is_s3_generic("mean"))
  expect_true(is_s3_method("mean.default"))

  expect_true(is_s3_generic("c"))
  expect_true(is_s3_method("c.Date"))
})

test_that("non-functions are not generics", {
  a <- TRUE
  b <- NULL
  c <- data.frame()

  expect_false(is_s3_generic("a"))
  expect_false(is_s3_generic("b"))
  expect_false(is_s3_generic("c"))
})

test_that("user defined generics & methods detected", {
  my_method <- function(x) UseMethod("mymethod")
  my_method.character <- function(x) x

  expect_true(is_s3_generic("my_method"))
  expect_true(is_s3_method("my_method.character"))
})

test_that("methods for group generics detected", {
  Ops.myclass <- function(x) x

  expect_false(is_s3_generic("Ops.myclass"))
  expect_true(is_s3_method("Ops.myclass"))
})

test_that("user defined generics detected even if use non-standard", {
  my_method <- function(x) {
    x <- 1
    if (x > 2) UseMethod("mymethod")
  }

  expect_true(is_s3_generic("my_method"))
})

test_that("user defined functions override primitives", {
  c <- function(x) x + 1
  c.test <- function(x) x + 3

  expect_false(is_s3_generic("c"))
  expect_false(is_s3_method("c"))
})

test_that("@method overrides auto-detection", {
  out <- parse_text("
    #' @export
    #' @method all.equal data.frame
    all.equal.data.frame <- function(...) 1
  ")[[1]]

  expect_equal(s3_method_info(out$object$value), c("all.equal", "data.frame"))
})

#' @export
roxy_tag_parse.roxy_tag_prop <- function(x) {
  x <- tag_two_part(x, "a property name", "a description", multiline = "always")
  if (is.null(x)) {
    return()
  }

  # Optionally specify a class
  if (grepl("@", x$val$name, fixed = TRUE)) {
    pieces <- strsplit(x$val$name, "@", fixed = TRUE)[[1]]
    if (length(pieces) != 2 || pieces[[1]] == "" || pieces[[2]] == "") {
      warn_roxy_tag(x, "must have form class@prop")
      return()
    }
    x$val$class <- pieces[[1]]
    x$val$name <- pieces[[2]]
  }

  x
}

#' @export
roxy_tag_rd.roxy_tag_prop <- function(x, base_path, env) {
  rd_section(
    x$tag,
    data.frame(
      class = x$val$class %||% NA_character_,
      name = x$val$name,
      description = x$val$description
    )
  )
}

#' @export
merge.rd_section_prop <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  rd_section(x$type, rbind(x$value, y$value))
}

#' @export
format.rd_section_prop <- function(x, ...) {
  props <- x$value
  classes <- unique(props$class)

  if (identical(classes, NA_character_)) {
    return(rd_section_description(
      "Additional properties",
      paste0("@", props$name),
      props$description
    ))
  }

  sections <- map_chr(classes, function(cls) {
    rows <- props[props$class == cls, ]
    rd_subsection_description(cls, paste0("@", rows$name), rows$description)
  })

  paste0(
    "\\section{Additional properties}{\n\n",
    paste0(sections, collapse = "\n"),
    "}\n"
  )
}

#' Parse a package, file, or inline code
#'
#' `parse_package()`, `parse_file()`, and `parse_text()` allow you to use
#' roxygen's parsing code to parse the roxygen blocks from a package, file, or
#' character vector of code. `env_package()` and `env_file()` provide
#' defaults that generate a temporary environment making it possible to
#' associate each block with the corresponding live object.
#'
#' @param path,file,text Either specify a `path` to the root directory of
#'   a package, an R `file`, or a character vector `text`.
#' @param env An environment environment containing the result of evaluating
#'   the input code. The defaults will do this for you in a test environment:
#'   for real code you'll need to generate the environment yourself.
#'
#'    You can also set to `NULL` if you only want to get the tokenized code
#'    blocks only. This suppresses evaluation of `@eval` tags, and will not
#'    find the code object associated with each block.
#' @return A list of roxy_block objects
#' @export
#' @keywords internal
parse_package <- function(path = ".", env = env_package(path)) {

  files <- package_files(path)
  list_of_blocks <- lapply(files, tokenize_file)

  blocks <- purrr::flatten(list_of_blocks)

  if (!is.null(env)) {
    blocks <- lapply(blocks, block_set_env, env = env)
  }

  blocks <- order_blocks(blocks)
  blocks
}

#' @export
#' @rdname parse_package
parse_file <- function(file,
                       env = env_file(file),
                       srcref_path = NULL) {

  blocks <- tokenize_file(file, srcref_path = srcref_path)

  if (!is.null(env)) {
    blocks <- lapply(blocks, block_set_env, env = env)
  }

  blocks <- order_blocks(blocks)
  blocks
}

#' @export
#' @rdname parse_package
parse_text <- function(text, env = env_file(file)) {

  file <- tempfile()
  write_lines(text, file)
  on.exit(unlink(file))

  blocks <- parse_file(file, env = env, srcref_path = "<text>")
  blocks
}

#' @export
#' @rdname parse_package
env_file <- function(file) {
  env <- new.env(parent = parent.env(globalenv()))
  methods::setPackageName("roxygen_devtest", env)

  sys.source(file, envir = env)
  env
}

#' @export
#' @rdname parse_package
env_package <- function(path) {
  load_pkgload(path)
}

# helpers -----------------------------------------------------------------

order_blocks <- function(blocks) {
  block_order <- function(x) {
    if (block_has_tags(x, "order")) {
      ord <- block_get_tag_value(x, "order")
      as.double(ord)
    } else {
      Inf
    }
  }

  ord <- vapply(blocks, block_order, double(1))
  blocks[order(ord)]
}

#' @export
roxy_tag_parse.roxy_tag_order <- function(x) {
  tag_value(x)
}

#' @export
roxy_tag_parse.roxy_tag_concept <- function(x) tag_value(x)
#' @export
roxy_tag_rd.roxy_tag_concept <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_concept <- function(x, ...) {
  format_rd(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_docType <- function(x) tag_name(x)
#' @export
roxy_tag_rd.roxy_tag_docType <- function(x, base_path, env) {
  rd_section("docType", x$val)
}
#' @export
format.rd_section_docType <- function(x, ...) {
  format_first(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_encoding <- function(x) tag_value(x)
#' @export
roxy_tag_rd.roxy_tag_encoding <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_encoding <- function(x, ...) {
  format_first(x, ...)
}

#' @export
roxy_tag_parse.roxy_tag_keywords <- function(x) tag_words(x, min = 1)
#' @export
roxy_tag_rd.roxy_tag_keywords <- function(x, base_path, env) {
  rd_section("keyword", x$val)
}
#' @export
format.rd_section_keyword <- function(x, ...) {
  format_rd(x, ...)
}

#' `roxy_tag` S3 constructor
#'
#' `roxy_tag()` is the constructor for tag objects.
#' `roxy_tag_warning()` is superseded by `warn_roxy_tag()`; use to generate a
#' warning that includes the location of the tag.
#'
#' @section Methods:
#' Define a method for `roxy_tag_parse` to support new tags. See [tag_parsers]
#' for more details.
#'
#' @keywords internal
#' @export
#' @param tag Tag name. Arguments starting with `.` are reserved for internal
#'   usage.
#' @param raw Raw tag value, a string.
#' @param val Parsed tag value, typically a character vector, but sometimes
#'   a list. Usually filled in by `tag_parsers`
#' @param file,line Location of the tag
roxy_tag <- function(tag, raw, val = NULL, file = NA_character_, line = NA_character_) {
  structure(
    list(
      file = file,
      line = line,
      raw = raw,
      tag = tag,
      val = val
    ),
    class = c(paste0("roxy_tag_", tag), "roxy_tag")
  )
}

roxy_generated_tag <- function(block, tag, val) {
  roxy_tag(
    tag = tag,
    raw = NULL,
    val = val,
    file = block$file,
    line = block$line
  )
}

roxy_test_tag <- function(raw = "", val = NULL) {
  roxy_tag("test", raw = raw, val = val, file = "test.R", line = 1)
}

#' @rdname roxy_tag
#' @param x A tag
#' @export
roxy_tag_parse <- function(x) {
  UseMethod("roxy_tag_parse")
}

#' @export
roxy_tag_parse.default <- function(x) {
  warn_roxy_tag(x, "is not a known tag")
  NULL
}

is.roxy_tag <- function(x) inherits(x, "roxy_tag")

#' @export
format.roxy_tag <- function(x, ..., file = NULL) {
  if (identical(x$file, file)) {
    file <- "line"
  } else if (is.na(x$file)) {
    file <- "????"
  } else {
    file <- basename(x$file)
  }
  line <- if (is.na(x$line)) "???" else format(x$line, width = 3)

  loc <- paste0("[", file, ":", line, "]")

  if (!is.null(x$raw)) {
    lines <- strsplit(x$raw, "\n")[[1]]
    ellipsis <- FALSE

    if (length(lines) > 1) {
      raw <- lines[[1]]
      ellipsis <- TRUE
    } else {
      raw <- x$raw
    }

    if (nchar(raw) > 50) {
      raw <- substr(raw, 1, 47)
      ellipsis <- TRUE
    }

    raw <- paste0(raw, if (ellipsis) "...")
  } else {
    raw <- "<generated>"
  }

  parsed <- if (is.null(x$val)) "{unparsed}" else "{parsed}"

  paste0(loc, " @", x$tag, " '", raw, "' ", parsed)
}

#' @export
print.roxy_tag <- function(x, ...) {
  cat_line(format(x, ...))
}

#' @export
#' @rdname roxy_tag
roxy_tag_warning <- function(x, ...) {
  # Should no longer be used internally

  message <- paste0(
    if (!is.na(x$file)) paste0("[", x$file, ":", x$line, "] "),
    ...,
    collapse = " "
  )

  warning(message, call. = FALSE, immediate. = TRUE)
  NULL
}

#' @export
#' @rdname roxy_tag
warn_roxy_tag <- function(tag, message, ...) {
  message[[1]] <- paste0(
    link_to(tag$file, tag$line), " @", tag$tag, " ",
    if (is.null(tag$raw)) ("(automatically generated) "),
    message[[1]]
  )
  cli::cli_warn(message, ..., .envir = parent.frame())
}

link_to <- function(file, line) {
  paste0("[", cli::style_hyperlink(
    paste0(basename(file), ":", line),
    paste0("file://", file),
    params = c(line = line, col = 1)
  ), "]")
}

# @R6method tag parsing ----------------------------------------------------

test_that("@R6method tag parser validates input", {
  tag <- roxy_tag("R6method", "Foo$bar")
  result <- roxy_tag_parse(tag)
  expect_equal(result$val, list(class = "Foo", method = "bar"))

  expect_snapshot({
    . <- roxy_tag_parse(roxy_tag("R6method", ""))
    . <- roxy_tag_parse(roxy_tag("R6method", "nomethod"))
    . <- roxy_tag_parse(roxy_tag("R6method", "Foo$bar\nextra"))
  })
})

# $set() ------------------------------------------------------------------

test_that("$set() infers @R6method automatically", {
  docs <- r6_doc(
    "
    #' Title
    C <- R6::R6Class('C', cloneable = FALSE)

    #' @description Method 2.
    #' @param x A number.
    C$set('public', 'meth2', function(x) { })
  "
  )
  meth <- docs$methods$self[[1]]
  expect_equal(meth$name, "meth2")
  expect_equal(meth$description, "Method 2.")
  expect_equal(meth$params, list(list(name = "x", description = "A number.")))
})

# explicit @R6method -------------------------------------------------------

test_that("explicit @R6method works on NULL block", {
  docs <- r6_doc(
    "
    #' Title
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        meth2 = function(x) {}
      )
    )
    #' @R6method C$meth2
    #' @description Method 2.
    #' @param x A number.
    NULL
  "
  )
  meth <- docs$methods$self[[1]]
  expect_equal(meth$name, "meth2")
  expect_equal(meth$description, "Method 2.")
  expect_equal(meth$params, list(list(name = "x", description = "A number.")))
})

test_that("@R6method warns on unknown class", {
  text <- "
      #' @R6method NoSuchClass$meth
      #' @description A method.
      NULL
    "
  expect_snapshot(roc_proc_text(rd_roclet(), text))
})

#' Access metadata about built-in or available tags
#'
#' @export
#' @family extending
#' @param built_in Logical. Whether to restrict the result to built-in tags
#' (`TRUE`) or to extend it to available tags (`FALSE`).
tags_list <- function(built_in = TRUE) {
  if (isTRUE(built_in)) {
    methods <- attr(methods('roxy_tag_parse'), "info")
    methods <- methods[methods$from == "roxygen2", ]
    methods <- rownames(methods)[-1]
  } else {
    # Needed since the info attribute doesn't seem to exist
    # during R CMD check
    methods <- as.character(methods('roxy_tag_parse'))[-1]
  }
  sort(sub('.*\\.roxy_tag_', '', methods))
}

#' @export
#' @rdname tags_list
tags_metadata <- function() {
  check_installed("yaml")

  meta <- yaml::read_yaml(yaml_path())
  data.frame(
    tag = map_chr(meta, \(x) x[["name"]]),
    description = map_chr(meta, \(x) x[["description"]]),
    # \n not useful outside of RStudio
    template = sub("\n", "", map_chr(meta, \(x) x[["template"]] %||% "")),
    vignette = map_chr(meta, \(x) x[["vignette"]] %||% NA_character_),
    recommend = map_lgl(meta, \(x) x[["recommend"]] %||% FALSE)
  )
}

yaml_path <- function() {
  system.file("roxygen2-tags.yml", package = "roxygen2")
}

tags_rd <- function(type, vignette = type) {
  tags <- tags_metadata()
  tags <- tags[tags$vignette == type & !is.na(tags$vignette), ]

  c(
    paste0("@name tags-", type),
    paste0("@aliases ", "@", tags$tag),
    "@description",
    paste0("Learn the full details in `vignette('", vignette, "')`."),
    "",
    if (any(tags$recommend)) {
      c(
        "Key tags:",
        tags_rd_desc(tags[tags$recommend, ])
      )
    },
    if (any(!tags$recommend)) {
      c(
        "Other less frequently used tags:",
        tags_rd_desc(tags[!tags$recommend, ])
      )
    },
    "@usage",
    paste0("#' @", tags$tag, tags$template)
  )
}
tags_rd_desc <- function(tags, section) {
  paste0("* `@", tags$tag, tags$template, "`: ", tags$description)
}

#' Tags for documenting functions
#'
#' @eval tags_rd("rd-functions")
#' @family documentation tags
NULL

#' Tags for documenting datasets
#'
#' @eval tags_rd("rd-datasets")
#' @family documentation tags
NULL

#' Tags for documenting S3
#'
#' @eval tags_rd("rd-S3")
#' @family documentation tags
NULL

#' Tags for documenting S4
#'
#' @eval tags_rd("rd-S4")
#' @family documentation tags
NULL

#' Tags for documenting S7
#'
#' @eval tags_rd("rd-S7")
#' @family documentation tags
NULL

#' Tags for documenting R6
#'
#' @eval tags_rd("rd-R6")
#' @family documentation tags
NULL

#' Tags that help you reuse documentation
#'
#' @eval tags_rd("reuse")
#' @family documentation tags
NULL

#' Tags for managing the `NAMESPACE`
#'
#' @eval tags_rd("namespace")
NULL

#' Tags related to markdown support
#'
#' @eval tags_rd("rd-formatting")
NULL

#' Tags for indexing and cross-references
#'
#' @eval tags_rd("index-crossref")
#' @family documentation tags
NULL

#' @export
roxy_tag_parse.roxy_tag_R6method <- function(x) {
  raw <- trimws(x$raw)
  if (raw == "") {
    warn_roxy_tag(x, "requires a value like {.code Class$method}")
    return(NULL)
  }
  raw <- check_multiline(x, raw, multiline = "never")

  pieces <- strsplit(raw, "\\$")[[1]]
  if (length(pieces) != 2 || pieces[1] == "" || pieces[2] == "") {
    warn_roxy_tag(x, "must be of the form {.code Class$method}")
    return(NULL)
  }

  x$val <- list(class = pieces[1], method = pieces[2])
  x
}

# Merge external @R6method blocks into their target R6 class blocks -----------

merge_external_r6methods <- function(blocks) {
  is_external <- map_lgl(blocks, function(b) {
    block_has_tags(b, "R6method") && !inherits(b, "roxy_block_r6class")
  })

  external <- blocks[is_external]
  if (length(external) == 0) {
    return(blocks)
  }

  blocks <- blocks[!is_external]

  # Build index of R6 class blocks by class name
  r6_index <- list()
  for (i in seq_along(blocks)) {
    if (inherits(blocks[[i]], "roxy_block_r6class")) {
      classname <- blocks[[i]]$object$value$classname
      if (!is.null(classname)) {
        r6_index[[classname]] <- i
      }
    }
  }

  for (ext_block in external) {
    r6method_tag <- block_get_tag(ext_block, "R6method")
    classname <- r6method_tag$val$class
    methodname <- r6method_tag$val$method

    idx <- r6_index[[classname]]
    if (is.null(idx)) {
      warn_roxy_tag(r6method_tag, "Can't find R6 class {.cls {classname}}")
      next
    }

    target <- blocks[[idx]]
    # Stamp all tags with the method name so r6_extract_methods()
    # can assign them by name; non-method tags will be filtered out
    # by the existing r6_tag_type() logic
    ext_tags <- discard(ext_block$tags, \(t) t$tag == "R6method")
    for (j in seq_along(ext_tags)) {
      ext_tags[[j]]$r6method <- methodname
    }
    target$tags <- c(target$tags, ext_tags)
    blocks[[idx]] <- target
  }

  blocks
}

#' @export
roxy_tag_parse.roxy_tag_field <- function(x) {
  tag_two_part(x, "a field name", "a description", multiline = TRUE)
}
#' @export
roxy_tag_rd.roxy_tag_field <- function(x, base_path, env) {
  value <- setNames(x$val$description, x$val$name)
  rd_section(x$tag, value)
}
#' @export
format.rd_section_field <- function(x, ...) {
  rd_section_description("Fields", names(x$value), x$value)
}

#' @export
roxy_tag_parse.roxy_tag_slot <- function(x) {
  tag_two_part(x, "a slot name", "a description", multiline = TRUE)
}
#' @export
roxy_tag_rd.roxy_tag_slot <- function(x, base_path, env) {
  value <- setNames(x$val$description, x$val$name)
  rd_section(x$tag, value)
}
#' @export
format.rd_section_slot <- function(x, ...) {
  rd_section_description("Slots", names(x$value), x$value)
}

#' @export
roxy_tag_parse.roxy_tag_backref <- function(x) {
  tag_value(x)
}

#' @export
roxy_tag_rd.roxy_tag_backref <- function(x, base_path, env) {
  rd_section("backref", x$val)
}

#' @export
format.rd_section_backref <- function(x, ...) {
  filename <- unique(x$value)
  filename <- file.path(basename(dirname(filename)), basename(filename), fsep = "/")

  lines <- stringi::stri_wrap(
    paste0("Please edit documentation in ", paste(filename, collapse = ", ")),
    initial = "% ",
    prefix = "%   ",
    width = 80,
    whitespace_only = TRUE
  )

  paste0(paste0(lines, collapse = "\n"))
}

#' @export
roxy_tag_parse.roxy_tag_examples <- function(x) {
  tag_examples(x)
}
#' @export
roxy_tag_parse.roxy_tag_examplesIf <- function(x) {
  lines <- unlist(strsplit(x$raw, "\r?\n"))

  condition <- lines[1]
  parse_err <- tryCatch(
    {
      suppressWarnings(parse(text = condition))
      NULL
    },
    error = function(err) err
  )
  if (!is.null(parse_err)) {
    warn_roxy_tag(x, "condition failed to parse", parent = parse_err)
    return(NULL)
  }

  example_lines <- lines[-1]
  if (length(example_lines) == 0 || all(trimws(example_lines) == "")) {
    warn_roxy_tag(x, "requires example code after the condition")
    return(NULL)
  }

  x$raw <- paste(
    c(
      paste0("\\dontshow{if (", condition, ") withAutoprint(\\{ # examplesIf}"),
      lines[-1],
      "\\dontshow{\\}) # examplesIf}"
    ),
    collapse = "\n"
  )

  tag_examples(x)
}
#' @export
roxy_tag_parse.roxy_tag_example <- function(x) {
  x <- tag_value(x, multiline = TRUE)

  nl <- re_count(x$val, "\n")
  if (any(nl) > 0) {
    warn_roxy_tag(
      x,
      c(
        "must be a single line",
        i = "Do you want @examples?"
      )
    )
    return()
  }

  x
}

#' @export
roxy_tag_rd.roxy_tag_examples <- function(x, base_path, env) {
  rd_section("examples", x$val)
}
#' @export
roxy_tag_rd.roxy_tag_examplesIf <- function(x, base_path, env) {
  rd_section("examples", x$val)
}
#' @export
roxy_tag_rd.roxy_tag_example <- function(x, base_path, env) {
  path <- file.path(base_path, x$val)
  if (!file.exists(path)) {
    warn_roxy_tag(x, "{.path {path}} doesn't exist")
    return()
  }

  code <- read_lines(path)
  rd_section("examples", escape_examples(code))
}

#' @export
format.rd_section_examples <- function(x, ...) {
  value <- paste0(x$value, collapse = "\n")
  rd_macro(x$type, value, space = TRUE)
}

#' Escape examples
#'
#' This documentation topic is used primarily for testing and to record
#' our understanding of the `\example{}` escaping rules.
#' See <https://developer.r-project.org/parseRd.pdf> for the details provided
#' by R core.
#'
#' @keywords internal
#' @export
#' @examples
#' # In examples we automatically escape Rd comments (%):
#' 100 %% 30
#' # even if they are in strings
#' "50%"
#'
#' # and \\ and \v inside of strings and symbols
#' "\v" # vertical tab
#' "\\"
#' # but not comments: \l \v
#'
#' # other string escapes are left as is
#' "\""
#' "\n"
#'
#' # Otherwise, backslashes and parentheses are left as is. This
#' # means that you need to escape unbalanced parentheses, which typically only
#' # occur in \dontshow{}:
#' \dontshow{if (FALSE) \{ }
#' print("Hello")
#' \dontshow{ \} }
#'
#' # You also need to escape backslashes in infix operators and comments
#' # (this is generally rare)
#' `%\\%` <- function(x, y) x + y
#' 10 %\\% 20
#' # \\\\ (renders as two backslashes)
escape_examples <- function(x) {
  x <- paste0(x, collapse = "\n")
  rd(escapeExamples(x))
}

# Strip \dontrun{}, \donttest{}, and \dontshow{} from example code.
# Used for R6 method examples displayed in \preformatted{} blocks where
# Rd macros are not interpreted. \dontrun{} and \donttest{} are unwrapped
# (content kept), while \dontshow{} is removed entirely.
strip_rd_example_tags <- function(x) {
  # Process each tag type
  for (tag in c("\\dontrun", "\\donttest")) {
    x <- strip_rd_tag(x, tag, keep_contents = TRUE)
  }
  x <- strip_rd_tag(x, "\\dontshow", keep_contents = FALSE)
  # Clean up any resulting blank lines at start/end
  x <- gsub("^\n+", "", x)
  x <- gsub("\n+$", "", x)

  x
}

strip_rd_tag <- function(x, tag, keep_contents) {
  needle <- paste0(tag, "{")
  repeat {
    start <- regexpr(needle, x, fixed = TRUE)
    if (start == -1L) {
      break
    }

    tag_len <- attr(start, "match.length")
    # findEndOfTag uses 0-based indexing
    open_brace_0 <- start + tag_len - 2L
    end_0 <- findEndOfTag(x, is_code = TRUE, start = open_brace_0)
    if (end_0 == -1L) {
      break
    }

    close_pos <- end_0 + 1L

    inner <- substring(x, start + tag_len, close_pos - 1L)
    if (keep_contents) {
      # Remove leading/trailing whitespace from inner content
      inner <- gsub("^\n", "", inner)
      inner <- gsub("\n$", "", inner)
    } else {
      inner <- ""
    }

    before <- substring(x, 1L, start - 1L)
    after <- substring(x, close_pos + 1L)

    # Clean up surrounding newlines
    before <- gsub("\n$", "", before)
    if (nzchar(inner)) {
      after <- gsub("^\n", "", after)
      x <- paste0(
        before,
        if (nzchar(before)) "\n",
        inner,
        if (nzchar(after)) "\n",
        after
      )
    } else {
      after <- gsub("^\n", "", after)
      x <- paste0(before, if (nzchar(before) && nzchar(after)) "\n", after)
    }
  }

  x
}

roxygen_setup <- function(path = ".", cur_version = NULL) {
  if (!file.exists(file.path(path, "DESCRIPTION"))) {
    cli::cli_abort(
      "{.arg package.dir} ({.path {path}}) does not contain a DESCRIPTION"
    )
  }

  is_first <- first_time(path)
  if (is_first) {
    cli::cli_inform("First time using roxygen2. Upgrading automatically...")
  }

  update_roxygen_version(path, cur_version = cur_version)

  encoding <- desc::desc_get("Encoding", path)[[1]]
  if (!identical(encoding, "UTF-8")) {
    cli::cli_warn(c(
      "roxygen2 requires Encoding: {.val UTF-8}",
      i = "Current encoding is {.val {encoding}}"
    ))
  }

  man_path <- file.path(path, "man")
  dir.create(man_path, recursive = TRUE, showWarnings = FALSE)

  is_first
}

update_roxygen_version <- function(path, cur_version = NULL) {
  cur <- cur_version %||% as.character(utils::packageVersion("roxygen2"))
  prev <- roxygen_version(path)

  if (!is.na(cur) && !is.na(prev) && package_version(cur) < package_version(prev)) {
    cli::cli_warn(c(
      "Installed roxygen2 is older than the version used with this package",
      i = "You have {.str {cur}} but you need {.str {prev}}"
    ))
  } else if (!identical(cur, prev)) {
    if (!is.na(prev) && numeric_version(prev) <= "6.1.99") {
      cli::cli_rule()
      cli::cli_inform(c(
        "Changes in roxygen2 7.0.0:",
        "* `%` is now escaped automatically in Markdown mode.",
        "Please carefully check .Rd files for changes"
      ))
      cli::cli_rule()
    }

    cli::cli_inform("Setting {.var RoxygenNote} to {.str {cur}}")
    desc::desc_set(RoxygenNote = cur, file = path)
  }
}

first_time <- function(path) {
  if (!is.na(roxygen_version(path))) {
    return(FALSE)
  }

  generated <- dir(file.path(path, "man"), full.names = TRUE)
  generated <- generated[!file.info(generated)$isdir]

  namespace <- file.path(path, "NAMESPACE")
  if (file.exists(namespace)) {
    generated <- c(generated, namespace)
  }

  roxy <- map_lgl(generated, made_by_roxygen)
  all(!roxy)
}

roxygen_version <- function(path = ".") {
  stringr::str_trim(desc::desc_get("RoxygenNote", path)[[1]])
}

#' @details
#' See `vignette("roxygen2", package = "roxygen2")` for an overview
#' of the package, `vignette("rd", package = "roxygen2")` for generating
#' documentation, and `vignette("namespace", package = "roxygen2")` for
#' generating the namespace specification.
#'
#' If you have existing Rd files, check out the `Rd2roxygen` package
#' for a convenient way of converting Rd files to roxygen comments.
#'
#' @useDynLib roxygen2, .registration=TRUE
#' @keywords internal
#' @importFrom Rcpp sourceCpp
#' @importFrom R6 R6Class
#' @importFrom purrr map map_int map_chr map_lgl map2
#' @import rlang
"_PACKAGE"

# Returns list of roxy_blocks
tokenize_file <- function(file, srcref_path = NULL) {
  lines <- read_lines(file)

  parsed <- parse(
    text = lines,
    keep.source = TRUE,
    srcfile = srcfilecopy(srcref_path %||% file, lines, isFile = TRUE)
  )
  if (length(parsed) == 0)
    return(list())

  refs <- utils::getSrcref(parsed)

  comment_refs <- comments(refs)
  tokens <- lapply(comment_refs, tokenise_ref)

  has_tokens <- !purrr::map_lgl(tokens, purrr::is_empty)

  blocks <- purrr::pmap(
    list(
      call = as.list(parsed)[has_tokens],
      srcref = refs[has_tokens],
      tokens = tokens[has_tokens]
    ),
    block_create
  )
  purrr::compact(blocks)
}

tokenise_ref <- function(x) {
  tokenise_block(
    as.character(x),
    file = attr(x, "srcfile")$filename,
    offset = x[[1]]
  )
}

# For each src ref, find the comment block preceding it
comments <- function(refs) {
  srcfile <- attr(refs[[1]], "srcfile")

  # first_line, first_byte, last_line, last_byte
  com <- vector("list", length(refs))
  for (i in seq_along(refs)) {
    # Comments begin after last line of last block, and this block is included
    # so that it can be parsed for additional comments
    if (i == 1) {
      first_byte <- 1
      first_line <- 1
    } else {
      first_byte <- refs[[i - 1]][4] + 1
      first_line <- refs[[i - 1]][3]
    }

    last_line <- refs[[i]][3]
    last_byte <- refs[[i]][4]

    lloc <- c(first_line, first_byte, last_line, last_byte)
    com[[i]] <- srcref(srcfile, lloc)
  }

  com
}

#' Parse tags
#'
#' These functions parse the `raw` tag value, convert a string into a richer R
#' object and storing it in `val`, or provide an informative warning and
#' returning `NULL`.
#'
#' @section New tag:
#' To create a new `@mytag` define `roxy_tag_parse.roxy_tag_mytag()`. It should
#' either call one of the functions here, or directly set `x$val`.
#'
#' @param x A [roxy_tag] object to parse
#' @returns A [roxy_tag] object with the `val` field set to the parsed value.
#' @name tag_parsers
#' @family extending
NULL

#' @export
#' @rdname tag_parsers
#' @param multiline Controls how the tag may span multiple lines:
#'   * `"never"` (the default): the tag must be a single line, and spanning
#'     multiple lines generates a warning.
#'   * `"indent"`: the tag may span multiple lines, but continuation lines must
#'     use a hanging indent (i.e. be indented more than the first line). The
#'     first line that is not indented (including a blank line) ends the tag,
#'     and anything after it is ignored, with a warning. Use this for tags where
#'     multiline input is convenient but a flush line almost always signals a
#'     missing tag (e.g., `@importFrom`).
#'   * `"always"`: the tag may span any number of lines and paragraphs. Use this
#'     for tags where multiline content is expected (e.g., `@usage`, `@rawRd`).
#'
#'   For backward compatibility, `FALSE` and `TRUE` are accepted as synonyms for
#'   `"never"` and `"always"` respectively.
tag_value <- function(x, multiline = "never") {
  x$val <- trimws(x$raw)

  if (x$val == "") {
    warn_roxy_tag(x, "requires a value")
    return(NULL)
  }

  x$val <- check_multiline(x, x$val, multiline)

  if (!rdComplete(x$raw, is_code = FALSE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    return(NULL)
  }

  x
}

# Also recorded in tags.yml
inherit_components <- c(
  "params",
  "return",
  "title",
  "description",
  "details",
  "seealso",
  "sections",
  "references",
  "examples",
  "author",
  "source",
  "note",
  "format"
)

#' @export
#' @rdname tag_parsers
tag_inherit <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    NULL
  } else if (!rdComplete(x$raw, is_code = FALSE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    NULL
  } else {
    pieces <- strsplit(trimws(x$raw), "\\s+")[[1]]
    fields <- pieces[-1]

    all <- inherit_components
    if (length(fields) == 0) {
      fields <- all
    } else {
      unknown <- setdiff(fields, all)
      if (length(unknown) > 0) {
        warn_roxy_tag(
          x,
          "attempts to inherit from unknown type {.str {unknown}}"
        )
        fields <- intersect(fields, all)
      }
    }

    x$val <- list(
      source = pieces[1],
      fields = fields
    )

    x
  }
}

#' @export
#' @rdname tag_parsers
tag_name <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    NULL
  } else if (!rdComplete(x$raw, is_code = FALSE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    NULL
  } else {
    n <- re_count(x$raw, "\\s+")
    if (n > 1) {
      warn_roxy_tag(x, "must have only one argument, not {n + 1}")
      NULL
    } else {
      x$val <- trimws(x$raw)
      x
    }
  }
}

#' @export
#' @rdname tag_parsers
#' @param first,second Name of first and second parts of two part tags
#' @param required Is the second part required (TRUE) or can it be blank
#'   (FALSE)?
#' @param markdown Should the second part be parsed as markdown?
tag_two_part <- function(
  x,
  first,
  second,
  required = TRUE,
  markdown = TRUE,
  multiline = "never"
) {
  if (trimws(x$raw) == "") {
    if (!required) {
      warn_roxy_tag(x, "requires {first}")
    } else {
      warn_roxy_tag(x, "requires two parts: {first} and {second}")
    }
    NULL
  } else if (!rdComplete(x$raw, is_code = FALSE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    NULL
  } else {
    val <- check_multiline(x, trimws(x$raw), multiline)
    pieces <- split_two_part(val)

    if (required && pieces[[2]] == "") {
      warn_roxy_tag(x, "requires two parts: {first} and {second}")
      return(NULL)
    }

    pieces[[2]] <- trim_docstring(pieces[[2]])
    if (markdown) {
      pieces[[2]] <- markdown_if_active(pieces[[2]], x)
    }

    x$val <- list(
      name = pieces[[1]],
      description = pieces[[2]]
    )
    x
  }
}

# Split a string into two parts: a name and a description.
# Handles backtick-quoted names that may contain spaces (e.g. `arg 1`).
split_two_part <- function(x) {
  if (grepl("^`", x)) {
    match <- regexpr("^`[^`]*`", x)
    if (match == -1L || attr(match, "match.length") == -1L) {
      # No closing backtick; fall back to space splitting
      re_split_half(x, "[[:space:]]+")
    } else {
      end <- attr(match, "match.length")
      # Strip backticks so name matches names(formals(fn))
      name <- substr(x, 2, end - 1)
      rest <- trimws(substr(x, end + 1, nchar(x)))
      c(name, rest)
    }
  } else {
    re_split_half(x, "[[:space:]]+")
  }
}

#' @export
#' @rdname tag_parsers
tag_name_description <- function(x) {
  tag_two_part(x, "a name", "a description")
}

#' @export
#' @rdname tag_parsers
#' @param min,max Minimum and maximum number of words
tag_words <- function(x, min = 0, max = Inf, multiline = "never") {
  val <- trimws(x$raw)

  val <- check_multiline(x, val, multiline)

  if (!rdComplete(val, is_code = FALSE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    return(NULL)
  }

  words <- if (nzchar(val)) strsplit(val, "\\s+")[[1]] else ""
  if (length(words) < min) {
    warn_roxy_tag(x, "must have at least {min} word{?s}, not {length(words)}")
    NULL
  } else if (length(words) > max) {
    warn_roxy_tag(x, "must have at most {max} word{?s}, not {length(words)}")
    NULL
  } else {
    x$val <- words
    x
  }
}

#' @export
#' @rdname tag_parsers
tag_words_line <- function(x) {
  lifecycle::deprecate_warn("8.0.0", "tag_words_line()", "tag_words()")
  tag_words(x)
}

# Normalises the `multiline` argument to one of "never", "indent", or "always",
# silently translating the legacy `FALSE`/`TRUE` values for backward
# compatibility.
as_multiline <- function(multiline, error_call = caller_env()) {
  if (isTRUE(multiline)) {
    return("always")
  }
  if (isFALSE(multiline)) {
    return("never")
  }

  arg_match0(multiline, c("never", "indent", "always"), error_call = error_call)
}

# Applies the multiline policy for a tag's value, warning when it is violated
# and returning the value to use (possibly truncated to its hanging-indented
# continuation). See the `multiline` parameter of `tag_value()` for the meaning
# of each mode.
check_multiline <- function(x, val, multiline) {
  multiline <- as_multiline(multiline)

  if (multiline == "always") {
    return(val)
  }

  if (multiline == "indent") {
    return(check_indent(x, val))
  }

  n_lines <- re_count(val, "\n")
  if (n_lines >= 1) {
    first_line <- re_split_half(val, "\n")[[1]]
    warn_roxy_tag(
      x,
      c(
        "must be only 1 line long, not {n_lines + 1}",
        i = "The first line is {.str {first_line}}"
      )
    )
  }

  val
}

# Keeps the first line of `val` plus any immediately following lines that use a
# hanging indent (indented more than the first line). The first flush or blank
# line ends the tag; anything after it is dropped with a warning, since a flush
# line usually signals a forgotten tag (e.g. a missing `@examples`).
check_indent <- function(x, val) {
  lines <- strsplit(val, "\n", fixed = TRUE)[[1]]
  if (length(lines) <= 1) {
    return(val)
  }

  indent <- leadingSpaces(lines)
  continues <- nzchar(trimws(lines[-1])) & indent[-1] > indent[[1]]

  ends_at <- if (all(continues)) length(lines) else which(!continues)[[1]]
  if (ends_at < length(lines)) {
    warn_roxy_tag(
      x,
      c(
        "must use a hanging indent to span multiple lines",
        i = "Continuation lines must be indented; did you forget a tag like {.code @examples}?"
      )
    )
  }

  paste(lines[seq_len(ends_at)], collapse = "\n")
}

#' @export
#' @rdname tag_parsers
tag_toggle <- function(x) {
  x$val <- trimws(x$raw)

  if (x$val != "") {
    warn_roxy_tag(x, "must not be followed by any text")
    NULL
  } else {
    x
  }
}

#' @export
#' @rdname tag_parsers
tag_code <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    NULL
  } else {
    tryCatch(
      {
        x$val <- parse(text = x$raw)
        x
      },
      error = function(e) {
        warn_roxy_tag(x, "failed to parse", parent = e)
        NULL
      }
    )
  }
}

# Examples need special parsing because escaping rules are different
#' @export
#' @rdname tag_parsers
tag_examples <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    return(NULL)
  }

  x$val <- escape_examples(gsub("^\n", "", x$raw))
  if (!rdComplete(x$val, is_code = TRUE)) {
    warn_roxy_tag(x, "has mismatched braces or quotes")
    NULL
  } else {
    x
  }
}

#' @export
#' @rdname tag_parsers
tag_markdown <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    NULL
  } else {
    x$val <- markdown_if_active(x$raw, x)
    x
  }
}

#' @export
#' @rdname tag_parsers
tag_markdown_with_sections <- function(x) {
  if (trimws(x$raw) == "") {
    warn_roxy_tag(x, "requires a value")
    return(NULL)
  }

  x$val <- markdown_if_active(x$raw, x, sections = TRUE)
  x
}

markdown_if_active <- function(text, tag, sections = FALSE) {
  if (markdown_on()) {
    out <- markdown(text, tag, sections)

    for (i in seq_along(out)) {
      if (sections && !rdComplete(out[[i]], is_code = FALSE)) {
        warn_roxy_tag(tag, "has mismatched braces or quotes")
        out[[i]] <- ""
      } else {
        out[[i]] <- trimws(out[[i]])
      }
    }
    out
  } else {
    if (!rdComplete(text, is_code = FALSE)) {
      warn_roxy_tag(tag, "has mismatched braces or quotes")
      ""
    } else {
      trimws(text)
    }
  }
}

test_that("tags containing only whitespace generate warning", {
  expect_snapshot({
    tag <- roxy_test_tag(" ")
    expect_parse_failure(tag_value(tag))
    expect_parse_failure(tag_inherit(tag))
    expect_parse_failure(tag_name(tag))
    expect_parse_failure(tag_name_description(tag))
    expect_parse_failure(tag_code(tag))
    expect_parse_failure(tag_examples(tag))
    expect_parse_failure(tag_markdown(tag))
    expect_parse_failure(tag_markdown_with_sections(tag))
  })
})

test_that("tags check for mismatched parents gives useful warnings", {
  expect_snapshot({
    tag <- roxy_test_tag("a {")
    expect_parse_failure(tag_value(tag))
    expect_parse_failure(tag_inherit(tag))
    expect_parse_failure(tag_name(tag))
    expect_parse_failure(tag_two_part(tag))
    expect_parse_failure(tag_words(tag))
    expect_parse_failure(tag_examples(tag))

    "markdown tags return empty values"
    tag_markdown(tag)
    tag_markdown_with_sections(tag)
  })

  local_markdown()
  markdown_on()
  expect_snapshot({
    tag <- roxy_test_tag("# one\ntwo\n# three\nfour {")
    tag_markdown_with_sections(tag)
  })
})

test_that("tag_inhert checks for valid inherits", {
  expect_snapshot({
    tag <- roxy_test_tag("foo params section")
    . <- tag_inherit(tag)
  })
})

test_that("tag_name() checks for valid names", {
  expect_snapshot({
    tag <- roxy_test_tag("a b c")
    expect_parse_failure(tag_name(tag))
  })
})

test_that("tag_two_part() gives useful warnings", {
  local_markdown()
  expect_snapshot({
    tag <- roxy_test_tag("")
    expect_parse_failure(tag_two_part(
      tag,
      "a name",
      "a value",
      required = FALSE
    ))
    expect_parse_failure(tag_two_part(tag, "a name", "a value"))
  })
})

test_that("tag_two_part() falls back to space split for unclosed backtick", {
  tag <- roxy_test_tag("`unclosed description")
  out <- tag_two_part(tag, "a name", "a value", required = FALSE)
  expect_equal(out$val, list(name = "`unclosed", description = "description"))
})

test_that("tag_words() gives useful warnings", {
  expect_snapshot({
    tag <- roxy_test_tag("a b")
    expect_parse_failure(tag_words(tag, 3, 3))
    expect_parse_failure(tag_words(tag, 1, 1))
  })
})

test_that("tag_words() warns on multi-line content and preserves value", {
  expect_snapshot({
    tag <- roxy_test_tag("a\nb")
    out <- tag_words(tag)
  })
  expect_equal(out$val, c("a", "b"))
})

test_that("tag_two_part() warns on multi-line content and preserves value", {
  expect_snapshot({
    tag <- roxy_test_tag("foo bar\nbaz")
    out <- tag_two_part(tag, "a name", "a value")
  })
  expect_equal(out$val, list(name = "foo", description = "bar\nbaz"))

  tag <- roxy_test_tag("foo bar\nbaz")
  out <- expect_silent(tag_two_part(tag, "a name", "a value", multiline = TRUE))
  expect_equal(out$val, list(name = "foo", description = "bar\nbaz"))
})

test_that("multiline accepts string strategies and legacy booleans", {
  expect_equal(as_multiline("never"), "never")
  expect_equal(as_multiline("indent"), "indent")
  expect_equal(as_multiline("always"), "always")
  expect_equal(as_multiline(FALSE), "never")
  expect_equal(as_multiline(TRUE), "always")
  expect_error(as_multiline("nope"))
})

test_that("tag_value() warns on multi-line content and preserves value", {
  expect_snapshot({
    tag <- roxy_test_tag("a\nb")
    out <- tag_value(tag)
  })
  expect_equal(out$val, "a\nb")
})

test_that("tag_words_line() is deprecated", {
  expect_snapshot({
    tag <- roxy_test_tag("a b")
    tag_words_line(tag)
  })
})

test_that("tag_toggle() gives useful warnings", {
  expect_snapshot({
    tag <- roxy_test_tag("x")
    tag_toggle(tag)
  })
})

test_that("tag_code() gives useful warnings", {
  expect_snapshot({
    tag <- roxy_test_tag("a + ")
    tag_code(tag)
  })
})

#' Add link reference definitions for functions to a markdown text
#'
#' We find the `[text][ref]` and the `[ref]` forms. There must be no
#' spaces between the closing and opening bracket in the `[text][ref]`
#' form.
#'
#' These are the link references we add for local links:
#'
#' ```
#' MARKDOWN           LINK TEXT  CODE RD
#' --------           ---------  ---- --
#' [fun()]            fun()       T   \\link[=fun]{fun()}
#' [obj]              obj         F   \\link{obj}
#' [`obj`]            obj         T   \\link{obj}
#' [text][fun()]      text        F   \\link[=fun]{text}
#' [text][obj]        text        F   \\link[=obj]{text}
#' [s4-class]         s4          F   \\link[=s4-class]{s4}
#' ```
#'
#' And for cross-package links:
#'
#' ```
#' MARKDOWN           LINK TEXT  CODE RD
#' --------           ---------  ---- --
#' [fun()]            fun()       T   \\link[pkg:fun]{pkg::fun()}
#' [obj]              obj         F   \\link[pkg:obj]{pkg::obj}
#' [`obj`]            obj         T   \\link[pkg:obj]{pkg::obj}
#' [pkg::fun()]       pkg::fun()  T   \\link[pkg:fun]{pkg::fun()}
#' [pkg::obj]         pkg::obj    F   \\link[pkg:obj]{pkg::obj}
#' [text][fun()]      text        F   \\link[pkg:fun]{text}
#' [text][obj]        text        F   \\link[pkg:obj]{text}
#' [text][pkg::fun()] text        F   \\link[pkg:fun]{text}
#' [text][pkg::obj]   text        F   \\link[pkg:obj]{text}
#' [s4-class]         s4          F   \\link[pkg:s4-class]{s4}
#' [pkg::s4-class]    pkg::s4     F   \\link[pkg:s4-class]{pkg::s4}
#' ```
#'
#' The reference links will always look like `R:ref` for `[ref]` and
#' `[text][ref]`. We add in a special `R:` marker to the URL to avoid
#' picking up other links, that were specified via `<url>` or
#' `[text](link)`. In the parsed XML tree these look the same as
#' our `[link]` and `[text][link]` links.
#'
#' In the link references, we need to URL encode the reference,
#' otherwise commonmark does not use it (see issue #518).
#'
#' @param text Input markdown text.
#' @return The input text and all dummy reference link definitions
#'   appended.
#'
#' @noRd
NULL


# Append link references for R functions ---------------------------------------

add_linkrefs_to_md <- function(text) {
  ref_lines <- get_md_linkrefs(text)
  if (length(ref_lines) == 0) {
    return(text)
  }
  ref_text <- paste0(ref_lines, collapse = "\n")
  paste0(text, "\n\n", ref_text, "\n")
}

get_md_linkrefs <- function(text) {
  refs <- regmatches(
    text,
    gregexec(
      paste0(
        "(?x)",
        "(?<=[^\\]\\\\]|^)", # must not be preceded by ] or \
        "\\[([^\\]\\[]+)\\]", # match anything inside of []
        "(?:\\[([^\\]\\[]+)\\])?", # match optional second pair of []
        "(?=[^\\[{]|$)" # must not be followed by [ or {
      ),
      text,
      perl = TRUE
    )
  )[[1]]
  if (length(refs) > 0) {
    refs <- t(refs)
  }

  if (length(refs) == 0) {
    return(character())
  }

  ## For the [fun] form the link text is the same as the destination.
  refs[, 3] <- ifelse(refs[, 3] == "", refs[, 2], refs[, 3])

  refs3encoded <- map_chr(refs[, 3], URLencode)
  paste0("[", refs[, 3], "]: ", "R:", refs3encoded)
}

# Link parsing -----------------------------------------------------------------

parse_link <- function(destination, contents, state) {
  ## Not a [] or [][] type link, remove prefix if it is
  if (!grepl("^R:", destination)) {
    return(NULL)
  }
  destination <- sub("^R:", "", URLdecode(destination))

  ## if contents is a `code tag`, then we need to move this outside
  is_code <- FALSE
  if (length(contents) == 1 && xml_name(contents) == "code") {
    is_code <- TRUE

    contents <- xml_contents(contents)
    destination <- sub("`$", "", sub("^`", "", destination))

    local_bindings(.env = state, in_link_code = TRUE)
  }

  ## If the supplied link text is the same as the reference text,
  ## then we assume that the link text was automatically generated and
  ## it was not specified explicitly. In this case `()` links are
  ## turned to `\\code{}`.
  ## We also assume link text if we see a non-text XML tag in contents.
  has_link_text <- paste(xml_text(contents), collapse = "") != destination ||
    any(xml_name(contents) != "text")

  ## if (is_code) then we'll need \\code
  ## `pkg` is package or NA
  ## `fun` is fun() or topic (fun is with parens)
  ## `topic` is fun or topic (fun is without parens)
  ## `s4` is TRUE if we link to an S4 class (i.e. have -class suffix)
  ## `noclass` is fun with -class removed

  is_code <- is_code || (grepl("[(][)]$", destination) && !has_link_text)

  pkg <- regmatches(destination, regexec("^(.*)::", destination))[[1]][2]
  explicit_pkg <- !is.na(pkg)
  fun <- utils::tail(strsplit(destination, "::", fixed = TRUE)[[1]], 1)
  topic <- sub("[(][)]$", "", fun)
  if (!has_link_text && grepl("-class$", destination)) {
    fun <- regmatches(fun, regexec("^(.*)-class$", fun))[[1]][2]
  }

  # Standardise links: cross-packagae always get prefix;
  # within package never gets prefix
  if (is.na(pkg)) {
    pkg <- find_package(topic, tag = state$tag)
  } else if (identical(pkg, roxy_meta_get("current_package"))) {
    pkg <- NA_character_
    explicit_pkg <- FALSE
  }
  if (explicit_pkg) {
    check_topic(pkg, topic, state$tag)
  }

  ## To understand this, look at the RD column of the table above
  if (!has_link_text) {
    text <- fun
    if (explicit_pkg) {
      text <- paste0(pkg, "::", text)
    }
    text <- escape(text)
  } else {
    text <- mdxml_link_text(contents, state)
  }
  rd_link(pkg, escape(topic), text, code = is_code)
}

check_topic <- function(pkg, topic, tag = NULL) {
  if (is.na(pkg) || identical(roxy_meta_get("current_package"), pkg)) {
    return(invisible())
  }

  if (!has_topic(topic, pkg)) {
    if (is_installed(pkg)) {
      warn_roxy_tag(tag, "refers to unavailable topic {pkg}::{topic}")
    } else {
      warn_roxy_tag(tag, "refers to un-installed package {pkg}")
    }
  }

  invisible()
}

fun_suffix <- function(name, env) {
  if (is_infix_fun(name)) {
    name
  } else if (!env_has(env, name)) {
    name
  } else {
    obj <- env_get(env, name)
    if (!is.function(obj)) {
      name
    } else {
      paste0(name, "()")
    }
  }
}

rd_link <- function(pkg, topic, text, code = FALSE) {
  if (is.na(pkg) && topic == text) {
    out <- paste0("\\link{", text, "}")
  } else {
    anchor <- if (is.na(pkg)) paste0("=", topic) else paste0(pkg, ":", topic)
    out <- paste0("\\link[", anchor, "]{", text, "}")
  }
  if (code) {
    out <- paste0("\\code{", out, "}")
  }
  out
}

#' Dummy page to test roxygen's markdown formatting
#'
#' Links are very tricky, so I'll put in some links here:
#' Link to a function: [roxygenize()].
#' Link to an object: [roxygenize] (we just treat it like an object here.
#'
#' Link to another package, function: [desc::desc()].
#' Link to another package, non-function: [desc::desc].
#'
#' Link with link text: [this great function][roxygenize()],
#' [`roxygenize`][roxygenize()], or [that great function][roxygenize].
#'
#' In another package: [and this one][desc::desc].
#'
#' This is a table:
#'
#' | __foo__ | __bar__ |
#' | :-- | --: |
#' | 1   | 2   |
#' | 100 | 200 |
#'
#' @name markdown-test
#' @keywords internal
NULL

internal_f <- function(p, f) {
  check_string(p)
  check_string(f)

  get(f, envir = asNamespace(p))
}

"%||%" <- function(a, b) {
  if (length(a) > 0) a else b
}

subs <- c(
  # Common special function names
  "[<-" = "-subset-",
  "[" = "-sub-",
  "<-" = "-set-",
  "::" = "-",

  # Infix verbs
  "!" = "-not-",
  "&" = "-and-",
  "|" = "-or-",
  "*" = "-times-",
  "+" = "-plus-",
  "^" = "-pow-",

  # Others
  '"' = "-quote-",
  "#" = "-hash-",
  "$" = "-cash-",
  "%" = "-grapes-",
  "'" = "-single-quote-",
  "(" = "-open-paren-",
  ")" = "-close-paren-",
  ":" = "-colon-",
  ";" = "-semi-colon-",
  "<" = "-less-than-",
  "==" = "-equals-",
  "=" = "-equals-",
  ">" = "-greater-than-",
  "?" = "-help-",
  "@" = "-at-",
  "]" = "-close-brace-",
  "\\" = "-backslash-",
  "/" = "-slash-",
  "`" = "-tick-",
  "{" = "-open-curly-",
  "}" = "-close-",
  "~" = "-twiddle-"
)

nice_name <- function(x) {
  for (i in seq_along(subs)) {
    x <- gsub(names(subs)[[i]], subs[[i]], x, fixed = TRUE)
  }

  # Clean up any remaining
  x <- gsub("[^A-Za-z0-9_.-]+", "-", x)
  x <- gsub("-+", "-", x)
  x <- gsub("^-|-$", "", x)
  x <- gsub("^\\.", "dot-", x)
  x
}

write_if_different <- function(path, contents, command = NULL, check = TRUE) {
  if (!file.exists(dirname(path))) {
    dir.create(dirname(path), showWarnings = FALSE)
  }

  name <- basename(path)
  if (check && !made_by_roxygen(path)) {
    cli::cli_inform(c(
      x = "Skipping {.href [{name}](file://{path})}",
      i = "It already exists and was not generated by {.pkg roxygen2}."
    ))
    return(FALSE)
  }

  line_ending <- detect_line_ending(path)
  contents <- paste0(paste0(contents, collapse = line_ending), line_ending)
  contents <- enc2utf8(gsub("\r?\n", line_ending, contents))
  if (same_contents(path, contents)) {
    # Touch so mtime reflects last run, even though file wasn't changed
    Sys.setFileTime(path, Sys.time())
    return(FALSE)
  }

  if (!grepl("^[a-zA-Z][a-zA-Z0-9_.-]*$", name)) {
    cli::cli_inform(c(
      x = "Skipping {.path {name}}",
      i = "Invalid file name"
    ))
    FALSE
  } else {
    if (!is.null(command)) {
      scheme <- "x-r-run"
      url <- paste0(scheme, ":", command)
      name <- cli::style_hyperlink(name, url)
    }
    cli::cli_inform("Writing {.path {name}}")

    writeBin(charToRaw(contents), path)
    TRUE
  }
}

same_contents <- function(path, contents) {
  if (length(contents) != 1) {
    cli::cli_abort("{.arg contents} must be a single string.", .internal = TRUE)
  }
  if (!file.exists(path)) {
    return(FALSE)
  }

  text_hash <- cli::hash_sha256(contents)

  path <- normalizePath(path, mustWork = TRUE)
  file_hash <- cli::hash_file_sha256(path)

  identical(text_hash, file_hash)
}

compact <- function(x) {
  x[!map_lgl(x, is.null)]
}

invert <- function(x) {
  if (length(x) == 0) {
    return()
  }
  stacked <- utils::stack(x)
  tapply(as.character(stacked$ind), stacked$values, list)
}

is_namespaced <- function(x) {
  tryCatch(
    {
      expr <- parse_expr(x)
      is_call(expr, "::", n = 2)
    },
    error = function(err) FALSE
  )
}

# Collapse the values associated with duplicated keys
collapse <- function(key, value, fun, ...) {
  check_character(key)
  stopifnot(length(key) == length(value))

  dedup <- tapply(value, key, fun, ..., simplify = FALSE)
  # tapply orders alphabetically, so reorder to match original order
  dedup <- dedup[unique(key)]

  list(
    key = names(dedup),
    value = unname(dedup)
  )
}

cat_line <- function(...) {
  cat(paste0(..., "\n", collapse = ""))
}

tag_aliases <- function(f) {
  paste0("@aliases ", paste0("@", names(f()), collapse = " "))
}

pkg_env <- function() {
  env <- new.env(parent = globalenv())
  env$.packageName <- "roxygen2"
  env
}

uuid <- function(nchar = 8) {
  paste(
    sample(c(letters, LETTERS, 0:9), nchar, replace = TRUE),
    collapse = ""
  )
}

paste_c <- function(...) {
  paste(c(...), collapse = "")
}

# quoting -----------------------------------------------------------------
auto_backtick <- function(x) {
  needs_backtick <- !has_quotes(x) & !is_syntactic(x)
  x[needs_backtick] <- encodeString(x[needs_backtick], quote = "`")
  x
}

auto_quote <- function(x) {
  needs_quotes <- !has_quotes(x) & !is_syntactic(x)
  x[needs_quotes] <- encodeString(x[needs_quotes], quote = '"')
  x
}

re_split_half <- function(x, pattern) {
  m <- regexpr(pattern, x)
  if (m > 0L) {
    left <- substr(x, 1, m - 1)
    right <- substr(x, m + attr(m, "match.length"), nchar(x))
  } else {
    left <- x
    right <- ""
  }
  c(left, right)
}

re_count <- function(x, pattern, fixed = FALSE) {
  m <- gregexpr(pattern, x, fixed = fixed)
  vapply(m, \(i) sum(i > 0L), integer(1))
}

re_replace_all <- function(x, pattern, fun) {
  m <- gregexpr(pattern, x, perl = TRUE)
  regmatches(x, m) <- lapply(regmatches(x, m), \(matches) {
    vapply(matches, fun, character(1))
  })
  x
}

is_syntactic <- function(x) make.names(x) == x
has_quotes <- function(x) grepl(r"[^(`|'|").*\1$]", x)
strip_quotes <- function(x) sub(r"[^(`|'|")(.*)\1$]", r"(\2)", x)

test_that("nice_name leaves ok chars unchanged", {
  expect_equal(nice_name("abc"), "abc")
  expect_equal(nice_name("a_b-c.R"), "a_b-c.R")
})

test_that("nice_name protects against invalid characters", {
  expect_equal(nice_name("a<-"), "a-set")
  expect_equal(nice_name("[.a"), "sub-.a")
})

test_that("is_namespaced works as expected", {
  expect_true(is_namespaced("a::b"))
  expect_false(is_namespaced("b::"))
  expect_false(is_namespaced("'::'"))
})

test_that("write_if_different produces informative messages", {
  dir <- withr::local_tempdir()
  path <- file.path(dir, "test.R")

  write_lines(made_by("#"), path)
  expect_snapshot(write_if_different(path, "a <- 2"))

  write_lines("a <- 1", path)
  expect_snapshot(write_if_different(path, "a <- 2"))

  path <- file.path(dir, "+.R")
  expect_snapshot(write_if_different(path, "a <- 2"))
})

test_that("write_if_different and end of line", {
  cnt_unix <- c("foo\nbar\nbaz", "foobar")
  cnt_win  <- c("foo\r\nbar\r\nbaz", "foobar")
  cnt_mix  <- c("foo\nbar\r\nbaz", "foobar")

  tmp <- tempfile("roxy-", fileext = ".Rd")
  on.exit(unlink(tmp), add = TRUE)

  # do not change unix le
  write_lines(cnt_unix, tmp, line_ending = "\n")
  expect_message(write_if_different(tmp, cnt_unix, check = FALSE), NA)
  expect_message(write_if_different(tmp, cnt_win,  check = FALSE), NA)
  expect_message(write_if_different(tmp, cnt_mix,  check = FALSE), NA)

  # do not change windows le
  write_lines(cnt_win, tmp, line_ending = "\r\n")
  expect_message(write_if_different(tmp, cnt_unix, check = FALSE), NA)
  expect_message(write_if_different(tmp, cnt_win,  check = FALSE), NA)
  expect_message(write_if_different(tmp, cnt_mix,  check = FALSE), NA)

  # change mixed le to windows
  tmp_win <- tempfile("roxy-", fileext = ".Rd")
  on.exit(unlink(tmp_win), add = TRUE)
  write_lines(cnt_win, tmp_win, line_ending = "\r\n")

  # write_lines changes line endings, so we use writeBin to create a file with mixed
  # line endings
  raw_mix <- charToRaw(paste0(paste0(cnt_mix, collapse = "\r\n"), "\r\n"))
  writeBin(raw_mix, tmp)
  expect_message(write_if_different(tmp, cnt_unix, check = FALSE), "Writing ")
  expect_identical(readBin(tmp, "raw", 100), readBin(tmp_win, "raw", 100))

  writeBin(raw_mix, tmp)
  expect_message(write_if_different(tmp, cnt_win, check = FALSE), "Writing ")
  expect_identical(readBin(tmp, "raw", 100), readBin(tmp_win, "raw", 100))

  writeBin(raw_mix, tmp)
  expect_message(write_if_different(tmp, cnt_mix, check = FALSE), "Writing ")
  expect_identical(readBin(tmp, "raw", 100), readBin(tmp_win, "raw", 100))
})

test_that("write_if_different produces correct command hyperlink", {
  testthat::local_reproducible_output(hyperlinks = TRUE)

  dir <- withr::local_tempdir()
  path <- file.path(dir, "test.R")

  write_lines(made_by("#"), path)
  expect_snapshot(write_if_different(path, "a <- 2", command = "rlang::inform('hi')"))
})

#' @export
roxy_tag_parse.roxy_tag_evalRd <- function(x) tag_code(x)
#' @export
roxy_tag_rd.roxy_tag_evalRd <- function(x, base_path, env) {
  rd_section("rawRd", roxy_tag_eval(x, env))
}

#' @export
roxy_tag_parse.roxy_tag_rawRd <- function(x) tag_value(x, multiline = "always")
#' @export
roxy_tag_rd.roxy_tag_rawRd <- function(x, base_path, env) {
  rd_section(x$tag, x$val)
}
#' @export
format.rd_section_rawRd <- function(x, ...) {
  paste(x$value, collapse = "\n")
}

roxy_tag_eval <- function(tag, env = new.env(parent = baseenv())) {
  tryCatch(
    {
      out <- roxy_eval(tag$val, env)

      if (!is.character(out)) {
        warn_roxy_tag(tag, "must evaluate to a character vector")
        NULL
      } else if (anyNA(out)) {
        warn_roxy_tag(tag, "must not contain any missing values")
        NULL
      } else {
        out
      }
    },
    error = function(e) {
      warn_roxy_tag(tag, "failed to evaluate", parent = e)
      NULL
    }
  )
}

#' Re-build outdated vignettes
#'
#' @description
#' This roclet rebuilds outdated vignettes with [tools::buildVignette],
#' but we no longer recommend it because we no longer recommend storing
#' built vignettes in a package.
#'
#' By default, it will rebuild all vignettes if the source file is newer than
#' the output pdf or html. (This means it will automatically re-build the
#' vignette if you change the vignette source, but _not_ when you
#' change the R code). If you want finer control, add a Makefile to
#' `vignettes/` and roxygen2 will use that instead.
#'
#' To prevent RStudio from re-building the vignettes again when checking
#' your package, add `--no-build-vignettes` to the "Build Source Package"
#' field in your project options.
#'
#' @keywords internal
#' @export
vignette_roclet <- function() {
  roclet("vignette")
}

#' @export
roclet_process.roclet_vignette <- function(x, blocks, env, base_path) {
}

#' @export
roclet_output.roclet_vignette <- function(x, results, base_path, ...) {
  vign_update_all(base_path)
}

# Determine if a vignette is out-of-date; i.e. it has no related files, or
# any of the related files are older than the vignette.
vign_outdated <- function(vign) {
  vign <- normalizePath(vign, mustWork = TRUE)
  name <- tools::file_path_sans_ext(basename(vign))

  # Currently, the final product of a vignette can only be pdf or html
  related <- dir(dirname(vign), pattern = paste0(name, "\\.(pdf|html)$"),
    full.names = TRUE)
  related <- setdiff(related, vign)

  length(related) == 0 || mtime(vign) > mtime(related)
}

vign_update <- function(vign) {
  if (!vign_outdated(vign)) return(FALSE)

  cli::cli_inform("Rebuilding {.file {basename(vign)}}")
  output <- tools::buildVignette(vign, dirname(vign), tangle = FALSE,
    clean = FALSE)

  TRUE
}

vign_update_all <- function(pkg_path) {
  vig_path <- file.path(pkg_path, "vignettes")
  if (!file.exists(vig_path)) return()

  if (file.exists(file.path(vig_path, "Makefile"))) {
    cli::cli_inform("Updating vignettes with make")

    make <- Sys.getenv("MAKE", "make")
    old <- setwd(vig_path)
    on.exit(setwd(old), add = TRUE)

    system(make)
  } else {
    cli::cli_inform("Updating vignettes")

    vigs <- tools::pkgVignettes(dir = pkg_path)
    invisible(map_lgl(vigs$docs, vign_update))
  }
}

mtime <- function(x) {
  max(file.info(x)$mtime)
}

#' Escape fragile Rd tags
#'
#' @description
#' `escape_rd_for_md()` replaces fragile Rd tags with placeholders, to avoid
#' interpreting them as markdown. `unescape_rd_for_md()` puts the original
#' text back in place of the placeholders after the markdown parsing is done.
#' The fragile tags are listed in `escaped_for_md`.
#'
#' Some Rd macros are treated specially:
#'
#' * For `if`, markdown is only allowed in the second argument.
#' * For `ifelse` markdown is allowed in the second and third arguments.
#'
#' @param text Input text. Potentially contains Rd and/or
#'   markdown markup.
#' @returns
#' * `escape_rd_for_md`: a "safe" version of the input text, where
#'   each fragile Rd tag is replaced by a placeholder. The
#'   original text is added as an attribute for each placeholder.
#' * `unescape_rd_for_md`: the original Rd text.
#' @rdname markdown-internals
#' @keywords internal
escape_rd_for_md <- function(text) {
  rd_tags <- find_fragile_rd_tags(text, escaped_for_md)
  protected <- protect_rd_tags(text, rd_tags)
  double_escape_md(protected)
}

escaped_for_md <- paste0(
  "\\",
  c(
    "acronym",
    "code",
    "command",
    "CRANpkg",
    "deqn",
    "doi",
    "dontrun",
    "dontshow",
    "donttest",
    "email",
    "env",
    "eqn",
    "figure",
    "file",
    "if",
    "ifelse",
    "kbd",
    "link",
    "linkS4class",
    "method",
    "mjeqn",
    "mjdeqn",
    "mjseqn",
    "mjsdeqn",
    "mjteqn",
    "mjtdeqn",
    "newcommand",
    "option",
    "out",
    "packageAuthor",
    "packageDescription",
    "packageDESCRIPTION",
    "packageIndices",
    "packageMaintainer",
    "packageTitle",
    "pkg",
    "PR",
    "preformatted",
    "renewcommand",
    "S3method",
    "S4method",
    "samp",
    "special",
    "testonly",
    "url",
    "var",
    "verb"
  )
)

#' @param rd_text The markdown parsed and interpreted text.
#' @param esc_text The original escaped text from
#'   `escape_rd_for_md()`.
#' @rdname markdown-internals
unescape_rd_for_md <- function(rd_text, esc_text) {
  id <- attr(esc_text, "roxygen-markdown-subst")$id
  tags <- attr(esc_text, "roxygen-markdown-subst")$tags

  for (i in seq_len(nrow(tags))) {
    ph <- paste0(id, "-", i, "-")
    rd_text <- sub(ph, tags$text[i], rd_text, fixed = TRUE)
  }

  rd_text
}

#' Find all fragile tags (int the supplied list) in the text
#'
#' Ignore the tags that are embedded into a fragile tag.
#'
#' @param text Input text, character scalar.
#' @param fragile Character vector of fragile tags to find.
#' @return Data frame of fragile tags, with columns:
#'   `tag`, `start`, `end`, `argend`,
#'   `text`.
#'
#' @noRd

find_fragile_rd_tags <- function(text, fragile) {
  tags <- find_all_rd_tags(text)
  ftags <- tags[tags$tag %in% fragile, ]

  ## Remove embedded ones
  keep <- map_lgl(seq_len(nrow(ftags)), function(i) {
    sum(ftags$start <= ftags$start[i] & ftags$argend >= ftags$argend[i]) == 1
  })

  ftags <- ftags[keep, ]

  if (nrow(ftags)) {
    ftags$text <- substring(text, ftags$start, ftags$argend)
  }

  ftags
}

#' Find all (complete) Rd tags in a string
#'
#' Complete means that we include the argument(s) as well.
#'
#' @param text Input text, character scalar.
#'
#' @noRd

find_all_rd_tags <- function(text) {
  text_len <- nchar(text)

  ## Find the tag names
  tags <- find_all_tag_names(text)

  ## Find the end of the argument list for each tag. Note that
  ## tags might be embedded into the arguments of other tags.
  tags$argend <- map_int(seq_len(nrow(tags)), function(i) {
    tag_plus <- substr(text, tags$end[i], text_len)
    findEndOfTag(tag_plus, is_code = FALSE, start = 0L) + tags$end[i]
  })

  tags
}

#' Find all tag names in a string
#'
#' Note that we also protect these tags within code, strings
#' and comments, for now. We'll see if this causes any
#' problems.
#'
#' @param text Input text, scalar.
#' @return Data frame, with columns: `tag`, `start`,
#'   `end`.
#'
#' @noRd

find_all_tag_names <- function(text) {
  ## Find the tags without arguments first
  m <- gregexpr(r"(\\[a-zA-Z][a-zA-Z0-9]*)", text)[[1]]
  if (m[[1]] == -1L) {
    tag_pos <- matrix(
      integer(),
      ncol = 2,
      dimnames = list(NULL, c("start", "end"))
    )
  } else {
    tag_pos <- cbind(
      start = as.integer(m),
      end = as.integer(m) + attr(m, "match.length") - 1L
    )
  }

  if (nrow(tag_pos) == 0) {
    data.frame(tag = character(), start = integer(), end = integer())
  } else {
    data.frame(
      tag = substring(text, tag_pos[, "start"], tag_pos[, "end"]),
      as.data.frame(tag_pos)
    )
  }
}

#' Replace fragile Rd tags with placeholders
#'
#' @param text The text, character scalar.
#' @param rd_tags Fragile Rd tags, in a data frame,
#'   as returned by `find_fragile_rd_tags`.
#' @return Text, after the substitution. The original
#'   text is added as an attribute.
#'
#' @noRd

protect_rd_tags <- function(text, rd_tags) {
  id <- make_random_string()

  text <- re_sub_same(text, rd_tags, id)

  attr(text, "roxygen-markdown-subst") <-
    list(tags = rd_tags, id = id)

  text
}

#' Replace parts of the same string
#'
#' It assumes that the intervals to be replaced do not
#' overlap. Gives an error otherwise.
#'
#' @param str String scalar.
#' @param repl Data frame with columns: `start`, `end`,
#'   `argend`, `text`.
#' @param id Placeholder string.
#' @return Input string with the replacements performed.
#'   Note that all replacements are performed in parallel,
#'   at least conceptually.
#'
#' @noRd

re_sub_same <- function(str, repl, id) {
  repl <- repl[order(repl$start), ]

  if (is.unsorted(repl$end) || is.unsorted(repl$argend)) {
    cli::cli_abort("Replacement intervals must not overlap.", .internal = TRUE)
  }

  for (i in seq_len(nrow(repl))) {
    ## The trailing - is needed, to distinguish between -1 and -10
    new_text <- paste0(id, "-", i, "-")
    str <- paste0(
      substr(str, 1, repl$start[i] - 1),
      new_text,
      substr(str, repl$argend[i] + 1, nchar(str))
    )

    ## Need to shift other coordinates (we shift everything,
    ## it is just simpler).
    inc <- nchar(new_text) - (repl$argend[i] - repl$start[i] + 1)
    repl$start <- repl$start + inc
    repl$end <- repl$end + inc
    repl$argend <- repl$argend + inc
  }

  str
}

#' Make a random string
#'
#' We use this as the placeholder, to make sure that the
#' placeholder does not appear in the text.
#'
#' @return String scalar
#'
#' @noRd

make_random_string <- function(length = 32) {
  paste(
    sample(c(LETTERS, letters, 0:9), length, replace = TRUE),
    collapse = ""
  )
}

#' Check markdown escaping
#'
#' This is a regression test for Markdown escaping.
#'
#' @details
#' Each of the following bullets should look the same when rendered:
#'
#' * Backticks: `\`, `\%`, `\$`, `\_`
#' * `\verb{}`: \verb{\\}, \verb{\\%}, \verb{\$}, \verb{\_}
#'
#' \[ this isn't a link \]
#' \\[ neither is this \\]
#'
#' @param text Input text.
#' @return Double-escaped text.
#' @keywords internal
#' @examples
#' "%" # percent
#' "\"" # double quote
#' '\'' # single quote
double_escape_md <- function(text) {
  text <- gsub(r"(\)", r"(\\)", text, fixed = TRUE)

  # De-dup escaping used to avoid [] creating a link
  text <- gsub(r"(\\[)", r"(\[)", text, fixed = TRUE)
  text <- gsub(r"(\\])", r"(\])", text, fixed = TRUE)
  text
}

tag_df <- function(tag, start, end, argend = NULL) {
  df <- data.frame(
    tag = tag,
    start = start,
    end = end
  )
  if (!is.null(argend)) {
    df$argend <- argend
  }
  df
}

test_that("find_all_tag_names", {
  text <- r"(blah blah \mytag blah blah)"
  expect_equal(
    find_all_tag_names(text),
    tag_df(r"(\mytag)", 11, 16)
  )
})

test_that("find_all_rd_tags", {
  cases <- list(
    ## No tags
    list("", character(), numeric(), numeric(), numeric()),
    list("nothing to see here", character(), numeric(), numeric(), numeric()),
    list("\nstill\nnothing\n", character(), numeric(), numeric(), numeric()),

    ## One tag
    list(r"(blah blah \mytag blah blah)", r"(\mytag)", 11, 16, 16),
    list(r"(blah blah \mytag{arg1} blah blah)", r"(\mytag)", 11, 16, 22),
    list(r"(blah blah \mytag{arg1}{arg2} blah blah)", r"(\mytag)", 11, 16, 28),
    list(r"(blah\mytag)", r"(\mytag)", 5, 10, 10),
    list(r"(blah \mytag)", r"(\mytag)", 6, 11, 11),
    list(r"(blah\mytag{arg})", r"(\mytag)", 5, 10, 15),
    list(r"(\mytag hoohoo)", r"(\mytag)", 1, 6, 6),
    list(r"(\mytag)", r"(\mytag)", 1, 6, 6),
    list(r"(\mytag{arg})", r"(\mytag)", 1, 6, 11),
    list("blah \\mytag\nblah blah", r"(\mytag)", 6, 11, 11),

    ## Multiple tags
    list(
      r"(blah \tag1 \tag2{arg} blah)",
      c(r"(\tag1)", r"(\tag2)"),
      c(6, 12),
      c(10, 16),
      c(10, 21)
    ),
    list(
      r"(blah \tag1{ \tag2{arg} } blah)",
      c(r"(\tag1)", r"(\tag2)"),
      c(6, 13),
      c(10, 17),
      c(24, 22)
    ),
    list(
      "blah \\tag1{\n\\tag2{arg}\n} blah",
      c(r"(\tag1)", r"(\tag2)"),
      c(6, 13),
      c(10, 17),
      c(24, 22)
    )
  )

  for (case in cases) {
    expect_equal(
      find_all_rd_tags(case[[1]]),
      do.call(tag_df, case[-1]),
      info = case[[1]]
    )
  }
})

test_that("find_all_rd_tags uses character offsets for multibyte text", {
  expect_equal(
    find_all_rd_tags("x \\code{café} y"),
    tag_df("\\code", 3, 7, 13)
  )
  expect_equal(
    find_all_rd_tags("é \\mytag{ééé}{ü} é"),
    tag_df("\\mytag", 3, 8, 16)
  )
})

test_that("find_fragile_rd_tags", {
  fragile <- c(r"(\frag)", r"(\frag1)", r"(\frag2)")

  cases <- list(
    list(r"(This is \frag{here}, \this{arg} not)", r"(\frag)"),
    list(r"(Embedded \frag{ into \frag1{arg} plus })", r"(\frag)"),
    list(
      r"(blah \cmd{ \frag{arg} \frag{arg} } \frag2 blah)",
      c(r"(\frag)", r"(\frag)", r"(\frag2)")
    )
  )

  for (case in cases) {
    expect_equal(
      find_fragile_rd_tags(case[[1]], fragile)$tag,
      case[[2]],
      info = case[[1]]
    )
  }
})


test_that("re_sub_same", {
  expect_equal(
    re_sub_same(
      "123456789ab",
      data.frame(start = c(1, 6), end = c(2, 10), argend = c(2, 10)),
      "xxx"
    ),
    "xxx-1-345xxx-2-b"
  )

  expect_equal(
    re_sub_same(
      "123456789ab",
      data.frame(start = c(1, 8), end = c(7, 10), argend = c(7, 10)),
      "xxx"
    ),
    "xxx-1-xxx-2-b"
  )

  expect_equal(
    re_sub_same(
      "123456789ab",
      data.frame(start = numeric(), end = numeric(), argend = numeric()),
      "xxx"
    ),
    "123456789ab"
  )
})

#' @export
roxy_tag_parse.roxy_tag_template <- function(x) {
  tag_value(x)
}

#' @export
roxy_tag_parse.roxy_tag_templateVar <- function(x) {
  tag_two_part(x, "a variable name", "a value", multiline = "always")
}

process_templates <- function(block, base_path) {
  tags <- block_get_tags(block, "template")
  if (length(tags) == 0) {
    return(block)
  }

  templates <- map_chr(tags, \(x) x[["val"]])
  paths <- map_chr(templates, template_find, base_path = base_path)

  var_tags <- block_get_tags(block, "templateVar")
  vars <- set_names(
    map(var_tags, \(x) x[["val"]][["description"]]),
    map_chr(var_tags, \(x) x[["val"]][["name"]])
  )
  vars <- lapply(vars, utils::type.convert, as.is = TRUE)

  results <- lapply(paths, template_eval, vars = list2env(vars))
  tokens <- lapply(results, tokenise_block, file = "TEMPLATE", offset = 0L)
  tags <- lapply(tokens, parse_tags)

  # Insert templates back in the location where they came from
  block_replace_tags(block, "template", tags)
}

# Helpers -----------------------------------------------------------------

template_find <- function(base_path, template_name) {
  file_name <- paste0(template_name, ".", c("R", "r"))
  path <- c(
    file.path(base_path, "man-roxygen", file_name),
    file.path(base_path, "man", "roxygen", "templates", file_name)
  )
  path_exists <- file.exists(path)

  if (!any(path_exists)) {
    # This should really use warn_roxy_tag() but it's not worth refactoring
    # for this rarely used feature
    cli::cli_abort("Can't find template {.str {template_name}}.", call = NULL)
  }

  path[path_exists][[1]]
}

template_eval <- function(template_path, vars) {
  utils::capture.output(brew::brew(template_path, envir = vars))
}

#' @export
object_defaults.r6class <- function(x, block) {
  r6on <- roxy_meta_get("r6", TRUE)
  if (isTRUE(r6on)) {
    list(
      roxy_generated_tag(block, "docType", NULL),
      roxy_generated_tag(block, ".r6data", extract_r6_data(x$value))
    )
  } else {
    NextMethod()
  }
}

extract_r6_data <- function(x) {
  list(
    self = extract_r6_self_data(x),
    super = drop_clone_maybe(x, extract_r6_super_data(x))
  )
}

drop_clone_maybe <- function(x, data) {
  if (! "clone" %in% names(x$public_methods)) {
    cline <- which(data$members$name == "clone" & data$members$type == "method")
    if (length(cline)) data$members <- data$members[-cline, ]
  }
  data
}

extract_r6_self_data <- function(x) {
  rbind(
    extract_r6_methods(x),
    extract_r6_fields(x),
    extract_r6_bindings(x)
  )
}

default_r6_methods <- function() {
  "clone"
}

extract_r6_methods <- function(x) {
  method_nms <- setdiff(names(x$public_methods), default_r6_methods())
  method_loc <- map_int(
    x$public_methods[method_nms],
    function(m) {
      ref <- utils::getSrcref(m)
      if (is.null(ref)) {
        name <- x$classname %||% "unknown"

        cli::cli_abort(
          c(
            "R6 class {.cls {name}} lacks source references.",
            i = paste0(
              "If you are using the `installed` load method in `DESCRIPTION`, then ",
              "try re-installing the package with option '--with-keep.source', e.g. ",
              "{.code install.packages(..., INSTALL_OPTS = \"--with-keep.source\")}."
            )
          ),
          call = NULL
        )
      }
      utils::getSrcLocation(ref)
    }
  )
  method_fnm <- map_chr(
    x$public_methods[method_nms],
    function(m) {
      utils::getSrcFilename(utils::getSrcref(m))
    }
  )
  method_formals <- map(x$public_methods[method_nms], formals)

  methods <- data.frame(
    stringsAsFactors = FALSE,
    type = if (length(method_loc)) "method" else character(),
    class = if (length(method_loc)) x$classname %||% NA_character_ else character(),
    name = unname(method_nms),
    file = unname(method_fnm),
    line = unname(method_loc),
    formals = I(unname(method_formals))
  )

  add_default_method_data(x, methods)
}

add_default_method_data <- function(obj, methods) {
  pubm <- obj$public_methods
  defaults <- list(
    clone = list(
      formals = if ("clone" %in% names(pubm)) I(list(formals(pubm$clone)))
    )
  )

  for (mname in names(defaults)) {
    if (mname %in% methods$name) next
    if (! mname %in% names(obj$public_methods)) next
    rec <- data.frame(
      stringsAsFactors = FALSE,
      type = defaults[[mname]]$type %||% "method",
      class = defaults[[mname]]$class %||% obj$classname %||% "unknown",
      name = defaults[[mname]]$name %||% mname,
      file = defaults[[mname]]$file %||% NA_character_,
      line = defaults[[mname]]$line %||% NA_integer_,
      formals = defaults[[mname]]$formals %||% NULL
    )
    methods <- rbind(methods, rec)
  }

  methods
}

extract_r6_fields <- function(x) {
  field_nms <- names(x$public_fields)
  data.frame(
    stringsAsFactors = FALSE,
    type = rep("field", length(field_nms)),
    name = as.character(field_nms),
    class = rep(x$classname %||% NA_character_, length(field_nms)),
    file = rep(NA, length(field_nms)),
    line = rep(NA, length(field_nms)),
    formals = I(replicate(length(field_nms), NULL))
  )
}

extract_r6_bindings <- function(x) {
  bind_nms <- names(x$active)
  data.frame(
    stringsAsFactors = FALSE,
    type = if (length(bind_nms)) "active" else character(),
    name = as.character(bind_nms),
    class = rep(x$classname %||% NA_character_, length(bind_nms)),
    file = rep(NA, length(bind_nms)),
    line = rep(NA, length(bind_nms)),
    formals = I(replicate(length(bind_nms), NULL))
  )
}

extract_r6_super_data <- function(x) {
  if (is.null(x$inherit)) return()
  super <- x$get_inherit()
  super_data <- extract_r6_super_data(super)

  method_nms <- names(super$public_methods)
  field_nms <- names(super$public_fields)
  active_nms <- names(super$active)
  classname <- super$classname %||% NA_character_
  pkg <- environmentName(topenv(super$parent_env))

  cls <- rbind(
    data.frame(
      stringsAsFactors = FALSE,
      package = pkg,
      classname = classname
    ),
    super_data$classes
  )

  types <- rep(
    c("method", "field", "active"),
    c(length(method_nms), length(field_nms), length(active_nms))
  )
  rsort <- function(x) sort_c(x, decreasing = TRUE)
  names <-c(rsort(method_nms), rsort(field_nms), rsort(active_nms))
  mth <- rbind(
    data.frame(
      stringsAsFactors = FALSE,
      package = rep(pkg, length(names)),
      classname = rep(classname , length(names)),
      type = types,
      name = names
    ),
    super_data$members
  )

  list(classes = cls, members = mth)
}

test_that("extract_r6_data without source refs", {
  txt <- "R6::R6Class('foo',
     public = list(
       field1 = NULL,
       meth1 = function(Z) { },
       meth2 = function(Z = 10, ...) { },
       field2 = \"foobar\",
       meth3 = function() { }
     )
   )"
  C <- eval(parse(text = txt, keep.source = FALSE))
  expect_snapshot(extract_r6_data(C), error = TRUE)
})

test_that("extract_r6_methods", {
  txt <-
    "R6::R6Class(
       public = list(
         field1 = NULL,
         meth1 = function(Z) { },
         meth2 = function(Z = 10, ...) { },
         field2 = \"foobar\",
         meth3 = function() { }
       )
     )"
  C <- eval(parse(text = txt, keep.source = TRUE))
  M <- extract_r6_methods(C)
  expect_equal(M$type, rep("method", 4))
  expect_equal(M$name, c(paste0("meth", 1:3), "clone"))
  expect_equal(M$line, c(4L, 5L, 7L, NA_integer_))
  expect_equal(
    M$formals,
    I(list(
      as.pairlist(alist(Z = )),
      as.pairlist(alist(Z = 10, ... = )),
      NULL,
      as.pairlist(alist(deep = FALSE))
    ))
  )
})

test_that("extract_r6_super_data", {
  eval(parse(test_path("testR6/R/classes.R"), keep.source = TRUE))

  D <- extract_r6_super_data(C)
  mypkg <- environmentName(topenv())
  expect_equal(D$classes$package, rep(mypkg, 2))
  expect_equal(D$classes$classname, c("B", "A"))
  expect_equal(D$members$package, rep(mypkg, 18))
  expect_equal(D$members$classname, rep(c("B", "A"), c(8, 10)))
  expect_equal(
    D$members$type,
    c(
      rep("method", 3),
      rep("field", 2),
      rep("active", 3),
      rep("method", 4),
      rep("field", 3),
      rep("active", 3)
    )
  )
  expect_equal(
    D$members$name,
    c(
      "meth4",
      "meth1",
      "clone",
      "field4",
      "field1",
      "active5",
      "active4",
      "active1",
      "meth3",
      "meth2",
      "meth1",
      "clone",
      "field3",
      "field2",
      "field1",
      "active3",
      "active2",
      "active1"
    )
  )
})

test_that("extract_r6_fields", {
  C <- R6::R6Class(
    public = list(
      field1 = NULL,
      meth1 = function() {},
      field2 = "foobar"
    )
  )
  F <- extract_r6_fields(C)
  expect_equal(F$type, rep("field", 2))
  expect_equal(F$name, c("field1", "field2"))

  C <- R6::R6Class(
    public = list(
      meth1 = function() {}
    )
  )
  F <- extract_r6_fields(C)
  expect_s3_class(F, "data.frame")
  expect_equal(F$type, character())
  expect_equal(F$name, character())

  C <- R6::R6Class()
  F <- extract_r6_fields(C)
  expect_s3_class(F, "data.frame")
  expect_equal(F$type, character())
  expect_equal(F$name, character())
})

test_that("extract_r6_bindings", {
  C <- R6::R6Class(
    active = list(
      bind1 = function(x) {},
      bind2 = function(x) {}
    ),
    public = list(
      meth1 = function() {}
    )
  )
  F <- extract_r6_bindings(C)
  expect_equal(F$type, rep("active", 2))
  expect_equal(F$name, c("bind1", "bind2"))

  C <- R6::R6Class(
    public = list(
      meth1 = function() {}
    )
  )
  F <- extract_r6_bindings(C)
  expect_s3_class(F, "data.frame")
  expect_equal(F$type, character())
  expect_equal(F$name, character())

  C <- R6::R6Class()
  F <- extract_r6_bindings(C)
  expect_s3_class(F, "data.frame")
  expect_equal(F$type, character())
  expect_equal(F$name, character())
})

test_that("can disable R6 handling", {
  text <- "
    #' Class
    C <- R6::R6Class(
      public = list(
        field = NULL,
        #' @description Method desc.
        #' @param arg Method arg.
        meth = function(arg) { }
      )
    )"
  local_roxy_meta_set("r6", FALSE)

  out <- roc_proc_text(rd_roclet(), text)[[1]]
  rd <- format(out)
  expect_false(grepl("section{Methods}", rd, fixed = TRUE))
  expect_true(grepl("arguments{", rd, fixed = TRUE))
})

#' @export
roxy_tag_parse.roxy_tag_includeRmd <- function(x) {
  if (!is_installed("rmarkdown")) {
    warn_roxy_tag(x, "requires the rmarkdown package")
    return()
  }

  tag_two_part(x, "path", "section", required = FALSE, markdown = FALSE)
}

#' @export
roxy_tag_rd.roxy_tag_includeRmd <- function(x, base_path, env) {
  rmd <- rel_rmd <- x$val$path
  section <- x$val$section

  if (!file.exists(rmd)) {
    warn_roxy_tag(x, "Can't find Rmd {.path {rmd}}")
    return(NULL)
  }

  if (section == "") section <- "details"
  stopifnot(is.character(rmd), length(rmd) == 1, !is.na(rmd))

  rmd_path <- tempfile(fileext = ".Rmd")
  md_path <- tempfile(fileext = ".md")
  on.exit(unlink(c(rmd_path, md_path), recursive = TRUE), add = TRUE)

  wd <- getwd()
  setwd(base_path)
  on.exit(setwd(wd), add = TRUE)

  # This will create an absolute path
  rmd <- normalizePath(rmd)

  cache_path <- paste0(sub("\\.Rmd$", "", rmd), "_cache/")
  fig_path <- file.path(dirname(rmd), "figure/")
  linkrefs <- rmd_linkrefs_from_file(rmd)
  opts <- c(
    root.dir = dirname(rmd),
    cache.path = cache_path,
    fig.path = fig_path,
    child = rmd
  )
  optss <- paste0(names(opts), "=", encodeString(opts, quote = '"'))
  txt <- sprintf(
    "```{r %s}\n```\n\n%s\n",
    paste(optss, collapse = ","),
    linkrefs
  )
  cat(txt, file = rmd_path)

  local_reproducible_output()
  tryCatch(
    rmarkdown::render(
      rmd_path,
      output_format = "github_document",
      output_options = c(
        list(html_preview = FALSE),
        if (utils::packageVersion("rmarkdown") >= "2.12") list(math_method = NULL)
      ),
      output_file = md_path,
      quiet = TRUE,
      envir = new_environment(parent = global_env())
    ),
    error = function(e) {
      warn_roxy_tag(x, "failed to evaluate {.path {rel_rmd}}", parent = e)
    }
  )

  if (!file.exists(md_path)) {
    return(NULL)
  }

  tryCatch(
    value <- rmd_eval_rd(md_path, x),
    error =  function(e) {
      warn_roxy_tag(x, "failed to process result of {.path {rel_rmd}}", parent = e)
    }
  )

  rd_section_markdown(section, value)
}

# Helpers -----------------------------------------------------------------

rmd_linkrefs_from_file <- function(path) {
  lines <- read_lines(path)
  txt <- paste(lines, collapse = "\n")
  paste(get_md_linkrefs(txt), collapse = "\n")
}

rmd_eval_rd <- function(path, tag) {
  mdtxt <- paste(read_lines(path), collapse = "\n")
  mdesc <- add_linkrefs_to_md(mdtxt)
  mdxml <- md_to_mdxml(mdesc)
  state <- new.env(parent = emptyenv())
  state$tag <- tag
  state$has_sections <- TRUE
  rd <- mdxml_children_to_rd_top(mdxml, state = state)
  rd
}

select_args_text <- function(fun, select = "", topic) {
  pieces <- strsplit(select, " +")[[1]]

  tryCatch(
    {
      parsed <- lapply(pieces, function(x) parse(text = x)[[1]])
      select_args(fun, parsed)
    },
    error = function(e) {
      warn_roxy_topic(topic$get_name(), "@inheritDotsParam failed", parent = e)
      character()
    }
  )
}

# Figure out which arguments that the user wants given a function and
# unevaluated list
select_args <- function(fun, select = list()) {
  stopifnot(is.function(fun))
  stopifnot(is.list(select))

  args <- names(formals(fun))
  args <- args[args != "..."]

  if (length(select) == 0) {
    return(args)
  }

  # Construct environment that allow minimal select-style semantics
  arg_idx <- as.list(setNames(seq_along(args), args))
  arg_env <- list2env(arg_idx, parent = emptyenv())
  arg_env$`:` <- `:`
  arg_env$`-` <- `-`
  arg_env$`(` <- `(`

  indices <- lapply(select, eval, envir = arg_env)
  for (i in seq_along(select)) {
    select_check(indices[[i]], select[[i]])
  }

  # If first is negative, start with all vars
  # If first is positive, start with no vars
  select <- rep(select_sign(indices[[1]]) < 0, length(args))

  for (idx in indices) {
    select[abs(idx)] <- select_sign(idx) > 0
  }

  args[select]
}

select_check <- function(x, call) {
  if (!is.numeric(x)) {
    cli::cli_abort(c(
      "Argument specification must evaluate to a numeric vector",
      "Problem in {.code {deparse(call)}}"
    ))
  }

  if (!(all(x > 0) || all(x < 0))) {
    cli::cli_abort(
      c(
        "Argument specification must be all positive or all negative, not a mixture",
        i = "Problem in {.code {deparse(call)}}"
      ),
      call = NULL
    )
  }

  invisible()
}

select_sign <- function(x) {
  if (all(x > 0)) {
    1
  } else if (all(x < 0)) {
    -1
  } else {
    NA
  }
}

test_that("warns on invalid input", {
  expect_snapshot({
    select_args_text(c("x", "na.rm"), "-xlab:", "test")
    select_args_text(c("x", "na.rm"), '"a"', "test")
    select_args_text(c("x", "y", "z"), "-x:z", "test")
  })
})

test_that("positive initial values starts from nothing", {
  expect_equal(select_args_text(c("x", "y", "z"), "x y", "test"), c("x", "y"))
})

test_that("negative initial starts from everything", {
  expect_equal(select_args_text(c("x", "y", "z"), "-z", "test"), c("x", "y"))
})

test_that("can alternative exclusion and inclusion", {
  expect_equal(
    select_args_text(c("x", "y", "z"), "-z z", "test"),
    c("x", "y", "z")
  )
  expect_equal(select_args_text(c("x", "y", "z"), "z -z", "test"), character())
})

#' @export
roxy_tag_parse.roxy_tag_usage <- function(x) {
  x <- tag_value(x)
  x$val <- rd(x$val)
  x
}

#' @export
roxy_tag_rd.roxy_tag_usage <- function(x, base_path, env) {
  if (identical(x$val, rd("NULL"))) {
    usage <- NULL
  } else {
    usage <- x$val
  }
  rd_section("usage", usage)
}

#' @export
format.rd_section_usage <- function(x, ...) {
  rd_macro(x$type, build_rd(x$value, collapse = "\n\n"), space = TRUE)
}

# object_usage ------------------------------------------------------------

object_usage <- function(x) {
  UseMethod("object_usage")
}

object_usage.default <- function(x) {
  NULL
}

object_usage.data <- function(x) {
  rd(x$alias)
}

object_usage.function <- function(x) {
  function_usage(x$alias, formals(x$value), identity)
}

object_usage.s3generic <- object_usage.function

object_usage.s3method <- function(x) {
  method <- attr(x$value, "s3method")
  s3method <- function(name) {
    paste0("\\method{", name, "}{", auto_backtick(method[2]), "}")
  }
  function_usage(method[1], formals(x$value), s3method)
}

object_usage.s4generic <- function(x) {
  function_usage(x$value@generic, formals(x$value), identity)
}

object_usage.s4method <- function(x) {
  s4method <- function(name) {
    classes <- auto_backtick(as.character(x$value@defined))
    paste0("\\S4method{", name, "}{", paste0(classes, collapse = ","), "}")
  }
  function_usage(x$value@generic, formals(x$value), s4method)
}

# Function usage ----------------------------------------------------------

# Usage:
# replacement, infix, regular
# function, s3 method, s4 method, data

function_usage <- function(name, formals, format_name = identity) {
  if (is_replacement_fun(name) && !is_infix_fun(name)) {
    name <- str_replace(name, fixed("<-"), "")
    if (identical(format_name, identity)) {
      name <- auto_backtick(name)
    }
    name <- gsub("%", "\\%", name, fixed = TRUE)
    formals$value <- NULL

    wrap_usage(name, format_name, formals, suffix = " <- value")
  } else if (is_infix_fun(name) && identical(format_name, identity)) {
    # If infix, and regular function, munge format
    arg_names <- names(formals)
    name <- format_name(name)
    if (is_padded_infix_fun(name)) {
      name <- paste0(" ", name, " ")
    }
    build_rd(arg_names[1], name, arg_names[2])
  } else {
    if (identical(format_name, identity)) {
      name <- auto_backtick(name)
    }
    name <- gsub("%", "\\%", name, fixed = TRUE)
    wrap_usage(name, format_name, formals)
  }
}

is_replacement_fun <- function(name) {
  str_detect(name, fixed("<-"))
}
is_infix_fun <- function(name) {
  ops <- c(
    "+", "-", "*", "^", "/",
    "==", ">", "<", "!=", "<=", ">=",
    "&", "|",
    "[[", "[", "$", ":", "::", ":::"
  )
  str_detect(name, "^%.*%$") || name %in% ops
}
is_padded_infix_fun <- function(name) {
  ops <- c(
    "+", "-", "*", "/",
    "==", ">", "<", "!=", "<=", ">=",
    "&", "|"
  )
  str_detect(name, "^%.*%$") || name %in% ops
}

usage_args <- function(args) {
  is.missing.arg <- function(arg) {
    is.symbol(arg) && deparse(arg) == ""
  }
  arg_to_text <- function(arg) {
    if (is.missing.arg(arg)) return("")
    text <- enc2utf8(deparse(arg, backtick = TRUE, width.cutoff = 500L))
    text <- paste0(text, collapse = "\n")
    Encoding(text) <- "UTF-8"

    text
  }
  map_chr(args, arg_to_text)
}

args_string <- function(x, space = " ") {
  sep <- ifelse(names(x) != "" & x != "", paste0(space, "=", space), "")

  nms <- names2(x)
  arg_names <- ifelse(nms == "", "", escape(auto_backtick(nms)))
  paste0(arg_names, sep, escape(x))
}

args_call <- function(call, args) {
  paste0(call, "(", paste0(args, collapse = ", "), ")")
}

#' @param name Function name
#' @param format_name Single argument that returns formatted function name
#' @param formals List of function formals
#' @param suffix Optional suffix, used for replacement functions
#' @noRd
wrap_usage <- function(name, format_name, formals, suffix = NULL, width = 80L) {
  if (roxy_meta_get("old_usage", FALSE)) {
    # Use nbsp to keep argument name & default value on same line
    args <- args_string(usage_args(formals), "\u{A0}")
    x <- args_call(format_name(name), args)
    out <- wrapUsage(x, width = as.integer(width), indent = 2)

    out <- gsub("\u{A0}", " ", out, useBytes = TRUE)
    Encoding(out) <- "UTF-8"

    return(rd(paste0(out, suffix)))
  }

  args <- args_string(usage_args(formals))
  bare <- args_call(name, args)

  if (!str_detect(bare, "\n") && nchar(bare, type = "width") < width) {
    # Don't need to wrap
    out <- args_call(format_name(name), args)
  } else {
    # Wrap each argument and put on own line
    args <- paste0("  ", args)
    args <- map_chr(args, wrapUsage, width = 90, indent = 4)
    out <- paste0(format_name(name), "(\n", paste0(args, collapse = ",\n"), "\n)")
  }

  rd(paste0(out, suffix))
}

# helpers -----------------------------------------------------------------

# used for testing
call_to_usage <- function(code, env = pkg_env()) {
  obj <- call_to_object(!!enexpr(code), env)
  as.character(object_usage(obj))
}

#' Build a new roclet.
#'
#' To create a new roclet, you will need to create a constructor function
#' that wraps `roclet`, and then implement the methods described below.
#'
#' @section Methods:
#'
#' * `roclet_preprocess()` is called after blocks have been parsed but before
#'   code has been evaluated. This should only be needed if your roclet affects
#'   how code will evaluated. Should return a roclet.
#'
#' * `roclet_process()` called after blocks have been evaluated; i.e. the
#'   `@eval` tag has been processed, and the object associated with each block
#'   has been determined.
#'
#' * `roclet_output()` is given the output from `roclet_process()` and should
#'   produce files on disk.
#'
#' * `roclet_clean()` called when `roxygenise(clean = TRUE)`. Should remove
#'   any files created by the roclet.
#'
#' ### Deprecated methods
#'
#' `roclet_tags()` is no longer used; instead provide a [roxy_tag_parse()]
#' method for each tag.
#'
#' @param x A `roclet` object.
#' @param blocks A list of [roxy_block] objects.
#' @param results Value returned from your `roclet_process()` method.
#' @param base_path Path to root of source package.
#' @param env Package environment.
#' @keywords internal
#' @name roclet
NULL

#' @export
#' @rdname roclet
roclet <- function(subclass, ...) {
  structure(list(...), class = c(paste0("roclet_", subclass), "roclet"))
}

#' @export
#' @rdname roclet
roclet_preprocess <- function(x, blocks, base_path) {
  UseMethod("roclet_preprocess")
}

#' @export
roclet_preprocess.default <- function(x, blocks, base_path) {
  x
}

#' @export
#' @rdname roclet
roclet_process <- function(x, blocks, env, base_path) {
  UseMethod("roclet_process")
}

#' @export
#' @rdname roclet
roclet_output <- function(x, results, base_path, ...) {
  UseMethod("roclet_output", x)
}

#' @export
#' @rdname roclet
roclet_clean <- function(x, base_path) {
  UseMethod("roclet_clean")
}

#' @export
#' @rdname roclet
roclet_tags <- function(x) {
  UseMethod("roclet_tags")
}

#' Create a roclet from a string.
#'
#' This provides a flexible way of specifying a roclet in a string.
#'
#' @param x Arbitrary R code evaluated in roxygen2 package.
#' @keywords internal
#' @export
#' @examples
#' # rd, namespace, and vignette work for backward compatibility
#' roclet_find("rd")
#'
#' # But generally you should specify the name of a function that
#' # returns a roclet
#' roclet_find("rd_roclet")
#'
#' # If it lives in another package, you'll need to use ::
#' roclet_find("roxygen2::rd_roclet")
#'
#' # If it takes parameters (which no roclet does currently), you'll need
#' # to call the function
#' roclet_find("roxygen2::rd_roclet()")
roclet_find <- function(x) {
  env <- new.env(parent = getNamespace("roxygen2"))
  env$rd <- rd_roclet
  env$namespace <- namespace_roclet
  env$vignette <- vignette_roclet

  expr <- parse(text = x)
  res <- eval(expr, env)

  if (is.function(res)) {
    res <- res()
  }

  if (!is.roclet(res)) {
    cli::cli_abort("Must return a roclet")
  }

  res
}

is.roclet <- function(x) inherits(x, "roclet")

#' Process roclet on string and capture results.
#'
#' Useful for testing.
#'
#' @param roclet Name of roclet to use for processing.
#' @param input Source string
#' @param wd Working directory
#' @export
#' @keywords internal
roc_proc_text <- function(roclet, input, wd = NULL) {
  stopifnot(is.roclet(roclet))

  file <- tempfile()
  write_lines(input, file)
  on.exit(unlink(file))

  if (!is.null(wd)) {
    withr::local_dir(wd)
  }

  env <- env_file(file)
  blocks <- parse_text(input, env = env)
  roclet_process(roclet, blocks, env = env, base_path = ".")
}

##' Title
##'
##' Description
##'
##' Details
##'
##' @param x,y,z Descriptions for x, y, z
##' @param a Description of a
##' @param b
##'   Description of b
##' @section Important:
##' Don't run with scissors!
##' @export

markdown <- function(text, tag = NULL, sections = FALSE) {
  tag <- tag %||% list(file = NA, line = NA)
  expanded_text <- tryCatch(
    markdown_evaluate(text),
    error = function(e) {
      warn_roxy_tag(tag, "failed to evaluate inline markdown code", parent = e)
      text
    }
  )
  escaped_text <- escape_rd_for_md(expanded_text)
  tryCatch(
    markdown_pass2(escaped_text, tag = tag, sections = sections),
    error = function(e) {
      warn_roxy_tag(tag, "markdown failed to process", parent = e)
      text
    }
  )
}

#' Expand the embedded inline code
#'
#' @details
#' For example this becomes two: `r 1+1`.
#' Variables can be set and then reused, within the same
#' tag: `r x <- 100; NULL`
#' The value of `x` is `r x`.
#'
#' We have access to the internal functions of the package, e.g.
#' since this is _roxygen2_, we can refer to the internal `markdown`
#' function, and this is `TRUE`: `r is.function(markdown)`.
#'
#' To insert the name of the current package: `r packageName()`.
#'
#' The `iris` data set has `r ncol(iris)` columns:
#' `r paste0("\x60\x60", colnames(iris), "\x60\x60", collapse = ", ")`.
#'
#' ```{r}
#' # Code block demo
#' x + 1
#' ```
#'
#' Chunk options:
#'
#' ```{r results = "hold"}
#' names(mtcars)
#' nrow(mtcars)
#' ```
#'
#' Plots:
#'
#' ```{r test-figure}
#' plot(1:10)
#' ```
#'
#' Alternative knitr engines:
#'
#' ```{verbatim}
#' #| file = "tests/testthat/example.Rmd"
#' ```
#'
#' Also see `vignette("rd-formatting")`.
#'
#' @param text Input text.
#' @return
#' Text with R code expanded.
#' A character vector of the same length as the input `text`.
#'
#' @keywords internal

markdown_pass1 <- function(text) {
  text <- paste(text, collapse = "\n")
  mdxml <- xml_ns_strip(md_to_mdxml(text, sourcepos = TRUE))
  code_nodes <- xml_find_all(mdxml, ".//code | .//code_block")
  rcode_nodes <- keep(code_nodes, is_markdown_code_node)
  if (length(rcode_nodes) == 0) {
    return(text)
  }
  rcode_pos <- parse_md_pos(map_chr(rcode_nodes, xml_attr, "sourcepos"))
  rcode_pos <- work_around_cmark_sourcepos_bug(text, rcode_pos)
  out <- eval_code_nodes(rcode_nodes)
  re_set_all_pos(text, rcode_pos, out, rcode_nodes)
}

# Work around commonmark sourcepos bug for inline R code
# https://github.com/r-lib/roxygen2/issues/1353
work_around_cmark_sourcepos_bug <- function(text, rcode_pos) {
  if (Sys.getenv("ROXYGEN2_NO_SOURCEPOS_WORKAROUND", "") != "") {
    return(rcode_pos)
  }

  lines <- strsplit(text, "\n", fixed = TRUE)[[1]]

  for (l in seq_len(nrow(rcode_pos))) {
    # Do not try to fix multi-line code, we error for that (below)
    if (rcode_pos$start_line[l] != rcode_pos$end_line[l]) {
      next
    }
    line <- lines[rcode_pos$start_line[l]]
    start <- rcode_pos$start_column[l]

    # Maybe correct? At some point this will be fixed upstream, hopefully.
    if (substr(line, start - 1, start + 1) == "`r ") {
      next
    }

    # Maybe indented and we can shift it?
    # It is possible that the shift that we try accidentally matches
    # "`r ", but it seems to be extremely unlikely. An example is this:
    # #'       ``1`r `` `r 22*10`
    # (seven spaces after the #', so an indent of six spaces. If we shift
    # the real "`r " left by six characters, there happens to be another
    # "`r " there.

    m <- regexpr("^[ ]+", line)
    indent <- attr(m, "match.length")
    if (
      m > 0L &&
        substr(line, start - 1 + indent, start + 1 + indent) == "`r "
    ) {
      rcode_pos$start_column[l] <- rcode_pos$start_column[l] + indent
      rcode_pos$end_column[l] <- rcode_pos$end_column[l] + indent
    }
  }

  rcode_pos
}

is_markdown_code_node <- function(x) {
  info <- xml_attr(x, "info")
  substr(xml_text(x), 1, 2) == "r " ||
    (!is.na(info) && grepl("^[{][a-zA-z]+[}, ]", info))
}

parse_md_pos <- function(text) {
  nums <- map(strsplit(text, "[:-]"), as.integer)
  data.frame(
    start_line = map_int(nums, \(x) x[[1]]),
    start_column = map_int(nums, \(x) x[[2]]),
    end_line = map_int(nums, \(x) x[[3]]),
    end_column = map_int(nums, \(x) x[[4]])
  )
}

eval_code_nodes <- function(nodes) {
  evalenv <- roxy_meta_get("evalenv")
  # This should only happen in our test cases
  if (is.null(evalenv)) {
    evalenv <- new.env(parent = baseenv())
  }

  map_chr(nodes, eval_code_node, env = evalenv)
}

eval_code_node <- function(node, env) {
  if (xml_name(node) == "code") {
    # write knitr markup for inline code
    text <- paste0("`", xml_text(node), "`")
  } else {
    lang <- xml_attr(node, "info")
    # write knitr markup for fenced code
    text <- paste0("```", if (!is.na(lang)) lang, "\n", xml_text(node), "```\n")
  }

  chunk_opts <- utils::modifyList(
    knitr_chunk_defaults(),
    as.list(roxy_meta_get("knitr_chunk_options", NULL))
  )

  roxy_knit(text, env, chunk_opts)
}

knitr_chunk_defaults <- function() {
  list(
    error = FALSE,
    fig.path = "man/figures/",
    fig.process = basename,
    comment = "#>",
    collapse = TRUE
  )
}

re_set_all_pos <- function(text, pos, value, nodes) {
  # Cmark has a bug when reporting source positions for multi-line
  # code tags, and it does not count the indenting space in the
  # continuation lines: https://github.com/commonmark/cmark/issues/296
  types <- xml_name(nodes)
  if (any(types == "code" & pos$start_line != pos$end_line)) {
    cli::cli_abort("Multi-line `r ` markup is not supported.", call = NULL)
  }

  # Need to split the string, because of the potential multi-line
  # code tags, and then also recode the positions
  lens <- nchar(strsplit(text, "\n", fixed = TRUE)[[1]])
  shifts <- c(0, cumsum(lens + 1L))
  shifts <- shifts[-length(shifts)]
  start <- shifts[pos$start_line] + pos$start_column
  end <- shifts[pos$end_line] + pos$end_column

  # Create intervals for the parts we keep
  keep_start <- c(1, end + 2L)
  keep_end <- c(start - 2L, nchar(text))

  # Now piece them together
  out <- paste0(
    substring(text, keep_start, keep_end),
    c(value, ""),
    collapse = ""
  )
  attributes(out) <- attributes(text)
  out
}

markdown_pass2 <- function(text, tag = NULL, sections = FALSE) {
  esc_text_linkrefs <- add_linkrefs_to_md(text)

  mdxml <- md_to_mdxml(esc_text_linkrefs)
  state <- new.env(parent = emptyenv())
  state$tag <- tag
  state$has_sections <- sections
  rd <- mdxml_children_to_rd_top(mdxml, state)

  map_chr(rd, unescape_rd_for_md, text)
}

md_to_mdxml <- function(x, ...) {
  md <- commonmark::markdown_xml(
    x,
    hardbreaks = TRUE,
    extensions = "table",
    ...
  )
  xml2::read_xml(md)
}

mdxml_children_to_rd_top <- function(xml, state) {
  state$section_tag <- uuid()
  out <- map_chr(xml_children(xml), mdxml_node_to_rd, state)
  out <- c(out, mdxml_close_sections(state))
  rd <- trimws(paste0(out, collapse = ""))
  if (state$has_sections) {
    secs <- strsplit(rd, state$section_tag, fixed = TRUE)[[1]] %||% ""
    titles <- c("", state$titles)
    # strsplit drops trailing empty strings, so pad to match titles length
    secs <- c(secs, rep("", length(titles) - length(secs)))
    rd <- structure(trimws(secs), names = titles)
  }
  rd
}

mdxml_children_to_rd <- function(xml, state) {
  out <- map_chr(xml_children(xml), mdxml_node_to_rd, state)
  paste0(out, collapse = "")
}

mdxml_node_to_rd <- function(xml, state) {
  if (
    !inherits(xml, "xml_node") ||
      !xml_type(xml) %in% c("text", "element")
  ) {
    warn_roxy_tag(
      state$tag,
      c(
        "markdown translation failed",
        x = "Unexpected internal error",
        i = "Please file an issue at https://github.com/r-lib/roxygen2/issues"
      )
    )
    return("")
  }

  switch(
    xml_name(xml),
    html = ,
    document = ,
    unknown = mdxml_children_to_rd(xml, state),

    paragraph = paste0("\n\n", mdxml_children_to_rd(xml, state)),
    text = if (is_true(state$in_link_code)) {
      escape_verb(xml_text(xml))
    } else {
      escape_comment(xml_text(xml))
    },
    emph = paste0("\\emph{", mdxml_children_to_rd(xml, state), "}"),
    strong = paste0("\\strong{", mdxml_children_to_rd(xml, state), "}"),
    softbreak = mdxml_break(state),
    linebreak = mdxml_break(state),

    code = mdxml_code(xml, state),
    code_block = mdxml_code_block(xml, state),

    table = mdxml_table(xml, state),
    list = mdxml_list(xml, state),
    item = mdxml_item(xml, state),
    link = mdxml_link(xml, state),
    image = mdxml_image(xml),
    heading = mdxml_heading(xml, state),

    # Only supported when including Rmds
    html_block = mdxml_html_block(xml, state),
    html_inline = mdxml_html_inline(xml, state),

    # Not supported
    block_quote = mdxml_unsupported(xml, state$tag, "block quotes"),
    thematic_break = mdxml_unsupported(xml, state$tag, "horizontal rules"),
    mdxml_unknown(xml, state$tag)
  )
}

mdxml_unknown <- function(xml, tag) {
  warn_roxy_tag(
    tag,
    c(
      "markdown translation failed",
      x = "Internal error: unknown xml node {xml_name(xml)}",
      i = "Please file an issue at https://github.com/r-lib/roxygen2/issues"
    )
  )
  escape_comment(xml_text(xml))
}
mdxml_unsupported <- function(xml, tag, feature) {
  warn_roxy_tag(
    tag,
    c(
      "markdown translation failed",
      x = "{feature} are not currently supported"
    )
  )
  escape_comment(xml_text(xml))
}

mdxml_break <- function(state) {
  if (isTRUE(state$inlink)) " " else "\n"
}

mdxml_code <- function(xml, tag) {
  code <- xml_text(xml)

  if (grepl("^Rd ", code)) {
    paste0(
      "\\Sexpr[stage=render,results=rd]{",
      substr(code, 4, nchar(code)),
      "}"
    )
  } else if (can_parse(code) || code %in% special) {
    # See escaping details at
    # https://cran.rstudio.com/doc/manuals/r-devel/R-exts.html#Insertions
    paste0("\\code{", gsub("%", "\\\\%", code), "}")
  } else {
    paste0("\\verb{", escape_verb(code), "}")
  }
}

special <- c(
  "-",
  ":",
  "::",
  ":::",
  "!",
  "!=",
  "(",
  "[",
  "[[",
  "@",
  "*",
  "/",
  "&",
  "&&",
  "%*%",
  "%/%",
  "%%",
  "%in%",
  "%o%",
  "%x%",
  "^",
  "+",
  "<",
  "<=",
  "=",
  "==",
  ">",
  ">=",
  "|",
  "||",
  "~",
  "$",
  "for",
  "function",
  "if",
  "repeat",
  "while"
)

mdxml_code_block <- function(xml, state) {
  info <- xml_attr(xml, "info", default = "")[1]
  if (nchar(info[1]) == 0) {
    info <- NA_character_
  }
  paste0(
    "\n\n",
    "\\if{html}{\\out{<div class=\"sourceCode",
    if (!is.na(info)) paste0(" ", info),
    "\">}}",
    "\\preformatted{",
    escape_verb(xml_text(xml)),
    "}",
    "\\if{html}{\\out{</div>}}"
  )
}

can_parse <- function(x) {
  tryCatch(
    {
      parse_expr(x)
      TRUE
    },
    error = function(x) FALSE
  )
}

escape_verb <- function(x) {
  # Don't need to escape \\ because that's already handled in double_escape_md()
  x <- gsub("%", "\\%", x, fixed = TRUE)
  x <- gsub("{", "\\{", x, fixed = TRUE)
  x <- gsub("}", "\\}", x, fixed = TRUE)
  x
}

mdxml_table <- function(xml, state) {
  head <- xml_children(xml)[[1]]
  align <- substr(xml_attr(xml_children(head), "align", default = "left"), 1, 1)

  rows <- xml_find_all(xml, "d1:table_row|d1:table_header")
  cells <- map(rows, xml_find_all, "d1:table_cell")

  cells_rd <- map(cells, \(x) map(x, mdxml_children_to_rd, state = state))
  rows_rd <- map_chr(cells_rd, paste0, collapse = " \\tab ")

  paste0(
    "\\tabular{",
    paste(align, collapse = ""),
    "}{\n",
    paste("  ", rows_rd, "\\cr\n", collapse = ""),
    "}\n"
  )
}

# A list, either bulleted or numbered
mdxml_list <- function(xml, state) {
  type <- xml_attr(xml, "type")
  if (type == "ordered") {
    paste0("\n\\enumerate{", mdxml_children_to_rd(xml, state), "\n}")
  } else {
    paste0("\n\\itemize{", mdxml_children_to_rd(xml, state), "\n}")
  }
}

mdxml_item <- function(xml, state) {
  ## A single item within a list. We remove the first paragraph
  ## tag, to avoid an empty line at the beginning of the first item.
  children <- xml_children(xml)
  if (length(children) == 0) {
    cnts <- ""
  } else if (xml_name(children[[1]]) == "paragraph") {
    cnts <- paste0(
      mdxml_children_to_rd(children[[1]], state),
      paste0(map_chr(children[-1], mdxml_node_to_rd, state), collapse = "")
    )
  } else {
    cnts <- mdxml_children_to_rd(xml, state)
  }
  paste0("\n\\item ", cnts)
}

mdxml_link <- function(xml, state) {
  ## Hyperlink, this can also be a link to a function
  dest <- xml_attr(xml, "destination")
  contents <- xml_contents(xml)

  link <- parse_link(dest, contents, state)

  if (!is.null(link)) {
    link
  } else if (dest == "" || dest == xml_text(xml)) {
    paste0("\\url{", escape_comment(xml_text(xml)), "}")
  } else {
    paste_c(
      c("\\href{", escape_comment(dest), "}"),
      c("{", mdxml_link_text(contents, state), "}")
    )
  }
}

mdxml_link_text <- function(xml_contents, state) {
  # Newlines in markdown get converted to softbreaks/linebreaks by
  # markdown_xml(), which then get interpreted as empty strings by
  # xml_text(). So we preserve newlines as spaces.
  inlink <- state$inlink
  on.exit(state$inlink <- inlink, add = TRUE)
  state$inlink <- TRUE

  text <- map_chr(xml_contents, mdxml_node_to_rd, state)
  paste0(text, collapse = "")
}

mdxml_image <- function(xml) {
  dest <- xml_attr(xml, "destination")
  title <- xml_attr(xml, "title")
  fmt <- get_image_format(dest)
  paste0(
    if (fmt == "html") "\\if{html}{",
    if (fmt == "pdf") "\\if{pdf}{",
    "\\figure{",
    dest,
    "}",
    if (nchar(title)) paste0("{", title, "}"),
    if (fmt %in% c("html", "pdf")) "}"
  )
}

get_image_format <- function(path) {
  should_restrict <- roxy_meta_get("restrict_image_formats") %||% TRUE
  if (!should_restrict) {
    return("all")
  }

  path <- tolower(path)
  rx <- default_image_formats()
  html <- grepl(rx$html, path)
  pdf <- grepl(rx$pdf, path)
  if (html && pdf) {
    "all"
  } else if (html) {
    "html"
  } else if (pdf) {
    "pdf"
  } else {
    "all"
  }
}

default_image_formats <- function() {
  list(
    html = "[.](jpg|jpeg|gif|png|svg)$",
    pdf = "[.](jpg|jpeg|gif|png|pdf)$"
  )
}

escape_comment <- function(x) {
  gsub("%", "\\%", x, fixed = TRUE)
}

mdxml_heading <- function(xml, state) {
  level <- xml_attr(xml, "level")
  if (!state$has_sections && level == 1) {
    warn_roxy_tag(
      state$tag,
      c(
        "markdown translation failed",
        x = "Level 1 headings are not supported in @{state$tag$tag}",
        i = "Do you want to put the heading in @description or @details?"
      )
    )
    return(escape_comment(xml_text(xml)))
  }

  txt <- map_chr(xml_contents(xml), mdxml_node_to_rd, state)
  if (level == 1) {
    state$titles <- c(state$titles, paste(txt, collapse = ""))
  }
  head <- paste0(
    mdxml_close_sections(state, level),
    "\n",
    if (level == 1) state$section_tag else "\\subsection{",
    if (level > 1) paste(txt, collapse = ""),
    if (level > 1) "}{"
  )
  state$section <- c(state$section, level)
  head
}

mdxml_html_block <- function(xml, state) {
  txt <- xml_text(xml)
  txt <- gsub("}", "\\}", txt, fixed = TRUE)
  txt <- gsub("{", "\\{", txt, fixed = TRUE)
  paste0(
    "\\if{html}{\\out{\n",
    txt,
    "}}\n"
  )
}

mdxml_html_inline <- function(xml, state) {
  paste0(
    "\\if{html}{\\out{",
    gsub("}", "\\}", xml_text(xml), fixed = TRUE),
    "}}"
  )
}

mdxml_close_sections <- function(state, upto = 1L) {
  hmy <- 0L
  upto <- max(upto, 2L)
  while (length(state$section) && tail(state$section, 1) >= upto) {
    hmy <- hmy + 1L
    state$section <- head(state$section, -1L)
  }

  paste0(rep("\n}\n", hmy), collapse = "")
}

#' @export
roxy_tag_parse.roxy_tag_md <- function(x) tag_toggle(x)

#' @export
roxy_tag_parse.roxy_tag_noMd <- function(x) tag_toggle(x)

## Hacky global switch - this uses the fact that blocks are parsed
## one after the another, and that we set markdown on/off before each
## block

markdown_env <- new.env(parent = emptyenv())
markdown_on <- function(value = NULL) {
  if (!is.null(value)) {
    assign("markdown-support", isTRUE(value), envir = markdown_env)
  }
  return(isTRUE(markdown_env$`markdown-support`))
}

local_markdown <- function(env = parent.frame()) {
  old <- markdown_env$`markdown-support`
  markdown_on(TRUE)
  withr::defer(markdown_on(old), envir = env)
}

markdown_activate <- function(tags) {
  ## markdown on/off based on global flag and presence of @md & @nomd

  names <- purrr::map_chr(tags, "tag")
  has_md <- "md" %in% names
  has_nomd <- "noMd" %in% names

  if (has_md && has_nomd) {
    md_tag <- tags[names == "md"][[1]]
    warn_roxy_tag(md_tag, "conflicts with @noMd; turning markdown parsing off")

    md <- FALSE
  } else {
    md <- roxy_meta_get("markdown", FALSE)
    if (has_md) md <- TRUE
    if (has_nomd) md <- FALSE
  }

  markdown_on(md)
}

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(comment = "#>", collapse = TRUE)

## ---- echo = FALSE------------------------------------------------------------
example <- "#' Add together two numbers
#'
#' @param x A number.
#' @param y A number.
#' @return A number.
#' @examples
#' add(1, 1)
#' add(10, 1)
add <- function(x, y) {
  x + y
}
"

## ----code=example-------------------------------------------------------------
#' Add together two numbers
#'
#' @param x A number.
#' @param y A number.
#' @return A number.
#' @examples
#' add(1, 1)
#' add(10, 1)
add <- function(x, y) {
  x + y
}


## ---- echo=FALSE, results="asis"----------------------------------------------
cat(
  "\x60\x60\x60rd\n",
  format(roxygen2::roc_proc_text(roxygen2::rd_roclet(), example)[[1]]),
  "\n\x60\x60\x60", sep = ""
)

## ---- eval = FALSE------------------------------------------------------------
#  #' Do values in a numeric vector fall in specified range?
#  #'
#  #' This is a shortcut for `x >= left & x <= right`, implemented
#  #' efficiently in C++ for local values, and translated to the
#  #' appropriate SQL for remote tables.
#  #'
#  #' @param x A numeric vector of values.
#  #' @param left,right Boundary values.
#  #' @export
#  #' @examples
#  #' between(1:12, 7, 9)
#  #'
#  #' x <- rnorm(1e2)
#  #' x[between(x, -1, 1)]
#  between <- function(x, left, right) {
#  }

roxy_eval <- function(expr, env) {
  local_reproducible_output()
  eval(expr, env)
}

roxy_knit <- function(text, envir, options) {
  old_opts <- purrr::exec(opts_chunk$set, options)
  withr::defer(purrr::exec(opts_chunk$set, old_opts))

  local_reproducible_output()
  knit(text = text, quiet = TRUE, envir = envir)
}

# Simplified from testthat::local_reproducible_output
local_reproducible_output <- function(.envir = parent.frame()) {
  withr::local_options(
    crayon.enabled = FALSE,
    cli.unicode = FALSE,
    cli.dynamic = FALSE,
    rlang_interactive = FALSE,
    width = 80,
    .local_envir = .envir
  )
  withr::local_envvar(RSTUDIO = NA, .local_envir = .envir)
  withr::local_collate("C", .local_envir = .envir)
}

rd_r6_class <- function(
  class,
  alias = class,
  superclasses = rd_r6_super(class),
  fields = rd_r6_fields(type = "field"),
  active_bindings = rd_r6_fields(type = "active"),
  methods = rd_r6_methods(alias)
) {
  structure(
    list(
      superclasses = superclasses,
      fields = fields,
      active_bindings = active_bindings,
      methods = methods
    ),
    class = "rd_r6_class"
  )
}

#' @export
format.rd_r6_class <- function(x, ...) {
  c(
    format(x$superclasses),
    format(x$fields),
    format(x$active_bindings),
    format(x$methods)
  )
}

# Extraction ---------------------------------------------------------------

r6_class_from_block <- function(block, env) {
  r6data <- block_get_tag_value(block, ".r6data")
  class <- block$object$value$classname
  alias <- block$object$alias

  rd_r6_class(
    class = class,
    alias = alias,
    superclasses = r6_extract_superclasses(r6data, env, class),
    fields = r6_extract_field_tags(block, r6data, type = "field"),
    active_bindings = r6_extract_field_tags(block, r6data, type = "active"),
    methods = r6_extract_methods(r6data, alias, block)
  )
}

#' Check if documentation needs to be updated
#'
#' @description
#' A lightweight check that compares modification times of `.Rd` files in
#' `man/` with the source files listed in their backrefs. This is much faster
#' than running [roxygenize()] but can suffer from both false negatives (e.g.
#' if an inherited documentation topic has changed) and false positives (e.g.
#' if a source file was modified but the change doesn't affect the
#' documentation).
#'
#' @inheritParams roxygenize
#' @param quiet If `TRUE`, suppresses the message listing out-of-date
#'   man pages.
#' @return A logical value, invisibly. `TRUE` if any man pages appear
#'   to be out of date; `FALSE` otherwise.
#' @export
#' @examples
#' \dontrun{
#' needs_roxygenize()
#' }
needs_roxygenize <- function(package.dir = ".", quiet = FALSE) {
  check_string(package.dir)
  check_bool(quiet)

  base_path <- normalizePath(package.dir)
  man_path <- file.path(base_path, "man")
  rd_files <- sort(dir(man_path, pattern = "\\.Rd$", full.names = TRUE))
  outdated <- map_lgl(rd_files, rd_outdated, base_path = base_path)
  out_of_date <- basename(rd_files[outdated])

  if (length(out_of_date) > 0 && !quiet) {
    cli::cli_inform(c(
      "!" = "{length(out_of_date)} man page{?s} may be out of date:",
      set_names(out_of_date, "*")
    ))
  }

  invisible(length(out_of_date) > 0)
}

rd_outdated <- function(rd_file, base_path) {
  lines <- read_lines(rd_file, n = 10)
  source_files <- rd_backref_sources(lines, base_path)
  if (length(source_files) == 0) {
    return(FALSE)
  }

  if (!all(file.exists(source_files))) {
    return(TRUE)
  }

  rd_mtime <- file.mtime(rd_file)
  source_mtimes <- file.mtime(source_files)
  any(source_mtimes > rd_mtime)
}

rd_backref_sources <- function(lines, base_path) {
  if (length(lines) == 0 || !check_made_by(lines[[1]])) {
    return(character())
  }

  regexp <- "^%\\s*(Please edit documentation in|  )"
  lines <- lines[grepl(regexp, lines)]
  if (length(lines) == 0) {
    return(character())
  }

  lines <- gsub("^%(\\s*Please edit documentation in)?", "", lines)
  files <- unlist(strsplit(lines, ","))
  files <- trimws(files)
  files <- files[nzchar(files)]

  file.path(base_path, files)
}

test_that("needs_roxygenize returns FALSE when all docs are current", {
  path <- local_package_copy(test_path("up-to-date"))
  suppressMessages(roxygenize(path))

  expect_false(needs_roxygenize(path))
})

test_that("needs_roxygenize returns TRUE when source is newer", {
  path <- local_package_copy(test_path("up-to-date"))
  suppressMessages(roxygenize(path))

  # Backdate the Rd file so source appears newer
  Sys.setFileTime(file.path(path, "man", "foo.Rd"), as.POSIXct("2000-01-01"))

  expect_snapshot(out <- needs_roxygenize(path))
  expect_true(out)
})

# rd_outdated ------------------------------------------------------------------

test_that("rd_outdated returns TRUE when source file is deleted", {
  path <- local_package_copy(test_path("up-to-date"))
  suppressMessages(roxygenize(path))

  file.remove(file.path(path, "R", "foo.R"))
  expect_true(rd_outdated(file.path(path, "man", "foo.Rd"), path))
})

# rd_backref_sources -----------------------------------------------------------

test_that("rd_backref_sources parses single source file", {
  lines <- c(
    "% Generated by roxygen2: do not edit by hand",
    "% Please edit documentation in R/foo.R",
    "\\name{foo}"
  )
  expect_equal(rd_backref_sources(lines, "/base"), "/base/R/foo.R")
})

test_that("rd_backref_sources parses multiple source files", {
  lines <- c(
    "% Generated by roxygen2: do not edit by hand",
    "% Please edit documentation in R/foo.R, R/bar.R",
    "\\name{foo}"
  )
  expect_equal(
    rd_backref_sources(lines, "/base"),
    c("/base/R/foo.R", "/base/R/bar.R")
  )
})

test_that("rd_backref_sources handles wrapped backref lines", {
  lines <- c(
    "% Generated by roxygen2: do not edit by hand",
    "% Please edit documentation in R/foo.R,",
    "%   R/bar.R",
    "\\name{foo}"
  )
  expect_equal(
    rd_backref_sources(lines, "/base"),
    c("/base/R/foo.R", "/base/R/bar.R")
  )
})

test_that("rd_backref_sources handles missing source file", {
  lines <- c(
    "% Generated by roxygen2: do not edit by hand",
    "\\name{foo}"
  )
  expect_equal(rd_backref_sources(lines, "/base"), character())
})

#' @export
roxy_tag_parse.roxy_tag_section <- function(x) {
  tag_markdown(x)
}

#' @export
roxy_tag_rd.roxy_tag_section <- function(x, base_path, env) {
  pieces <- split_section_title(x$val)
  title <- strsplit(pieces[1], "\n", fixed = TRUE)[[1]]

  if (length(title) > 1) {
    warn_roxy_tag(
      x,
      c(
        "title spans multiple lines.",
        i = "Did you forget a colon (:) at the end of the title?"
      )
    )
    return()
  }

  rd_section_section(pieces[1], pieces[2])
}

# Split a section into title and content at the first `:`. Colons inside `{}`
# are ignored so that titles can contain code like `\code{base::split()}`
# (#1878).
split_section_title <- function(x) {
  chars <- strsplit(x, "", fixed = TRUE)[[1]]
  depth <- 0L
  escaped <- FALSE

  for (i in seq_along(chars)) {
    char <- chars[[i]]
    if (escaped) {
      escaped <- FALSE
    } else if (char == "\\") {
      escaped <- TRUE
    } else if (char == "{") {
      depth <- depth + 1L
    } else if (char == "}") {
      depth <- depth - 1L
    } else if (char == ":" && depth == 0L) {
      return(c(substr(x, 1, i - 1L), substr(x, i + 1L, nchar(x))))
    }
  }
  c(x, "")
}

rd_section_section <- function(title, content) {
  check_character(title)
  check_character(content)
  stopifnot(length(title) == length(content))

  rd_section("section", list(title = title, content = content))
}

#' @export
format.rd_section_section <- function(x, ...) {
  paste0(
    "\\section{",
    x$value$title,
    "}{\n",
    x$value$content,
    "\n}\n",
    collapse = "\n"
  )
}

#' @export
merge.rd_section_section <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))

  dedup <- collapse(
    c(x$value$title, y$value$title),
    c(x$value$content, y$value$content),
    paste,
    collapse = "\n\n"
  )
  rd_section("section", list(title = dedup$key, content = unlist(dedup$value)))
}

#' @export
#' @rdname roxy_tag
warn_roxy_tag <- function(tag, message, parent = NULL, envir = parent.frame()) {
  tag_name <- cli::format_inline("{.strong @{tag$tag}} ")
  if (is.null(tag$raw)) tag_name <- paste(tag_name, "(automatically generated) ")
  message[[1]] <- paste0(tag_name, message[[1]])

  warn_roxy(tag$file, tag$line, message, parent = parent, envir = envir)
}

warn_roxy_block <- function(block, message, parent = NULL, envir = parent.frame()) {
  warn_roxy(block$file, block$line, message, parent = parent, envir = envir)
}

warn_roxy_function <- function(fun, message, parent = NULL, envir = parent.frame()) {
  srcref <- attr(fun, "srcref")
  file <- attr(srcref, "srcfile")$filename
  line <- as.vector(srcref)[[1]]

  warn_roxy(file, line, message, parent = parent, envir = envir)
}

warn_roxy <- function(file, line, message, parent = NULL, envir = parent.frame()) {
  link <- cli::style_hyperlink(
    paste0(basename(file), ":", line),
    paste0("file://", file),
    params = c(line = line, col = 1)
  )

  message[[1]] <- paste0(link, ": ", message[[1]], ".")
  names(message)[[1]] <- "x"
  cli::cli_inform(message, parent = parent, .envir = envir)
}

warn_roxy_topic <- function(topic, message, parent = NULL, envir = parent.frame()) {
  message[[1]] <- paste0("In topic '", topic, "': ", message[[1]], ".")
  names(message)[[1]] <- "x"
  cli::cli_inform(message, parent = parent, .envir = envir)
}

package_seealso <- function(URL, BugReports) {
  itemize("Useful links:", package_seealso_urls(URL, BugReports))
}

package_seealso_urls <- function(URL = NULL, BugReports = NULL) {
  if (!is.null(URL)) {
    links <- strsplit(URL, ",\\s+")[[1]]
    links <- map_chr(links, wrap_urls)
    links <- gsub("\\url\\{https://doi.org/", "\\doi{", links)
  } else {
    links <- character()
  }
  if (!is.null(BugReports)) {
    links <- c(links, paste0("Report bugs at \\url{", escape(BugReports), "}"))
  }

  links
}

wrap_urls <- function(x) {
  gsub(
    "(https?://[^\\s,]+|ftp://[^\\s,]+)",
    "\\\\url{\\1}",
    escape(x),
    perl = TRUE
  )
}

package_authors <- function(authors) {
  authors <- tryCatch(eval(parse(text = authors %||% "")), error = function(e) {
    cli::cli_inform(c(x = "Failed to evaluate Authors@R."), parent = e)
    NULL
  })
  if (is.null(authors)) {
    return()
  }

  desc <- map_chr(unclass(authors), author_desc)
  type <- map_chr(unclass(authors), author_type)
  # People who are both maintainer and author should appear in both sections
  is_cre_aut <- map_lgl(unclass(authors), \(x) all(c("cre", "aut") %in% x$role))
  by_type <- split(desc, type)
  by_type$aut <- c(desc[is_cre_aut], by_type$aut)

  paste(
    c(
      paste0("\\strong{Maintainer}: ", by_type$cre[[1]], "\n"),
      itemize("Authors:", by_type$aut),
      itemize("Other contributors:", by_type$other)
    ),
    collapse = "\n"
  )
}

author_desc <- function(x) {
  if (inherits(x, "person")) {
    cli::cli_abort("Person class must be stripped.", .internal = FALSE)
  }

  desc <- paste0(x$given, collapse = " ")

  if (!is.null(x$family)) {
    desc <- paste0(desc, " ", paste0(x$family, collapse = " "))
  }

  if (!is.null(x$email)) {
    desc <- paste0(desc, " ", paste0("\\email{", x$email, "}", collapse = " "))
  }

  if (!is.null(x$comment)) {
    if (has_name(x$comment, "ORCID")) {
      orcid <- x$comment[["ORCID"]]

      if (grepl("https?://", orcid)) {
        desc <- paste0(desc, " (\\href{", orcid, "}{ORCID})")
      } else {
        desc <- paste0(desc, " (\\href{https://orcid.org/", orcid, "}{ORCID})")
      }
      x$comment <- x$comment[!names(x$comment) %in% "ORCID"]
    }
    if (has_name(x$comment, "ROR")) {
      ror <- x$comment[["ROR"]]

      if (grepl("https?://", ror)) {
        desc <- paste0(desc, " (\\href{", ror, "}{ROR})")
      } else {
        desc <- paste0(desc, " (\\href{https://ror.org/", ror, "}{ROR})")
      }
      x$comment <- x$comment[!names(x$comment) %in% "ROR"]
    }

    if (length(x$comment) > 0) {
      sep <- ifelse(nzchar(names2(x$comment)), ": ", "")
      comments <- paste0(names2(x$comment), sep, x$comment)
      desc <- paste0(desc, " (", paste(comments, collapse = ", "), ")")
    }
  }

  extra_roles <- setdiff(x$role, c("cre", "aut"))
  if (length(extra_roles) > 0) {
    desc <- paste0(
      desc,
      " [",
      paste0(role_lookup[extra_roles], collapse = ", "),
      "]"
    )
  }

  desc
}

author_type <- function(x) {
  if ("cre" %in% x$role) {
    "cre"
  } else if ("aut" %in% x$role) {
    "aut"
  } else {
    "other"
  }
}

role_lookup <- c(
  "abr" = "abridger",
  "act" = "actor",
  "adp" = "adapter",
  "rcp" = "addressee",
  "anl" = "analyst",
  "anm" = "animator",
  "ann" = "annotator",
  "apl" = "appellant",
  "ape" = "appellee",
  "app" = "applicant",
  "arc" = "architect",
  "arr" = "arranger",
  "acp" = "art copyist",
  "adi" = "art director",
  "art" = "artist",
  "ard" = "artistic director",
  "asg" = "assignee",
  "asn" = "associated name",
  "att" = "attributed name",
  "auc" = "auctioneer",
  "aut" = "author",
  "aqt" = "author in quotations or text abstracts",
  "aft" = "author of afterword, colophon, etc.",
  "aud" = "author of dialog",
  "aui" = "author of introduction, etc.",
  "ato" = "autographer",
  "ant" = "bibliographic antecedent",
  "bnd" = "binder",
  "bdd" = "binding designer",
  "blw" = "blurb writer",
  "bkd" = "book designer",
  "bkp" = "book producer",
  "bjd" = "bookjacket designer",
  "bpd" = "bookplate designer",
  "bsl" = "bookseller",
  "brl" = "braille embosser",
  "brd" = "broadcaster",
  "cll" = "calligrapher",
  "ctg" = "cartographer",
  "cas" = "caster",
  "cns" = "censor",
  "chr" = "choreographer",
  "cng" = "cinematographer",
  "cli" = "client",
  "cor" = "collection registrar",
  "col" = "collector",
  "clt" = "collotyper",
  "clr" = "colorist",
  "cmm" = "commentator",
  "cwt" = "commentator for written text",
  "com" = "compiler",
  "cpl" = "complainant",
  "cpt" = "complainant-appellant",
  "cpe" = "complainant-appellee",
  "cmp" = "composer",
  "cmt" = "compositor",
  "ccp" = "conceptor",
  "cnd" = "conductor",
  "con" = "conservator",
  "csl" = "consultant",
  "csp" = "consultant to a project",
  "cos" = "contestant",
  "cot" = "contestant-appellant",
  "coe" = "contestant-appellee",
  "cts" = "contestee",
  "ctt" = "contestee-appellant",
  "cte" = "contestee-appellee",
  "ctr" = "contractor",
  "ctb" = "contributor",
  "cpc" = "copyright claimant",
  "cph" = "copyright holder",
  "crr" = "corrector",
  "crp" = "correspondent",
  "cst" = "costume designer",
  "cou" = "court governed",
  "crt" = "court reporter",
  "cov" = "cover designer",
  "cre" = "creator",
  "cur" = "curator",
  "dnc" = "dancer",
  "dtc" = "data contributor",
  "dtm" = "data manager",
  "dte" = "dedicatee",
  "dto" = "dedicator",
  "dfd" = "defendant",
  "dft" = "defendant-appellant",
  "dfe" = "defendant-appellee",
  "dgg" = "degree granting institution",
  "dgs" = "degree supervisor",
  "dln" = "delineator",
  "dpc" = "depicted",
  "dpt" = "depositor",
  "dsr" = "designer",
  "drt" = "director",
  "dis" = "dissertant",
  "dbp" = "distribution place",
  "dst" = "distributor",
  "dnr" = "donor",
  "drm" = "draftsman",
  "dub" = "dubious author",
  "edt" = "editor",
  "edc" = "editor of compilation",
  "edm" = "editor of moving image work",
  "elg" = "electrician",
  "elt" = "electrotyper",
  "enj" = "enacting jurisdiction",
  "eng" = "engineer",
  "egr" = "engraver",
  "etr" = "etcher",
  "evp" = "event place",
  "exp" = "expert",
  "fac" = "facsimilist",
  "fld" = "field director",
  "fmd" = "film director",
  "fds" = "film distributor",
  "flm" = "film editor",
  "fmp" = "film producer",
  "fmk" = "filmmaker",
  "fpy" = "first party",
  "frg" = "forger",
  "fmo" = "former owner",
  "fnd" = "funder",
  "gis" = "geographic information specialist",
  "hnr" = "honoree",
  "hst" = "host",
  "his" = "host institution",
  "ilu" = "illuminator",
  "ill" = "illustrator",
  "ins" = "inscriber",
  "itr" = "instrumentalist",
  "ive" = "interviewee",
  "ivr" = "interviewer",
  "inv" = "inventor",
  "isb" = "issuing body",
  "jud" = "judge",
  "jug" = "jurisdiction governed",
  "lbr" = "laboratory",
  "ldr" = "laboratory director",
  "lsa" = "landscape architect",
  "led" = "lead",
  "len" = "lender",
  "lil" = "libelant",
  "lit" = "libelant-appellant",
  "lie" = "libelant-appellee",
  "lel" = "libelee",
  "let" = "libelee-appellant",
  "lee" = "libelee-appellee",
  "lbt" = "librettist",
  "lse" = "licensee",
  "lso" = "licensor",
  "lgd" = "lighting designer",
  "ltg" = "lithographer",
  "lyr" = "lyricist",
  "mfp" = "manufacture place",
  "mfr" = "manufacturer",
  "mrb" = "marbler",
  "mrk" = "markup editor",
  "med" = "medium",
  "mdc" = "metadata contact",
  "mte" = "metal-engraver",
  "mtk" = "minute taker",
  "mod" = "moderator",
  "mon" = "monitor",
  "mcp" = "music copyist",
  "msd" = "musical director",
  "mus" = "musician",
  "nrt" = "narrator",
  "osp" = "onscreen presenter",
  "opn" = "opponent",
  "orm" = "organizer",
  "org" = "originator",
  "oth" = "other",
  "own" = "owner",
  "pan" = "panelist",
  "ppm" = "papermaker",
  "pta" = "patent applicant",
  "pth" = "patent holder",
  "pat" = "patron",
  "prf" = "performer",
  "pma" = "permitting agency",
  "pht" = "photographer",
  "ptf" = "plaintiff",
  "ptt" = "plaintiff-appellant",
  "pte" = "plaintiff-appellee",
  "plt" = "platemaker",
  "pra" = "praeses",
  "pre" = "presenter",
  "prt" = "printer",
  "pop" = "printer of plates",
  "prm" = "printmaker",
  "prc" = "process contact",
  "pro" = "producer",
  "prn" = "production company",
  "prs" = "production designer",
  "pmn" = "production manager",
  "prd" = "production personnel",
  "prp" = "production place",
  "prg" = "programmer",
  "pdr" = "project director",
  "pfr" = "proofreader",
  "prv" = "provider",
  "pup" = "publication place",
  "pbl" = "publisher",
  "pbd" = "publishing director",
  "ppt" = "puppeteer",
  "rdd" = "radio director",
  "rpc" = "radio producer",
  "rce" = "recording engineer",
  "rcd" = "recordist",
  "red" = "redaktor",
  "ren" = "renderer",
  "rpt" = "reporter",
  "rps" = "repository",
  "rth" = "research team head",
  "rtm" = "research team member",
  "res" = "researcher",
  "rsp" = "respondent",
  "rst" = "respondent-appellant",
  "rse" = "respondent-appellee",
  "rpy" = "responsible party",
  "rsg" = "restager",
  "rsr" = "restorationist",
  "rev" = "reviewer",
  "rbr" = "rubricator",
  "sce" = "scenarist",
  "sad" = "scientific advisor",
  "aus" = "screenwriter",
  "scr" = "scribe",
  "scl" = "sculptor",
  "spy" = "second party",
  "sec" = "secretary",
  "sll" = "seller",
  "std" = "set designer",
  "stg" = "setting",
  "sgn" = "signer",
  "sng" = "singer",
  "sds" = "sound designer",
  "spk" = "speaker",
  "spn" = "sponsor",
  "sgd" = "stage director",
  "stm" = "stage manager",
  "stn" = "standards body",
  "str" = "stereotyper",
  "stl" = "storyteller",
  "sht" = "supporting host",
  "srv" = "surveyor",
  "tch" = "teacher",
  "tcd" = "technical director",
  "tld" = "television director",
  "tlp" = "television producer",
  "ths" = "thesis advisor",
  "trc" = "transcriber",
  "trl" = "translator",
  "tyd" = "type designer",
  "tyg" = "typographer",
  "uvp" = "university place",
  "vdg" = "videographer",
  "vac" = "voice actor",
  "wit" = "witness",
  "wde" = "wood engraver",
  "wdc" = "woodcutter",
  "wam" = "writer of accompanying material",
  "wac" = "writer of added commentary",
  "wal" = "writer of added lyrics",
  "wat" = "writer of added text",
  "win" = "writer of introduction",
  "wpr" = "writer of preface",
  "wst" = "writer of supplementary textual content"
)

itemize <- function(header, x) {
  if (length(x) == 0) {
    return()
  }

  paste0(
    header,
    "\n",
    "\\itemize{\n",
    paste0("  \\item ", x, "\n", collapse = ""),
    "}\n"
  )
}

package_url_parse <- function(x) {
  # <doi:XX.XXX> -> \doi{XX.XXX} to avoid CRAN Notes, etc.
  # URL-decode %XX sequences because \doi{} is fully verbatim:
  # raw % starts an Rd comment and \% is not supported (#1321)
  x <- re_replace_all(x, "<(doi|DOI):(.*?)>", function(match) {
    match <- gsub("^<(doi|DOI):|>$", "", match)
    match <- utils::URLdecode(match)
    paste0("\\doi{", match, "}")
  })

  # <http:XX.XXX> -> \url{http:XX.XXX}
  x <- re_replace_all(x, "<(http|https):\\/\\/(.*?)>", function(match) {
    match <- gsub("^<|>$", "", match)
    paste0("\\url{", escape(match), "}")
  })

  # <arxiv:XXX> -> \href{https://arxiv.org/abs/XXX}{arXiv:XXX}
  # https://github.com/wch/r-source/blob/trunk/src/library/tools/R/Rd2pdf.R#L149-L151
  patt_arxiv <- "<(arXiv:|arxiv:)([[:alnum:]/.-]+)([[:space:]]*\\[[^]]+\\])?>"
  x <- re_replace_all(x, patt_arxiv, function(match) {
    match <- gsub("^<(arXiv:|arxiv:)|>$", "", match)
    # Special cases has <arxiv:id [code]>.
    # See https://CRAN.R-project.org/package=ciccr
    # Extract arxiv id, split by space
    arxiv_id <- re_split_half(match, " ")[[1]]

    paste0(
      "\\href{https://arxiv.org/abs/",
      escape(arxiv_id),
      "}{arXiv:",
      match,
      "}"
    )
  })

  x
}

test_that("person turned into meaningful text", {
  person_desc <- function(
    given = "H",
    family = "W",
    email = "h@w.com",
    role = "aut",
    comment = NULL
  ) {
    out <- person(
      given = given,
      family = family,
      email = email,
      role = role,
      comment = comment
    )
    author_desc(unclass(out)[[1]])
  }

  expect_snapshot({
    "Multiple given/family names"
    person_desc(c("First", "Second"), c("Family1", "Family2"))

    "Multiple roles"
    person_desc(role = "ctb")

    "ORCID comments"
    person_desc(comment = c("ORCID" = "0000-0003-4757-117X"))
    person_desc(comment = c("ORCID" = "https://orcid.org/0000-0003-4757-117X"))
    person_desc(comment = c("ORCID" = "0000-0003-4757-117X", "extra"))

    "ROR comments"
    person_desc(comment = c("ROR" = "03wc8by49"))
    person_desc(comment = c("ROR" = "https://ror.org/03wc8by49"))
    person_desc(comment = c("ROR" = "03wc8by49", "extra"))

    "Arbitrary comments (#1746)"
    person_desc(comment = c(acronym = "rajo", "Contributed XY"))
    person_desc(comment = c("comment 1", "comment 2"))
  })
})

test_that("cre+aut person appears in both maintainer and authors (#1588)", {
  authors <- 'c(
    person("A", "B", role = c("aut", "cre"), email = "a@b.com"),
    person("C", "D", role = "aut")
  )'
  out <- package_authors(authors)
  expect_match(out, "Maintainer.*A B")
  expect_match(out, "Authors.*A B")
  expect_match(out, "Authors.*C D")
})

test_that("useful message if Authors@R is corrupted", {
  expect_snapshot({
    package_authors("1 + ")
    package_authors("stop('Uhoh')")
  })
})

test_that("can convert quote percentage signs in urls", {
  expect_equal(
    package_seealso_urls("https://www.foo.bar/search?q=see%20also"),
    "\\url{https://www.foo.bar/search?q=see\\%20also}"
  )

  expect_equal(
    package_seealso_urls(
      BugReports = "https://www.foo.bar/search?q=bug%20report"
    ),
    "Report bugs at \\url{https://www.foo.bar/search?q=bug\\%20report}"
  )
})

test_that("can convert DOIs in url", {
  expect_equal(
    package_seealso_urls("https://doi.org/10.5281/zenodo.1485309"),
    "\\doi{10.5281/zenodo.1485309}"
  )
})

test_that("annotated URLs are extracted correctly (#1420)", {
  expect_equal(
    package_seealso_urls("https://x (XYZ), https://u (ABC)"),
    c("\\url{https://x} (XYZ)", "\\url{https://u} (ABC)")
  )
})

test_that("can autolink urls on package Description", {
  expect_equal(
    package_url_parse("x <https://x.com> y"),
    "x \\url{https://x.com} y"
  )
  expect_equal(
    package_url_parse("x <https://x.com/%3C-%3E> y"),
    "x \\url{https://x.com/\\%3C-\\%3E} y"
  )
})

test_that("can autolink DOIs", {
  expect_equal(package_url_parse("x <doi:abcdef> y"), "x \\doi{abcdef} y")
  expect_equal(package_url_parse("x <DOI:abcdef> y"), "x \\doi{abcdef} y")
  # URL-decodes %XX sequences since \doi{} can't contain % (#1321)
  expect_equal(package_url_parse("x <DOI:%3C-%3E> y"), "x \\doi{<->} y")
  expect_equal(
    package_url_parse(
      "x <doi:10.1175/1520-0469(1996)053%3C2365:PORWON%3E2.0.CO;2>"
    ),
    "x \\doi{10.1175/1520-0469(1996)053<2365:PORWON>2.0.CO;2}"
  )
})

test_that("can autolink arxiv", {
  expect_equal(
    package_url_parse("x <arXiv:abc> y"),
    "x \\href{https://arxiv.org/abs/abc}{arXiv:abc} y"
  )
  expect_equal(
    package_url_parse("x <arxiv:abc> y"),
    "x \\href{https://arxiv.org/abs/abc}{arXiv:abc} y"
  )
  expect_equal(
    package_url_parse("x <arxiv:abc [def]> y"),
    "x \\href{https://arxiv.org/abs/abc}{arXiv:abc [def]} y"
  )
})

test_that("autolink several matching patterns", {
  text <- "url <http://a.com> doi <doi:xx> arxiv <arXiv:xx>"
  expect_equal(
    package_url_parse(text),
    paste(
      "url \\url{http://a.com}",
      "doi \\doi{xx}",
      "arxiv \\href{https://arxiv.org/abs/xx}{arXiv:xx}"
    )
  )
})

test_that("multiple email addresses for a person are acceptable #1487", {
  me <- person("me", email = c("one@email.me", "two@email.me"))
  expect_equal(
    author_desc(unclass(me)[[1]]),
    "me \\email{one@email.me} \\email{two@email.me}"
  )
})

rd_r6_inherited <- function(
  package = character(),
  classname = character(),
  name = character()
) {
  structure(
    list(package = package, classname = classname, name = name),
    class = "rd_r6_inherited"
  )
}

#' @export
format.rd_r6_inherited <- function(x, ...) {
  if (length(x$name) == 0) {
    return()
  }

  details <- paste0(
    "<details",
    if (length(x$name) <= 5) " open",
    "><summary>Inherited methods</summary>"
  )

  # Which parent classes are documented?
  cls <- unique(x$classname)
  pkgs <- x$package[match(cls, x$classname)]
  ht <- map2_lgl(cls, pkgs, has_topic)
  topic_ok <- ht[match(x$classname, cls)]

  # Build display components for each inherited method
  anchor <- sprintf("method-%s-%s", x$classname, x$name)
  self_pkg <- roxy_meta_get("current_package") %||% ""
  same_pkg <- x$package == self_pkg
  prefix <- ifelse(same_pkg, "", paste0(x$package, "::"))
  label <- sprintf("%s%s$%s()", prefix, x$classname, x$name)

  # Linked version: clickable reference to parent class docs
  href <- sprintf("../../%s/html/%s.html#%s", x$package, x$classname, anchor)
  data_attrs <- paste(
    sprintf('data-pkg="%s"', x$package),
    sprintf('data-topic="%s"', x$classname),
    sprintf('data-id="%s"', x$name)
  )
  linked <- sprintf(
    '<span class="pkg-link" %s><a href=\'%s\'><code>%s</code></a></span>',
    data_attrs,
    href,
    label
  )

  # Plain version: just the method name, no link
  plain <- sprintf("<code>%s</code>", label)
  items <- paste0("  <li>", ifelse(topic_ok, linked, plain), "</li>")

  rd_if_html(paste(
    c(details, "<ul>", items, "</ul>", "</details>"),
    collapse = "\n"
  ))
}

r6_extract_inherited_methods <- function(r6data) {
  super <- r6data$super
  if (is.null(super)) {
    return(rd_r6_inherited())
  }

  super_meth <- super$members[super$members$type == "method", ]
  self <- r6data$self
  super_meth <- super_meth[!super_meth$name %in% self$name, ]
  super_meth <- super_meth[!duplicated(super_meth$name), ]
  if (nrow(super_meth) == 0) {
    return(rd_r6_inherited())
  }

  super_meth <- super_meth[rev(seq_len(nrow(super_meth))), ]
  rd_r6_inherited(
    package = super_meth$package,
    classname = super_meth$classname,
    name = super_meth$name
  )
}

#' Process a package with the Rd, namespace and collate roclets
#'
#' This is the workhorse function that uses roclets, the built-in document
#' transformation functions, to build all documentation for a package. See
#' the documentation for the individual roclets, [rd_roclet()],
#' [namespace_roclet()], and for [update_collate()],
#' for more details.
#'
#' Note that roxygen2 is a dynamic documentation system: it works by
#' inspecting loaded objects in the package. This means that you must
#' be able to load the package in order to document it: see [load] for
#' details.
#'
#' @param package.dir Location of package top level directory. Default is
#'   working directory.
#' @param roclets Character vector of roclet names to use with package.
#'   The default, `NULL`, uses the roxygen `roclets` option,
#'   which defaults to `c("collate", "namespace", "rd")`.
#' @param load_code A function used to load all the R code in the package
#'   directory. The default, `NULL`, uses the strategy defined by
#'   the `load` roxygen option, which defaults to [load_pkgload()].
#'   See [load] for more details.
#' @param clean If `TRUE`, roxygen will delete all files previously
#'   created by roxygen before running each roclet.
#' @return `NULL`
#' @export
roxygenize <- function(package.dir = ".",
                       roclets = NULL,
                       load_code = NULL,
                       clean = FALSE) {

  base_path <- normalizePath(package.dir)
  is_first <- roxygen_setup(base_path)

  roxy_meta_load(base_path)
  # Load required packages for method registration
  packages <- roxy_meta_get("packages")
  lapply(packages, loadNamespace)

  roclets <- roclets %||% roxy_meta_get("roclets")

  # To load code, we need a up-to-date Collate field and NAMESPACE
  if ("collate" %in% roclets) {
    update_collate(base_path)
    roclets <- setdiff(roclets, "collate")
  }
  if ("namespace" %in% roclets) {
    update_namespace_imports(base_path)
  }

  if (length(roclets) == 0)
    return(invisible())

  roclets <- lapply(roclets, roclet_find)

  if (!is_interactive()) {
    withr::local_options(warn = 1)
  }

  # Now load code
  load_code <- find_load_strategy(load_code)
  env <- load_code(base_path)
  local_roxy_meta_set("env", env)

  # Tokenise each file
  blocks <- parse_package(base_path, env = NULL)

  if (clean) {
    purrr::walk(roclets, roclet_clean, base_path = base_path)
  }

  roclets <- lapply(roclets, roclet_preprocess,
    blocks = blocks,
    base_path = base_path
  )

  blocks <- lapply(blocks, block_set_env, env = env)

  results <- lapply(roclets, roclet_process,
    blocks = blocks,
    env = env,
    base_path = base_path
  )

  out <- purrr::map2(
    roclets, results,
    roclet_output,
    base_path = base_path,
    is_first = is_first
  )
  invisible(out)
}

#' @rdname roxygenize
#' @export
roxygenise <- roxygenize

rd_r6_methods <- function(alias, self = list(), inherited = rd_r6_inherited()) {
  structure(
    list(alias = alias, self = self, inherited = inherited),
    class = "rd_r6_methods"
  )
}

#' @export
format.rd_r6_methods <- function(x, ...) {
  if (length(x$self) == 0) {
    return()
  }

  lines <- character()
  push <- function(...) lines <<- c(lines, ...)

  nms <- map_chr(x$self, \(m) m$name)
  classes <- map_chr(x$self, \(m) m$class)
  dest <- sprintf("method-%s-%s", classes, nms)
  code <- sprintf("\\code{%s()}", r6_method_name(classes, nms))

  push("\\section{Methods}{")
  push(
    "\\subsection{Public methods}{",
    "  \\itemize{",
    sprintf("    \\item \\href{#%s}{%s}", dest, code),
    "  }",
    "}"
  )
  push(format(x$inherited))
  for (method in x$self) {
    push(format(method))
  }
  push("}")

  lines
}

r6_extract_methods <- function(r6data, alias, block) {
  self <- r6data$self
  methods_df <- self[self$type == "method", ]
  methods_df <- methods_df[order(methods_df$file, methods_df$line), ]
  methods_df$tags <- replicate(nrow(methods_df), list(), simplify = FALSE)

  # Move method tags to methods data structure
  for (i in seq_along(block$tags)) {
    tag <- block$tags[[i]]
    if (r6_tag_type(tag, block) != "method") {
      next
    }

    # Tags from external blocks already have method name; ow need to find
    meth <- tag$r6method %||% find_method_for_tag(methods_df, tag)

    midx <- which(meth == methods_df$name)
    if (length(midx) == 0) {
      warn_roxy_tag(tag, "can't find matching R6 method")
      next
    }
    methods_df$tags[[midx]] <- c(methods_df$tags[[midx]], list(tag))
  }

  # Flatten markdown sections
  for (i in seq_along(methods_df$tags)) {
    methods_df$tags[[i]] <- lapply(methods_df$tags[[i]], r6_flatten_sections)
  }

  methods_df <- add_default_methods(methods_df, block)

  nodoc <- map_int(methods_df$tags, length) == 0
  if (any(nodoc)) {
    warn_roxy_block(
      block,
      "Undocumented R6 method{?s}: {methods_df$name[nodoc]}"
    )
  }

  # Methods with @noRd are deliberately suppressed
  has_noRd <- map_lgl(methods_df$tags, function(tags) {
    any(map_lgl(tags, \(t) t$tag == "noRd"))
  })
  methods_df <- methods_df[!has_noRd, ]

  self_methods <- lapply(
    seq_len(nrow(methods_df)),
    function(i) r6_method_from_row(methods_df[i, ], block)
  )
  inherited <- r6_extract_inherited_methods(r6data)
  rd_r6_methods(alias, self = self_methods, inherited = inherited)
}

add_default_methods <- function(methods, block) {
  defaults <- list(
    clone = list(
      roxy_generated_tag(
        block,
        "description",
        "The objects of this class are cloneable with this method."
      ),
      roxy_generated_tag(
        block,
        "param",
        list(name = "deep", description = "Whether to make a deep clone.")
      )
    )
  )

  for (mname in names(defaults)) {
    mline <- match(mname, methods$name)
    if (is.na(mline)) {
      next
    }
    if (length(methods$tags[[mline]]) > 0) {
      next
    }
    methods$tags[[mline]] <- defaults[[mname]]
  }

  methods
}

find_method_for_tag <- function(methods, tag) {
  if (nrow(methods) == 0) {
    return(NA_character_)
  }
  if (tag$file == "<text>") {
    # for testing
    same_file <- TRUE
  } else {
    same_file <- basename(methods$file) == basename(tag$file)
  }
  w <- which(same_file & methods$line > tag$line)[1]
  methods$name[w]
}

r6_flatten_sections <- function(tag) {
  if (!tag$tag %in% c("description", "details")) {
    return(tag)
  }
  if (length(tag$val) <= 1) {
    return(tag)
  }
  titles <- names(tag$val)
  sections <- map_chr(
    seq_along(tag$val)[-1],
    \(i) paste0("\\subsection{", titles[[i]], "}{\n", tag$val[[i]], "\n}")
  )
  parts <- if (nzchar(tag$val[[1]])) c(tag$val[[1]], sections) else sections
  tag$val <- paste(parts, collapse = "\n\n")
  tag
}

r6_all_examples <- function(methods) {
  unlist(lapply(methods$self, function(method) {
    if (length(method$examples) == 0) {
      return()
    }
    c(
      "\n## ------------------------------------------------",
      paste0("## Method `", r6_method_name(method$class, method$name), "()`"),
      "## ------------------------------------------------\n",
      paste(method$examples, collapse = "\n")
    )
  }))
}

test_that("format.rd_r6_methods with one method", {
  methods <- rd_r6_methods(
    "Foo",
    self = list(
      rd_r6_method(
        name = "run",
        class = "Foo",
        formals = NULL,
        description = "Run it."
      )
    )
  )
  expect_snapshot(cat(format(methods), sep = "\n"))
})

test_that("format.rd_r6_methods returns nothing when empty", {
  expect_null(format(rd_r6_methods("Foo")))
})

test_that("r6_all_examples aggregates across methods", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Greet.
        #' @examples c$greet()
        greet = function() 'hi',
        #' @description Run.
        run = function() 1,
        #' @description Stop.
        #' @param force Force it
        #' @examples c$stop()
        #' @examples c$stop(force = TRUE)
        stop = function(force = FALSE) NULL
      )
    )"
  docs <- r6_doc(text)
  expect_snapshot(cat(r6_all_examples(docs$methods), sep = "\n"))
})

test_that("@noRd suppresses entire class documentation", {
  text <- "
    #' Class
    #' @noRd
    #' @field field1 Foo.
    C <- R6::R6Class(
      public = list(
        field1 = NULL,
        field2 = 'bar'
      )
    )"
  expect_silent(out <- roc_proc_text(rd_roclet(), text))
  expect_equal(out, list())
})

test_that("@noRd suppresses method documentation", {
  text <- "
    #' Class
    C <- R6::R6Class('C', cloneable = FALSE,
      public = list(
        #' @description Greet.
        greet = function() 'hi',
        #' @noRd
        initialize = function(x) NULL
      )
    )"
  expect_silent(docs <- r6_doc(text))

  nms <- map_chr(docs$methods$self, \(m) m$name)
  expect_equal(nms, "greet")
})

test_that("r6_flatten_sections collapses markdown sections", {
  local_markdown()

  tag <- roxy_tag("details", "Some details.\n\n# A heading\n\nBody.")
  tag <- tag_markdown_with_sections(tag)
  expect_length(tag$val, 2)

  tag <- r6_flatten_sections(tag)
  expect_length(tag$val, 1)
  expect_match(tag$val, "Some details.", fixed = TRUE)
  expect_match(tag$val, "\\subsection{A heading}", fixed = TRUE)
  expect_match(tag$val, "Body.", fixed = TRUE)
})

rd_r6_method <- function(
  name,
  class,
  formals,
  description = character(),
  details = character(),
  params = list(),
  return = NULL,
  examples = character()
) {
  structure(
    list(
      name = name,
      class = class,
      formals = formals,
      description = description,
      details = details,
      params = params,
      return = return,
      examples = examples
    ),
    class = "rd_r6_method"
  )
}

#' @export
format.rd_r6_method <- function(x, ...) {
  lines <- character()
  push <- function(...) lines <<- c(lines, ...)
  push_subsection <- function(title, ...) {
    push(
      paste0("  \\subsection{", title, "}{"),
      paste0("    ", c(...)),
      "  }"
    )
  }

  # Anchor and heading
  call <- r6_method_name(x$class, x$name)

  id <- paste0("method-", x$class, "-", x$name)
  push(rd_if_html("<hr>"))
  push(rd_if_html('<a id="', id, '"></a>'))
  push(rd_if_latex("\\hypertarget{", id, "}{}"))

  push(paste0("\\subsection{\\code{", call, "()}}{"))

  # Description
  if (length(x$description) > 0) {
    push(
      paste0(
        "  ",
        sub("\n?\n?$", "\n\n", head(x$description, -1)),
        recycle0 = TRUE
      ),
      paste0("  ", utils::tail(x$description, 1), recycle0 = TRUE)
    )
  }

  # Usage
  fake <- paste(rep("X", nchar(call)), collapse = "")
  usage <- format(function_usage(fake, x$formals))
  push_subsection(
    "Usage",
    rd_if_html('<div class="r">'),
    paste0("\\preformatted{", sub(paste0("^", fake), call, usage), "}"),
    rd_if_html("</div>")
  )

  # Params
  if (length(x$params) > 0) {
    nms <- map_chr(x$params, \(p) p$name)
    vals <- map_chr(x$params, \(p) p$description)
    push_subsection(
      "Arguments",
      rd_if_html('<div class="arguments">'),
      "\\describe{",
      paste0("  \\item{\\code{", nms, "}}{", vals, "}"),
      "}",
      rd_if_html("</div>")
    )
  }

  # Details
  if (length(x$details) > 0) {
    push_subsection(
      "Details",
      sub("\n?\n?$", "\n\n", head(x$details, -1)),
      utils::tail(x$details, 1)
    )
  }

  # Return
  if (!is.null(x$return)) {
    push_subsection("Returns", x$return)
  }

  # Examples
  if (length(x$examples) > 0) {
    push_subsection(
      "Examples",
      rd_if_html('<div class="r example copy">'),
      paste0("\\preformatted{", strip_rd_example_tags(x$examples), "\n", "}"),
      rd_if_html("</div>")
    )
  }

  # End
  push("}\n")

  lines
}

r6_method_from_row <- function(method, block) {
  tags <- method$tags[[1]]

  desc_tags <- keep(tags, \(t) t$tag == "description")
  description <- map_chr(desc_tags, \(x) x[["val"]])

  det_tags <- keep(tags, \(t) t$tag == "details")
  details <- map_chr(det_tags, \(x) x[["val"]])

  params <- r6_resolve_params(method, block)

  ret_tags <- keep(tags, \(t) t$tag %in% c("return", "returns"))
  if (length(ret_tags) > 1) {
    warn_roxy_block(block, "Must use one @return(s) per R6 method")
  }
  ret <- if (length(ret_tags) > 0) ret_tags[[1]]$val else NULL

  exa_tags <- keep(tags, \(t) t$tag == "examples")
  examples <- map_chr(exa_tags, \(x) x[["val"]])

  rd_r6_method(
    name = method$name,
    class = method$class,
    formals = method$formals[[1]],
    description = description,
    details = details,
    params = params,
    return = ret,
    examples = examples
  )
}

# Resolve params within a class: method @param -> class-level @param -> @field.
# Cross-class inheritance from parent classes is handled later by
# r6_resolve_method_params().
r6_resolve_params <- function(method, block) {
  tags <- method$tags[[1]]
  par_tags <- keep(tags, \(t) t$tag == "param")

  params <- r6_tags_to_params(par_tags)
  pnames <- r6_param_names(params)

  dup <- unique(pnames[duplicated(pnames)])
  for (m in dup) {
    warn_roxy_block(
      block,
      c(
        "Must use one @param for each argument",
        x = "${method$name}({m}) is documented multiple times"
      )
    )
  }

  fnames <- names(method$formals[[1]])
  if (length(fnames) == 0) {
    return(list())
  }

  # 1. Add class-level @param
  miss <- setdiff(fnames, pnames)
  cls_tags <- keep(block$tags, function(t) {
    !is.na(t$line) &&
      t$line < block$line &&
      tag_is(t, "param") &&
      tag_has_name(t, miss)
  })
  params <- c(params, r6_tags_to_params(cls_tags))

  # 2. For initialize() only, inherit from @field
  if (method$name == "initialize") {
    miss <- setdiff(fnames, r6_param_names(params))

    if (length(miss) > 0) {
      field_tags <- keep(block$tags, function(t) {
        tag_is(t, "field") && tag_has_name(t, miss)
      })
      params <- c(params, r6_tags_to_params(field_tags))
    }
  }

  # Order them according to formals
  firstnames <- map_chr(
    strsplit(map_chr(params, \(x) x$name), ","),
    \(x) trimws(x[[1]])
  )
  params[order(match(firstnames, fnames))]
}

r6_tags_to_params <- function(tags) {
  lapply(tags, function(t) {
    list(
      name = gsub(",", ", ", t$val$name),
      description = t$val$description
    )
  })
}

r6_param_names <- function(params) {
  if (length(params) == 0) {
    return(character())
  }
  trimws(unlist(strsplit(map_chr(params, \(x) x$name), ",")))
}

r6_method_name <- function(class, method) {
  paste0(class, "$", ifelse(method == "initialize", "new", method))
}

rd_if_html <- function(...) {
  paste0("\\if{html}{\\out{", ..., "}}")
}

rd_if_latex <- function(...) {
  paste0("\\if{latex}{\\out{", ..., "}}")
}

object_defaults <- function(x, block) UseMethod("object_defaults")

#' @export
object_defaults.default <- function(x, block) list()

#' @export
object_defaults.r6method <- function(x, block) {
  list(
    roxy_generated_tag(block, "R6method", x$value)
  )
}

#' @export
object_defaults.function <- function(x, block) {
  list(
    roxy_generated_tag(block, "usage", object_usage(x)),
    # Used in process_inherit_params()
    roxy_generated_tag(block, ".formals", names(formals(x$value)))
  )
}

#' @export
object_defaults.s3generic <- object_defaults.function

#' @export
object_defaults.s3method <- object_defaults.function

#' @export
object_defaults.s4generic <- object_defaults.function

#' @export
object_defaults.s4method <- object_defaults.function

#' @export
object_defaults.data <- function(x, block) {
  str_out <- rd(object_format(x$value))

  list(
    roxy_generated_tag(block, "docType", "data"),
    roxy_generated_tag(block, "format", str_out),
    roxy_generated_tag(block, "keywords", "datasets"),
    roxy_generated_tag(block, "usage", object_usage(x))
  )
}

#' @export
object_defaults.value <- function(x, block) {
  list(
    roxy_generated_tag(block, "usage", object_usage(x))
  )
}

#' @export
object_defaults.package <- function(x, block) {
  desc <- x$value$desc

  logo_path <- file.path(x$value$path, "man", "figures", "logo.svg")
  if (!file.exists(logo_path)) {
    logo_path <- file.path(x$value$path, "man", "figures", "logo.png")
  }
  if (file.exists(logo_path)) {
    logo_file <- basename(logo_path)
    fig <- paste_c(
      c("\\if{html}{\\figure{", logo_file, "}"),
      "{options: style='float: right' alt='logo' width='120'}}\n\n"
    )
  } else {
    fig <- ""
  }

  name <- desc$get_field("Package")
  title <- desc$get_field("Title")

  description <- desc$get_field("Description")
  description <- package_url_parse(description)
  description <- paste0(fig, description)

  seealso <- package_seealso(
    desc$get_field("URL", NULL),
    desc$get_field("BugReports", NULL)
  )
  authors <- package_authors(desc$get_field("Authors@R", NULL))

  list(
    roxy_generated_tag(block, ".package", name),
    roxy_generated_tag(block, "docType", "package"),
    roxy_generated_tag(block, "name", package_suffix(name)),
    # default aliases are added in topics_add_package_alias()
    roxy_generated_tag(block, "title", paste0(name, ": ", title)),
    roxy_generated_tag(block, "description", description),
    roxy_generated_tag(block, "seealso", seealso),
    roxy_generated_tag(block, "author", authors)
  )
}

#' @export
object_defaults.import <- function(x, block) {
  importFrom <- roxy_generated_tag(
    block,
    "importFrom",
    c(x$value$pkg, x$value$fun)
  )

  if (block_has_tags(block, c("rdname", "name"))) {
    obj <- object_from_name(x$value$fun, asNamespace(x$value$pkg), block)
    return(c(list(importFrom), object_defaults(obj, block)))
  }

  list(
    importFrom,
    roxy_generated_tag(block, "docType", "import"),
    roxy_generated_tag(block, "name", "reexports"),
    roxy_generated_tag(block, "keywords", "internal"),
    roxy_generated_tag(block, "title", "Objects exported from other packages"),
    roxy_generated_tag(
      block,
      ".reexport",
      list(pkg = x$value$pkg, fun = x$value$fun)
    )
  )
}

#' @export
object_defaults.s7class <- object_defaults.function

#' @export
object_defaults.s7generic <- object_defaults.function

#' @export
object_defaults.s7method <- function(x, block) {
  list(
    roxy_generated_tag(block, "usage", object_usage(x)),
    roxy_generated_tag(block, ".formals", names(formals(x$value$fn)))
  )
}

#' @export
object_defaults.s4class <- function(x, block) {
  list(
    roxy_generated_tag(block, "docType", "class")
  )
}

#' @export
object_defaults.rcclass <- function(x, block) {
  list(
    roxy_generated_tag(block, "docType", "class"),
    if (!is.null(x$methods)) {
      roxy_generated_tag(block, ".methods", x$methods)
    }
  )
}

# Helpers -----------------------------------------------------------------

package_suffix <- function(name) {
  paste0(name, "-package")
}

# Re-export ----------------------------------------------------------------
rd_section_reexport <- function(pkg, fun) {
  check_character(pkg)
  check_character(fun)
  stopifnot(length(pkg) == length(fun))

  rd_section("reexport", list(pkg = pkg, fun = fun))
}

#' @export
roxy_tag_rd.roxy_tag_.reexport <- function(x, base_path, env) {
  rd_section_reexport(x$val$pkg, x$val$fun)
}
#' @export
merge.rd_section_reexport <- function(x, y, ...) {
  stopifnot(identical(class(x), class(y)))
  rd_section_reexport(
    c(x$value$pkg, y$value$pkg),
    c(x$value$fun, y$value$fun)
  )
}
#' @export
format.rd_section_reexport <- function(x, ...) {
  info <- data.frame(
    pkg = x$value$pkg,
    fun = x$value$fun
  )

  pkgs <- split(info, x$value$pkg)
  pkg_links <- map(pkgs, function(pkg) {
    pkg <- pkg[order(pkg$fun), ]
    links <- paste0(reexport_link(pkg$pkg, pkg$fun), collapse = ", ")
    paste0("\\item{", pkg$pkg[[1]], "}{", links, "}")
  })

  paste0(
    "\\description{\n",
    "These objects are imported from other packages. Follow the links\n",
    "below to see their documentation.\n",
    "\n",
    "\\describe{\n",
    paste0("  ", unlist(pkg_links), collapse = "\n\n"),
    "\n}}\n"
  )
}

reexport_link <- function(pkg, fun) {
  env <- tryCatch(asNamespace(pkg), error = function(e) emptyenv())
  map_chr(seq_along(fun), function(i) {
    rd_link(
      pkg[[i]],
      escape(fun[[i]]),
      escape(fun_suffix(fun[[i]], env)),
      code = TRUE
    )
  })
}

TopoSort <- R6::R6Class(
  "TopoSort",
  public = list(
    vertices = list(),

    add = function(name) {
      if (is.null(self$vertices[[name]])) {
        self$vertices[[name]] <- Vertex$new(name)
      }
      invisible(self$vertices[[name]])
    },

    add_ancestor = function(predecessor_name, ancestor_name) {
      predecessor <- self$add(predecessor_name)
      ancestor <- self$add(ancestor_name)

      predecessor$add_ancestor(ancestor)
    },

    sort = function() {
      sorted <- list()

      visit <- function(predecessor) {
        predecessor$discovered <- TRUE
        for (ancestor in predecessor$ancestors) {
          if (!ancestor$discovered) {
            visit(ancestor)
          }
        }
        sorted <<- append(sorted, predecessor)
      }

      for (vertex in self$vertices) {
        if (!vertex$discovered) {
          visit(vertex)
        }
      }
      map_chr(sorted, \(x) x[["name"]])
    }
  )
)

Vertex <- R6::R6Class(
  "Vertex",
  public = list(
    name = NA_character_,
    discovered = FALSE,
    ancestors = list(),

    initialize = function(name) {
      self$name <- name
    },

    has_ancestor = function(ancestor) {
      for (vertex in self$ancestors) {
        if (identical(ancestor, vertex)) {
          return(TRUE)
        }
      }
      FALSE
    },

    add_ancestor = function(ancestor) {
      if (!self$has_ancestor(ancestor)) {
        self$ancestors <- append(ancestor, self$ancestors)
      }
    }
  )
)

#' Update Collate field in DESCRIPTION
#'
#' @description
#' By default, R loads files in alphabetical order. This is fine if your
#' package doesn't have any cross-file dependencies, but if you're using a tool
#' like S4, you'll need to make sure that classes are loaded before subclasses
#' and generics are defined before methods. You can do this by hand by setting
#' the `Collate` field in the `DESCRIPTION` or automate it by using `@include`
#' tags to specify the cross-file dependencies:
#'
#' ```R
#' #' @include before.R
#' NULL
#' ```
#'
#' If there are no `@include` tags, roxygen2 will leave the `Collate` field as
#' is. This makes it easier to use roxygen2 with an existing collate directive,
#' but if you remove all your `@include` tags, you'll need to also
#' manually delete the collate field.
#'
#' Generally, you should not need to run this function yourself; it will be
#' run automatically by any package that needs to load your R files in
#' collation order.
#'
#' `update_collate()` is not not technically a [roclet], like [rd_roclet()]
#' and [namespace_roclet()], because you have to be able to load the pacakge
#' before you can process it with roclets. However, because it was historical
#' implemented as a roclet, it's still controlled by the `roclets` argument of
#' [roxygenize()].
#'
#' @param base_path Path to package directory.
#' @examples
#' #' If `example-a.R`, `example-b.R` and `example-c.R` live in `R/`
#' #' and we're in `example-a.R`, then the following @include tag
#' #' ensures that example-b and example-c are sourced before example-a.
#' #' @include example-b.R example-c.R
#' NULL
#' @export
#' @aliases @include
update_collate <- function(base_path) {
  check_string(base_path)
  if (!file.exists(base_path)) {
    cli::cli_abort("{.path {base_path}} doesn't exist.")
  }

  new <- generate_collate(file.path(base_path, "R"))
  if (is.null(new)) {
    return(invisible())
  }

  desc_path <- file.path(base_path, "DESCRIPTION")
  old <- desc::desc_get_collate(file = desc_path)

  if (!identical(old, new)) {
    cli::cli_inform("Updating collate directive in {.path {desc_path}}")
    desc::desc_set_collate(new, file = desc_path)
  }

  invisible()
}

generate_collate <- function(base_path) {
  paths <- sort_c(dir(base_path, pattern = "[.][Rr]$", full.names = TRUE))

  includes <- lapply(paths, find_and_filter_includes, paths = basename(paths))
  names(includes) <- paths

  n <- sum(map_int(includes, length))
  if (n == 0) {
    return()
  }

  topo <- TopoSort$new()
  for (path in paths) {
    file <- base_path(path, base_path)

    topo$add(file)
    for (include in includes[[path]]) {
      topo$add_ancestor(file, include)
    }
  }

  topo$sort()
}

find_and_filter_includes <- function(path, paths) {
  includes <- find_includes(path)

  invalid <- setdiff(includes, paths)
  if (length(invalid) > 0) {
    for (include in invalid) {
      path <- cli::style_hyperlink(basename(path), paste0("file://", path))
      cli::cli_inform(c(
        x = "{path}: unknown path in `@include {invalid}`."
      ))
    }
  }

  intersect(includes, paths)
}

base_path <- function(path, base) {
  path <- normalizePath(path, winslash = "/")
  base <- normalizePath(base, winslash = "/")

  sub(paste0(base, "/"), "", path, fixed = TRUE)
}

test_that("collation as expected", {
  names <- generate_collate(test_path("collate"))

  before <- function(a, b) {
    all(which(names %in% a) < which(names %in% b))
  }

  expect_true(before("undershorts", "pants"))
  expect_true(before(c("tie", "belt"), "jacket"))
  expect_true(before(c("socks", "undershorts", "pants"), "shoes"))
})

test_that("update_collate() checks that directory exists", {
  expect_snapshot(update_collate("doesn't-exist"), error = TRUE)
})

test_that("Collate field unchanged when no @includes", {
  path <- local_package_copy(test_path('testCollateNoIncludes'))

  update_collate(path)
  expect_equal(desc::desc_get_field("Collate", file = path), "b.R a.R")
})

test_that("DESCRIPTION file is re-written only if collate changes", {
  path <- local_package_copy(test_path("testCollateOverwrite"))

  expect_snapshot(
    {
      update_collate(path)
      "Second run should be idempotent"
      update_collate(path)
    },
    transform = function(x) gsub(path, "<path>", x, fixed = TRUE)
  )
})

test_that("drops bad collect directives", {
  path <- local_package_copy(test_path("empty"))
  write_lines(c("#' @include foo", "NULL"), file.path(path, "R", "a.R"))
  write_lines(c("#' @include a.R", "NULL"), file.path(path, "R", "b.R"))
  unlink(file.path(path, "R/empty-package.R"))

  withr::with_dir(path, expect_snapshot(update_collate(".")))
  expect_equal(desc::desc_get_collate(file = path), c("a.R", "b.R"))
})

test_that("can read from file name with utf-8 path", {
  path <- withr::local_tempfile(
    pattern = "Universit\u00e0-",
    lines = c("#' @include foo.R", NULL)
  )
  expect_equal(find_includes(path), "foo.R")
})

#' Default format for data
#'
#' This function is called to generate the default "Format" section for each
#' data object. The default implementation will return the class and dimension
#' information.
#'
#' @param x A data object
#' @return A `character` value with valid `Rd` syntax, or `NULL`.
#' @keywords internal
#' @export
object_format <- function(x) {
  UseMethod("object_format")
}

#' @export
object_format.default <- function(x) {
  paste0("An object of class ", format_classes(x), " ", format_dim(x), ".")
}

format_classes <- function(x) {
  classes <- paste0("\\code{", escape_rd_braces(class(x)), "}")

  base_classes <- NULL
  if (length(classes) > 1L) {
    base_classes <- paste0(
      " (inherits from ",
      paste(classes[-1L], collapse = ", "),
      ")"
    )
  }

  paste0(classes[[1L]], base_classes)
}

format_dim <- function(x) {
  if (length(dim(x)) == 2L) {
    paste0("with ", nrow(x), " rows and ", ncol(x), " columns")
  } else if (length(dim(x)) > 2L) {
    paste0("of dimension ", paste(dim(x), collapse = " x "))
  } else {
    paste0("of length ", length(x))
  }
}

escape_rd_braces <- function(x) {
  x <- gsub("{", "\\{", x, fixed = TRUE)
  x <- gsub("}", "\\}", x, fixed = TRUE)
  x
}

# helpers -----------------------------------------------------------------

# used for testing
call_to_format <- function(code, env = pkg_env()) {
  obj <- call_to_object(!!enexpr(code), env)
  object_format(obj$value)
}

test_that("format has nice defaults for bare vectors", {
  skip_if(getRversion() <= "4.0.0")

  expect_snapshot({
    call_to_format(x <- list(a = 1, b = 2))
    call_to_format(x <- ordered(letters[1:5]))
    call_to_format(x <- diag(10))
    call_to_format(x <- array(1:27, dim = c(3, 3, 3)))
    call_to_format(x <- data.frame(a = 1, b = 2))
  })
})

test_that("format escapes braces in class names (#1744)", {
  expect_snapshot({
    call_to_format(x <- quote({}))
  })
})

find_package <- function(topic, tag = NULL) {
  cur_pkg <- roxy_meta_get("current_package")
  cur_pkg_dir <- roxy_meta_get("current_package_dir")
  if (is.null(cur_pkg)) {
    # Don't try and link in basic tests
    return(NA_character_)
  }

  deps <- rdtools::pkg_search_deps(cur_pkg_dir %||% cur_pkg)
  pkg <- rdtools::topic_qualifier(topic, from = cur_pkg, packages = deps)
  if (is.null(pkg)) {
    warn_roxy_tag(
      tag,
      c(
        "Could not resolve link to topic {.val {topic}} in the dependencies or base packages.",
        "i" = paste(
          "If you haven't documented {.val {topic}} yet, or just changed its name, this is normal.",
          "Once {.val {topic}} is documented, this warning goes away."
        ),
        "i" = "Make sure that the name of the topic is spelled correctly.",
        "i" = "Always list the linked package as a dependency.",
        "i" = "Alternatively, you can fully qualify the link with a package name."
      )
    )
    NA_character_
  } else if (length(pkg) == 1) {
    pkg
  } else {
    warn_roxy_tag(
      tag,
      c(
        "Topic {.val {topic}} is available in multiple packages: {.pkg {pkg}}.",
        i = "Qualify topic explicitly with a package name when linking to it."
      )
    )
    NA_character_
  }
}

has_topic <- function(topic, package) {
  rdtools::topic_exists(topic, package)
}

object_from_call <- function(call, env, block, file) {
  if (is.character(call)) {
    if (identical(call, "_PACKAGE")) {
      parser_package(file)
    } else {
      parser_data(call, env, file)
    }
  } else if (is_set_call(call)) {
    parser_r6_set(call, env)
  } else if (is.call(call)) {
    if (is_s7_method_call(call)) {
      return(parser_s7_method(call, env, block))
    }

    call <- call_match(call, eval(call[[1]], env))
    name <- deparse(call[[1]])
    switch(
      name,
      "=" = ,
      "<-" = ,
      "<<-" = parser_assignment(call, env, block),
      "delayedAssign" = parser_delayedAssign(call, env, block),
      "::" = parser_import(call, env, block),

      "methods::setClass" = ,
      "setClass" = parser_setClass(call, env, block),
      "methods::setClassUnion" = ,
      "setClassUnion" = parser_setClassUnion(call, env, block),
      "methods::setRefClass" = ,
      "setRefClass" = parser_setRefClass(call, env, block),
      "methods::setGeneric" = ,
      "setGeneric" = parser_setGeneric(call, env, block),
      "methods::setMethod" = ,
      "setMethod" = parser_setMethod(call, env, block),
      "methods::setReplaceMethod" = ,
      "setReplaceMethod" = parser_setReplaceMethod(call, env, block),

      "R.methodsS3::setMethodS3" = ,
      "setMethodS3" = parser_setMethodS3(call, env, block),

      "R.oo::setConstructorS3" = ,
      "setConstructorS3" = parser_setConstructorS3(call, env, block),
      NULL
    )
  } else {
    # Patch @docType package to ensure that it gets a default alias
    # and other "_PACKAGE" features
    if (block_has_tags(block, "docType")) {
      docType <- block_get_tag_value(block, "docType")
      if (docType == "package") {
        warn_roxy_block(
          block,
          c(
            '`@docType "package"` is deprecated',
            i = 'Please document "_PACKAGE" instead.'
          )
        )
        return(parser_package(file))
      }
    }
    NULL
  }
}

object_from_name <- function(name, env, block) {
  value <- get(name, env)
  if (inherits(value, "R6ClassGenerator")) {
    type <- "r6class"
  } else if (methods::is(value, "refObjectGenerator")) {
    value <- methods::getClass(as.character(value@className), where = env)
    type <- "rcclass"
  } else if (methods::is(value, "classGeneratorFunction")) {
    value <- methods::getClass(as.character(value@className), where = env)
    type <- "s4class"
  } else if (methods::is(value, "MethodDefinition")) {
    # S4 methods need munging to get real function def
    value@.Data <- extract_method_fun(value@.Data)
    type <- "s4method"
  } else if (methods::is(value, "standardGeneric")) {
    type <- "s4generic"
  } else if (inherits(value, "S7_class")) {
    type <- "s7class"
  } else if (inherits(value, "S7_generic")) {
    type <- "s7generic"
  } else if (is.function(value)) {
    # Potential S3 methods/generics need metadata added
    method <- block_get_tag_value(block, "method")
    value <- add_s3_metadata(value, name, env, block)
    if (inherits(value, "s3generic")) {
      type <- "s3generic"
    } else if (inherits(value, "s3method")) {
      type <- "s3method"
    } else {
      type <- "function"
    }
  } else {
    type <- "value"
  }

  object(value, name, type)
}

# Parsers for individual calls --------------------------------------------

is_set_call <- function(call) {
  is_call(call) &&
    is_call(call[[1]], "$", n = 2) &&
    is_symbol(call[[1]][[3]], "set")
}

parser_r6_set <- function(call, env) {
  lhs <- call[[1]]

  obj_name <- deparse(lhs[[2]])
  obj <- tryCatch(get(obj_name, envir = env), error = function(e) NULL)
  if (!inherits(obj, "R6ClassGenerator")) {
    return(NULL)
  }
  class_name <- obj$classname

  method_name <- call[[3]]
  if (!is.character(method_name)) {
    return(NULL)
  }

  object(list(class = class_name, method = method_name), NULL, "r6method")
}

parser_data <- function(call, env, block) {
  if (isNamespace(env)) {
    value <- getExportedValue(call, ns = asNamespace(env))
  } else {
    value <- get(call, envir = env)
  }
  object(value, call, type = "data")
}

parser_package <- function(file) {
  pkg_path <- dirname(dirname(file))
  value <- list(
    desc = desc::desc(file = pkg_path),
    path = pkg_path
  )
  object(value, NULL, type = "package")
}

parser_assignment <- function(call, env, block) {
  name <- as.character(call[[2]])

  # If it's a compound assignment like x[[2]] <- ignore it
  if (length(name) > 1) {
    return()
  }

  # If it doesn't exist (any more), don't document it.
  if (!exists(name, env)) {
    return()
  }

  object_from_name(name, env, block)
}

parser_delayedAssign <- function(call, env, block) {
  name <- as.character(call$x)
  object_from_name(name, env, block)
}

parser_setClass <- function(call, env, block) {
  name <- as.character(call$Class)
  value <- methods::getClass(name, where = env)

  object(value, NULL, "s4class")
}

parser_setClassUnion <- function(call, env, block) {
  name <- as.character(call$name)
  value <- methods::getClass(name, where = env)

  object(value, NULL, "s4class")
}

parser_setRefClass <- function(call, env, block) {
  name <- as.character(call$Class)
  value <- methods::getClass(name, where = env)

  object(value, NULL, "rcclass")
}

parser_setGeneric <- function(call, env, block) {
  name <- as.character(call$name)
  value <- methods::getGeneric(name, where = env)

  object(value, NULL, "s4generic")
}

parser_setMethod <- function(call, env, block) {
  name <- as.character(call$f)
  value <- methods::getMethod(name, eval(call$signature), where = env)
  value@.Data <- extract_method_fun(value@.Data)

  object(value, NULL, "s4method")
}

parser_setReplaceMethod <- function(call, env, block) {
  name <- paste0(as.character(call$f), "<-")
  value <- methods::getMethod(name, eval(call[[3]]), where = env)
  value@.Data <- extract_method_fun(value@.Data)

  object(value, NULL, "s4method")
}

parser_import <- function(call, env, block) {
  pkg <- as.character(call[[2]])
  fun <- as.character(call[[3]])

  object(list(pkg = pkg, fun = fun), alias = fun, type = "import")
}

parser_setMethodS3 <- function(call, env, block) {
  # R.methodsS3::setMethodS3(name, class, ...)
  method <- as.character(call[[2]])
  class <- as.character(call[[3]])
  name <- paste(method, class, sep = ".")

  value <- add_s3_metadata(get(name, env), name, env, block)

  object(value, name, "s3method")
}

parser_setConstructorS3 <- function(call, env, block) {
  # R.oo::setConstructorS3(name, ...)
  name <- as.character(call[[2]])
  object(get(name, env), name, "function")
}

# method(generic, class) <- fn
# `<-`(method(generic, class), fn)
is_s7_method_call <- function(call) {
  is_call(call, "<-", n = 2) && is_call(call[[2]], "method", ns = c("", "S7"))
}

parser_s7_method <- function(call, env, block) {
  generic_call <- call[[2]][[2]]
  class_call <- call[[2]][[3]]
  method_call <- call[[3]]

  generic <- eval(generic_call, env)
  if (inherits(generic, "S7_generic")) {
    generic_name <- generic@name
  } else {
    # S3 or S4 generic passed by name
    generic_name <- deparse(generic_call)
  }

  # Evaluate class spec: either a single class, a union, or list() for
  # multi-dispatch
  classes <- eval(class_call, env)
  if (!is_bare_list(classes)) {
    classes <- list(classes)
  }
  class_names <- lapply(classes, s7_class_name, block = block)

  fn <- eval(method_call, env)

  value <- list(fn = fn, generic = generic_name, classes = class_names)
  aliases <- s7_method_aliases(generic_name, class_names)
  object(value, aliases, "s7method")
}

s7_method_aliases <- function(generic, classes) {
  if (!any(lengths(classes) > 1)) {
    return(NULL)
  }

  combos <- expand.grid(classes, stringsAsFactors = FALSE)
  apply(combos, 1, function(row) {
    paste0(generic, ",", paste0(row, collapse = ","), "-method")
  })
}

# https://github.com/RConsortium/S7/issues/594
s7_class_name <- function(cls, block) {
  name <- nameOfClass(cls)
  if (!is.null(name)) {
    # Regular S7 class + base wrappers
    name
  } else if (inherits(cls, "S7_union")) {
    # Unions return vector of member names, recursing for nested types
    unlist(lapply(cls$classes, s7_class_name, block = block))
  } else if (inherits(cls, "S7_S3_class")) {
    cls$class
  } else if (inherits(cls, "S7_any")) {
    "any"
  } else if (inherits(cls, "S7_missing")) {
    "missing"
  } else {
    warn_roxy_block(block, "Unknown S7 class type")
    paste0(deparse(cls), collapse = " ")
  }
}

# helpers -----------------------------------------------------------------

add_s3_metadata <- function(val, name, env, block) {
  if (block_has_tags(block, "method")) {
    method <- block_get_tag_value(block, "method")
    return(s3_method(val, method))
  }

  if (block_has_tags(block, "exportS3Method")) {
    method <- block_get_tag_value(block, "exportS3Method")
    if (length(method) == 1 && grepl("::", method, fixed = TRUE)) {
      generic <- re_split_half(method, "::")[[2]]
      class <- gsub(paste0("^", generic, "\\."), "", name)
      return(s3_method(val, c(generic, class)))
    }
  }

  if (is_s3_generic(name, env)) {
    class(val) <- c("s3generic", "function")
    return(val)
  }

  method <- find_generic(name, env)
  if (is.null(method)) {
    val
  } else {
    s3_method(val, method)
  }
}

# When a generic has ... and a method adds new arguments, the S4 method
# wraps the definition inside another function which has the same arguments
# as the generic. This function figures out if that's the case, and extracts
# the original function if so.
#
# It's based on expression processing based on the structure of the
# constructed method which looks like:
#
# function (x, ...) {
#   .local <- function (x, ..., y = 7) {}
#   .local(x, ...)
# }
extract_method_fun <- function(fun) {
  method_body <- body(fun)
  if (!is_call(method_body, "{")) {
    return(fun)
  }
  if (length(method_body) < 2) {
    return(fun)
  }

  first_line <- method_body[[2]]
  if (!is_call(first_line, name = "<-", n = 2)) {
    return(fun)
  }
  if (!identical(first_line[[2]], quote(`.local`))) {
    return(fun)
  }

  local_fun <- eval(first_line[[3]])
  if (!is.function(local_fun)) {
    return(fun)
  }

  local_fun
}

#' Constructors for S3 object to represent R objects
#'
#' These objects are usually created by the parsers, but it is also
#' useful to generate them by hand for testing.
#'
#' @param value The object itself.
#' @param alias Alias for object being documented, in case you create a
#'   generator function with different name.
#' @export
#' @keywords internal
#' @param type Type of the object, character. E.g. `"data"` or `"s4method"`.
object <- function(value, alias, type) {
  structure(
    list(
      alias = alias,
      value = value,
      methods = if (type == "rcclass") rc_methods(value),
      topic = object_topic(value, alias, type)
    ),
    class = c(type, "object")
  )
}

#' @export
format.object <- function(x, ...) {
  c(
    paste0("<", class(x)[1], "> ", x$name),
    paste0("  $topic ", x$topic),
    if (!is.null(x$alias)) paste0("  $alias ", x$alias)
  )
}

#' @export
print.object <- function(x, ...) {
  cat_line(format(x, ...))
}

object_topic <- function(value, alias, type) {
  switch(
    type,
    s4method = method_topic(value@generic, value@defined),
    s4class = paste0(value@className, "-class"),
    s4generic = value@generic,
    rcclass = paste0(value@className, "-class"),
    r6class = alias,
    r6method = alias,
    rcmethod = value@name,
    s7class = alias,
    s7generic = alias,
    s7method = method_topic(value$generic, value$classes),
    s3generic = alias,
    s3method = alias,
    import = alias,
    `function` = alias,
    package = alias,
    data = alias,
    value = alias,
    cli::cli_abort("Unsupported type {.str {type}}.", .internal = TRUE)
  )
}

method_topic <- function(generic, classes) {
  class_strings <- vapply(classes, paste0, character(1), collapse = "/")
  paste0(generic, ",", paste0(class_strings, collapse = ","), "-method")
}

call_to_object <- function(code, env = pkg_env(), file = NULL) {
  code <- enexpr(code)

  eval(code, envir = env)
  if (is_call(code, "{")) {
    call <- code[[length(code)]]
  } else {
    call <- code
  }
  object_from_call(call, env, block = NULL, file = file)
}

test_that("undocumentable things return null", {
  expect_null(call_to_object(NULL))
  expect_null(call_to_object(10))
  expect_null(call_to_object(1 + 2))
})

# data / package -------------------------------------------------------

test_that("recommends use of _PACKAGE", {
  path <- local_package_copy(test_path("empty"))

  block <- "
    #' @docType package
    NULL
  "
  withr::with_dir(path, expect_snapshot(out <- parse_text(block)[[1]]))

  expect_s3_class(out$object, "package")
  expect_equal(out$object$value$desc$get_field("Package"), "empty")
})

test_that("finds package description", {
  obj <- call_to_object("_PACKAGE", file = test_path("testEagerData/R/a.r"))
  expect_s3_class(obj, "package")
  expect_equal(obj$value$desc$get_field("Package"), "testEagerData")
})


test_that("finds datasets given by name", {
  obj <- call_to_object({
    df <- data.frame(x = 1, y = 2)
    "df"
  })
  expect_s3_class(obj, "data")
  expect_equal(obj$alias, "df")
  expect_s3_class(obj$value, "data.frame")
})

test_that("can document eager data", {
  path <- local_package_copy(test_path('testEagerData'))
  suppressMessages(roxygenise(path))
  withr::defer(pkgload::unload("testEagerData"))

  expect_true(file.exists(file.path(path, "man/a.Rd")))
})

test_that("can document lazy data", {
  path <- local_package_copy(test_path('testLazyData'))
  suppressMessages(roxygenise(path))
  withr::defer(pkgload::unload("testLazyData"))

  expect_true(file.exists(file.path(path, "man/a.Rd")))
})

# imports -----------------------------------------------------------------

test_that("find function imported from another package", {
  obj <- call_to_object(purrr::map_int)
  expect_s3_class(obj, "import")
  expect_equal(obj$alias, "map_int")
  expect_equal(obj$value$pkg, "purrr")
})

# assignment ------------------------------------------------------------

test_that("finds function created with assignment", {
  obj <- call_to_object({
    foo <- function(x, y, z) {}
  })
  expect_s3_class(obj, "function")
})

test_that("finds S3 generic created with assignment", {
  obj <- call_to_object({
    foo <- function(x, y, z) UseMethod("foo")
  })
  expect_s3_class(obj, "s3generic")
})

test_that("finds S3 method created with assignment", {
  obj <- call_to_object({
    foo <- function(x, y, z) UseMethod("foo")
    foo.method <- function(x, y, z) {}
  })
  expect_s3_class(obj, "s3method")
})

test_that("finds data created with assignment", {
  obj <- call_to_object({
    foo <- 1:10
  })
  expect_s3_class(obj, "data")
})

test_that("finds class generator", {
  obj <- call_to_object({
    newFoo <- setClass("Foo")
  })
  expect_s3_class(obj, "s4class")
  expect_equal(obj$alias, "newFoo")
  expect_s4_class(obj$value, "classRepresentation")

  obj <- call_to_object({
    newFoo <- setRefClass("Foo")
  })
  expect_s3_class(obj, "rcclass")
  expect_equal(obj$alias, "newFoo")
  expect_s4_class(obj$value, "classRepresentation")
})

test_that("ignored compound assignment", {
  obj <- call_to_object({
    foo <- list()
    foo[[1]] <- function(x, y, z) {}
  })
  expect_null(obj)
})

test_that("finds function created with delayed assignment", {
  obj <- call_to_object({
    delayedAssign("foo", function(x, y, z) {})
  })
  expect_s3_class(obj, "function")
})

# S3 ----------------------------------------------------------------------

test_that("can derive S3 metadata for base generics", {
  block <- "
    #' @export
    mean.foo <- function(...) 1
  "
  out <- parse_text(block)[[1]]

  expect_equal(s3_method_info(out$object$value), c("mean", "foo"))
})

test_that("@method overrides auto-detection", {
  block <- "
    #' @export
    #' @method all.equal data.frame
    all.equal.data.frame <- function(...) 1
  "
  out <- parse_text(block)[[1]]

  expect_equal(s3_method_info(out$object$value), c("all.equal", "data.frame"))
})

test_that("exportS3Method registers S3 metadata", {
  block <- "
    #' @exportS3Method stats::median
    median.foo <- function(...) 1
  "
  out <- parse_text(block)[[1]]
  expect_equal(s3_method_info(out$object$value), c("median", "foo"))
})


# S4 ----------------------------------------------------------------------

test_that("finds S4 and RC classes", {
  obj <- call_to_object(setClass("Foo"))
  expect_s3_class(obj, "s4class")
  expect_equal(obj$topic, "Foo-class")
  expect_equal(obj$alias, NULL)

  obj <- call_to_object(setRefClass("Foo"))
  expect_s3_class(obj, "rcclass")
  expect_equal(obj$topic, "Foo-class")

  obj <- call_to_object({
    setClass("Foo")
    setClassUnion("Foo2", "Foo")
  })
  expect_s3_class(obj, "s4class")
  expect_equal(obj$topic, "Foo2-class")
})

test_that("finds S4 generics and methods", {
  obj <- call_to_object({
    setGeneric("bar", function(x) standardGeneric("bar"))
  })
  expect_s3_class(obj, "s4generic")

  obj <- call_to_object({
    setGeneric("bar", function(x) standardGeneric("bar"))
    setMethod('bar', 'Foo', function(x) {})
  })
  expect_s3_class(obj, "s4method")

  obj <- call_to_object({
    setGeneric("bar<-", function(x, value) standardGeneric("bar<-"))
    setReplaceMethod("bar", "Foo", function(x, value) {})
  })
  expect_s3_class(obj, "s4method")
})

test_that("finds correct parser even when namespaced", {
  obj <- call_to_object({
    setClass("Foo")
    setGeneric("baz", function(x) standardGeneric("baz"))
    methods::setMethod('baz', 'Foo', function(x) {})
  })
  expect_s3_class(obj, "s4method")
})

test_that("finds arguments when S4 method wrapped inside .local()", {
  obj <- call_to_object({
    setClass("Foo")
    setMethod('subset', 'Foo', function(x, foo, ...) {})
  })
  expect_s3_class(obj, "s4method")
  expect_named(formals(obj$value@.Data), c("x", "foo", "..."))
})


# R.oo / R.methodsS3 ------------------------------------------------------

test_that("can define constructor with R.oo", {
  skip_if_not_installed("R.oo")
  obj <- call_to_object({
    R.oo::setConstructorS3("Foo", function(x, y, z) {})
  })
  expect_s3_class(obj, "function")
  expect_equal(obj$alias, "Foo")
})

test_that("can define method for R.methodsS3", {
  skip_if_not_installed("R.methodsS3")
  obj <- call_to_object({
    R.methodsS3::setMethodS3("foo", "default", function(x, ...) {})
  })
  expect_s3_class(obj, "s3method")
  expect_equal(obj$alias, "foo.default")
})

# extract_method_fun ------------------------------------------------------

test_that("fails gracefully on bad inputs", {
  fun1 <- function() {}
  fun2 <- function() 1 + 2
  fun3 <- function() {
    1 + 2
  }
  fun4 <- function() {
    x <- 1
  }
  fun5 <- function() {
    .local <- 1
  }

  expect_equal(extract_method_fun(fun1), fun1)
  expect_equal(extract_method_fun(fun2), fun2)
  expect_equal(extract_method_fun(fun3), fun3)
  expect_equal(extract_method_fun(fun4), fun4)
  expect_equal(extract_method_fun(fun5), fun5)
})

readLines <- function(...) cli::cli_abort("Use read_lines!", .internal = TRUE)
writeLines <- function(...) cli::cli_abort("Use write_lines!", .internal = TRUE)

read_lines <- function(path, n = -1L) {
  base::readLines(path, n = n, encoding = "UTF-8", warn = FALSE)
}

write_lines <- function(text, path, line_ending = detect_line_ending(path)) {
  # we need to convert any embedded newlines as well
  text <- gsub("\r?\n", line_ending, text)

  path <- file(path, open = "wb")
  base::writeLines(enc2utf8(text), path, sep = line_ending, useBytes = TRUE)
  close(path)
}

detect_line_ending <- function(path) {
  tryCatch(
    {
      samp <- suppressWarnings(readChar(path, nchars = 500))
      if (isTRUE(grepl("\r\n", samp))) "\r\n" else "\n"
    },
    error = function(e) "\n"
  )
}

test_that("empty file gives empty list", {
  out <- roc_proc_text(rd_roclet(), "")
  expect_identical(out, list())
})

test_that("NULL gives empty list", {
  out <- roc_proc_text(rd_roclet(), "NULL")
  expect_identical(out, list())
})

test_that("@noRd inhibits documentation", {
  out <- roc_proc_text(rd_roclet(), "
    #' Would be title
    #' @title Overridden title
    #' @name a
    #' @noRd
    NULL")

  expect_equal(length(out), 0)
})

test_that("deleted objects not documented", {
  out <- roc_proc_text(rd_roclet(), "
    f <- function(){
      .a <- 0
      function(x = 1){
        .a <<- .a + x
        .a
      }
    }

    #' Addition function.
    f2 <- f()
    rm(f)
  ")
  expect_equal(names(out), "f2.Rd")
})

test_that("documenting unknown function requires name", {
  block <- "
    #' Virtual Class To Enforce Max Slot Length
    setClass('A')

    #' Validity function.
    setValidity('A', function(object) TRUE)
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("can't set description and re-export", {
  block <- "
      #' @description NOPE
      #' @export
      magrittr::`%>%`
      "
  expect_snapshot(out <- roc_proc_text(rd_roclet(), block))
  expect_length(out, 0)
})

test_that("documenting NA gives useful error message (#194)", {
  block <- "
    #' Missing value
    NA
  "
  expect_snapshot(. <- roc_proc_text(rd_roclet(), block))
})

test_that("@description NULL", {
  # Just ignore in this case
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @description NULL
    #' @format NULL
    foobar <- 1:10
  ")
  expect_identical(out[[1]]$get_value("description"), "Title")

  # Still ignore
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #' @description NULL
    #' @description desc
    #' @format NULL
    foobar <- 1:10
  ")
  expect_identical(out[[1]]$get_value("description"), "desc")

  # Still ignore for objects as well
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #' @description NULL
    #' @format NULL
    foobar <- 1:10
  ")
  expect_identical(out[[1]]$get_value("description"), "Title")

  # But drop for package docs
  block <- "
    #' Title
    #'
    #' @docType package
    #' @description NULL
    #' @name pkg
    '_PACKAGE'
  "
  out <- roc_proc_text(rd_roclet(), block, wd = test_path("empty"))
  expect_null(out[[1]]$get_value("description"))
})

test_that("@details NULL", {
  # Just ignore in this case
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #'
    #' @details NULL
    #' @format NULL
    foobar <- 1:10
  ")
  expect_null(out[[1]]$get_value("details"))

  # Still ignore
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #' @details NULL
    #' @details desc
    #' @format NULL
    foobar <- 1:10
  ")
  expect_identical(out[[1]]$get_value("details"), "desc")

  # Still ignore for objects as well
  out <- roc_proc_text(rd_roclet(), "
    #' Title
    #' @details NULL
    #' @format NULL
    foobar <- 1:10
  ")
  expect_null(out[[1]]$get_value("details"))
})


# package docs ------------------------------------------------------------


test_that("package docs don't get alias if function present", {

  block <- "
    #' Title
    #'
    '_PACKAGE'

    #' Empty
    empty <- function() {}
  "

  out <- roc_proc_text(rd_roclet(), block, test_path("empty"))[[1]]
  expect_equal(out$get_value("alias"), "empty-package")
})

test_that("package docs preserve existing aliases", {
  block <- "
    #' Title
    #' @aliases a b
    #'
    '_PACKAGE'
  "

  out <- roc_proc_text(rd_roclet(), block, test_path("empty"))[[1]]
  expect_equal(out$get_value("alias"), c("empty", "empty-package", "a", "b"))

  block <- paste0(block, "
    #' Empty
    empty <- function() {}
  ")
  out <- roc_proc_text(rd_roclet(), block, test_path("empty"))[[1]]
  expect_equal(out$get_value("alias"), c("empty-package", "a", "b"))
})

test_that("get correct alias even if user has overriden name", {
  block <- "
    #' Title
    #' @name foo
    #' @aliases bar
    #'
    '_PACKAGE'
  "

  out <- roc_proc_text(rd_roclet(), block, test_path("empty"))[[1]]
  expect_equal(
    out$get_value("alias"),
    c("empty", "empty-package", "foo", "bar")
  )
})

# UTF-8 -------------------------------------------------------------------

test_that("can generate nonASCII document", {
  path <- local_package_copy(test_path('testNonASCII'))
  withr::defer(pkgload::unload("testNonASCII"))

  expect_snapshot({
    roxygenise(path, roclets = "rd")
    "Second run should be idempotent"
    roxygenise(path, roclets = "rd")
  })

  rd_path <- file.path(path, "man", "printChineseMsg.Rd")
  expect_true(file.exists(rd_path))
  rd <- read_lines(rd_path)

  expect_true(any(grepl("\u6211\u7231\u4e2d\u6587", rd)))
  expect_true(any(grepl("\u4e2d\u6587\u6ce8\u91ca", rd)))
})

test_that("unicode escapes are ok", {
  path <- local_package_copy(test_path('testUtf8Escape'))
  withr::defer(pkgload::unload("testUtf8Escape"))

  expect_snapshot({
    roxygenise(path, roclets = "rd")
    "Second run should be idempotent"
    roxygenise(path, roclets = "rd")
  })

  rd_path <- file.path(path, "man", "a.Rd")
  expect_true(file.exists(rd_path))
  rd <- read_lines(rd_path)

  expect_true(any(grepl("7\u00b0C", rd)))
})

test_that("automatically deletes unused files", {
  path <- local_package_copy(test_path("empty"))
  dir.create(file.path(path, "man"))
  suppressMessages(roxygenise(path))
  withr::defer(pkgload::unload("empty"))

  write_lines(made_by("%"), file.path(path, "man/test.Rd"))
  expect_snapshot(roxygenise(path))
})

test_that("write_lines uses specified line_ending", {
  win_nl <- withr::local_tempfile()
  write_lines("baz", win_nl, line_ending = "\r\n")
  expect_equal(readChar(win_nl, 100), "baz\r\n")

  unix_nl <- withr::local_tempfile()
  write_lines("baz", unix_nl, line_ending = "\n")
  expect_equal(readChar(unix_nl, 100), "baz\n")
})

test_that("detect_line_ending captures existing line ending", {
  win_nl <- withr::local_tempfile()
  write_lines("baz", win_nl, line_ending = "\r\n")
  expect_equal(detect_line_ending(win_nl), "\r\n")

  unix_nl <- withr::local_tempfile()
  write_lines("baz", unix_nl, line_ending = "\n")
  expect_equal(detect_line_ending(unix_nl), "\n")
})

test_that("detect_line_ending defaults to unix", {
  non_existent_file <- withr::local_tempfile()
  expect_equal(detect_line_ending(non_existent_file), "\n")

  empty_file <- withr::local_tempfile()
  file.create(empty_file)
  expect_equal(detect_line_ending(empty_file), "\n")
})

test_that("write_lines preserves existing line ending", {
  win_nl <- withr::local_tempfile()
  write_lines("baz", win_nl, line_ending = "\r\n")
  write_lines("baz", win_nl)
  expect_equal(readChar(win_nl, 100), "baz\r\n")

  unix_nl <- withr::local_tempfile()
  write_lines("baz", unix_nl, line_ending = "\n")
  write_lines("baz", unix_nl)
  expect_equal(readChar(unix_nl, 100), "baz\n")
})

#' Roclet: make Rd files
#'
#' @description
#' This [roclet] automates the production of the `.Rd` files that
#' R uses to document functions, datasets, packages, classes, and more.
#' See `vignette("rd-functions")` for details.
#'
#' It is run by default by [roxygenize()].
#'
#' @seealso [tags-rd-functions], [tags-rd-datasets], [tags-rd-S3], [tags-rd-S4],
#'   [tags-rd-S7], [tags-rd-R6], [tags-reuse],
#'   [tags-index-crossref] for tags provided by this roclet.
#' @export
#' @examples
#' #' Add together two numbers
#' #' @param x A number.
#' #' @param y A number.
#' #' @return A number.
#' #' @export
#' #' @examples
#' #' add(1, 1)
#' #' add(10, 1)
#' add <- function(x, y) {
#'   x + y
#' }
rd_roclet <- function() {
  roclet("rd")
}

#' @export
roclet_process.roclet_rd <- function(x, blocks, env, base_path) {
  blocks <- merge_external_r6methods(blocks)

  # Convert each block into a topic, indexed by filename
  topics <- RoxyTopics$new()
  for (block in blocks) {
    rd <- block_to_rd(block, base_path, env)
    topics$add(rd, block)
  }

  topics_process_r6_inherit(topics)
  topics_process_family(topics, env)
  topics_process_inherit(topics, env)
  topics$drop_invalid()
  topics_fix_params_order(topics)
  topics_add_default_description(topics)
  topics_add_package_alias(topics)

  topics$topics
}

#' @export
roclet_output.roclet_rd <- function(
  x,
  results,
  base_path,
  ...,
  is_first = FALSE
) {
  man <- normalizePath(file.path(base_path, "man"))

  contents <- map_chr(results, format)
  paths <- file.path(man, names(results))

  names <- unname(map_chr(results, \(x) x$get_name()[[1]]))
  if (length(names) > 0) {
    commands <- paste0("pkgload::dev_help('", names, "')")
  } else {
    commands <- character()
  }

  # Always check for roxygen2 header before overwriting NAMESPACE (#436),
  # even when running for the first time
  mapply(write_if_different, paths, contents, command = commands)

  if (!is_first) {
    # Automatically delete any files in man directory that were generated
    # by roxygen in the past, but weren't generated in this sweep.

    old_paths <- setdiff(dir(man, full.names = TRUE), paths)
    old_paths <- old_paths[!file.info(old_paths)$isdir]
    old_roxygen <- Filter(made_by_roxygen, old_paths)
    if (length(old_roxygen) > 0) {
      cli::cli_inform("Deleting {.file {basename(old_roxygen)}}")
      unlink(old_roxygen)
    }
  }

  paths
}

#' @export
roclet_clean.roclet_rd <- function(x, base_path) {
  rd <- dir(file.path(base_path, "man"), full.names = TRUE)
  rd <- rd[!file.info(rd)$isdir]
  unlink(keep(rd, made_by_roxygen))
}

# Does this block get an Rd file?
needs_doc <- function(block) {
  if (block_has_tags(block, "noRd")) {
    return(FALSE)
  }

  block_has_tags(
    block,
    c(
      "description",
      "param",
      "return",
      "title",
      "example",
      "examples",
      "name",
      "rdname",
      "details",
      "inherit",
      "describeIn"
    )
  )
}

# Tag processing functions ------------------------------------------------

block_to_rd <- function(block, base_path, env) {
  UseMethod("block_to_rd")
}

#' @export

block_to_rd.default <- function(block, ...) {
  cli::cli_abort("Unknown block type.", .internal = TRUE)
}

#' @export

block_to_rd.roxy_block <- function(block, base_path, env) {
  # Must start by processing templates
  block <- process_templates(block, base_path)

  if (!needs_doc(block)) {
    return()
  }

  name <- block_get_tag(block, "name")$val %||% block$object$topic
  if (is.null(name)) {
    warn_roxy_block(
      block,
      c(
        "Block must have a @name",
        i = "Either document an existing object or manually specify with @name"
      )
    )
    return()
  }

  rd <- RoxyTopic$new()
  topic_add_name_aliases(rd, block, name)
  for (tag in block$tags) {
    rd$add(roxy_tag_rd(tag, env = env, base_path = base_path))
  }

  if (rd$has_section("description") && rd$has_section("reexport")) {
    warn_roxy_block(
      block,
      "Block must not include a description when re-exporting a function"
    )
    return()
  }

  describe_rdname <- topic_add_describe_in(rd, block, env)
  filename <- describe_rdname %||%
    block_get_tag(block, "rdname")$val %||%
    nice_name(name)
  rd$filename <- paste0(filename, ".Rd")

  rd
}

#' @export

block_to_rd.roxy_block_r6class <- function(block, base_path, env) {
  r6on <- roxy_meta_get("r6", TRUE)
  if (!isTRUE(r6on)) {
    return(NextMethod())
  }

  # Must start by processing templates
  block <- process_templates(block, base_path)

  if (!needs_doc(block)) {
    return()
  }

  name <- block_get_tag(block, "name")$val %||% block$object$topic
  if (is.null(name)) {
    warn_roxy_block(block, "must have a @name")
    return()
  }

  rd <- RoxyTopic$new()
  topic_add_name_aliases(rd, block, name)

  rd$add(roxy_tag_rd(
    block_get_tag(block, "name"),
    env = env,
    base_path = base_path
  ))
  rd$add(roxy_tag_rd(
    block_get_tag(block, "title"),
    env = env,
    base_path = base_path
  ))

  if (rd$has_section("description") && rd$has_section("reexport")) {
    warn_roxy_block(
      block,
      "Block must not include a description when re-exporting a function"
    )
    return()
  }

  topic_add_r6_methods(rd, block, env, base_path)

  describe_rdname <- topic_add_describe_in(rd, block, env)
  filename <- describe_rdname %||%
    block_get_tag(block, "rdname")$val %||%
    nice_name(name)
  rd$filename <- paste0(filename, ".Rd")

  rd
}

# Special cases -----------------------------------------------------------

topics_add_default_description <- function(topics) {
  for (topic in topics$topics) {
    if (length(topic$get_section("description")) > 0) {
      next
    }

    # rexport manually generates a own description, so don't need to
    if (
      !topic$has_section("reexport") &&
        !identical(topic$get_value("docType"), "package")
    ) {
      topic$add(rd_section("description", topic$get_value("title")))
    }
  }

  invisible()
}

topics_add_package_alias <- function(topics) {
  aliases <- unlist(topics$simple_values("alias"), use.names = FALSE)

  for (topic in topics$topics) {
    if (!identical(topic$get_value("docType"), "package")) {
      next
    }

    package <- topic$get_value("package")
    defaults <- c(package, package_suffix(package))

    aliases <- union(setdiff(defaults, aliases), topic$get_value("alias"))
    topic$add(rd_section("alias", aliases), overwrite = TRUE)
    break
  }

  invisible(NULL)
}

# Tag-wise processing -----------------------------------------------------

#' Generate Rd output from a tag
#'
#' Provide a method for this generic if you want a tag to generate output
#' in `.Rd` files. See `vignette("extending")` for more details.
#'
#' @param x The tag
#' @param base_path Path to package root directory.
#' @param env Environment in which to evaluate code (if needed)
#' @return Methods must return a [rd_section].
#' @export
#' @family extending
roxy_tag_rd <- function(x, base_path, env) {
  UseMethod("roxy_tag_rd")
}

#' @export
roxy_tag_rd.default <- function(x, base_path, env) {}

# Special tags ------------------------------------------------------------
# These tags do not directly affect the output, and are no complicated enough
# to require their own files.

#' @export
roxy_tag_rd.roxy_tag_.formals <- function(x, base_path, env) {
  rd_section("formals", x$val)
}
#' @export
format.rd_section_formals <- function(x, ...) NULL

#' @export
roxy_tag_rd.roxy_tag_.package <- function(x, base_path, env) {
  rd_section("package", x$val)
}
#' @export
format.rd_section_package <- function(x, ...) NULL

#' @export
roxy_tag_parse.roxy_tag_method <- function(x) tag_words(x, 2, 2)

#' @export
roxy_tag_parse.roxy_tag_noRd <- function(x) tag_toggle(x)

#' @export
roxy_tag_parse.roxy_tag_rdname <- function(x) tag_value(x)

# Generated by cpp11: do not edit by hand

escapeExamples <- function(x) {
  .Call(`_roxygen2_escapeExamples`, x)
}

findEndOfTag <- function(string, is_code, start) {
  .Call(`_roxygen2_findEndOfTag`, string, is_code, start)
}

rdComplete <- function(string, is_code) {
  .Call(`_roxygen2_rdComplete`, string, is_code)
}

leadingSpaces <- function(lines) {
  .Call(`_roxygen2_leadingSpaces`, lines)
}

tokenise_block <- function(lines, file, offset) {
  .Call(`_roxygen2_tokenise_block`, lines, file, offset)
}

find_includes <- function(path) {
  .Call(`_roxygen2_find_includes`, path)
}

wrapUsage <- function(string, width, indent) {
  .Call(`_roxygen2_wrapUsage`, string, width, indent)
}

r6_extract_field_tags <- function(block, r6data, type = c("field", "active")) {
  type <- match.arg(type)
  other_type <- if (type == "field") "active" else "field"
  label <- if (type == "field") "field" else "active binding"

  self <- r6data$self
  expected <- self$name[self$type == type]
  other <- self$name[self$type == other_type]

  tags <- keep(block$tags, \(t) tag_is(t, "field") && !tag_has_name(t, other))
  docd <- unlist(lapply(tags, tag_names))

  dup <- unique(docd[duplicated(docd)])
  if (length(dup) > 0) {
    warn_roxy_block(block, "R6 {label}{?s} documented multiple times: {dup}")
  }

  if (type == "field") {
    xtra <- setdiff(docd, expected)
    if (length(xtra) > 0) {
      warn_roxy_block(block, "Unknown R6 {label}{?s}: {xtra}")
    }
  }

  # @field name NULL suppresses documentation for that field/binding
  tags <- discard(tags, function(t) toupper(t$val$description) == "NULL")

  items <- lapply(tags, function(t) {
    rd_r6_field(
      name = gsub(",", ", ", t$val$name),
      description = t$val$description
    )
  })

  rd_r6_fields(items, type = type, expected = expected)
}

r6_field_names <- function(rd_fields) {
  if (length(rd_fields) == 0) {
    return(character())
  }
  labels <- map_chr(rd_fields, \(x) x$name)
  trimws(unlist(strsplit(labels, ",")))
}

# Rd ---------------------------------------------------------------------------

rd_r6_fields <- function(
  fields = list(),
  type = c("field", "active"),
  expected = character()
) {
  type <- match.arg(type)
  structure(
    list(fields = fields, type = type, expected = expected),
    class = "rd_r6_fields"
  )
}

rd_r6_field <- function(name, description) {
  structure(
    list(name = name, description = description),
    class = "rd_r6_field"
  )
}

#' @export
format.rd_r6_field <- function(x, ...) {
  paste0("\\item{\\code{", x$name, "}}{", x$description, "}")
}

#' @export
format.rd_r6_fields <- function(x, ...) {
  if (x$type == "field") {
    format_r6_field_section(x$fields, "Public fields", "r6-fields")
  } else {
    format_r6_field_section(x$fields, "Active bindings", "r6-active-bindings")
  }
}

format_r6_field_section <- function(fields, title, css_class) {
  if (length(fields) == 0) {
    return()
  }

  c(
    paste0("\\section{", title, "}{"),
    paste0("  ", rd_if_html(paste0('<div class="', css_class, '">'))),
    "  \\describe{",
    paste0("    ", map_chr(fields, format), collapse = "\n\n"),
    "  }",
    paste0("  ", rd_if_html("</div>")),
    "}"
  )
}

test_that("r6_extract_field_tags builds rd_r6_field objects", {
  text <- "
    #' @field field1 Foo.
    #' @field field2 Bar.
    C <- R6::R6Class(
      public = list(
        field1 = NULL,
        field2 = \"foobar\"
      )
    )"
  block <- parse_text(text)[[1]]
  r6data <- block_get_tag_value(block, ".r6data")
  expect_silent(fields <- r6_extract_field_tags(block, r6data, "field"))
  expect_equal(
    fields$fields,
    list(
      rd_r6_field("field1", "Foo."),
      rd_r6_field("field2", "Bar.")
    )
  )
})

test_that("comma-separated @field works (#1600)", {
  text <- "
    #' Class
    C <- R6::R6Class(
      public = list(
        #' @field var_1,var_2 do something vars
        var_1 = 1,
        var_2 = 2
      )
    )"
  expect_silent(docs <- r6_doc(text))
  expect_equal(
    docs$fields$fields,
    list(rd_r6_field("var_1, var_2", "do something vars"))
  )
})

test_that("r6_extract_field_tags builds active binding objects", {
  text <- "
    #' @field bind1 Active binding.
    #' @field bind2 Active 2.
    C <- R6::R6Class(
      active = list(
        bind1 = function(x) { },
        bind2 = function(x) { }
      )
    )"
  block <- parse_text(text)[[1]]
  r6data <- block_get_tag_value(block, ".r6data")
  expect_silent(bindings <- r6_extract_field_tags(block, r6data, "active"))
  expect_equal(
    bindings$fields,
    list(
      rd_r6_field("bind1", "Active binding."),
      rd_r6_field("bind2", "Active 2.")
    )
  )
})

test_that("@field name NULL suppresses field documentation", {
  text <- "
    #' Class
    #' @field field1 Foo.
    #' @field field2 NULL
    C <- R6::R6Class(
      public = list(
        field1 = NULL,
        field2 = 'bar'
      )
    )"
  block <- parse_text(text)[[1]]
  r6data <- block_get_tag_value(block, ".r6data")
  expect_silent(fields <- r6_extract_field_tags(block, r6data, "field"))
  expect_equal(
    fields$fields,
    list(rd_r6_field("field1", "Foo."))
  )
})

test_that("warns about undocumented fields", {
  text <- "
    #' Class
    C <- R6::R6Class(
      public = list(
        undocumented_field = NULL
      )
    )"
  expect_snapshot(docs <- r6_doc(text))
  expect_equal(docs$fields$fields, list())
})

test_that("warns about fields documented multiple times", {
  text <- "
    #' Class
    #' @field x First.
    #' @field x Second.
    C <- R6::R6Class(
      public = list(x = NULL)
    )"
  expect_snapshot(docs <- r6_doc(text))
})

test_that("warns about unknown fields", {
  text <- "
    #' Class
    #' @field nosuch Doesn't exist.
    C <- R6::R6Class(
      public = list(x = NULL)
    )"
  expect_snapshot(docs <- r6_doc(text))
})

test_that("format.rd_r6_field produces \\item markup", {
  expect_snapshot(cat(format(rd_r6_field("x", "A number."))))
})

test_that("format.rd_r6_fields produces fields & bindings sections", {
  fields <- rd_r6_fields(list(
    rd_r6_field("x", "A number."),
    rd_r6_field("y", "A string.")
  ))
  expect_snapshot(cat(format(fields), sep = "\n"))

  bindings <- rd_r6_fields(
    list(rd_r6_field("val", "A value.")),
    type = "active"
  )
  expect_snapshot(cat(format(bindings), sep = "\n"))
})

test_that("format.rd_r6_fields returns nothing when empty", {
  expect_null(format(rd_r6_fields()))
})

# Superclass field inheritance -----------------------------------------------

test_that("fields inherit docs from superclass", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @field x A field.
        x = NULL
      )
    )

    #' Child
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        x = NULL
      )
    )"
  expect_silent(docs <- r6_doc(text))
  expect_equal(
    docs$fields$fields,
    list(rd_r6_field("x", "A field."))
  )
})

test_that("field docs in subclass override superclass", {
  text <- "
    #' Parent
    A <- R6::R6Class('A', cloneable = FALSE,
      public = list(
        #' @field x Parent field.
        x = NULL
      )
    )

    #' Child
    #' @field x Child field.
    B <- R6::R6Class('B', cloneable = FALSE,
      inherit = A,
      public = list(
        x = NULL
      )
    )"
  docs <- r6_doc(text)
  expect_equal(
    docs$fields$fields,
    list(rd_r6_field("x", "Child field."))
  )
})

set_collate <- function(locale) {
  cur <- Sys.getlocale(category = "LC_COLLATE")
  Sys.setlocale(category = "LC_COLLATE", locale = locale)
  cur
}

with_collate <- function(locale, code) {
  old <- set_collate(locale)
  on.exit(set_collate(old))

  force(code)
}

sort_c <- function(x, ...) {
  with_collate("C", sort(x, ...))
}

order_c <- function(x, ...) {
  with_collate("C", order(x, ...))
}

rd_r6_super <- function(
  class,
  package = character(),
  classname = character(),
  has_topic = logical()
) {
  structure(
    list(
      class = class,
      package = package,
      classname = classname,
      has_topic = has_topic
    ),
    class = "rd_r6_super"
  )
}

#' @export
format.rd_r6_super <- function(x, ...) {
  if (length(x$classname) == 0) {
    return()
  }

  cls <- x$classname
  pkgs <- x$package
  ht <- x$has_topic

  title <- if (length(cls) > 1) "Super classes" else "Super class"

  self_pkg <- roxy_meta_get("current_package") %||% ""
  same_pkg <- pkgs == self_pkg

  label <- ifelse(same_pkg, cls, paste0(pkgs, "::", cls))
  path <- map_chr(seq_along(cls), function(i) {
    if (ht[[i]]) {
      rd_link(pkgs[[i]], cls[[i]], label[[i]], code = TRUE)
    } else {
      paste0("\\code{", label[[i]], "}")
    }
  })
  me <- sprintf("\\code{%s}", x$class)

  c(
    paste0("\\section{", title, "}{"),
    paste(c(rev(path), me), collapse = " -> "),
    "}"
  )
}

r6_extract_superclasses <- function(r6data, env, class) {
  super <- r6data$super
  cls <- unique(super$classes$classname)
  if (length(cls) == 0) {
    return(rd_r6_super(class))
  }

  pkgs <- super$classes$package[match(cls, super$classes$classname)]
  ht <- map2_lgl(cls, pkgs, has_topic)
  rd_r6_super(class, package = pkgs, classname = cls, has_topic = ht)
}

---
title: "Indexing and cross-references"
output: rmarkdown::html_vignette
description: >
  Make it easier for users to find your functions
  by cross-referencing them and control their
  indexing.
vignette: >
  %\VignetteIndexEntry{Indexing and cross-references}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette discusses tags that help users finding documentation through cross-references and indexes.

## See also

`@seealso` allows you to point to other useful resources, either on the web (using a url) or to related functions (with a function link like `[function_name()]`).
For `sum()`, this might look like:

```{r}
#' @seealso [prod()] for products, [cumsum()] for cumulative sums, and
#'   [colSums()]/[rowSums()] marginal sums over high-dimensional arrays.
```

## Family

If you have a family of related functions, you can use `@family {family}` to cross-reference each function to every member of the family.
A function can be a member of multiple families.

By default `@family {family}`, will generate the see also text "Other {family}:", so the `@family` name should be plural (i.e., "model building helpers" not "model building helper").

If you want to override the default title, you can provide an `rd_family_title` element in a list stored in `man/roxygen/meta.R`:

```{r}
#| eval: false

list(
  rd_family_title = list(aggregations = "Aggregation functions")
)
```

## References

If the object you're documenting has connections to the scientific literature, use `@reference` to provide a citation.

## Aliases

`?` and `help()` look for topic aliases; `?foo` will find any topic that contains the `foo` alias.

roxygen2 generates a default alias for you based on the object you're documenting.
You can add additional aliases with `@aliases alias1 alias2 alias3` or remove default alias with `@aliases NULL`.

## Search

As well as looking in the aliases, `help.search()` and `???` also look in the `@title`, `@keywords`, and `@concept`s tags.

- `@keywords` adds standard keywords, which must be present in `file.path(R.home("doc"), "KEYWORDS")`.

- `@concept` adds arbitrary key words or phrases.
  Each `@concept` should contain a single word or phrase.

Generally speaking, `@keywords` and `@concepts` are not terribly useful because most people find documentation using Google, not R's built-in search.

There's one exception: `@keywords internal`.
It's useful because it removes the function from the documentation index; it's useful for functions aimed primarily at other developers, not typical users of the package.

## Back references

The original source location is added as a comment to the second line of each generated `.Rd` file in the following form:

```
% Please edit documentation in ...
```

`roxygen2` tries to capture all locations from which the documentation is assembled.
For code that *generates* R code with Roxygen comments (e.g., the Rcpp package), the `@backref` tag is provided.
This allows specifying the "true" source of the documentation, and will substitute the default list of source files.
Use one tag per source file:

```{r}
#' @backref src/file.cpp
#' @backref src/file.h
```

---
title: "Documenting S4"
description: >
  How to document S4 generics, methods, and classes.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting S4}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

There are three things that you might document for [S4](https://adv-r.hadley.nz/s4.html):

- **Generics**: document as functions; mention that they are generics.
- **Methods**: unlike S3/S7, all S4 methods must be documented.
- **Classes**: document the class definition and use `@slot` for slots.

## Generics

S4 **generics** are functions, so document them as such.
`@export` a generic if you want users to call it or other developers to write methods for it.
If the generic is internal, you don't need to export or document it.

### Supported syntax

- `setGeneric("balance", function(x) standardGeneric("balance"))`

- `setGroupGeneric("Ops2", function(e1, e2) ...)`, which defines a group
  generic and is otherwise documented just like `setGeneric()`.

## Classes

Document **S4 classes** by adding a roxygen block before `setClass()`.
`@export` a class if you want users to create instances or other developers to extend it (e.g. by creating subclasses).
Internal classes don't need documentation.
Use `@slot` to document the slots of the class.
Here's a simple example:

```{r}
#| eval: false

#' An S4 class to represent a bank account
#'
#' @slot balance A length-one numeric vector
#' @export
Account <- setClass("Account", slots = list(balance = "numeric"))
```

### Supported syntax

- `setClass("Account", slots = list(balance = "numeric"))`

- `setClassUnion("Number", c("numeric", "integer"))`, which defines a class
  and is otherwise documented just like `setClass()`.

## Methods

S4 **methods** are a little more complicated.
Unlike S3 and S7 methods, all S4 methods must be documented.
You only need to `@export` a method if the generic lives in another package.

You can document methods in three places:

- In the class.
  Most appropriate if the corresponding generic uses single dispatch and you created the class.

- In the generic.
  Most appropriate if the generic uses multiple dispatches and you control it.

- In its own file.
  Most appropriate if the method is complex or the either two options don't apply.

Use either `@rdname` or `@describeIn` to control where method documentation goes.
See `vignette("reuse")` for more details.

### Supported syntax

- `setMethod("show", "Account", function(object) ...)`

- `setReplaceMethod("balance", "Account", function(x, value) ...)`, which
  documents the `balance<-` method, aliased as
  `balance<-,Account-method`.

- `setAs("numeric", "Account", function(from) ...)`, which documents the
  `coerce()` method. The usage comes from `coerce()`, so must you document
  `@param to` and `@param strict` even if your definition only takes `from`.

   If you supply `replace`, only the `coerce()` method is documented; use
  `@aliases` if you also want to document `coerce<-`.

---
title: "(R)Markdown support"
description: >
  The details of the (R)Markdown support provided by roxygen2.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{(R)Markdown support}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(comment = "#>", collapse = TRUE)
```

We expect most roxygen2 users will write documentation using markdown rather than Rd syntax, since it's familiar, and doesn't require learning any new syntax.
In most cases, you can just use your existing RMarkdown knowledge and it'll work as you expect.
When it doesn't, you can read this vignette to figure out what's going on and how to fix it.

## Enabling markdown support

To turn on Markdown support for a package, insert this entry into the `DESCRIPTION` file of the package:

```
Config/roxygen2/markdown: TRUE
```

If you use devtools/usethis, this will be automatically inserted for you when you create a new package.
If you're updating an existing package, we recommend `usethis::use_roxygen_md()` which will modify the `DESCRIPTION` and prompt you to use the [roxygen2md](https://roxygen2md.r-lib.org) package to convert your existing docs.

If needed, you can also use `@md` or `@noMd` to turn markdown support on or off for a documentation block.

Here is an example roxygen chunk that uses Markdown.

```r
#' Use roxygen to document a package
#'
#' This function is a wrapper for the [roxygen2::roxygenize()] function from
#' the roxygen2 package. See the documentation and vignettes of
#' that package to learn how to use roxygen.
#'
#' @param pkg package description, can be path or package name.  See
#'   [as.package()] for more information.
#' @param clean,reload Deprecated.
#' @inheritParams roxygen2::roxygenise
#' @seealso [roxygen2::roxygenize()], `browseVignettes("roxygen2")`
#' @export
```

## Basic syntax

roxygen uses the [commonmark package](https://github.com/r-lib/commonmark), based on the "CommonMark Reference Implementation".
See <https://commonmark.org/help/> for more about the parser and the markdown language it supports.
The most important details are described below.

### Sections and subsections

The usual Markdown heading markup creates sections and subsections.
Top level headings (e.g. `# title`) create sections with the `\section{}` Rd tag.
This largely supersedes use of the older `@section` tag.

Top-level headings can only appear after the `@description` and `@details` tags.
Since `@details` can appear multiple times in a block, you can always precede a '`#`' section with `@details`, if you want put it near the end of the block, after `@return` for example:

```r
#' @details
#' Trim the leading and trailing whitespace from a character vector.
#'
#' @param x Character vector.
#' @return Character vector, with the whitespace trimmed.
#'
#' @details # This will be a new section
#' ...
```

Top level sections are placed at a fixed position in the manual page, after the parameters and the details, but before `\note{}`, `\seealso{}` and the `\examples{}`.
Their order will be the same as in the roxygen block.

Headings at level two and above may appear inside any roxygen tag that formats lines of text, e.g. `@description`, `@details`, `@return`, and create subsections with the `\subsection{}` Rd tag.

```r
#' @details
#' ## Subsection within details
#' ### Sub-subsection
#' ... text ...
```

### Inline formatting

For *emphasis*, put the text between asterisks or underline characters.
For **strong** text, use two asterisks at both sides.

```r
#' @references
#' Robert E Tarjan and Mihalis Yannakakis. (1984). Simple
#' linear-time algorithms to test chordality of graphs, test acyclicity
#' of hypergraphs, and selectively reduce acyclic hypergraphs.
#' *SIAM Journal of Computation* **13**, 566-579.
```

```r
#' See `::is_falsy` for the definition of what is _falsy_
#' and what is _truthy_.
```

### Code

Inline code is supported via backticks.

```r
#' @param ns Optionally, a named vector giving prefix-url pairs, as
#'   produced by `xml_ns`. If provided, all names will be explicitly
#'   qualified with the ns prefix, i.e. if the element `bar` is defined ...
```

For blocks of code, put your code between triple backticks:

````r
#' ```
#' pkg <- make_packages(
#'   foo1 = { f <- function() print("hello!") ; d <- 1:10 },
#'   foo2 = { f <- function() print("hello again!") ; d <- 11:20 }
#' )
#' foo1::f()
#' foo2::f()
#' foo1::d
#' foo2::d
#' dispose_packages(pkg)
#' ```
````

You can also include executable code chunks using the usual knitr syntax.
See below for more details.

### Lists

Regular Markdown lists are recognized and converted to `\enumerate{}` or `\itemize{}` lists:

```r
#' There are two ways to use this function:
#' 1. If its first argument is not named, then it returns a function
#'    that can be used to color strings.
#' 1. If its first argument is named, then it also creates a
#'    style with the given name. This style can be used in
#'    `style`. One can still use the return value
#'    of the function, to create a style function.
```

```r
#' The style (the `...` argument) can be anything of the
#' following:
#' * An R color name, see `colors()`.
#' * A 6- or 8-digit hexa color string, e.g. `#ff0000` means
#'   red. Transparency (alpha channel) values are ignored.
#' * A one-column matrix with three rows for the red, green,
#'   and blue channels, as returned by [grDevices::col2rgb()].
```

Note that you do not have to leave an empty line before the list.
This is different from some Markdown parsers.

### Tables

Use [GFM table formatting](https://github.github.com/gfm/#tables-extension-):

```md
| foo | bar |
| --- | --- |
| baz | bim |
```

By default, columns are left-aligned.
Use colons to generate right and center aligned columns:

```md
| left | center | right |
| :--- | :----: | ----: |
| 1    | 2      | 3     |
```

### Links

Markdown hyperlinks work as usual:

```r
#' See more about the Markdown markup at the
#' [Commonmark web site](http://commonmark.org/help)
```

URLs inside angle brackets are also automatically converted to hyperlinks:

```r
#' The main R web site is at <https://r-project.org>.
```

### Images

Markdown syntax for inline images works.
The image files must be in the `man/figures` directory:

```r
#' Here is an example plot:
#' ![](example-plot.jpg "Example Plot Title")
```

## Function links

Markdown notation can also be used to create links to other help topics.
There are three basic forms for linking to topics in the current package or its dependencies.

  | Markdown        | Code | Rd                            |
  | :-------------- | :--- | :---------------------------- |
  | `[func()]`      | yes  | `\code{\link[=func]{func()}}` |
  | `[topic]`       | no   | `\link{topic}`                |
  | `` [`topic`] `` | yes  | `\code{\link{topic}}`         |

R 4.5.0 and later requires that links to all topics outside of your package be explicitly qualified with the package name.
roxygen2 will attempt to do this for you by looking at the dependencies of your package.
If this fails, e.g. if multiple dependencies document the same topic, you'll need to manually resolve using the syntax below.

  | Markdown             | Code | Rd                                    |
  | :------------------- | :--- | :------------------------------------ |
  | `[pkg::func()]`      | yes  | `\code{\link[pkg:func]{pkg::func()}}` |
  | `[pkg::topic]`       | no   | `\link[pkg:topic]{pkg::topic}`        |
  | `` [`pkg::topic`] `` | yes  | `\code{\link[pkg:topic]{pkg::topic}}` |

You can also provide your own link text.
In this case there's no advantage to supplying a function call, as you control the formatting of the text.

  | Markdown                   | Code | Rd                              |
  | :------------------------- | :--- | :------------------------------ |
  | `[text][topic]`            | no   | `\link[=topic]{text}`           |
  | `` [`text`][topic] ``      | yes  | `\code{\link[=topic]{text}}`    |
  | `[text][pkg::topic]`       | no   | `\link[pkg:topic]{text}`        |
  | `` [`text`][pkg::topic] `` | yes  | `\code{\link[pkg:topic]{text}}` |

## Code chunks

You can insert executable code with ```` ```{r} ```` just like in knitr documents.
For example:

````{r}
#| echo: false

simple_fenced <- "#' @title Title
#' @details Details
#' ```{r lorem}
#' 1+1
#' ```
#' @md
foo <- function() NULL
"
````

```{r, code=simple_fenced}
```

becomes:

```{r}
#| echo: false
#| results: "asis"

cat(
  "\x60\x60\x60rd\n",
  format(roxygen2:::roc_proc_text(roxygen2::rd_roclet(), simple_fenced)[[1]]),
  "\n\x60\x60\x60"
)
```

This code is run every time you call `roxygenize()` (or `devtools::document()`) to generate the Rd files.
This potentially makes `roxygenize()` (much) slower.
Either avoid expensive computations, or turn on knitr caching with `cache = TRUE`.
Make sure to omit the cache from the package with `usethis::use_build_ignore()`.

Note that knitr will call the appropriate `print()` or (if available) `knitr::knit_print()` method on the result.
This may generate markdown not supported by roxygen2.
If needed, override the automatic methods to have your R calls return your own markdown as a character vector, wrapped in `knitr::asis_output()`.

You can also use `` `r ` `` to insert the results of running R code.
This is particularly useful for reducing duplication in your documentation so is documented in `vignette("reuse")`.

### Chunk options

Code blocks support some knitr chunk options, e.g. to keep the output of several expressions together, you can specify `results="hold"`:

````r
#' ```{r results="hold"}
#' names(mtcars)
#' nrow(mtcars)
#' ```
````

Some knitr chunk options are reset at the start of every code block, so if you want to change these, you'll have to specify them for every chunk.
These are currently `r paste0("\x60", names(roxygen2:::knitr_chunk_defaults()), "\x60")`.

Alternatively, you can set the `knitr_chunk_options` option to override these defaults, or add new chunk options that are used for the whole package.
See `?load_options` for specifying roxygen2 options.

### Images

Plots will create `.png` files in the `man/figures` directory with file names coming from the chunk name.
Be aware that plots can quickly increase the size of your package leading to challenges for CRAN submission.

````r
#' ```{r iris-pairs-plot}
#' pairs(iris[1:4], main = "Anderson's Iris Data -- 3 species",
#'   pch = 21, bg = c("red", "green3", "blue")[unclass(iris$Species)])
#' ```
````

By default roxygen2 only includes PDF images in the PDF manual, and SVG images in the HTML manual.
If you want to avoid this restriction, set the `restrict_image_formats` roxygen2 option to `FALSE`, see `?load_options`.

## Possible problems

### Some Rd tags can't contain markdown

When mixing `Rd` and Markdown notation, most `Rd` tags may contain Markdown markup, the ones that can *not* are: `r paste0("\x60", roxygen2:::escaped_for_md, "\x60", collapse = ", ")`.

### Mixing Markdown and `Rd` markup

Note that turning on Markdown does *not* turn off the standard `Rd` syntax.
We suggest that you use the regular `Rd` tags in a Markdown roxygen chunk only if necessary.
The two parsers do occasionally interact, and the Markdown parser can pick up and reformat Rd syntax, causing an error, or corrupted manuals.

### Leading white space

Leading white space is interpreted by the commonmark parser, but is ignored by the `Rd` parser (except in `\preformatted{}`).
Make sure that you only include leading white space intentionally, for example, in nested lists.

### Spurious lists

The commonmark parser does not require an empty line before lists, and this might lead to unintended lists if a line starts with a number followed by a dot, or with an asterisk followed by white space:

```r
#' You can see more about this topic in the book cited below, on page
#' 42. Clearly, the numbered list that starts here is not intentional.
```

---
title: "Managing imports and exports"
description: >
  Generating the `NAMESPACE` file with roxygen2.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Managing imports and exports}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(comment = "#>", collapse = TRUE)
```

The package `NAMESPACE` is one of the most confusing parts of building a package.
roxygen2 aims to make it as easy as possible to build a package that is a well-behaved member of the R ecosystem.
This is a little frustrating at first, but soon becomes second-nature.

## Exports

In order for your users to use a function[^1] from your package, you must **export** it.
In most cases, you can just use the `@export` tag, and roxygen2 will automatically figure out which `NAMESPACE` directive (i.e. `export()`, `S3method()`, `exportClasses()`, or `exportMethods()`) you need.

[^1]: Including S3/S4/S7 generics and constructors.

For details and examples of exporting functions, and S3, S4, and S7 classes/generics/methods, see the corresponding vignettes:

- `vignette("rd-functions")` for regular functions.
- `vignette("rd-S3")` for S3 generics and methods, including delayed registration with `@exportS3Method`.
- `vignette("rd-S4")` for S4 generics, classes, and methods.
- S7 methods are registered at load time, not through `NAMESPACE`; see `vignette("rd-S7")`.

Datasets should never be exported as they are not found in `NAMESPACE`.
Learn more in `vignette("rd-datasets")`.

### Manual exports

If `@export` does not automatically generate the correct `NAMESPACE` directive, you can use one of the tags below to exercise greater control:

- `@export foo` generates `export(foo)`
- `@exportS3Method generic method` generates `S3method(generic, method)`
- `@exportClass foo` generates `exportClasses(foo)`
- `@exportMethod foo` generates `exportMethods(foo)`
- `@exportPattern foo` generates `exportPattern(foo)`

For even more specialised cases you can use `@rawNamespace code` which inserts `code` literally into the `NAMESPACE`.
This is useful if you need a conditional import or export, e.g.

```{r}
# From dplyr:
#' @rawNamespace import(vctrs, except = data_frame)
NULL

# From backports:
#' @rawNamespace if (getRversion() < "4.0.0") export(stopifnot)
NULL
```

If you need to automate this, `@evalNamespace fun()` will evaluate `fun()` in your package environment and insert the results into `NAMESPACE`.
Note that because `evalNamespace()` is run in the package environment, it can only generate exports, not imports.

## Imports

The `NAMESPACE` also controls which functions from other packages are made available to your package.

### Functions

If you are using just a few functions from another package, we recommending adding the package to the `Imports:` field of the `DESCRIPTION` file and calling the functions explicitly using `::`, e.g., `pkg::fun()`.

```r
my_function <- function(x, y) {
  pkg::fun(x) * y
}
```

If the repetition of the package name becomes annoying you can `@importFrom` and drop the `::`:

```r
#' @importFrom pkg fun
my_function <- function(x, y) {
  fun(x) * y
}
```

Imports affect every function in a package, so it's common to collect them in a central place, like `{packagename}-package.R`.
This is automated by `usethis::use_import_from()`.

```{r}
#' @importFrom pkg fun1 fun2
#' @importFrom pkg2 fun3
#' @importFrom pkg3 fun4
NULL
```

Note the use of `NULL` here: you must provide something for roxygen2 to document, so we use `NULL` as placeholder.

It is possible, but not generally recommended to import all functions from a package with `@import package`.
This is risky if you import functions from more than one package, because while it might be ok today, in the future the packages might end up with a function having the same name, and your users will get a warning every time your package is loaded.

### S3

S3 generics are just functions, so the same rules for functions apply.
S3 methods always accompany the generic, so as long as you can access the generic, the methods will also be available.
In other words, you don't need to do anything to import an S3 method.

### S4

- To use classes defined in another package, place `@importClassesFrom package ClassA ClassB ...` next to the classes that inherit from the imported classes, or next to the methods that implement a generic for the imported classes.
- To use generics defined in another package, place `@importMethodsFrom package GenericA GenericB ...` next to the methods that use the imported generics.

### Compiled code

To import compiled code from another package, use `@useDynLib`

- `@useDynLib package` imports all compiled functions.

- `@useDynLib package routinea routineb` imports selected compiled functions.

- Any `@useDynLib` specification containing a comma, e.g. `@useDynLib mypackage, .registration = TRUE` will be inserted as is into the the NAMESPACE, e.g. `useDynLib(mypackage, .registration = TRUE)`

---
title: "Documenting S3"
description: >
  How to document S3 generics, methods, and classes.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting S3}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

There are three things that you might document for [S3](https://adv-r.hadley.nz/s3.html):

- **Generics**: mention that the function is a generic and list the available methods.
- **Methods**: link back to the generic; only document individually when the method has unique behavior or arguments.
- **Classes**: document the constructor.

## Generics

S3 **generics** are regular functions, so document them as such.
`@export` a generic if you want users to call it or other developers to write methods for it.
If the generic is internal, you don't need to export or document it.

The documentation should mention that the function is a generic, because this tells the reader that the behavior may vary depending on the input and that they can write their own methods.
For simple generics, you can do this in the description:

```{r}
#| eval: false

#' Frobnpolicate an object
#'
#' @description
#' `frobnpolicate()` is an S3 generic that ..., with methods available for
#' the following classes:
#'
#' `r doclisting::methods_list("frobnpolicate")`
```

For more complicated generics, you can use a `# Methods` section to provide more detail:

```{r}
#| eval: false

#' Frobnpolicate an object
#'
#' @description
#' `frobnpolicate()` does ...
#'
#' # Methods
#' `frobnpolicate()` is an S3 generic with methods available for the following
#' classes:
#'
#' `r doclisting::methods_list("frobnpolicate")`
```

You might also want to include a section that provides additional details for developers implementing their own methods.

Both examples above use the [doclisting](https://doclisting.r-lib.org/) package to automatically generate a list of methods, with links to their help topics.
These examples use [inline R code](reuse.html#inline-code) (`` `r ` ``), which generates the list at documentation time (i.e. when you run `devtools::document()`).
This only requires including doclisting in `Suggests`.

If you want the list to dynamically reflect all methods that are currently registered (including methods registered by other packages), use an [inline Rd code](reuse.html#inline-rd-code) block (`` `Rd ` ``) instead:

```{r}
#| eval: false

#' `Rd doclisting::methods_list("frobnpolicate")`
```

Using `` `Rd ` `` requires doclisting in `Imports`, and you'll need a dummy call to eliminate the `R CMD check` NOTE about unused imports:

```{r}
#| eval: false

ignore_unused_imports <- function() {
  doclisting::methods_list
}
```

## Classes

S3 **classes** have no formal definition, so document the [constructor](https://adv-r.hadley.nz/s3.html#s3-constructor).
`@export` the constructor if you want users to create instances of your class or other developers to extend it (e.g. by creating subclasses).
Internal constructors don't need documentation.

Note that you don't need to list methods for a class: in S3, methods belong to generics, not to classes.
Users should look at the generic's help page to learn about available methods.

## Methods

It is your choice whether or not to document S3 **methods**.
Generally, it's not necessary to document straightforward methods for common generics like `print()`.

You must, however, always `@export` S3 methods, even for internal generics.
This **registers** the method so that the generic can find it; a user can't directly access the method definition by typing its name.
roxygen2 will warn you if you have forgotten.

```{r}
#| eval: false

#' @export
bizarro.character <- function(x, ...) {
  letters <- strsplit(x, "")
  letters_rev <- lapply(letters, rev)
  vapply(letters_rev, paste, collapse = "", FUN.VALUE = character(1))
}
```

If you are registering a method in some other way, you can use `@exportS3Method NULL` to suppress the warning.

It's good practice to document methods that have unique behavior or arguments.
For example, the clock package documents the methods for [`date_group()`](https://clock.r-lib.org/reference/date_group.html) on their own pages ([Date](https://clock.r-lib.org/reference/date-group.html), [POSIXt](https://clock.r-lib.org/reference/posixt-group.html)) because the methods accept different `precision` values and have type-specific return value semantics and examples.
The generic page serves only as a signpost linking to the individual method pages.

When documenting a method, always include a link back to the generic using `[generic_name()]` so the reader can easily find the full documentation and other methods.

### Methods for generics in other packages

It's common to write methods for generics defined in other packages.
There are three cases you need to be aware of:

- **Base packages**: you don't need to do anything special: just `@export` the method and roxygen2 will generate the correct `S3method()` directive.

- **Imported packages**: You have two options.
  Firstly, import the generic with `@importFrom` and `@export` the method:.

  ```{r}
  #| eval: false
  
  #' @importFrom pkg generic
  #' @export
  generic.foo <- function(x, ...) {}
  ```

  Alternatively, use `@exportS3Method pkg::generic`:

  ```{r}
  #| eval: false
  
  #' @exportS3Method pkg::generic
  generic.foo <- function(x, ...) {}
  ```

- **Suggested packages**: you can't import the generic for a suggested package, so you must use `@exportS3Method pkg::generic`.
  This uses delayed registration, so the method is only be registered when the suggested package is loaded.

Generally, roxygen2 can automatically figure out which generic the method belongs to.
But there is occasionally ambiguity if the generic name contains `.`.
You can avoid this problem in your own packages by not using `.` in generic names.
But you might encounter it when writing methods for generics in other packages.
For example, is `all.equal.data.frame()` the `equal.data.frame` method for `all()`, or the `data.frame` method for `all.equal()`?
If this happens to you, disambiguate with `@method`:

```{r}
#| eval: false

#' @method all.equal data.frame
#' @export
all.equal.data.frame <- function(target, current, ...) {
  # ...
}
```

roxygen2 does now automatically handle the `all.equal` case for you, so this should happen very very rarely.
And, again it can be avoided by not using `.` in generic or class names.

---
title: "Documenting S7"
description: >
  How to document S7 generics, methods, and classes.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting S7}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

There are three things that you might document for [S7](https://rconsortium.github.io/S7/):

- **Generics**: mention that the function is a generic and list the available methods.
- **Methods**: link back to the generic; only document individually when the method has unique behavior or arguments.
- **Classes**: document the constructor.

## Generics

S7 **generics** are functions, so document them as such.
Export a generic if you want users to call it or other developers to write methods for it.
If the generic is internal, you don't need to document it.

The documentation should mention that the function is a generic, because this tells the reader that the behavior may vary depending on the input and that they can write their own methods.
For simple generics, you can do this in the description:

```{r}
#| eval: false

#' Size of an object
#'
#' @description
#' `size()` is an S7 generic that determines the size of an object,
#' with methods available for the following classes:
#'
#' `r doclisting::methods_list("size")`
#'
#' @param x An object.
#' @param ... Not used.
#' @returns A single number.
#' @export
size <- new_generic("size", "x")
```

For more complicated generics, you can use a `# Methods` section to provide more detail:

```{r}
#| eval: false

#' Size of an object
#'
#' @description
#' `size()` determines the size of an object.
#'
#' # Methods
#' `size()` is an S7 generic with methods available for the following
#' classes:
#'
#' `r doclisting::methods_list("size")`
#'
#' @param x An object.
#' @param ... Not used.
#' @returns A single number.
#' @export
size <- new_generic("size", "x")
```

See the [S3 Generics section](rd-S3.html#generics) for more about using the [doclisting](https://doclisting.r-lib.org/) package to automatically generate method lists.

It's good practice to document the default method alongside the generic using `@rdname`:

```{r}
#| eval: false

#' @rdname size
method(size, class_any) <- function(x, ...) {
  length(x)
}
```

## Classes

S7 **classes** are constructor functions, so document them much like you'd document any other function.
Export a class if you want users to create instances or other developers to extend it (e.g. by creating subclasses).
Internal classes don't need documentation.

Use `@param` to document the constructor arguments (which correspond to class properties), and `@returns` to describe the object that is returned.
If the class has additional properties that are not part of the constructor (e.g. read-only computed properties), use `@prop` to document them.

```{r}
#| eval: false

#' A range
#'
#' Create a range represented by a numeric `start` and `end`. The start must
#' always be less than the end.
#'
#' @param start Start of range.
#' @param end End of range.
#' @prop length Length of the range (read-only).
#' @returns An `Range` S7 object.
#' @export
Range <- new_class(
  "Range",
  properties = list(
    start = class_numeric,
    end = class_numeric,
    length = new_property(getter = function(self) self@end - self@start),
    validator = function(self) {
      if (self@start > self@end) {
        "start must be less than or equal to end"
      }
    }
  )
)
```

If multiple classes share one Rd page (via `@rdname`), you can prefix the property name with the class name to group properties by class, e.g. `@prop ClassName@prop_name description`.

## Methods

It is your choice whether or not to document S7 **methods**.
S7 methods are registered with `method(generic, class) <- fn`.
Generally, it's not necessary to document straightforward methods.

It's good practice, however, to document methods that have unique behavior or arguments.
If you do document a method, give it its own roxygen block.
When documenting a method, always include a link back to the generic using `[generic_name()]` so the reader can easily find the full documentation and other methods.

```{r}
#| eval: false

#' Size of a range
#'
#' The size of a range is its [size()], i.e. its length.
#'
#' @param x A `Range` object.
#' @param ... Not used.
#' @returns A single number.
method(size, Range) <- function(x, ...) {
  x@length
}
```

S7 methods are registered at load time via `S7::methods_register()` in your `.onLoad()` function, not through `NAMESPACE` directives, so you never need to `@export` them.
See `vignette("packages", package = "S7")` for details.

---
title: "Documenting functions"
description: >
  The basics of roxygen2 tags and how to use them for
  documenting functions.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(comment = "#>", collapse = TRUE)
```

This vignette describes the most important roxygen2 tags for documenting functions.
See `vignette("reuse")` for reusing documentation across topics.

## Exporting

For a user to use a function in your package, you must **export** it with `@export`.
If you export a function, you must also document it, and since other people will use it, you need to be careful if you later change the function interface.
We usually put `@export` in between `@returns` and `@examples`:

```{r}
#| eval: false

#' Add two numbers together
#'
#' @param x,y A pair of numbers.
#' @returns A number.
#' @export
#' @examples
#' 1 + 2
add <- function(x, y) {
  x + y
}
```

You'll learn what all these are pieces are next!

## The introduction

Each documentation block starts with some text which defines the title, the description, and the details.
Here's an example showing what the documentation for `sum()` might look like if it had been written with roxygen:

```{r}
#' Sum of vector elements
#'
#' `sum` returns the sum of all the values present in its arguments.
#'
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.
sum <- function(..., na.rm = TRUE) {}
```

This introductory block is broken up as follows:

- The first sentence is the **title**: that's what you see when you look at `help(package = mypackage)` and is shown at the top of each help file.
  It should generally fit on one line, be written in sentence case, and not end in a full stop.

- The second paragraph is the **description**: this comes first in the documentation and should briefly describe what the function does.

- The third and subsequent paragraphs go into the **details**: this is a (often long) section that comes after the argument description and should provide any other important details of how the function operates.
  The details are optional.

### Title

The title appears in package indexes and search results, so think about how users will find your function.
Use synonyms to describe what the function does.
For example, dplyr uses titles like:

- "Keep rows that match a condition" (`filter()`)
- "Create, modify, and delete columns" (`mutate()`)
- "Summarise each group down to one row" (`summarise()`)

### Description

The description should briefly summarize the goal of the function, usually in one paragraph.
This can be challenging for simple functions, because it can feel like you're just repeating the title of the function.
Try to find a slightly different wording, if you can.
It's okay if this feels a little repetitive; it's often useful for users to see the same thing expressed in two different ways.
It's a little extra work, but the extra effort is often worth it.

```{r}
#| eval: false

#' Detect the presence/absence of a match
#'
#' `str_detect()` returns a logical vector with `TRUE` for each element of
#' `string` that matches `pattern` and `FALSE` otherwise. It's equivalent to
#' `grepl(pattern, string)`.
```

If you need a multi-paragraph description, use an explicit `@description` tag:

```{r}
#| eval: false

#' View strings and matches
#'
#' @description
#' `str_view()` is used to print the underlying representation of a string and
#' to see how a `pattern` matches.
#'
#' Matches are surrounded by `<>` and unusual whitespace (i.e. all whitespace
#' apart from `" "` and `"\n"`) are surrounded by `{}` and escaped.
```

Basically, if you're going to include an empty line in your description, you'll need to use an explicit `@description` tag.

### Details

The details section is optional and comes after the argument description in the rendered help.
Use informative markdown headings to break up long details.

```{r}
#' Sum of vector elements
#'
#' @description
#' `sum` returns the sum of all the values present in its arguments.
#'
#' @details
#' This is a generic function: methods can be defined for it directly
#' or via the [Summary()] group generic. For this to work properly,
#' the arguments `...` should be unnamed, and dispatch is on the
#' first argument.
```

## Required tags

Functions are the most commonly documented objects.
Functions require three tags: `@param`, `@returns`, and `@examples`.

### Inputs

Use `@param name description` to describe each input to the function.
The description should provide a succinct summary of the parameter type (e.g. a string, a numeric vector), and if not obvious from the name, what the parameter does.
The description is a sentence so should start with a capital letter and end with a full stop.
It can span multiple lines (or even paragraphs) if necessary.
All parameters must be documented.

```{r}
#| eval: false

#' @param pattern Pattern to look for.
#'
#'   The default interpretation is a regular expression, as described in
#'   `vignette("regular-expressions")`. Use [regex()] for finer control of the
#'   matching behaviour.
#'
#' @param string Input vector. Either a character vector, or something
#'   coercible to one.
```

It's a good idea to describe the default value in the `@param` description, particularly when the default is non-obvious or the signature and the rendered argument documentation are far apart.

```{r}
#| eval: false

#' @param na.rm Remove missing values? If `FALSE` (the default), the result
#'   will be `NA` if any element of `string` is `NA`.
```

For arguments with a small fixed set of possible values, list them in the description:

```{r}
#| eval: false

#' @param side Side on which to remove whitespace: `"left"`, `"right"`, or
#'   `"both"` (the default).
```

If each value needs more explanation, use a bulleted list:

```{r}
#| eval: false

#' @param whitespace_only A boolean.
#'   * `TRUE` (the default): wrapping will only occur at whitespace.
#'   * `FALSE`: can break on any non-word character (e.g. `/`, `-`).
```

If two or more arguments are tightly coupled, you can document them in one place by separating the names with commas (no spaces).
For example, to document both `x` and `y`, you can say `@param x,y Numeric vectors`.

### Outputs

`@returns description` describes the output from the function.
Briefly describe the type/shape of the output, not the details.

All functions must have a documented return value for initial CRAN submission.

For simple cases, a single sentence suffices:

```{r}
#| eval: false

#' @returns A logical vector the same length as `string`.
```

For functions returning data frames, describe how the output relates to the input:

```{r}
#| eval: false

#' @returns
#' An object of the same type as `.data`. The output has the following
#' properties:
#'
#' * Rows are a subset of the input, but appear in the same order.
#' * Columns are not modified.
#' * The number of groups may be reduced (if `.preserve` is not `TRUE`).
#' * Data frame attributes are preserved.
```

### Examples

`@examples` provides executable R code showing how to use the function in practice.
This is a very important part of the documentation because many people look at the examples before reading anything.
Example code must work without errors as it is run automatically as part of `R CMD check`.

Focus on demonstrating the most important features with realistic, typical usage.
Use comments to break examples into sections:

```{r}
#| eval: false

#' @examples
#' fruit <- c("apple", "banana", "pear", "pineapple")
#' str_detect(fruit, "a")
#' str_detect(fruit, "^a")
#' str_detect(fruit, "a$")
#'
#' # Also vectorised over pattern
#' str_detect("aecfg", letters)
```

Examples must be self-contained and not change the user's state: if you modify `options()`, reset them; create files in `tempdir()` and delete them; don't change the working directory or write to the clipboard.

For the purpose of illustration, it's often useful to include code that causes an error.
`try()` is the best way to do this, because users will see the error message:

```{r}
#| eval: false

#' @examples
#' # Row sizes must be compatible when column-binding
#' try(bind_cols(tibble(x = 1:3), tibble(y = 1:2)))
```

You can also use `\dontrun{}` to prevent code from executing, but `try()` is generally preferred.

For code that only works in certain environments, use `@examplesIf`:

```r
#' @examplesIf interactive()
#' browseURL("https://roxygen2.r-lib.org")
```

This generates

```
\examples{
\dontshow{if (interactive() (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
gh_organizations(since = 42)
\dontshow{\}) # examplesIf}
}
```

This way, the code evaluating whether the example can be run is not shown to users reading the help, but it still prevents R CMD check failures.
`@examplesIf` is preferred over wrapping code in `if()` because users don't see the machinery, and unlike `\dontrun{}`, the code still runs when the condition is met.

Instead of including examples directly in the documentation, you can put them in separate files and use `@example path/relative/to/package/root` to insert them into the documentation.

All functions must have examples for initial CRAN submission.

### Usage

In most cases, the function usage (which appears beneath the description in the generated docs) will be automatically derived from the function specification.
For the cases where it is not, please [file an issue](https://github.com/r-lib/roxygen2/issues) and use `@usage` to override the default with what you want.
If you want to suppress the usage altogether (which is sometimes useful for internal or deprecated functions), you can use `@usage NULL`.

---
title: "Documenting packages"
description: >
  How to document a package as a whole using the
  `"_PACKAGE"` sentinel.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting packages}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

As well as documenting every object inside the package, you can also document the package itself by documenting the special sentinel `"_PACKAGE"`.
This automatically includes information parsed from the `DESCRIPTION`, including title, description, list of authors, and useful URLs.

We recommend placing package documentation in `{pkgname}-package.R`, and have `@keywords internal`.
Use `usethis::use_package_doc()` to set up automatically.
Here's an example:

```{r}
#| eval: false

#' @keywords internal
"_PACKAGE"
```

Package documentation is a good place to put `# Package options` that documents options used by the package.

Some notes:

- By default, aliases will be added so that both `?pkgname` and `package?pkgname` will find the package help.
  If there's an existing function called `pkgname`, use `@aliases {pkgname}-package NULL` to override the default.

- Use `@references` to point to published material about the package that users might find helpful.

---
title: "Documenting datasets"
description: >
  How to document datasets stored in `data/`.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting datasets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Datasets are stored in `data/`, not as regular R objects in the package.
This means you need to document them in a slightly different way: instead of documenting the data directly, you quote the dataset's name.
For example, this is the roxygen2 block used for `ggplot2::diamonds`:

```{r}
#| eval: false

#' Prices of over 50,000 round cut diamonds
#'
#' A dataset containing the prices and other attributes of almost 54,000
#'  diamonds. The variables are as follows:
#'
#' @format A data frame with 53940 rows and 10 variables:
#' \describe{
#'   \item{price}{price in US dollars ($326--$18,823)}
#'   \item{carat}{weight of the diamond (0.2--5.01)}
#'   \item{cut}{quality of the cut (Fair, Good, Very Good, Premium, Ideal)}
#'   \item{color}{diamond colour, from D (best) to J (worst)}
#'   \item{clarity}{a measurement of how clear the diamond is (I1 (worst), SI2,
#'     SI1, VS2, VS1, VVS2, VVS1, IF (best))}
#'   \item{x}{length in mm (0--10.74)}
#'   \item{y}{width in mm (0--58.9)}
#'   \item{z}{depth in mm (0--31.8)}
#'   \item{depth}{total depth percentage = z / mean(x, y) = 2 * z / (x + y) (43--79)}
#'   \item{table}{width of top of diamond relative to widest point (43--95)}
#' }
#'
#' @source {ggplot2} tidyverse R package.
"diamonds"
```

Datasets should never be exported with `@export` because they are not found in the `NAMESPACE`.
Instead, datasets will either be automatically available if you set `LazyData: true` in your `DESCRIPTION`, or available after calling `data()` if not.
This field also affects the default usage.
If you have `LazyData: true`, the usage will be just the dataset name (e.g. `diamonds`).
Otherwise, the usage will be wrapped in `data()` (e.g. `data(diamonds)`).

Note the use of two additional tags that are particularly useful for documenting data:

- `@format`, which gives an overview of the structure of the dataset.
  This should include a **definition list** that describes each variable.
  There's currently no way to generate this with Markdown, so this is one of the few places you'll need to Rd markup directly.

- `@source` where you got the data form, often a URL.

---
title: "Reusing documentation"
description: >
  Tools for reusing documentation across topics, and between
  documentation and vignettes.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reusing documentation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

roxygen2 provides several ways to avoid repeating yourself in code documentation, while assembling information from multiple places in one documentation file:

-   Combine documentation for closely related functions into a single file with `@describe` in or `@rdname`.

-   Inherit components from another topic with `@inheritParams`, `@inheritSection`, or `@inherit`.

-   Use functions to generate repeated text with inline R code.

-   Share text between documentation and vignettes with child documents.

The chapter concludes by showing you how to update superseded reuse mechanisms that we no longer recommend: `@includeRmd`, `@eval`/`@evalRd`, and `@template`.

## Multiple functions in the same topic

You can document multiple functions in the same file by using either `@rdname` or `@describeIn` tag.
It's a technique best used with care: documenting too many functions in one place leads to confusion.
Use it when all functions have the same (or very similar) arguments.

### `@describeIn`

Use `@describeIn <destination> <description>` to document mutiple functions together.
It generates a new section named "**Related functions and methods**", further divided into subsections by the type of relationship between the source and the destination documentation.
Each subsection contains a bulleted list describing the function with the specified `<description>`.

You can document several functions which perform slight variations of a task or which use different defaults in one documentation file:

```{r}
#' Power
#' @param x base
#' @param exp exponent
power <- function(x, exp) x ^ exp

#' @describeIn power Square a number
square <- function(x) power(x, 2)

#' @describeIn power Cube a number
cube <- function(x) power(x, 3)
```

In this case, both destination and source are "normal" functions, and you can use `<description>` to explain their differences.

`@describeIn` is also well suited to explain the relationships between functions making up an object oriented (OO) scheme using S3 or S4.
In this case, roxygen will automatically detect whether a source function is a method and document the relationship to the relevant class or generic.
Your new methods will be organised in one of two ways:

1.  If your new method extends a generic in the `<destination>`, it will be listed in a section titled "**Methods for generic `<destination>`**".
    The subsection will include all methods for your new generic, labelled by the class they cover.

2.  If your new method extends another generic, for a class constructed in the destination, it will be listed in a subsection titled "**Methods for class `<destination>`**".
    It will be labelled by the generic which the method extends.

While you can mix and match (some) different types of functions with `@describeIn`, the output may quickly become disorienting for users if the functions are not tightly related.

### Order of includes

By default, roxygen blocks are processed in the order in which they appear in the file.
When you're combining multiple files, this can sometimes cause the function usage to appear in a suboptimal order.
You can override the default ordering with `@order`.
For example, the following the block would place `times` first in `arith.Rd` because 1 comes before 2.

```{r}
#' @rdname arith
#' @order 2
add <- function(x, y) x + y

#' @rdname arith
#' @order 1
times <- function(x, y) x * y
```

## Inheriting documentation

You can inherit documentation from other topics in a few ways:

-   `@inheritParams source_function` inherits just the parameter documentation from `source_function()`.

-   `@inherit source_function` will inherit all supported components from `source_function`.

    Alternatively, you can use (e.g.) `@inherit source_function return details` to just inherit the return value and details from `source_function()`.
    Supported components are `r paste0("\x60", roxygen2:::inherit_components, "\x60", collapse = ", ")`.

-   `@inheritSection source_function Section title` will inherit the single `@section` called "Section title" from `source_function()`.

-   `@inheritDotParams` automatically generates parameter documentation for `...` for the common case where you pass `...` on to another function.
    Because you often override some arguments, it comes with a flexible specification for argument selection:

    -   `@inheritDotParams foo` takes all parameters from `foo()`
    -   `@inheritDotParams foo a b e:h` takes parameters `a`, `b`, and all parameters between `e` and `h`
    -   `@inheritDotParams foo -x -y` takes all parameters except for `x` and `y`.

All of these tags also work to inherit documentation from functions in another package by using `pkg::source_function`.

## Inline code

To insert code inline, enclose it in `` `r ` ``.
Roxygen will interpret the rest of the text within backticks as R code and evaluate it, and replace the backtick expression with its value.
Here's a simple example:

```{r include = FALSE}
roxygen2:::markdown_on()

simple_inline <- "#' Title `r 1 + 1`
#'
#' Description `r 2 + 2`
foo <- function() NULL
"
```

```{r code=simple_inline}
```

This is equivalent to writing:

```{r code = roxygen2:::markdown(simple_inline)}
```

The resulting text, together with the whole tag is interpreted as markdown, as usual.
This means that you can use R to dynamically write markdown.
For example if you defined this function in your package:

```{r}
alphabet <- function(n) {
  paste0("`", letters[1:n], "`", collapse = ", ")
}
```

You could then write:

```{r echo=FALSE}
env <- new.env()
env$alphabet <- alphabet
roxygen2:::roxy_meta_set("evalenv", env)

backtick <- "#' Title
#' 
#' @param x A string. Must be one of `r alphabet(5)`
foo <- function(x) NULL
"
```

```{r code = backtick}
```

The result is equivalent to writing the following by hand:

```{r code = roxygen2:::markdown_pass1(backtick)}
```

This is a powerful technique for reducing duplication because you can flexibly parameterise the function however best meets your needs.
Note that the evaluation environment is deliberately a child of the package that you're documenting so you can call internal functions.

## Child documents

You can use the same `.Rmd` or `.md` document in the documentation, `README.Rmd`, and vignettes but using child documents:

    ```{r child = "common.Rmd"}`r ''`
    ```

The included Rmd file can have roxygen Markdown-style links to other help topics.
E.g. `[roxygen2::roxygenize()]` will link to the manual page of the `roxygenize` function in roxygen2.
See `vignette("rd-formatting")` for details.

If the Rmd file contains roxygen (Markdown-style) links to other help topics, then some care is needed, as those links will not work in Rmd files by default.
A workaround is to specify external HTML links for them.
These external locations will *not* be used for the manual which instead always links to the help topics in the manual.
Example:

    See also the [roxygen2::roxygenize()] function.

    [roxygen2::roxygenize()]: https://roxygen2.r-lib.org/reference/roxygenize.html

This example will link to the supplied URLs in HTML / Markdown files and it will link to the `roxygenize` help topic in the manual.

Note that if you add external link targets like these, then roxygen will emit a warning about these link references being defined multiple times (once externally, and once to the help topic).
This warning originates in Pandoc, and it is harmless.

## Superseded

Over the years, we have experimented with a number of other ways to reduce duplication across documentation files.
A number of these are now superseded and we recommend changing them to use the techniques described above:

-   Instead of `@includeRmd man/rmd/example.Rmd`, use a child document.

-   Instead of `@eval` or `@evalRd`, use inline R code.

-   Instead of `@template` and `@templateVars` write your own function and call it from inline R code.

Inline R markdown can only generate markdown text within a tag so in principle it is less flexible than `@eval`/`@evalRd`/`@template`.
However, our experience has revealed that generating multiple tags at once tends to be rather inflexible, and you often end up refactoring into smaller pieces so we don't believe this reflects a real loss of functionality.

---
title: "Documenting other objects"
description: > 
  How to document datasets, packages, and the classes,
  generics, and methods of S3, S4, and R6.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Documenting other objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Datasets

Datasets are stored in `data/`, not as regular R objects in the package.
This means you need to document them in a slightly different way: instead of documenting the data directly, you quote the dataset's name.

```{r}
#' Prices of over 50,000 round cut diamonds
#'
#' A dataset containing the prices and other attributes of almost 54,000
#'  diamonds. The variables are as follows:
#'
#' @format A data frame with 53940 rows and 10 variables:
#' \describe{
#'   \item{price}{price in US dollars ($326--$18,823)}
#'   \item{carat}{weight of the diamond (0.2--5.01)}
#'   \item{cut}{quality of the cut (Fair, Good, Very Good, Premium, Ideal)}
#'   \item{color}{diamond colour, from D (best) to J (worst)}
#'   \item{clarity}{a measurement of how clear the diamond is (I1 (worst), SI2,
#'     SI1, VS2, VS1, VVS2, VVS1, IF (best))}
#'   \item{x}{length in mm (0--10.74)}
#'   \item{y}{width in mm (0--58.9)}
#'   \item{z}{depth in mm (0--31.8)}
#'   \item{depth}{total depth percentage = z / mean(x, y) = 2 * z / (x + y) (43--79)}
#'   \item{table}{width of top of diamond relative to widest point (43--95)}
#' }
#' 
#' @source {ggplot2} tidyverse R package.
```

Note the use of two additional tags that are particularly useful for documenting data:

-   `@format`, which gives an overview of the structure of the dataset.
    This should include a **definition list** that describes each variable.
    There's currently no way to generate this with Markdown, so this is one of the few places you'll need to Rd markup directly.

-   `@source` where you got the data form, often a URL.

## Packages

As well as documenting every object inside the package, you can also document the package itself by documenting the special sentinel `"_PACKAGE"`.
This automatically includes information parsed from the `DESCRIPTION`, including title, description, list of authors, and useful URLs.

We recommend placing package documentation in `{pkgname}-package.R`, and have `@keywords internal`.
Use `usethis::use_package_doc()` to set up automatically.

Here's an example:

```{r, eval = FALSE}
#' @keywords internal 
"_PACKAGE"
```

Package documentation is a good place to put `# Package options` that documents options used by the package.

Some notes:

-   By default, aliases will be added so that both `?pkgname` and `package?pkgname` will find the package help.
    If there's an existing function called `pkgname`, use `@aliases {pkgname}-package NULL` to override the default.

-   Use `@references` to point to published material about the package that users might find helpful.

## S3

-   S3 **generics** are regular functions, so document them as such.
    If necessary, include a section that provides additional details for developers implementing methods.

-   S3 **classes** have no formal definition, so document the [constructor](https://adv-r.hadley.nz/s3.html#s3-constructor).

-   It is your choice whether or not to document S3 **methods**.
    Generally, it's not necessary to document straightforward methods for common generics like `print()`.
    (You should, however, always `@export` S3 methods).

    If your method is more complicated, you should document it by setting `@rdname` or `@describeIn`.
    For complicated methods, you might document in their own file (i.e. `@rdname generic.class`; for simpler methods you might document with the generic (i.e. `@describeIn generic)`.
    Learn more about these tags in `vignette("reuse")`.

-   Generally, roxygen2 will automatically figure out the generic that the method belongs to, and you should only need to use `@method` if there is ambiguity.
    For example, is `all.equal.data.frame()` the `equal.data.frame` method for `all()`, or the `data.frame` method for `all.equal()`?.
    If this happens to you, disambiguate with (e.g.) `@method all.equal data.frame`.

## S4

S4 **generics** are also functions, so document them as such.

Document **S4 classes** by adding a roxygen block before `setClass()`.
Use `@slot` to document the slots of the class.
Here's a simple example:

```{r}
#' An S4 class to represent a bank account
#'
#' @slot balance A length-one numeric vector
Account <- setClass("Account",
  slots = list(balance = "numeric")
)
```

S4 **methods** are a little more complicated.
Unlike S3 methods, all S4 methods must be documented.
You can document them in three places:

-   In the class.
    Most appropriate if the corresponding generic uses single dispatch and you created the class.

-   In the generic.
    Most appropriate if the generic uses multiple dispatches and you control it.

-   In its own file.
    Most appropriate if the method is complex.
    or the either two options don't apply.

Use either `@rdname` or `@describeIn` to control where method documentation goes.
See the next section for more details.

## R6

-   R6 methods can be documented in-line, i.e. the method's documentation comments come right before the definition of the method.

-   Method documentation can use the `@description`, `@details`, `@param`, `@return` and `@examples` tags.
    These are used to create a subsection for the method, within a separate 'Methods' section.
    All roxygen comment lines of a method documentation must appear after a tag.

-   `@param` tags that appear before the class definition are automatically inherited by all methods, if needed.

-   R6 fields and active bindings can make use of the `@field` tag.
    Their documentation should also be in-line.

-   roxygen2 checks that all public methods, public fields, active bindings and all method arguments are documented, and issues warnings otherwise.

-   To turn off the special handling of R6 classes and go back to the roxygen2 6.x.x behavior, use the `r6 = FALSE` option in `DESCRIPTION`, in the `Roxygen` entry: `Roxygen: list(r6 = FALSE)`.

roxygen2 automatically generates additional sections for an R6 class:

-   A section with information about the superclass(es) of the class, with links.
    In HTML this includes a list of all inherited methods, with links.

-   An 'Examples' section that contains all class and method examples.
    This section is run by `R CMD check`, so method examples must work without errors.

An example from the R6 tutorial:

```{r}
#' R6 Class Representing a Person
#'
#' @description
#' A person has a name and a hair color.
#'
#' @details
#' A person can also greet you.

Person <- R6::R6Class("Person",
public = list(

    #' @field name First or full name of the person.
    name = NULL,

    #' @field hair Hair color of the person.
    hair = NULL,

    #' @description
    #' Create a new person object.
    #' @param name Name.
    #' @param hair Hair color.
    #' @return A new `Person` object.
    initialize = function(name = NA, hair = NA) {
      self$name <- name
      self$hair <- hair
      self$greet()
    },

    #' @description
    #' Change hair color.
    #' @param val New hair color.
    #' @examples
    #' P <- Person("Ann", "black")
    #' P$hair
    #' P$set_hair("red")
    #' P$hair
    set_hair = function(val) {
      self$hair <- val
    },

    #' @description
    #' Say hi.
    greet = function() {
      cat(paste0("Hello, my name is ", self$name, ".\n"))
    }
  )
)
```

---
title: "Managing imports and exports"
description: >
  Generating the `NAMESPACE` file with roxygen2.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Managing imports and exports}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false

knitr::opts_chunk$set(comment = "#>", collapse = TRUE)
```

The package `NAMESPACE` is one of the most confusing parts of building a package.
roxygen2 aims to make it as easy as possible to build a package that is a well-behaved member of the R ecosystem.
This is a little frustrating at first, but soon becomes second-nature.

## Exports

In order for your users to use a function[^1] from your package, you must **export** it.
In most cases, you can just use the `@export` tag, and roxygen2 will automatically figure out which `NAMESPACE` directive (i.e. `export()`, `S3method()`, `exportClasses()`, or `exportMethods()`) you need.

[^1]: Including S3/S4/S7 generics and constructors.

For details and examples of exporting functions, and S3, S4, and S7 classes/generics/methods, see the corresponding vignettes:

- `vignette("rd-functions")` for regular functions.
- `vignette("rd-S3")` for S3 generics and methods, including delayed registration with `@exportS3Method`.
- `vignette("rd-S4")` for S4 generics, classes, and methods.
- S7 methods are registered at load time, not through `NAMESPACE`; see `vignette("rd-S7")`.

Datasets should never be exported as they are not found in `NAMESPACE`.
Learn more in `vignette("rd-datasets")`.

### Manual exports

If `@export` does not automatically generate the correct `NAMESPACE` directive, you can use one of the tags below to exercise greater control:

- `@export foo` generates `export(foo)`
- `@exportS3Method generic method` generates `S3method(generic, method)`
- `@exportClass foo` generates `exportClasses(foo)`
- `@exportMethod foo` generates `exportMethods(foo)`
- `@exportPattern foo` generates `exportPattern(foo)`

For even more specialised cases you can use `@rawNamespace code` which inserts `code` literally into the `NAMESPACE`.
This is useful if you need a conditional import or export, e.g.

```{r}
# From dplyr:
#' @rawNamespace import(vctrs, except = data_frame)
NULL

# From backports:
#' @rawNamespace if (getRversion() < "4.0.0") export(stopifnot)
NULL
```

If you need to automate this, `@evalNamespace fun()` will evaluate `fun()` in your package environment and insert the results into `NAMESPACE`.
Note that because `evalNamespace()` is run in the package environment, it can only generate exports, not imports.

## Imports

The `NAMESPACE` also controls which functions from other packages are made available to your package.

### Functions

If you are using just a few functions from another package, we recommending adding the package to the `Imports:` field of the `DESCRIPTION` file and calling the functions explicitly using `::`, e.g., `pkg::fun()`.

```r
my_function <- function(x, y) {
  pkg::fun(x) * y
}
```

If the repetition of the package name becomes annoying you can `@importFrom` and drop the `::`:

```r
#' @importFrom pkg fun
my_function <- function(x, y) {
  fun(x) * y
}
```

Imports affect every function in a package, so it's common to collect them in a central place, like `{packagename}-package.R`.
This is automated by `usethis::use_import_from()`.

```{r}
#' @importFrom pkg fun1 fun2
#' @importFrom pkg2 fun3
#' @importFrom pkg3 fun4
NULL
```

Note the use of `NULL` here: you must provide something for roxygen2 to document, so we use `NULL` as placeholder.

It is possible, but not generally recommended to import all functions from a package with `@import package`.
This is risky if you import functions from more than one package, because while it might be ok today, in the future the packages might end up with a function having the same name, and your users will get a warning every time your package is loaded.
Any `@import` specification containing a comma, e.g. `@import rlang, except = ":="` will be inserted as is into the the NAMESPACE, e.g. `import(rlang, except = ":=")`.

### S3

S3 generics are just functions, so the same rules for functions apply.
S3 methods always accompany the generic, so as long as you can access the generic, the methods will also be available.
In other words, you don't need to do anything to import an S3 method.

### S4

- To use classes defined in another package, place `@importClassesFrom package ClassA ClassB ...` next to the classes that inherit from the imported classes, or next to the methods that implement a generic for the imported classes.
- To use generics defined in another package, place `@importMethodsFrom package GenericA GenericB ...` next to the methods that use the imported generics.

### Compiled code

To import compiled code from another package, use `@useDynLib`

- `@useDynLib package` imports all compiled functions.

- `@useDynLib package routinea routineb` imports selected compiled functions.

- Any `@useDynLib` specification containing a comma, e.g. `@useDynLib mypackage, .registration = TRUE` will be inserted as is into the the NAMESPACE, e.g. `useDynLib(mypackage, .registration = TRUE)`

# update_collate() checks that directory exists

    Code
      update_collate("doesn't-exist")
    Condition
      Error in `update_collate()`:
      ! 'doesn't-exist' doesn't exist

# DESCRIPTION file is re-written only if collate changes

    Code
      update_collate(".")
    Message
      Updating collate directive in './DESCRIPTION'
    Code
      # Second run should be idempotent
      update_collate(".")

# exporting a call to :: produces re-exports documentation

    % Generated by roxygen2: do not edit by hand
    % Please edit documentation in ./<text>
    \docType{import}
    \name{reexports}
    \alias{reexports}
    \alias{auto_test}
    \title{Objects exported from other packages}
    \keyword{internal}
    \description{
    These objects are imported from other packages. Follow the links
    below to see their documentation.
    
    \describe{
      \item{testthat}{\code{\link[testthat]{auto_test}}}
    }}

# write_if_different produces informative messages

    Code
      write_if_different(path, "a <- 2")
    Message
      Writing 'test.R'
    Output
      [1] TRUE

---

    Code
      write_if_different(path, "a <- 2")
    Message
      x Skipping 'test.R'
      i It already exists and was not generated by roxygen2.
    Output
      [1] FALSE

---

    Code
      write_if_different(path, "a <- 2")
    Message
      x Skipping '+.R'
      i Invalid file name
    Output
      [1] FALSE

# write_if_different produces correct command hyperlink

    Code
      write_if_different(path, "a <- 2", command = "rlang::inform('hi')")
    Message
      Writing ']8;;x-r-run:rlang::inform('hi')test.R]8;;'
    Output
      [1] TRUE

# missing title generates useful message

    Code
      roc_proc_text(rd_roclet(), block)
    Condition
      Warning:
      Skipping 'foo.Rd'
      i File lacks name and/or title
    Output
      named list()

# new wrapping style doesn't change unexpectedly

    f(
      a = "                                    a",
      b = "                                    b",
      c = "                                    c",
      d = "                                    d"
    ) 
    
    f(
      a = c("abcdef", "abcdef", "abcdef", "abcdef", "abcdef", "abcdef", "abcdef", "abcdef",
        "abcdef", "abcdef")
    ) 
    
    \method{mean}{reallyratherquitelongclassname}(
      reallyreatherquitelongargument = "reallyratherquitelongvalue_____________________"
    ) 
    
    long_replacement_fun(
      x,
      a = "aaaaaaaaaaaaaaaa",
      b = "aaaaaaaaaaaaaaaa",
      c = "aaaaaaaaaaaaaaaa"
    ) <- value 
    
    function_name(
      x,
      y,
      xy = "abcdef",
      xyz = c(`word word word word` = "abcdef", `word word word` = "abcdef", `word word word`
        = "abcdef", `word word word` = "abcdef")
    ) 
    
    function_name(
      f = function(x) {
         1
         2
     }
    ) 
    

# old wrapping style doesn't change unexpectedly

    f(a = "                                    a",
      b = "                                    b",
      c = "                                    c",
      d = "                                    d") 
    
    f(a = c("abcdef", "abcdef", "abcdef", "abcdef", "abcdef", "abcdef", "abcdef",
      "abcdef", "abcdef", "abcdef")) 
    
    
      \method{mean}{reallyratherquitelongclassname}(reallyreatherquitelongargument = "reallyratherquitelongvalue_____________________") 
    
    long_replacement_fun(x, a = "aaaaaaaaaaaaaaaa", b = "aaaaaaaaaaaaaaaa",
      c = "aaaaaaaaaaaaaaaa") <- value 
    
    f(xxxxxxxxxxxxxxxxxx1, xxxxxxxxxxxxxxxxxx2, xxxxxxxxxxxxxxxxxx3, x = "\\"'",
      xxxxxxxxxxxxxxxxxx4, xxxxxxxxxxxxxxxxxx5, xxxxxxxxxxxxxxxxxx6,
      xxxxxxxxxxxxxxxxxx7) 
    

# invalid syntax gives useful warning

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:2: @includeRmd requires a path.

# useful warnings

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: @includeRmd Can't find Rmd 'path'.

---

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Output
      
    Message
      
      Quitting from <temp-path.Rmd>:1-3 [unnamed-chunk-2]
      
      Quitting from <another-temp-path.Rmd>:1-2 [unnamed-chunk-1]
      x <text>:3: @includeRmd failed to evaluate '<temp-path.Rmd>'.
      Caused by error:
      ! Error

# useful warning if no topic found

    Code
      . <- find_package("doesntexist")
    Message
      x Could not resolve link to topic "doesntexist" in the dependencies or base packages.
      i If you haven't documented "doesntexist" yet, or just changed its name, this is normal. Once "doesntexist" is documented, this warning goes away.
      i Make sure that the name of the topic is spelled correctly.
      i Always list the linked package as a dependency.
      i Alternatively, you can fully qualify the link with a package name.

# gives useful warning if same name in multiple packages

    Code
      . <- find_package("pkg_env")
    Message
      x Topic "pkg_env" is available in multiple packages: pkgload and rlang.
      i Qualify topic explicitly with a package name when linking to it.

# warns about multiple @return(s) tags

    Code
      docs <- r6_doc(text)
    Message
      x <text>:3: Must use one @return(s) per R6 method.

# warns about undocumented params

    Code
      docs <- r6_doc(text)
    Message
      x In topic 'C': Must use one @param for each argument.
      x run(x) is not documented
      x In topic 'C': Must use one @param for each argument.
      x run(y) is not documented

# warns about duplicated params

    Code
      docs <- r6_doc(text)
    Message
      x <text>:3: Must use one @param for each argument.
      x $run(x) is documented multiple times

# format.rd_r6_method produces method subsection

    Code
      cat(format(method), sep = "\n")
    Output
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Person-greet"></a>}}
      \if{latex}{\out{\hypertarget{method-Person-greet}{}}}
      \subsection{\code{Person$greet()}}{
        Say hello.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Person$greet(who, how = "nicely")}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{who}}{Name to greet.}
            \item{\code{how}}{Greeting style.}
          }
          \if{html}{\out{</div>}}
        }
      }
      

# format.rd_r6_method renames initialize to new

    Code
      cat(format(method), sep = "\n")
    Output
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Foo-initialize"></a>}}
      \if{latex}{\out{\hypertarget{method-Foo-initialize}{}}}
      \subsection{\code{Foo$new()}}{
        Create object.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Foo$new()}
          \if{html}{\out{</div>}}
        }
      }
      

# format.rd_r6_method includes optional sections

    Code
      cat(format(method), sep = "\n")
    Output
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Job-run"></a>}}
      \if{latex}{\out{\hypertarget{method-Job-run}{}}}
      \subsection{\code{Job$run()}}{
        Run the job.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Job$run()}
          \if{html}{\out{</div>}}
        }
        \subsection{Details}{
          Some details.
        }
        \subsection{Returns}{
          The result.
        }
        \subsection{Examples}{
          \if{html}{\out{<div class="r example copy">}}
          \preformatted{job$run()
      }
          \if{html}{\out{</div>}}
        }
      }
      

# format.rd_r6_method strips \dontrun etc from examples (#1072)

    Code
      cat(format(method), sep = "\n")
    Output
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Job-run"></a>}}
      \if{latex}{\out{\hypertarget{method-Job-run}{}}}
      \subsection{\code{Job$run()}}{
        Run the job.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Job$run()}
          \if{html}{\out{</div>}}
        }
        \subsection{Examples}{
          \if{html}{\out{<div class="r example copy">}}
          \preformatted{      
      donrun()
            
      donttest()
            
          
      }
          \if{html}{\out{</div>}}
        }
      }
      

# format.rd_r6_method omits empty optional sections

    Code
      cat(format(method), sep = "\n")
    Output
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Job-run"></a>}}
      \if{latex}{\out{\hypertarget{method-Job-run}{}}}
      \subsection{\code{Job$run()}}{
        Run.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Job$run()}
          \if{html}{\out{</div>}}
        }
      }
      

# @examples and @example interleave

    \examples{
    example <- 'example1'
    a <- 2
    example <- 'example2'
    } 

# @example gives warning if used instead of @examples

    Code
      out <- roc_proc_text(rd_roclet(), block)[[1]]
    Message
      x <text>:4: @example must be a single line.
      i Do you want @examples?

# warns if path doesn't exist

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:4: @example './this-path-doesnt-exist.R' doesn't exist.

# @examplesIf

    \examples{
    \dontshow{if (foo::bar()) withAutoprint(\{ # examplesIf}
    maybe-run-this-code
    \dontshow{\}) # examplesIf}
    \dontshow{if (foobar()) withAutoprint(\{ # examplesIf}
    and-this
    and-that
    \dontshow{\}) # examplesIf}
    } 

# @examplesIf warns about unparseable condition

    Code
      out <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:4: @examplesIf condition failed to parse.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: 1 +
         ^

# @examplesIf warns on empty body (#1695)

    Code
      out <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:4: @examplesIf requires example code after the condition.

# person turned into meaningful text

    Code
      # Multiple given/family names
      person_desc(c("First", "Second"), c("Family1", "Family2"))
    Output
      [1] "First Second Family1 Family2 \\email{h@w.com}"
    Code
      # Multiple roles
      person_desc(role = "ctb")
    Output
      [1] "H W \\email{h@w.com} [contributor]"
    Code
      # ORCID comments
      person_desc(comment = c(ORCID = "0000-0003-4757-117X"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://orcid.org/0000-0003-4757-117X}{ORCID})"
    Code
      person_desc(comment = c(ORCID = "https://orcid.org/0000-0003-4757-117X"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://orcid.org/0000-0003-4757-117X}{ORCID})"
    Code
      person_desc(comment = c(ORCID = "0000-0003-4757-117X", "extra"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://orcid.org/0000-0003-4757-117X}{ORCID}) (extra)"
    Code
      # ROR comments
      person_desc(comment = c(ROR = "03wc8by49"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://ror.org/03wc8by49}{ROR})"
    Code
      person_desc(comment = c(ROR = "https://ror.org/03wc8by49"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://ror.org/03wc8by49}{ROR})"
    Code
      person_desc(comment = c(ROR = "03wc8by49", "extra"))
    Output
      [1] "H W \\email{h@w.com} (\\href{https://ror.org/03wc8by49}{ROR}) (extra)"
    Code
      # Arbitrary comments (#1746)
      person_desc(comment = c(acronym = "rajo", "Contributed XY"))
    Output
      [1] "H W \\email{h@w.com} (acronym: rajo, Contributed XY)"
    Code
      person_desc(comment = c("comment 1", "comment 2"))
    Output
      [1] "H W \\email{h@w.com} (comment 1, comment 2)"

# useful message if Authors@R is corrupted

    Code
      package_authors("1 + ")
    Message
      x Failed to evaluate Authors@R.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: 1 + 
         ^
    Output
      NULL
    Code
      package_authors("stop('Uhoh')")
    Message
      x Failed to evaluate Authors@R.
      Caused by error:
      ! Uhoh
    Output
      NULL

# @R6method tag parser validates input

    Code
      . <- roxy_tag_parse(roxy_tag("R6method", ""))
    Message
      x NA:NA: @R6method requires a value like `Class$method`.
    Code
      . <- roxy_tag_parse(roxy_tag("R6method", "nomethod"))
    Message
      x NA:NA: @R6method must be of the form `Class$method`.
    Code
      . <- roxy_tag_parse(roxy_tag("R6method", "Foo$bar\nextra"))
    Message
      x NA:NA: @R6method must be only 1 line long, not 2.
      i The first line is "Foo$bar"

# @R6method warns on unknown class

    Code
      roc_proc_text(rd_roclet(), text)
    Message
      x <text>:2: @R6method Can't find R6 class <NoSuchClass>.
    Output
      list()

# short and sweet links work

    Code
      out1 <- roc_proc_text(rd_roclet(),
      "\n    #' Title\n    #'\n    #' See [11pkg::function()], [11pkg::object].\n    #' @md\n    foo <- function() {}")[[
        1]]
    Message
      x <text>:4: @description refers to un-installed package 11pkg.
      x <text>:4: @description refers to un-installed package 11pkg.

---

    Code
      out1 <- roc_proc_text(rd_roclet(), block)[[1]]
    Message
      x <text>:4: @description refers to unavailable topic cli::bar111.

# generates informative warnings

    Code
      tag <- roxy_test_tag()
      check_topic("11papaya", "foo", tag)
    Message
      x test.R:1: @test refers to un-installed package 11papaya.
    Code
      check_topic("cli", "foofofofoo", tag)
    Message
      x test.R:1: @test refers to unavailable topic cli::foofofofoo.

# extract_r6_data without source refs

    Code
      extract_r6_data(C)
    Condition
      Error:
      ! R6 class <foo> lacks source references.
      i If you are using the `installed` load method in `DESCRIPTION`, then try re-installing the package with option '--with-keep.source', e.g. `install.packages(..., INSTALL_OPTS = "--with-keep.source")`.

# warns on invalid input

    Code
      select_args_text(c("x", "na.rm"), "-xlab:", "test")
    Message
      x In topic 'test': argument selection failed.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: -xlab:
         ^
    Output
      character(0)
    Code
      select_args_text(c("x", "na.rm"), "\"a\"", "test")
    Message
      x In topic 'test': argument selection failed.
      Caused by error in `select_check()`:
      ! Argument specification must evaluate to a numeric vector.
      Problem in `"a"`.
    Output
      character(0)
    Code
      select_args_text(c("x", "y", "z"), "-x:z", "test")
    Message
      x In topic 'test': argument selection failed.
      Caused by error:
      ! Argument specification must be all positive or all negative, not a mixture.
      i Problem in `-x:z`.
    Output
      character(0)

# generates informative warnings

    Code
      tag <- roxy_test_tag()
      find_topic_filename("11papaya", "foo", tag)
    Message
      x test.R:1: @test refers to unavailable topic 11papaya::foo.
      Caused by error in `find.package()`:
      ! there is no package called '11papaya'
    Output
      [1] "foo"
    Code
      find_topic_filename("stringr", "foofofofoo", tag)
    Message
      x test.R:1: @test refers to unavailable topic stringr::foofofofoo.
    Output
      [1] "foofofofoo"

# invalid syntax generates useful warning

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: @field requires two parts: a field name and a description.
      x <text>:4: @slot requires two parts: a slot name and a description.

# warn if forgotten colon

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:4: @section title spans multiple lines..
      i Did you forget a colon (:) at the end of the title?

# \links are transformed

    \arguments{
    \item{algo}{The hashing algorithm to be used by \code{\link[digest]{digest}}. Defaults to
    "sha1"}
    } 

# invalid syntax gives useful warning

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:2: @inheritDotParams requires a source.
      x <text>:3: @inheritSection requires two parts: a topic name and a section title.

# warns on unknown inherit type

    Code
      parse_text(text)
    Message
      x <text>:2: @inherit attempts to inherit from unknown type "blah".
    Output
      [[1]]
      <roxy_block> [<text>:3]
        $tag
          [line:  2] @inherit 'fun blah' {parsed}
          [line:  3] @backref '<generated>' {parsed}
        $call   NULL
        $object NULL
        
      

# warns if can't find section

    Code
      . <- roc_proc_text(rd_roclet(), code)
    Message
      x In topic 'b': @inheritSection failed to find section "A" in topic a.

# warned if no params need documentation

    Code
      . <- roc_proc_text(rd_roclet(), code)
    Message
      x In topic 'x': @inheritParams failed.
      i All parameters are already documented; none remain to be inherited.

# can inherit all from single function

    [1] "test-rd-inherit-dots.txt"

# does not produce multiple ... args

    [1] "test-rd-inherit-dots-inherit.txt"

# can inherit dots from several functions

    \arguments{
    \item{...}{
      Arguments passed on to \code{\link{foo}}, \code{\link{bar}}
      \describe{
        \item{\code{x}}{x}
        \item{\code{y}}{y1}
        \item{\code{z}}{z}
      }}
    } 

# useful error for bad inherits

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x In topic 'bar': argument selection failed.
      Caused by error in `FUN()`:
      ! object 'z' not found
      x In topic 'bar': @inheritDotParams failed.
      i No arguments inherited from `foo()`.

# warns when no params to inherit (#1671)

    Code
      out <- roc_proc_text(rd_roclet(), text)
    Message
      x In topic 'bar': @inheritDotParams failed.
      i No arguments inherited from `foo()`.

# inheritDotParams warns when source not found (#1602)

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x In topic 'test': @inheritDotParams failed to find topic "format" in current package.
      x In topic 'test': @inheritDotParams failed.
      i No arguments inherited from `format()`.

# useful warnings if can't find topics

    Code
      get_rd("notinstalled::pkg", source = "source")
    Message
      x In topic 'source': @inherits failed because notinstalled is not installed.
    Output
      NULL
    Code
      get_rd("base::doesntexist", source = "source")
    Message
      x In topic 'source': @inherits failed to find topic base::doesntexist.
    Output
      NULL
    Code
      get_rd("doesntexist", RoxyTopics$new(), source = "source")
    Message
      x In topic 'source': @inherits failed to find topic "doesntexist" in current package.
    Output
      NULL

# has thoughtful print method

    Code
      parse_text(text)[[1]]
    Output
      <roxy_block> [<text>:5]
        $tag
          [line:  1] @title 'This is a title' {parsed}
          [line:  3] @param 'x,y A number' {parsed}
          [line:  4] @export '' {parsed}
          [line:  5] @usage '<generated>' {parsed}
          [line:  5] @.formals '<generated>' {parsed}
          [line:  5] @backref '<generated>' {parsed}
        $call   f <- function(x, y) x + y
        $object <function> 
          $topic f
          $alias f

# errors are propagated

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:5: @eval failed to evaluate.
      Caused by error in `foo()`:
      ! Uhoh

# must return non-NA string

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: @eval must evaluate to a character vector.

---

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: @eval must not contain any missing values.

# warns about duplicate tags

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:5: Block must contain only one @rdname.

# useful error if no DESCRIPTION

    Code
      roxygen_setup(path)
    Condition
      Error:
      ! `package.dir` ('<path>') does not contain a DESCRIPTION file.

# informs about initial setup

    Code
      roxygen_setup(path, cur_version = "8.0.0")
    Message
      First time using roxygen2. Upgrading automatically...
      i Setting Config/roxygen2/version to "8.0.0"
    Output
      [1] TRUE

# warns about non UTF-8 encoding

    Code
      roxygen_setup(path, cur_version = "8.0.0")
    Message
      x roxygen2 requires "Encoding: UTF-8"
      i Current encoding is "latin1"
    Output
      [1] FALSE

# warns if roxygen version is too new

    Code
      roxygen_setup(path, cur_version = "8.0.0")
    Message
      x Installed roxygen2 is older than the version used with this package
      i You have "8.0.0" but you need "10.0.0"
    Output
      [1] FALSE

# informs about major changes in 7.0.0

    Code
      roxygen_setup(path, cur_version = "8.0.0")
    Message
      --------------------------------------------------------------------------------
      Changes in roxygen2 7.0.0:
      * `%` is now escaped automatically in Markdown mode.
      Please carefully check .Rd files for changes
      --------------------------------------------------------------------------------
      i Setting Config/roxygen2/version to "8.0.0"
    Output
      [1] FALSE

# @prop creates Additional properties section

    Code
      out$get_section("prop")
    Output
      \section{Additional properties}{
      
      \describe{
      \item{\code{@a}}{prop a}
      
      \item{\code{@b}}{prop b}
      }}
       

# @prop class@name groups by class

    Code
      out$get_section("prop")
    Output
      \section{Additional properties}{
      
      \subsection{Parent}{
      
      \describe{
      \item{\code{@x}}{prop x}
      }
      }
      
      \subsection{Child}{
      
      \describe{
      \item{\code{@y}}{prop y}
      }
      }
      }
       

# @prop with mismatched braces warns and doesn't crash

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x <text>:4: @prop has mismatched braces or quotes.

# @prop class@name warns on invalid spec

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x <text>:4: @prop must have form class@prop.

# tags containing only whitespace generate warning

    Code
      tag <- roxy_test_tag(" ")
      expect_parse_failure(tag_value(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_inherit(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_name(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_name_description(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires two parts: a name and a description.
    Code
      expect_parse_failure(tag_code(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_examples(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_markdown(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.
    Code
      expect_parse_failure(tag_markdown_with_sections(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a value.

# tags check for mismatched parents gives useful warnings

    Code
      tag <- roxy_test_tag("a {")
      expect_parse_failure(tag_value(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_inherit(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_name(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_two_part(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_words(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_words_line(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      expect_parse_failure(tag_examples(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test has mismatched braces or quotes.
    Code
      # markdown tags return empty values
      tag_markdown(tag)
    Message
      x test.R:1: @test has mismatched braces or quotes.
    Output
      [test.R:  1] @test 'a {' {parsed}
    Code
      tag_markdown_with_sections(tag)
    Message
      x test.R:1: @test has mismatched braces or quotes.
    Output
      [test.R:  1] @test 'a {' {parsed}

---

    Code
      tag <- roxy_test_tag("# one\ntwo\n# three\nfour {")
      tag_markdown_with_sections(tag)
    Message
      x test.R:1: @test has mismatched braces or quotes.
    Output
      [test.R:  1] @test '# one...' {parsed}

# tag_inhert checks for valid inherits

    Code
      tag <- roxy_test_tag("foo params section")
      . <- tag_inherit(tag)
    Message
      x test.R:1: @test attempts to inherit from unknown type "section".

# tag_name() checks for valid names

    Code
      tag <- roxy_test_tag("a b c")
      expect_parse_failure(tag_name(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must have only one argument, not 2.

# tag_two_part() gives useful warnings

    Code
      tag <- roxy_test_tag("")
      expect_parse_failure(tag_two_part(tag, "a name", "a value", required = FALSE))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires a name.
    Code
      expect_parse_failure(tag_two_part(tag, "a name", "a value"))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test requires two parts: a name and a value.

# tag_words() gives useful warnings

    Code
      tag <- roxy_test_tag("a b")
      expect_parse_failure(tag_words(tag, 3, 3))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must have at least 3 words, not 2.
    Code
      expect_parse_failure(tag_words(tag, 1, 1))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must have at most 1 word, not 2.

# tag_words_line() gives useful warnings

    Code
      tag <- roxy_test_tag("a\nb")
      expect_parse_failure(tag_words_line(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must be a single line, not 2.
      i The first line is "a"
    Code
      tag <- roxy_test_tag("a\nb\n2")
      expect_parse_failure(tag_words_line(tag))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must be a single line, not 3.
      i The first line is "a"

# tag_toggle() gives useful warnings

    Code
      tag <- roxy_test_tag("x")
      tag_toggle(tag)
    Message
      x test.R:1: @test must not be followed by any text.
    Output
      NULL

# tag_code() gives useful warnings

    Code
      tag <- roxy_test_tag("a + ")
      tag_code(tag)
    Message
      x test.R:1: @test failed to parse.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: a + 
         ^
    Output
      NULL

# warn about unknown tags

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:2: @unknown is not a known tag.

# incomplete rd in prequel or tag raises issue

    Code
      out <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:2: @title has mismatched braces or quotes.
      x <text>:3: @aliases has mismatched braces or quotes.

# @describeIn suggests @rdname

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:2: @describeIn requires a name and description.
      i Did you want @rdname instead?

# Multiple @describeIn functions combined into one

    Code
      out$get_section("minidesc")
    Output
      \section{Functions}{
      \itemize{
      \item \code{square()}: Square a number
      
      \item \code{cube()}: Cube a number
      
      }} 

# multiple methods and others are combined into a generic

    Code
      out$get_section("minidesc")
    Output
      \section{Methods (by class)}{
      \itemize{
      \item \code{zap(numeric)}: method
      
      \item \code{zap(character)}: method
      
      }}
      \section{Functions}{
      \itemize{
      \item \code{print(qux)}: function (method for different generic)
      
      \item \code{zap_helper()}: function
      
      }} 

# multiple methods and others are combined into a class constructor

    Code
      out$get_section("minidesc")
    Output
      \section{Methods (by generic)}{
      \itemize{
      \item \code{print(foo)}: method
      
      \item \code{format(foo)}: method
      
      }}
      \section{Functions}{
      \itemize{
      \item \code{format(bar)}: function (method for different class)
      
      \item \code{is_foo()}: function
      
      }} 

---

    Code
      out$get_section("minidesc")
    Output
      \section{Methods (by generic)}{
      \itemize{
      \item \code{print(roxygen2_baz)}: method
      
      }}
      \section{Functions}{
      \itemize{
      \item \code{format(quuz_baz)}: function (method for another class)
      
      }} 

# infix and replacement names get nice label

    Code
      out$get_section("minidesc")
    Output
      \section{Functions}{
      \itemize{
      \item \code{x \%foo\% y}: infix
      
      \item \code{foo(x) <- value}: replacement for foo
      
      }} 

# s4 methods get nice label

    Code
      out$get_section("minidesc")
    Output
      \section{Methods (by generic)}{
      \itemize{
      \item \code{m_id(foo1)}: function
      
      }}
      \section{Functions}{
      \itemize{
      \item \code{m_id()}: generic
      
      }} 

---

    Code
      out$get_section("minidesc")
    Output
      \section{Methods (by class)}{
      \itemize{
      \item \code{bar1(x = foo2, y = foo3)}: method1
      
      \item \code{bar1(x = foo3, y = foo2)}: method2
      
      }} 

# complains about bad usage

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:6: @describeIn must be used with an object.

---

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:6: @describeIn can not be used with @name.

---

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:6: @describeIn can not be used with @rdname.

# error-ful evalRd generates warning

    Code
      expect_parse_failure(roxy_tag_eval(roxy_test_tag(val = 1)))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must evaluate to a character vector.
    Code
      expect_parse_failure(roxy_tag_eval(roxy_test_tag(val = NA_character_)))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test must not contain any missing values.
    Code
      expect_parse_failure(roxy_tag_eval(roxy_test_tag(val = quote(stop("Uhoh")))))
    Output
      <message/rlang_message>
      Message:
      x test.R:1: @test failed to evaluate.
      Caused by error:
      ! Uhoh

# documenting unknown function requires name

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:6: Block must have a @name.
      i Either document an existing object or manually specify with @name

# can't set description and re-export

    Code
      out <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:4: Block must not include a description when re-exporting a function.

# documenting NA gives useful error message (#194)

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: Block must have a @name.
      i Either document an existing object or manually specify with @name

# can generate nonASCII document

    Code
      roxygenise(path, roclets = "rd")
    Message
      i Loading testNonASCII
      Writing 'printChineseMsg.Rd'
    Code
      # Second run should be idempotent
      roxygenise(path, roclets = "rd")
    Message
      i Loading testNonASCII

# unicode escapes are ok

    Code
      roxygenise(path, roclets = "rd")
    Message
      i Loading testUtf8Escape
      Writing 'a.Rd'
    Code
      # Second run should be idempotent
      roxygenise(path, roclets = "rd")
    Message
      i Loading testUtf8Escape

# automatically deletes unused files

    Code
      roxygenise(path)
    Message
      i Loading empty
      Deleting 'test.Rd'

# invalid syntax generates useful warning

    Code
      . <- roc_proc_text(rd_roclet(), block)
    Message
      x <text>:3: @templateVar requires two parts: a variable name and a value.

# templates gives useful error if not found

    Code
      roc_proc_text(rd_roclet(), block)
    Condition
      Error:
      ! Can't find template "doesn't-exist".

# format.rd_r6_inherited renders method list

    Code
      cat(format(inherited), sep = "\n")
    Output
      \if{html}{\out{<details open><summary>Inherited methods</summary>
      <ul>
        <li><code>pkg::A$foo()</code></li>
        <li><code>pkg::A$bar()</code></li>
      </ul>
      </details>}}

# recommends use of _PACKAGE

    Code
      out <- parse_text(block)[[1]]
    Message
      x <text>:3: `@docType "package"` is deprecated.
      i Please document "_PACKAGE" instead.

# S7 method with unknown class type warns

    Code
      s7_class_name(42L, block)
    Message
      x test.R:1: Unknown S7 class type.
    Output
      [1] "42L"

# format.rd_r6_methods with one method

    Code
      cat(format(methods), sep = "\n")
    Output
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-Foo-run}{\code{Foo$run()}}
        }
      }
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-Foo-run"></a>}}
      \if{latex}{\out{\hypertarget{method-Foo-run}{}}}
      \subsection{\code{Foo$run()}}{
        Run it.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{Foo$run()}
          \if{html}{\out{</div>}}
        }
      }
      
      }

# r6_all_examples aggregates across methods

    Code
      cat(r6_all_examples(docs$methods), sep = "\n")
    Output
      
      ## ------------------------------------------------
      ## Method `C$greet()`
      ## ------------------------------------------------
      
      c$greet()
      
      ## ------------------------------------------------
      ## Method `C$stop()`
      ## ------------------------------------------------
      
      c$stop()
      c$stop(force = TRUE)

# format has nice defaults for bare vectors

    Code
      call_to_format(x <- list(a = 1, b = 2))
    Output
      [1] "An object of class \\code{list} of length 2."
    Code
      call_to_format(x <- ordered(letters[1:5]))
    Output
      [1] "An object of class \\code{ordered} (inherits from \\code{factor}) of length 5."
    Code
      call_to_format(x <- diag(10))
    Output
      [1] "An object of class \\code{matrix} (inherits from \\code{array}) with 10 rows and 10 columns."
    Code
      call_to_format(x <- array(1:27, dim = c(3, 3, 3)))
    Output
      [1] "An object of class \\code{array} of dimension 3 x 3 x 3."
    Code
      call_to_format(x <- data.frame(a = 1, b = 2))
    Output
      [1] "An object of class \\code{data.frame} with 1 rows and 2 columns."

# format escapes braces in class names (#1744)

    Code
      call_to_format(x <- quote({ }))
    Output
      [1] "An object of class \\code{\\{} of length 1."

# warns about undocumented fields

    Code
      docs <- r6_doc(text)
    Message
      x In topic 'C': Undocumented R6 field: undocumented_field.

# warns about fields documented multiple times

    Code
      docs <- r6_doc(text)
    Message
      x <text>:5: R6 field documented multiple times: x.

# warns about unknown fields

    Code
      docs <- r6_doc(text)
    Message
      x <text>:4: Unknown R6 field: nosuch.
      x In topic 'C': Undocumented R6 field: x.

# format.rd_r6_field produces \item markup

    Code
      cat(format(rd_r6_field("x", "A number.")))
    Output
      \item{\code{x}}{A number.}

# format.rd_r6_fields produces fields & bindings sections

    Code
      cat(format(fields), sep = "\n")
    Output
      \section{Public fields}{
        \if{html}{\out{<div class="r6-fields">}}
        \describe{
          \item{\code{x}}{A number.}
      
          \item{\code{y}}{A string.}
        }
        \if{html}{\out{</div>}}
      }

---

    Code
      cat(format(bindings), sep = "\n")
    Output
      \section{Active bindings}{
        \if{html}{\out{<div class="r6-active-bindings">}}
        \describe{
          \item{\code{val}}{A value.}
        }
        \if{html}{\out{</div>}}
      }

# warns about unmatched components 

    Code
      docs <- r6_doc(text)
    Message
      x <text>:6: @description can't find matching R6 method.

# format.rd_r6_class with fields

    Code
      cat(format(docs), sep = "\n")
    Output
      \section{Public fields}{
        \if{html}{\out{<div class="r6-fields">}}
        \describe{
          \item{\code{x}}{A number.}
      
          \item{\code{y}}{A string.}
        }
        \if{html}{\out{</div>}}
      }

# format.rd_r6_class with active bindings

    Code
      cat(format(docs), sep = "\n")
    Output
      \section{Active bindings}{
        \if{html}{\out{<div class="r6-active-bindings">}}
        \describe{
          \item{\code{val}}{A value.}
        }
        \if{html}{\out{</div>}}
      }

# format.rd_r6_class with no inherited methods

    Code
      cat(format(docs), sep = "\n")
    Output
      \section{Super class}{
      \code{R_GlobalEnv::C1} -> \code{C2}
      }
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-C2-meth1}{\code{C2$meth1()}}
        }
      }
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-C2-meth1"></a>}}
      \if{latex}{\out{\hypertarget{method-C2-meth1}{}}}
      \subsection{\code{C2$meth1()}}{
        method1
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{C2$meth1()}
          \if{html}{\out{</div>}}
        }
      }
      
      }

# format.rd_r6_class with inherited methods

    Code
      cat(format(docs), sep = "\n")
    Output
      \section{Super class}{
      \code{R_GlobalEnv::A} -> \code{B}
      }
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-B-shared}{\code{B$shared()}}
          \item \href{#method-B-clone}{\code{B$clone()}}
        }
      }
      \if{html}{\out{<details open><summary>Inherited methods</summary>
      <ul>
        <li><code>R_GlobalEnv::A$only_a()</code></li>
      </ul>
      </details>}}
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-B-shared"></a>}}
      \if{latex}{\out{\hypertarget{method-B-shared}{}}}
      \subsection{\code{B$shared()}}{
        Method from B.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{B$shared()}
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-B-clone"></a>}}
      \if{latex}{\out{\hypertarget{method-B-clone}{}}}
      \subsection{\code{B$clone()}}{
        The objects of this class are cloneable with this method.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{B$clone(deep = FALSE)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{deep}}{Whether to make a deep clone.}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      }

# format.rd_r6_class with markdown sections

    Code
      cat(format(docs), sep = "\n")
    Output
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-C-meth}{\code{C$meth()}}
        }
      }
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-C-meth"></a>}}
      \if{latex}{\out{\hypertarget{method-C-meth}{}}}
      \subsection{\code{C$meth()}}{
        Method description.
      
      \subsection{Description section}{
      Description section body.
      }
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{C$meth()}
          \if{html}{\out{</div>}}
        }
        \subsection{Details}{
          \subsection{Details section}{
      Details section body.
      }
        }
      }
      
      }

# code blocks work

    Before
    
    \if{html}{\out{<div class="sourceCode">}}\preformatted{x \%in\% 1:10
    }\if{html}{\out{</div>}}
    
    After

# code block with language creates HTML tag

    Before
    
    \if{html}{\out{<div class="sourceCode r">}}\preformatted{x \%in\% 1:10
    }\if{html}{\out{</div>}}
    
    After

# can insert block and inline html

    Code
      out$get_section("description")
    Output
      \description{
      \if{html}{\out{
      <p>This is a paragraph</p>
      <p>This is another paragraph</p>
      }}
      } 

---

    Code
      out$get_section("description")
    Output
      \description{
      This is a paragraph containing a manually inserted image
      before-\if{html}{\out{<img src='foo.png'>}}-after
      } 

# can convert table to Rd

    Code
      for (table in tables) {
        cat_line(table)
        cat_line(markdown(table))
        cat_line()
      }
    Output
      
      | x   | y   |
      | --- | --- |
      | 1   | 2   |
      \tabular{ll}{
         x \tab y \cr
         1 \tab 2 \cr
      }
      
      | x   | y   |
      | :-: | --: |
      | 1   | 2   |
      \tabular{cr}{
         x \tab y \cr
         1 \tab 2 \cr
      }
      
      | x     | y         |
      | ----- | --------- |
      | 1 _2_ | 3 *4* `5` |
        
      \tabular{ll}{
         x \tab y \cr
         1 \emph{2} \tab 3 \emph{4} \code{5} \cr
      }
      

# unhandled markdown generates warning

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x <text>:4: @description markdown translation failed.
      x block quotes are not currently supported

# horizontal rules generate warning (#1707)

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x <text>:6: @details markdown translation failed.
      x horizontal rules are not currently supported

# level 1 heading in markdown generates warning in some tags

    Code
      . <- roc_proc_text(rd_roclet(), text)
    Message
      x <text>:4: @seealso markdown translation failed.
      x Level 1 headings are not supported in @seealso
      i Do you want to put the heading in @description or @details?

# image formats work

    Code
      roc_proc_text(rd_roclet(),
      "\n      #' Title\n      #'\n      #' ![](example.svg \"Plot title 1\")\n      #' ![](example.pdf \"Plot title 2\")\n      #' ![](example.PNG \"Plot title 3\")\n      #' @md\n      foo <- function() { }\n    ")[[
        1]]
    Output
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo}
      \alias{foo}
      \title{Title}
      \usage{
      foo()
      }
      \description{
      \if{html}{\figure{example.svg}{Plot title 1}}
      \if{pdf}{\figure{example.pdf}{Plot title 2}}
      \figure{example.PNG}{Plot title 3}
      }

# careful ordering

    Code
      out
    Output
      $foo1.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo1}
      \alias{foo1}
      \title{foo1}
      \usage{
      foo1()
      }
      \description{
      foo1
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo]{foo()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      
      $foo2.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo2}
      \alias{foo2}
      \title{foo2}
      \usage{
      foo2()
      }
      \description{
      foo2
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo]{foo()}},
      \code{\link[=foo1]{foo1()}}
      }
      \concept{a}
      
      $Foo3.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{Foo3}
      \alias{Foo3}
      \title{Foo3}
      \usage{
      Foo3()
      }
      \description{
      Foo3
      }
      \seealso{
      Other a:
      \code{\link[=foo]{foo()}},
      \code{\link[=foo1]{foo1()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      
      $foo.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo}
      \alias{foo}
      \title{foo}
      \usage{
      foo()
      }
      \description{
      foo
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo1]{foo1()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      

# multi-line inline code gives useful warning

    Code
      out <- roc_proc_text(rd_roclet(), block)[[1]]
    Message
      x <text>:4: @description failed to evaluate inline markdown code.
      Caused by error:
      ! Multi-line `r ` markup is not supported.

# inline code gives useful warning

    Code
      out <- roc_proc_text(rd_roclet(), block)[[1]]
    Output
      
    Message
      
      Quitting from :1-1
      x <text>:4: @description failed to evaluate inline markdown code.
      Caused by error:
      ! Failed to parse the inline R code: `r 1 + `
      Reason: <text>:2:0: unexpected end of input
      1: 1 + 
         ^

# interleaving fences and inline code

    Code
      cat(out1$get_value("details"))
    Output
      Details 10
      
      \if{html}{\out{<div class="sourceCode r">}}\preformatted{y <- x + 10
      y
      #> [1] 20
      }\if{html}{\out{</div>}}

# preserves white space

    Code
      cat(out1$get_value("details"))
    Output
      \if{html}{\out{<div class="sourceCode r">}}\preformatted{a <- 1
      
      b <- 2
      }\if{html}{\out{</div>}}
      
      \if{html}{\out{<div class="sourceCode r">}}\preformatted{c <- 3
      }\if{html}{\out{</div>}}

# alternative knitr engines

    Code
      print(out1 <- roc_proc_text(rd_roclet(),
      "\n      #' Title\n      #'\n      #' Description.\n      #'\n      #' ```{verbatim}\n      #' #| file = testthat::test_path(\"example.Rmd\")\n      #' ```\n      #' @md\n      #' @name x\n      NULL\n    "))
    Output
      $x.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{x}
      \alias{x}
      \title{Title}
      \description{
      Description.
      }
      \details{
      \if{html}{\out{<div class="sourceCode default">}}\preformatted{```\{r\}
      # comment
      this <- 10
      is <- this + 10
      good <- this + is
      }\if{html}{\out{</div>}}
      }
      

# integration test

    Code
      cat(read_lines(rd_file), sep = "\n")
    Output
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in R/classes.R
      \name{A}
      \alias{A}
      \title{Class A}
      \description{
      Class A description.
      }
      \details{
      Class A details
      }
      \examples{
      a <- A$new()
      
      ## ------------------------------------------------
      ## Method `A$meth1()`
      ## ------------------------------------------------
      
      ## Example for meth1
      
      ## ------------------------------------------------
      ## Method `A$meth2()`
      ## ------------------------------------------------
      
      ## Example for meth2
      ## Second line
      ## Third line
      }
      \section{Public fields}{
        \if{html}{\out{<div class="r6-fields">}}
        \describe{
          \item{\code{field1}}{A field 1.}
      
          \item{\code{field2}}{A field 2.}
      
          \item{\code{field3}}{A field 3.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Active bindings}{
        \if{html}{\out{<div class="r6-active-bindings">}}
        \describe{
          \item{\code{active1}}{A binding 1.}
      
          \item{\code{active2}}{A binding 2.}
      
          \item{\code{active3}}{A binding 3.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-A-meth1}{\code{A$meth1()}}
          \item \href{#method-A-meth2}{\code{A$meth2()}}
          \item \href{#method-A-meth3}{\code{A$meth3()}}
          \item \href{#method-A-clone}{\code{A$clone()}}
        }
      }
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-A-meth1"></a>}}
      \if{latex}{\out{\hypertarget{method-A-meth1}{}}}
      \subsection{\code{A$meth1()}}{
        A method 1.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{A$meth1(Z)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{Z}}{zzzzzzz}
          }
          \if{html}{\out{</div>}}
        }
        \subsection{Examples}{
          \if{html}{\out{<div class="r example copy">}}
          \preformatted{## Example for meth1
      }
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-A-meth2"></a>}}
      \if{latex}{\out{\hypertarget{method-A-meth2}{}}}
      \subsection{\code{A$meth2()}}{
        Method 2 description.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{A$meth2(Z = 10, ...)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{Z}}{Overriding Z argument for meth2.}
            \item{\code{...}}{Rest.}
          }
          \if{html}{\out{</div>}}
        }
        \subsection{Details}{
          Method 2 details.
        }
        \subsection{Returns}{
          A value.
        }
        \subsection{Examples}{
          \if{html}{\out{<div class="r example copy">}}
          \preformatted{## Example for meth2
      ## Second line
      ## Third line
      }
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-A-meth3"></a>}}
      \if{latex}{\out{\hypertarget{method-A-meth3}{}}}
      \subsection{\code{A$meth3()}}{
        Method 3.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{A$meth3(x)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{x}}{An argument.}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-A-clone"></a>}}
      \if{latex}{\out{\hypertarget{method-A-clone}{}}}
      \subsection{\code{A$clone()}}{
        The objects of this class are cloneable with this method.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{A$clone(deep = FALSE)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{deep}}{Whether to make a deep clone.}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      }

---

    Code
      cat(read_lines(rd_file), sep = "\n")
    Output
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in R/classes.R
      \name{B}
      \alias{B}
      \title{Class B}
      \description{
      Class B Description.
      }
      \details{
      Class B details.
      }
      \section{Super class}{
      \code{A} -> \code{B}
      }
      \section{Public fields}{
        \if{html}{\out{<div class="r6-fields">}}
        \describe{
          \item{\code{field1}}{B field 1.}
      
          \item{\code{field4}}{B field 4.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Active bindings}{
        \if{html}{\out{<div class="r6-active-bindings">}}
        \describe{
          \item{\code{active1}}{B binding 1.}
      
          \item{\code{active4}}{B binding 4.}
      
          \item{\code{active5}}{B binding 5.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-B-meth1}{\code{B$meth1()}}
          \item \href{#method-B-meth4}{\code{B$meth4()}}
          \item \href{#method-B-clone}{\code{B$clone()}}
        }
      }
      \if{html}{\out{<details open><summary>Inherited methods</summary>
      <ul>
        <li><code>A$meth2()</code></li>
        <li><code>A$meth3()</code></li>
      </ul>
      </details>}}
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-B-meth1"></a>}}
      \if{latex}{\out{\hypertarget{method-B-meth1}{}}}
      \subsection{\code{B$meth1()}}{
        B method 1.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{B$meth1(Z)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{Z}}{Still zzzzzzzz.}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-B-meth4"></a>}}
      \if{latex}{\out{\hypertarget{method-B-meth4}{}}}
      \subsection{\code{B$meth4()}}{
        B method 4.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{B$meth4()}
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-B-clone"></a>}}
      \if{latex}{\out{\hypertarget{method-B-clone}{}}}
      \subsection{\code{B$clone()}}{
        The objects of this class are cloneable with this method.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{B$clone(deep = FALSE)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{deep}}{Whether to make a deep clone.}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      }

---

    Code
      cat(read_lines(rd_file), sep = "\n")
    Output
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in R/classes.R
      \name{C}
      \alias{C}
      \title{Class C}
      \description{
      Class C Description.
      }
      \details{
      Class C details.
      }
      \section{Super classes}{
      \code{A} -> \code{B} -> \code{C}
      }
      \section{Public fields}{
        \if{html}{\out{<div class="r6-fields">}}
        \describe{
          \item{\code{field2}}{C field 2.}
      
          \item{\code{field5}}{C field 5.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Active bindings}{
        \if{html}{\out{<div class="r6-active-bindings">}}
        \describe{
          \item{\code{active2}}{C binding 2.}
      
          \item{\code{active4}}{C binding 4.}
      
          \item{\code{active6}}{C binding 6.}
        }
        \if{html}{\out{</div>}}
      }
      \section{Methods}{
      \subsection{Public methods}{
        \itemize{
          \item \href{#method-C-meth2}{\code{C$meth2()}}
          \item \href{#method-C-meth5}{\code{C$meth5()}}
        }
      }
      \if{html}{\out{<details open><summary>Inherited methods</summary>
      <ul>
        <li><code>A$meth3()</code></li>
        <li><code>B$meth1()</code></li>
        <li><code>B$meth4()</code></li>
      </ul>
      </details>}}
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-C-meth2"></a>}}
      \if{latex}{\out{\hypertarget{method-C-meth2}{}}}
      \subsection{\code{C$meth2()}}{
        C method 2.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{C$meth2(Z = 10, ...)}
          \if{html}{\out{</div>}}
        }
        \subsection{Arguments}{
          \if{html}{\out{<div class="arguments">}}
          \describe{
            \item{\code{Z}}{zzzzz}
            \item{\code{...}}{etc}
          }
          \if{html}{\out{</div>}}
        }
      }
      
      \if{html}{\out{<hr>}}
      \if{html}{\out{<a id="method-C-meth5"></a>}}
      \if{latex}{\out{\hypertarget{method-C-meth5}{}}}
      \subsection{\code{C$meth5()}}{
        C method 5.
        \subsection{Usage}{
          \if{html}{\out{<div class="r">}}
          \preformatted{C$meth5()}
          \if{html}{\out{</div>}}
        }
      }
      
      }

# parsing warnings only fire once

    Code
      roxygenise(path)
    Message
      Writing 'NAMESPACE'
      i Loading testNamespace
      x multiline.R:1: @importFrom must be only 1 line long, not 2.
      i The first line is "stats median"

# @exportS3Method generates fully automatically

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @exportS3Method must be used with a known S3 method.

# @exportS3Method can extract class from generic

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @exportS3Method must have form package::generic.

---

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @exportS3Method must be used with a function.

---

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @exportS3Method generic ("foo") doesn't match function ("foo1.bar").

# poorly formed importFrom throws error

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @importFrom must have at least 2 words, not 1.

# multiline importFrom generates warning

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @importFrom must be only 1 line long, not 2.
      i The first line is "test test1"

# can regenerate NAMESPACE even if its broken

    Code
      update_namespace_imports(path)
    Message
      Writing 'NAMESPACE'

# rawNamespace must be valid code

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @rawNamespace failed to parse.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: a +
         ^

# evalNamespace warns for bad code

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @evalNamespace failed to parse.
      Caused by error in `parse()`:
      ! <text>:2:0: unexpected end of input
      1: a +
         ^

---

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @evalNamespace failed to evaluate.
      Caused by error:
      ! Uhoh

---

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @evalNamespace must evaluate to a character vector.

# invalid imports generate helpful message

    Code
      out <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @importFrom Excluding unknown export from utils: `InvalidUtilsFunction1`.

---

    Code
      out <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:2: @importFrom Excluding unknown exports from utils: `InvalidUtilsFunction1` and `InvalidUtilsFunction2`.

# warns if S3 method not documented

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:5: S3 method `mean.myclass` needs @export or @exportS3Method tag.
      x <text>:3: S3 method `foo.numeric` needs @export or @exportS3Method tag.

---

    Code
      . <- roc_proc_text(namespace_roclet(), block)
    Message
      x <text>:3: S3 method `foo.{` needs @export or @exportS3Method tag.

# look up string

    Code
      find_load_strategy("blahblahb")
    Condition
      Error:
      ! Unknown value of `load` option: "blahblahb".

# informative errors for bad inputs

    Code
      find_load_strategy(1)
    Condition
      Error:
      ! `load_code` must be a single string, not the number 1.
    Code
      find_load_strategy(NULL, list())
    Condition
      Error:
      ! roxygen2 `load` option must be a single string, not an empty list.

# format.rd_r6_super with one superclass

    Code
      cat(format(supers), sep = "\n")
    Output
      \section{Super class}{
      \code{mypkg::Parent} -> \code{Child}
      }

# format.rd_r6_super omits pkg:: for same-package classes

    Code
      cat(format(supers), sep = "\n")
    Output
      \section{Super classes}{
      \code{otherpkg::GrandParent} -> \code{Parent} -> \code{Child}
      }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{markdown_pass1}
\alias{markdown_pass1}
\title{Expand the embedded inline code}
\usage{
markdown_pass1(text)
}
\arguments{
\item{text}{Input text.}
}
\value{
Text with the inline code expanded. A character vector of the
same length as the input \code{text}.
}
\description{
Expand the embedded inline code
}
\details{
For example this becomes two: 2.
Variables can be set and then reused, within the same
tag:
The value of \code{x} is 100.

We have access to the internal functions of the package, e.g.
since this is \emph{roxygen2}, we can refer to the internal \code{markdown}
function, and this is \code{TRUE}: TRUE.

To insert the name of the current package: roxygen2.

The \code{iris} data set has 5 columns:
\code{Sepal.Length}, \code{Sepal.Width}, \code{Petal.Length}, \code{Petal.Width}, \code{Species}.\if{html}{\out{<div class="r">}}\preformatted{# Code block demo
x + 1
}\if{html}{\out{</div>}}\preformatted{## [1] 101
}

Chunk options:\if{html}{\out{<div class="r">}}\preformatted{names(mtcars)
nrow(mtcars)
}\if{html}{\out{</div>}}\preformatted{##  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
## [1] 32
}

Plots:\if{html}{\out{<div class="r">}}\preformatted{plot(1:10)
}\if{html}{\out{</div>}}

\figure{test-figure-1.png}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxygenize-needs.R
\name{needs_roxygenize}
\alias{needs_roxygenize}
\title{Check if documentation needs to be updated}
\usage{
needs_roxygenize(package.dir = ".", quiet = FALSE)
}
\arguments{
\item{package.dir}{Location of package top level directory. Default is
working directory.}

\item{quiet}{If \code{TRUE}, suppresses the message listing out-of-date
man pages.}
}
\value{
A logical value, invisibly. \code{TRUE} if any man pages appear
to be out of date; \code{FALSE} otherwise.
}
\description{
A lightweight check that compares modification times of \code{.Rd} files in
\verb{man/} with the source files listed in their backrefs. This is much faster
than running \code{\link[=roxygenize]{roxygenize()}} but can suffer from both false negatives (e.g.
if an inherited documentation topic has changed) and false positives (e.g.
if a source file was modified but the change doesn't affect the
documentation).
}
\examples{
\dontrun{
needs_roxygenize()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roclet.R
\name{roclet_find}
\alias{roclet_find}
\title{Create a roclet from a string.}
\usage{
roclet_find(x)
}
\arguments{
\item{x}{Arbitrary R code evaluated in roxygen2 package.}
}
\description{
This provides a flexible way of specifying a roclet in a string.
}
\examples{
# rd, namespace, and vignette work for backward compatibility
roclet_find("rd")

# But generally you should specify the name of a function that
# returns a roclet
roclet_find("rd_roclet")

# If it lives in another package, you'll need to use ::
roclet_find("roxygen2::rd_roclet")

# If it takes parameters (which no roclet does currently), you'll need
# to call the function
roclet_find("roxygen2::rd_roclet()")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-escaping.R
\name{escape_rd_for_md}
\alias{escape_rd_for_md}
\alias{unescape_rd_for_md}
\title{Escape fragile Rd tags}
\usage{
escape_rd_for_md(text)

unescape_rd_for_md(rd_text, esc_text)
}
\arguments{
\item{text}{Input text. Potentially contains Rd and/or
markdown markup.}

\item{rd_text}{The markdown parsed and interpreted text.}

\item{esc_text}{The original escaped text from
\code{escape_rd_for_md()}.}
}
\value{
\itemize{
\item \code{escape_rd_for_md}: a "safe" version of the input text, where
each fragile Rd tag is replaced by a placeholder. The
original text is added as an attribute for each placeholder.
\item \code{unescape_rd_for_md}: the original Rd text.
}
}
\description{
\code{escape_rd_for_md()} replaces fragile Rd tags with placeholders, to avoid
interpreting them as markdown. \code{unescape_rd_for_md()} puts the original
text back in place of the placeholders after the markdown parsing is done.
The fragile tags are listed in \code{escaped_for_md}.

Some Rd macros are treated specially:
\itemize{
\item For \code{if}, markdown is only allowed in the second argument.
\item For \code{ifelse} markdown is allowed in the second and third arguments.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-link.R
\name{markdown-test}
\alias{markdown-test}
\title{Dummy page to test roxygen's markdown formatting}
\description{
Links are very tricky, so I'll put in some links here:
Link to a function: \code{\link[=roxygenize]{roxygenize()}}.
Link to an object: \link{roxygenize} (we just treat it like an object here.
}
\details{
Link to another package, function: \code{\link[desc:desc]{desc::desc()}}.
Link to another package, non-function: \link[desc:desc]{desc::desc}.

Link with link text: \link[=roxygenize]{this great function},
\code{\link{roxygenize}}, or \link[=roxygenize]{that great function}.

In another package: \link[desc:desc]{and this one}.

This is a table:\tabular{lr}{
   \strong{foo} \tab \strong{bar} \cr
   1 \tab 2 \cr
   100 \tab 200 \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roclet.R
\name{roc_proc_text}
\alias{roc_proc_text}
\title{Process roclet on string and capture results}
\usage{
roc_proc_text(roclet, input, wd = NULL)
}
\arguments{
\item{roclet}{Name of roclet to use for processing.}

\item{input}{Source string}

\item{wd}{Working directory}
}
\description{
Useful for testing.
}
\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd-examples.R
\name{escape_examples}
\alias{escape_examples}
\title{Escape examples}
\usage{
escape_examples(x)
}
\description{
This documentation topic is used primarily for testing and to record
our understanding of the \verb{\\example\{\}} escaping rules.
See \url{https://developer.r-project.org/parseRd.pdf} for the details provided
by R core.
}
\examples{
# In examples we automatically escape Rd comments (\%):
100 \%\% 30
# even if they are in strings
"50\%"

# and \\ and \v inside of strings and symbols
"\\v" # vertical tab
"\\\\"
# but not comments: \l \v

# other string escapes are left as is
"\""
"\n"

# Otherwise, backslashes and parentheses are left as is. This
# means that you need to escape unbalanced parentheses, which typically only
# occur in \dontshow{}:
\dontshow{if (FALSE) \{ }
print("Hello")
\dontshow{ \} }

# You also need to escape backslashes in infix operators and comments
# (this is generally rare)
`\%\\\\\%` <- function(x, y) x + y
10 \%\\\% 20
# \\\\ (renders as two backslashes)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-parser.R
\name{tag_parsers}
\alias{tag_parsers}
\alias{tag_value}
\alias{tag_inherit}
\alias{tag_name}
\alias{tag_two_part}
\alias{tag_name_description}
\alias{tag_words}
\alias{tag_words_line}
\alias{tag_toggle}
\alias{tag_code}
\alias{tag_examples}
\alias{tag_markdown}
\alias{tag_markdown_with_sections}
\title{Parse tags}
\usage{
tag_value(x)

tag_inherit(x)

tag_name(x)

tag_two_part(x, first, second, required = TRUE, markdown = TRUE)

tag_name_description(x)

tag_words(x, min = 0, max = Inf)

tag_words_line(x)

tag_toggle(x)

tag_code(x)

tag_examples(x)

tag_markdown(x)

tag_markdown_with_sections(x)
}
\arguments{
\item{x}{A \link{roxy_tag} object to parse}

\item{first, second}{Name of first and second parts of two part tags}

\item{required}{Is the second part required (TRUE) or can it be blank
(FALSE)?}

\item{markdown}{Should the second part be parsed as markdown?}

\item{min, max}{Minimum and maximum number of words}
}
\value{
A \link{roxy_tag} object with the \code{val} field set to the parsed value.
}
\description{
These functions parse the \code{raw} tag value, convert a string into a richer R
object and storing it in \code{val}, or provide an informative warning and
returning \code{NULL}.
}
\section{New tag}{

To create a new \verb{@mytag} define \code{roxy_tag_parse.roxy_tag_mytag()}. It should
either call one of the functions here, or directly set \code{x$val}.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd}
\alias{tags-rd}
\alias{@description}
\alias{@details}
\alias{@example}
\alias{@examples}
\alias{@examplesIf}
\alias{@noRd}
\alias{@param}
\alias{@rawRd}
\alias{@return}
\alias{@returns}
\alias{@title}
\alias{@usage}
\title{Tags for documenting functions}
\usage{
#' @description${1:A short description...}

#' @details${1:Additional details...}

#' @example ${1:path}.R
#' @examples${1:# example code}

#' @examplesIf ${1:condition}${2:# example code}

#' @noRd
#' @param ${1:name} ${2:description}
#' @rawRd ${1:rd}
#' @return ${1:description}
#' @returns ${1:description}
#' @title ${1:title}
#' @usage ${1:fun}(${2:arg1, arg2 = default, ...})
}
\description{
Learn the full details in \code{vignette('rd')}.

Key tags:
\itemize{
\item \verb{@description$\{1:A short description...\} }: A short description of the purpose of the function. Usually around a paragraph, but can be longer if needed.
\item \verb{@example $\{1:path\}.R}: Embed examples stored in another file.
\item \verb{@examples$\{1:# example code\} }: Executable R code that demonstrates how the function works. Code must run without error.
\item \verb{@examplesIf $\{1:condition\}$\{2:# example code\} }: Run examples only when \code{condition} is \code{TRUE}.
\item \verb{@noRd}: Suppress \code{.Rd} generation for a block. Use for documentation blocks that should only be visible in the source code.
\item \verb{@param $\{1:name\} $\{2:description\}}: Describe a function input. Should describe acceptable input types and how it affects the output. \code{description} is usually one or two sentences but can be as long as needed.
Document multiple arguments by separating their names with commas without spaces.
\item \verb{@returns $\{1:description\}}: Describe the function's output. Typically will be a 1-2 sentence description of the output type, but might also include discussion of important errors or warnings.
\item \verb{@title $\{1:title\}}: A one-line description of the function shown in various indexes. An explicit \verb{@title} is not usually needed as by default it is taken from the first paragraph in the roxygen block.
\item \verb{@usage $\{1:fun\}($\{2:arg1, arg2 = default, ...\})}: Override the default usage generated by roxygen2. Only needed when roxygen2 fails to correctly derive the usage of your function.
}

Other less frequently used tags:
\itemize{
\item \verb{@details$\{1:Additional details...\} }: Additional details about the function. Generally superseded by instead using a level 1 heading.
\item \verb{@rawRd $\{1:rd\}}: Insert literal text directly into the \code{.Rd} file.
\item \verb{@return $\{1:description\}}: Describe the function's output. Superseded in favour of \verb{@returns}.
}
}
\seealso{
Other documentation tags: 
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-other}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags_list}
\alias{tags_list}
\alias{tags_metadata}
\title{Access metadata about built-in or available tags}
\usage{
tags_list(built_in = TRUE)

tags_metadata()
}
\arguments{
\item{built_in}{Logical. Whether to restrict the result to built-in tags
(\code{TRUE}) or to extend it to available tags (\code{FALSE}).}
}
\description{
Access metadata about built-in or available tags
}
\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-datasets}
\alias{tags-rd-datasets}
\alias{@format}
\alias{@source}
\title{Tags for documenting datasets}
\usage{
#' @format ${1:description}
#' @source ${1:description}
}
\description{
Learn the full details in \code{vignette('rd-datasets')}.

Key tags:
\itemize{
\item \verb{@format $\{1:description\}}: Describe the type/shape of a dataset. If the dataset is a data frame, include a description of each column. If not supplied, will be automatically generated by \code{object_format()}.
\item \verb{@source $\{1:description\}}: Describe where the dataset came from. Provide a link to the original source (if possible) and briefly describe any manipulation that you performed when importing the data.
}
}
\seealso{
Other documentation tags:
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-R6}},
\code{\link{tags-rd-S3}},
\code{\link{tags-rd-S4}},
\code{\link{tags-rd-S7}},
\code{\link{tags-rd-functions}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-namespace}
\alias{tags-namespace}
\alias{@evalNamespace}
\alias{@export}
\alias{@exportClass}
\alias{@exportMethod}
\alias{@exportPattern}
\alias{@exportS3Method}
\alias{@import}
\alias{@importClassesFrom}
\alias{@importFrom}
\alias{@importMethodsFrom}
\alias{@rawNamespace}
\alias{@useDynLib}
\title{Tags for managing the \code{NAMESPACE}}
\usage{
#' @evalNamespace ${1:r-code}
#' @export
#' @exportClass ${1:class}
#' @exportMethod ${1:generic}
#' @exportPattern ${1:pattern}
#' @exportS3Method ${1:package}::${2:generic}
#' @import ${1:package}
#' @importClassesFrom ${1:package} ${2:class}
#' @importFrom ${1:package} ${2:function}
#' @importMethodsFrom ${1:package} ${2:generic}
#' @rawNamespace ${1:namespace directives}
#' @useDynLib ${1:package}
}
\description{
Learn full the details in in \code{vignette('namespace')}.

Key tags:
\itemize{
\item \verb{@export}: Export this function, method, generic, or class so it's available outside of the package.
\item \verb{@exportS3Method $\{1:package\}::$\{2:generic\}}: Export an S3 method. Only needed when the method is for a generic from a suggested package.
\item \verb{@importFrom $\{1:package\} $\{2:function\}}: Import specific functions from a package.
\item \verb{@useDynLib $\{1:package\}}: Import compiled code from another package.
}

Other less frequently used tags:
\itemize{
\item \verb{@evalNamespace $\{1:r-code\}}: Evaluate arbitrary code in the package namespace and insert the results into the \code{NAMESPACE}. Should return a character vector of directives.
\item \verb{@exportClass $\{1:class\}}: Export an S4 class. For expert use only; in most cases you should use \verb{@export} so roxygen2 can automatically generate the correct directive.
\item \verb{@exportMethod $\{1:generic\}}: Export S4 methods. For expert use only; in most cases you should use \verb{@export} so roxygen2 can automatically generate the correct directive.
\item \verb{@exportPattern $\{1:pattern\}}: Export all objects matching a regular expression.
\item \verb{@import $\{1:package\}}: Import all functions from a package. Use with extreme care.
\item \verb{@importClassesFrom $\{1:package\} $\{2:class\}}: Import S4 classes from another package.
\item \verb{@importMethodsFrom $\{1:package\} $\{2:generic\}}: Import S4 methods from a package.
\item \verb{@rawNamespace $\{1:namespace directives\}}: Insert literal text directly into the \code{NAMESPACE}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/field.R
\name{rd_section}
\alias{rd_section}
\title{Construct an \code{rd_section} object}
\usage{
rd_section(type, value)
}
\arguments{
\item{type}{Section type. Stored in \code{type} field, and in class
\verb{rd_section_\{type\}}. To avoid namespace clashes between different
extensions, this should include the package name.}

\item{value}{Section data. Only used by \code{format()} and \code{merge()} methods.}
}
\description{
An \code{rd_section} represents an Rd command that can appear at the top-level
of an Rd document, like \verb{\\name\{\}}, \verb{\\title\{\}}, \verb{\\description\{\}}, or
\verb{\\section\{\}}.
}
\section{Methods}{

If provide your own \code{rd_section} type, you'll also need to define a
\verb{format.rd_section_\{type\}} method that returns formatted Rd output. You
may also need to provide a \verb{merge.rd_section_\{type\}} method if two
sections can not be combined with \code{rd_section(x$type, c(x$value, y$value))}.
See \code{vignette("extending")} for more details.
}

\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block.R
\name{roxy_block}
\alias{roxy_block}
\alias{block_has_tags}
\alias{block_get_tags}
\alias{block_get_tag}
\alias{block_get_tag_value}
\title{Blocks}
\usage{
roxy_block(tags, file, line, call, object = NULL)

block_has_tags(block, tags)

block_get_tags(block, tags)

block_get_tag(block, tag)

block_get_tag_value(block, tag)
}
\arguments{
\item{tags}{A list of \link{roxy_tag}s.}

\item{file, line}{Location of the \code{call} (i.e. the line after the last
line of the block).}

\item{call}{Expression associated with block.}

\item{object}{Optionally, the object associated with the block, found
by inspecting/evaluating \code{call}.}

\item{block}{A \code{roxy_block} to manipulate.}

\item{tag}{A single tag name.}
}
\description{
A \code{roxy_block} represents a single roxygen2 block.

The \verb{block_*} functions provide a few helpers for common operations:
\itemize{
\item \code{block_has_tag(blocks, tags)}: does \code{block} contain any of these \code{tags}?
\item \code{block_get_tags(block, tags)}: get all instances of \code{tags}
\item \code{block_get_tag(block, tag)}: get single tag. Returns \code{NULL} if 0,
throws warning if more than 1.
\item \code{block_get_tag_value(block, tag)}: gets \code{val} field from single tag.
}
}
\examples{
# The easiest way to see the structure of a roxy_block is to create one
# using parse_text:
text <- "
  #' This is a title
  #'
  #' @param x,y A number
  #' @export
  f <- function(x, y) x + y
"

# parse_text() returns a list of blocks, so I extract the first
block <- parse_text(text)[[1]]
block
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/object-from-call.R
\name{object}
\alias{object}
\title{Constructors for S3 object to represent R objects}
\usage{
object(value, alias, type)
}
\arguments{
\item{value}{The object itself.}

\item{alias}{Alias for object being documented, in case you create a
generator function with different name.}

\item{type}{Type of the object, character. E.g. \code{"data"} or \code{"s4method"}.}
}
\description{
These objects are usually created by the parsers, but it is also
useful to generate them by hand for testing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-formatting}
\alias{tags-rd-formatting}
\alias{@md}
\alias{@noMd}
\alias{@section}
\title{Tags related to markdown support}
\usage{
#' @md
#' @noMd
#' @section ${1:section title}:
}
\description{
Learn full the details in in \code{vignette('rd-formatting')}.

Other less frequently used tags:
\itemize{
\item \verb{@md}: Force markdown processing for a block.
\item \verb{@noMd}: Suppress markdown processing for a block.
\item \verb{@section $\{1:section title\}: }: Add an arbitrary section to the documentation. Now generally superseded in favour of using a level 1 heading.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/namespace.R
\name{namespace_roclet}
\alias{namespace_roclet}
\title{Roclet: make \code{NAMESPACE}}
\usage{
namespace_roclet()
}
\description{
This \link{roclet} automates the production of a \code{NAMESPACE} file, which controls
the functions imported and exported by your package, as described in
\href{https://cran.r-project.org/doc/manuals/r-release/R-exts.html}{Writing R extensions}.
It is run by default by \code{\link[=roxygenize]{roxygenize()}}.

The \code{NAMESPACE} is generated in two passes: the first generates only
import directives (because this can be computed without evaluating package
code), and the second generates everything (after the package has been
loaded).

See \code{vignette("namespace")} for details.
}
\examples{
# The most common namespace tag is @export, which declares that a function
# is part of the external interface of your package
#' @export
foofy <- function(x, y, z) {
}
# This results in the following line in `NAMESPACE`:
# export(foofy)

# You'll also often find global imports living in a file called
# R/{package}-package.R.
#' @importFrom magrittr \%>\%
#' @import rlang
NULL

# This results in the following lines in `NAMESPACE`:
# importFrom(magrittr,"\%>\%")
# import(rlang)
}
\seealso{
\link{tags-namespace} for tags that generate \code{NAMESPACE} directives.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collate.R
\name{update_collate}
\alias{update_collate}
\alias{@include}
\title{Update Collate field in DESCRIPTION}
\usage{
update_collate(base_path)
}
\arguments{
\item{base_path}{Path to package directory.}
}
\description{
By default, R loads files in alphabetical order. This is fine if your
package doesn't have any cross-file dependencies, but if you're using a tool
like S4, you'll need to make sure that classes are loaded before subclasses
and generics are defined before methods. You can do this by hand by setting
the \code{Collate} field in the \code{DESCRIPTION} or automate it by using \verb{@include}
tags to specify the cross-file dependencies:

\if{html}{\out{<div class="sourceCode R">}}\preformatted{#' @include before.R
NULL
}\if{html}{\out{</div>}}

If there are no \verb{@include} tags, roxygen2 will leave the \code{Collate} field as
is. This makes it easier to use roxygen2 with an existing collate directive,
but if you remove all your \verb{@include} tags, you'll need to also
manually delete the collate field.

Generally, you should not need to run this function yourself; it will be
run automatically by any package that needs to load your R files in
collation order.

\code{update_collate()} is not not technically a \link{roclet}, like \code{\link[=rd_roclet]{rd_roclet()}}
and \code{\link[=namespace_roclet]{namespace_roclet()}}, because you have to be able to load the pacakge
before you can process it with roclets. However, because it was historical
implemented as a roclet, it's still controlled by the \code{roclets} argument of
\code{\link[=roxygenize]{roxygenize()}}.
}
\examples{
#' If `example-a.R`, `example-b.R` and `example-c.R` live in `R/`
#' and we're in `example-a.R`, then the following @include tag
#' ensures that example-b and example-c are sourced before example-a.
#' @include example-b.R example-c.R
NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{vignette_roclet}
\alias{vignette_roclet}
\title{Re-build outdated vignettes}
\usage{
vignette_roclet()
}
\description{
This rebuilds outdated vignettes with \link[tools:buildVignette]{tools::buildVignette}.
By default, it will rebuild all vignettes if the source file is newer than
the output pdf or html. (This means it will automatically re-build the
vignette if you change the vignette source, but \emph{not} when you
change the R code). If you want finer control, add a Makefile to
\verb{vignettes/} and roxygen2 will use that instead.
}
\details{
To prevent RStudio from re-building the vignettes again when checking
your package, add \code{--no-build-vignettes} to the "Build Source Package"
field in your project options.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-reuse}
\alias{tags-reuse}
\alias{@describeIn}
\alias{@eval}
\alias{@evalRd}
\alias{@includeRmd}
\alias{@inherit}
\alias{@inheritDotParams}
\alias{@inheritParams}
\alias{@inheritSection}
\alias{@order}
\alias{@rdname}
\alias{@template}
\alias{@templateVar}
\title{Tags that help you reuse documentation}
\usage{
#' @describeIn ${1:destination} ${2:description}
#' @eval ${1:r-code}
#' @evalRd ${1:r-code}
#' @includeRmd man/rmd/${1:filename}.Rmd
#' @inherit ${1:source} ${2:components}
#' @inheritDotParams ${1:source} ${2:arg1 arg2 arg3}
#' @inheritParams ${1:source}
#' @inheritSection ${1:source} ${2:section name}
#' @order ${1:number}
#' @rdname ${1:topic-name}
#' @template ${1:path-to-template}
#' @templateVar ${1:name} ${2:value}
}
\description{
Learn full the details in in \code{vignette('reuse')}.

Key tags:
\itemize{
\item \verb{@describeIn $\{1:destination\} $\{2:description\}}: Document a function or method in the \code{destination} topic.
\item \verb{@inherit $\{1:source\} $\{2:components\}}: Inherit one or more documentation components from another topic. If \code{components} is omitted, all supported components will be inherited.
Otherwise, specify individual components to inherit by picking one or more of \code{params}, \code{return}, \code{title}, \code{description}, \code{details}, \code{seealso}, \code{sections}, \code{references}, \code{examples}, \code{author}, \code{source}, and \code{note}.
\item \verb{@inheritDotParams $\{1:source\} $\{2:arg1 arg2 arg3\}}: Automatically generate documentation for \code{...} when you're passing dots along to another function.
\item \verb{@inheritParams $\{1:source\}}: Inherit argument documentation from another function. Only inherits documentation for arguments that aren't already documented locally.
\item \verb{@inheritSection $\{1:source\} $\{2:section name\}}: Inherit a specific named section from another topic.
\item \verb{@order $\{1:number\}}: Override the default (lexigraphic) order in which multiple blocks are combined into a single topic.
\item \verb{@rdname $\{1:topic-name\}}: Override the file name of generated \code{.Rd} file. Can be used to combine multiple blocks into a single documentation topic.
}

Other less frequently used tags:
\itemize{
\item \verb{@eval $\{1:r-code\}}: Evaluate arbitrary code in the package namespace and insert the results back into the block. Should return a character vector of lines.
\item \verb{@evalRd $\{1:r-code\}}: Evaluate arbitrary code in the package namespace and insert the results back as into the block. Should return a character vector of lines.
\item \verb{@includeRmd man/rmd/$\{1:filename\}.Rmd}: Insert the contents of an \code{.Rmd} into the current block. Superseded in favour of using a code chunk with a child document.
\item \verb{@template $\{1:path-to-template\}}: Use a roxygen2 template. Now superseded in favour of inline R code.
\item \verb{@templateVar $\{1:name\} $\{2:value\}}: Define variables for use in a roxygen2 template.
}
}
\seealso{
Other documentation tags: 
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-other}},
\code{\link{tags-rd}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd.R
\name{roxy_tag_rd}
\alias{roxy_tag_rd}
\title{Generate Rd output from a tag}
\usage{
roxy_tag_rd(x, base_path, env)
}
\arguments{
\item{x}{The tag}

\item{base_path}{Path to package root directory.}

\item{env}{Environment in which to evaluate code (if needed)}
}
\value{
Methods must return a \link{rd_section}.
}
\description{
Provide a method for this generic if you want a tag to generate output
in \code{.Rd} files. See \code{vignette("extending")} for more details.
}
\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse.R
\name{parse_package}
\alias{parse_package}
\alias{parse_file}
\alias{parse_text}
\alias{env_file}
\alias{env_package}
\title{Parse a package, file, or inline code}
\usage{
parse_package(path = ".", env = env_package(path))

parse_file(file, env = env_file(file), srcref_path = NULL)

parse_text(text, env = env_file(file))

env_file(file)

env_package(path)
}
\arguments{
\item{path, file, text}{Either specify a \code{path} to the root directory of
a package, an R \code{file}, or a character vector \code{text}.}

\item{env}{An environment environment containing the result of evaluating
the input code. The defaults will do this for you in a test environment:
for real code you'll need to generate the environment yourself.

You can also set to \code{NULL} if you only want to get the tokenized code
blocks only. This suppresses evaluation of \verb{@eval} tags, and will not
find the code object associated with each block.}

\item{srcref_path}{Path to be used as source ref.}
}
\value{
A list of roxy_block objects
}
\description{
\code{parse_package()}, \code{parse_file()}, and \code{parse_text()} allow you to use
roxygen's parsing code to parse the roxygen blocks from a package, file, or
character vector of code. \code{env_package()} and \code{env_file()} provide
defaults that generate a temporary environment making it possible to
associate each block with the corresponding live object.
}
\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxygen2-package.R
\docType{package}
\name{roxygen2-package}
\alias{roxygen2}
\alias{roxygen2-package}
\title{roxygen2: In-Line Documentation for R}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Generate your Rd documentation, 'NAMESPACE' file, and collation field using specially formatted comments. Writing documentation in-line with code makes it easier to keep your documentation up-to-date as your requirements change. 'roxygen2' is inspired by the 'Doxygen' system for C++.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://roxygen2.r-lib.org/}
  \item \url{https://github.com/r-lib/roxygen2}
  \item Report bugs at \url{https://github.com/r-lib/roxygen2/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@rstudio.com} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Peter Danenberg \email{pcd@roxygen.org} [copyright holder]
  \item Gábor Csárdi \email{csardi.gabor@gmail.com}
  \item Manuel Eugster [copyright holder]
}

Other contributors:
\itemize{
  \item RStudio [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-escaping.R
\name{double_escape_md}
\alias{double_escape_md}
\title{Check markdown escaping}
\usage{
double_escape_md(text)
}
\arguments{
\item{text}{Input text.}
}
\value{
Double-escaped text.
}
\description{
This is a regression test for Markdown escaping.
}
\details{
Each of the following bullets should look the same when rendered:
\itemize{
\item Backticks: \verb{\\}, \verb{\\\%}, \verb{\\$}, \verb{\\_}
\item \verb{\verb{}}: \verb{\\}, \verb{\\\%}, \verb{\$}, \verb{\_}
}

[ this isn't a link ]
\[ neither is this \]
}
\examples{
"\%" # percent
"\"" # double quote
'\'' # single quote
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topic.R
\name{RoxyTopic}
\alias{RoxyTopic}
\title{A \code{RoxyTopic} is an ordered collection of unique rd_sections}
\description{
A \code{RoxyTopic} object corresponds to a generated \code{.Rd} file.
}
\keyword{internal}
\section{Public fields}{
  \if{html}{\out{<div class="r6-fields">}}
  \describe{
    \item{\code{sections}}{Named list of sections. Each item must be an
\code{\link[=rd_section]{rd_section()}} object.}

    \item{\code{filename}}{Path to the \code{.Rd} file to generate.}
  }
  \if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-RoxyTopic-format}{\code{RoxyTopic$format()}}
    \item \href{#method-RoxyTopic-is_valid}{\code{RoxyTopic$is_valid()}}
    \item \href{#method-RoxyTopic-has_section}{\code{RoxyTopic$has_section()}}
    \item \href{#method-RoxyTopic-get_section}{\code{RoxyTopic$get_section()}}
    \item \href{#method-RoxyTopic-get_value}{\code{RoxyTopic$get_value()}}
    \item \href{#method-RoxyTopic-get_rd}{\code{RoxyTopic$get_rd()}}
    \item \href{#method-RoxyTopic-get_name}{\code{RoxyTopic$get_name()}}
    \item \href{#method-RoxyTopic-inherits_from}{\code{RoxyTopic$inherits_from()}}
    \item \href{#method-RoxyTopic-inherits_section_from}{\code{RoxyTopic$inherits_section_from()}}
    \item \href{#method-RoxyTopic-add}{\code{RoxyTopic$add()}}
    \item \href{#method-RoxyTopic-add_section}{\code{RoxyTopic$add_section()}}
    \item \href{#method-RoxyTopic-clone}{\code{RoxyTopic$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-format"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-format}{}}}
\subsection{\code{RoxyTopic$format()}}{
  Format the \code{.Rd} file. It considers the sections in
particular order, even though Rd tools will reorder them again.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$format(...)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{Passed to the \code{format()} methods of the \code{\link[=rd_section]{rd_section()}}
objects, the sections.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Character string.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-is_valid"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-is_valid}{}}}
\subsection{\code{RoxyTopic$is_valid()}}{
  Check if an \code{.Rd} file is valid
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$is_valid()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Logical flag, \code{TRUE} for valid \code{.Rd} files
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-has_section"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-has_section}{}}}
\subsection{\code{RoxyTopic$has_section()}}{
  Check if an \code{.Rd} file has a certain section.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$has_section(type)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{type}}{Section type, a character scalar.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Logical flag.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-get_section"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-get_section}{}}}
\subsection{\code{RoxyTopic$get_section()}}{
  Query a section.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$get_section(type)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{type}}{Section type, a character scalar.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    The \link{rd_section} object representing the section, or \code{NULL}
if the topic has no such section.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-get_value"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-get_value}{}}}
\subsection{\code{RoxyTopic$get_value()}}{
  Query the value of a section. This is the value of
the \link{rd_section} object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$get_value(type)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{type}}{Section type, a character scalar.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Value.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-get_rd"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-get_rd}{}}}
\subsection{\code{RoxyTopic$get_rd()}}{
  Get the Rd code of a section.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$get_rd(type)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{type}}{Section type, a character scalar.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Character vector, one element per line.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-get_name}{}}}
\subsection{\code{RoxyTopic$get_name()}}{
  Get the value of the \code{name} section. This is the name
of the Rd topic.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$get_name()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Character scalar.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-inherits_from"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-inherits_from}{}}}
\subsection{\code{RoxyTopic$inherits_from()}}{
  Query the topics this topic inherits \code{type} from.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$inherits_from(type)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{type}}{Section type, a character scalar.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A character vector of topic names.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-inherits_section_from"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-inherits_section_from}{}}}
\subsection{\code{RoxyTopic$inherits_section_from()}}{
  Query the topics this topic inherits sections from.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$inherits_section_from()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A character vector of topic names.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-add"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-add}{}}}
\subsection{\code{RoxyTopic$add()}}{
  Add one or more sections to the topic.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$add(x, block = "???", overwrite = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{x}}{Section(s) to add. It may be
another \code{RoxyTopic} object, all of its sections will be added;
or an \link{rd_section} object;
or a list of \link{rd_section} objects to add.}
      \item{\code{block}}{Name of block to use in error messages.}
      \item{\code{overwrite}}{Whether to overwrite an existing section. If \code{FALSE}
then the two sections will be merged.}
    }
    \if{html}{\out{</div>}}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-add_section"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-add_section}{}}}
\subsection{\code{RoxyTopic$add_section()}}{
  Add a section.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$add_section(section, block = "???", overwrite = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{section}}{\link{rd_section} object to add.}
      \item{\code{block}}{Name of block to use in error messages.}
      \item{\code{overwrite}}{Whether to overwrite an existing section. If \code{FALSE}
then the two sections will be merged.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Details}{
    Ensures that each type of name (as given by its name), only appears
once in \code{self$sections}. This method if for internal use only.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RoxyTopic-clone"></a>}}
\if{latex}{\out{\hypertarget{method-RoxyTopic-clone}{}}}
\subsection{\code{RoxyTopic$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{RoxyTopic$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-S4}
\alias{tags-rd-S4}
\alias{@slot}
\title{Tags for documenting S4}
\usage{
#' @slot ${1:name} ${2:description}
}
\description{
Learn the full details in \code{vignette('rd-S4')}.

Key tags:
\itemize{
\item \verb{@slot $\{1:name\} $\{2:description\}}: Describe the slot of an S4 class.
}
}
\seealso{
Other documentation tags:
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-R6}},
\code{\link{tags-rd-S3}},
\code{\link{tags-rd-S7}},
\code{\link{tags-rd-datasets}},
\code{\link{tags-rd-functions}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxygenize.R
\name{roxygenize}
\alias{roxygenize}
\alias{roxygenise}
\title{Document a package with roxygen2}
\usage{
roxygenize(package.dir = ".", roclets = NULL, load_code = NULL, clean = FALSE)

roxygenise(package.dir = ".", roclets = NULL, load_code = NULL, clean = FALSE)
}
\arguments{
\item{package.dir}{Location of package top level directory. Default is
working directory.}

\item{roclets}{Character vector of \link[=roclet]{roclets} to use.

The default, \code{NULL}, uses the roxygen \code{roclets} option,
which defaults to \code{c("collate", "namespace", "rd")}. This will update
(if needed) the \code{Collate} field with \code{\link[=update_collate]{update_collate()}},
produce the \code{NAMESPACE} file with \code{\link[=namespace_roclet]{namespace_roclet()}}, and
produce the Rd files with \code{\link[=rd_roclet]{rd_roclet()}}.

(Note that \code{update_collate()} is not technically a roclet but is still
controlled with this argument for historical reasons.)}

\item{load_code}{A function used to load all the R code in the package
directory. The default, \code{NULL}, uses the strategy defined by
the \code{load} roxygen option, which defaults to \code{\link[=load_pkgload]{load_pkgload()}}.
See \link{load} for more details.}

\item{clean}{If \code{TRUE}, roxygen will delete all files previously
created by roxygen before running each roclet.}
}
\value{
\code{NULL}
}
\description{
This is the workhorse function that builds manual pages and metadata for a
package. It is powered by \link[=roclet]{roclets}, roxygen2's plugin system for
producing different types of output. See the documentation of the
individual components (\code{\link[=rd_roclet]{rd_roclet()}}, \code{\link[=namespace_roclet]{namespace_roclet()}},
\code{\link[=update_collate]{update_collate()}}) for more details, or learn how to make your own in
\code{vignette("extending")}.
}
\details{
Note that roxygen2 is a dynamic documentation system: it works by
inspecting loaded objects in the package. This means that you must
be able to load the package in order to document it: see \link{load} for
details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/object-s3.R
\name{is_s3_generic}
\alias{is_s3_generic}
\alias{is_s3_method}
\title{Determine if a function is an S3 generic or S3 method.}
\usage{
is_s3_generic(name, env = parent.frame())

is_s3_method(name, env = parent.frame())
}
\arguments{
\item{name}{Name of function.}

\item{env}{Base environment in which to look for function definition.}
}
\description{
\code{is_s3_generic} compares name to \code{.knownS3Generics} and
\code{.S3PrimitiveGenerics}, then looks at the function body to see if it
calls \code{\link[=UseMethod]{UseMethod()}}.

\code{is_s3_method} builds names of all possible generics for that function
and then checks if any of them actually is a generic.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-other}
\alias{tags-rd-other}
\alias{@field}
\alias{@format}
\alias{@method}
\alias{@slot}
\alias{@source}
\title{Tags for documenting datasets and classes}
\usage{
#' @field ${1:name} ${2:description}
#' @format ${1:description}
#' @method ${1:generic} ${2:class}
#' @slot ${1:name} ${2:description}
#' @source ${1:description}
}
\description{
Learn the full details in \code{vignette('rd-other')}.

Key tags:
\itemize{
\item \verb{@field $\{1:name\} $\{2:description\}}: Describe a R6 or refClass field.
\item \verb{@format $\{1:description\}}: Describe the type/shape of a dataset. If the dataset is a data frame, include a description of each column. If not supplied, will be automatically generated by \code{object_format()}.
\item \verb{@method $\{1:generic\} $\{2:class\}}: Force a function to be recognised as an S3 method. This affects the default usage and the \code{NAMESPACE} directive produced by \verb{@export}. Only needed if automatic detection fails.
\item \verb{@slot $\{1:name\} $\{2:description\}}: Describe the slot of an S4 class.
\item \verb{@source $\{1:description\}}: Describe where the dataset came from. Provide a link to the original source (if possible) and briefly describe any manipulation that you performed when importing the data.
}
}
\seealso{
Other documentation tags: 
\code{\link{tags-index-crossref}},
\code{\link{tags-rd}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-code.R
\name{markdown_evaluate}
\alias{markdown_evaluate}
\title{Expand embedded inline code}
\usage{
markdown_evaluate(text)
}
\arguments{
\item{text}{Input text.}
}
\value{
Text with R code expanded.
A character vector of the same length as the input \code{text}.
}
\description{
For example this becomes two: 2.
Variables can be set and then reused, within the same
tag:
The value of \code{x} is 100.

We have access to the internal functions of the package, e.g.
since this is \emph{roxygen2}, we can refer to the internal \code{markdown}
function, and this is \code{TRUE}: TRUE.

To insert the name of the current package: roxygen2.

The \code{iris} data set has 5 columns:
\code{Sepal.Length}, \code{Sepal.Width}, \code{Petal.Length}, \code{Petal.Width}, \code{Species}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Code block demo
x + 1
#> [1] 101
}\if{html}{\out{</div>}}

Chunk options:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{names(mtcars)
nrow(mtcars)
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
#> [1] 32
}\if{html}{\out{</div>}}

Plots:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{plot(1:10)
}\if{html}{\out{</div>}}

\figure{test-figure-1.png}

Alternative knitr engines:

\if{html}{\out{<div class="sourceCode default">}}\preformatted{```\{r\}
# comment
this <- 10
is <- this + 10
good <- this + is
}\if{html}{\out{</div>}}

Also see \code{vignette("rd-formatting")}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/object-format.R
\name{object_format}
\alias{object_format}
\title{Default format for data}
\usage{
object_format(x)
}
\arguments{
\item{x}{A data object}
}
\value{
A \code{character} value with valid \code{Rd} syntax, or \code{NULL}.
}
\description{
This function is called to generate the default "Format" section for each
data object. The default implementation will return the class and dimension
information.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roclet.R
\name{roclet}
\alias{roclet}
\alias{roclet_preprocess}
\alias{roclet_process}
\alias{roclet_output}
\alias{roclet_clean}
\alias{roclet_tags}
\title{Build a new roclet.}
\usage{
roclet(subclass, ...)

roclet_preprocess(x, blocks, base_path)

roclet_process(x, blocks, env, base_path)

roclet_output(x, results, base_path, ...)

roclet_clean(x, base_path)

roclet_tags(x)
}
\arguments{
\item{x}{A \code{roclet} object.}

\item{blocks}{A list of \link{roxy_block} objects.}

\item{base_path}{Path to root of source package.}

\item{env}{Package environment.}

\item{results}{Value returned from your \code{roclet_process()} method.}
}
\description{
To create a new roclet, you will need to create a constructor function
that wraps \code{roclet}, and then implement the methods described below.
}
\section{Methods}{

\itemize{
\item \code{roclet_preprocess()} is called after blocks have been parsed but before
code has been evaluated. This should only be needed if your roclet affects
how code will evaluated. Should return a roclet.
\item \code{roclet_process()} called after blocks have been evaluated; i.e. the
\verb{@eval} tag has been processed, and the object associated with each block
has been determined.
\item \code{roclet_output()} is given the output from \code{roclet_process()} and should
produce files on disk.
\item \code{roclet_clean()} called when \code{roxygenise(clean = TRUE)}. Should remove
any files created by the roclet.
}
\subsection{Deprecated methods}{

\code{roclet_tags()} is no longer used; instead provide a \code{\link[=roxy_tag_parse]{roxy_tag_parse()}}
method for each tag.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-index-crossref}
\alias{tags-index-crossref}
\alias{@aliases}
\alias{@backref}
\alias{@concept}
\alias{@family}
\alias{@keywords}
\alias{@references}
\alias{@seealso}
\title{Tags for indexing and cross-references}
\usage{
#' @aliases ${1:alias}
#' @backref ${1:path}
#' @concept ${1:concept}
#' @family ${1:family name}
#' @keywords ${1:keyword}
#' @references ${1:reference}
#' @seealso [${1:func}()]
}
\description{
Learn full the details in in \code{vignette('index-crossref')}.

Key tags:
\itemize{
\item \verb{@aliases $\{1:alias\}}: Add additional aliases to the topic.
Use \code{NULL} to suppress the default alias automatically generated by roxygen2.
\item \verb{@concept $\{1:concept\}}: Add additional keywords or phrases to be included in the \code{help.search()} index. Each \verb{@concept} should be a single term or phrase.
\item \verb{@family $\{1:family name\}}: Generate \verb{@seealso} entries to all other functions in \verb{family name}.
\item \verb{@keywords $\{1:keyword\}}: Add a standardised keyword, indexed by \code{help.search()}. These are generally not useful apart from \verb{@keyword internal} which flags the topic as internal and removes from topic indexes.
\item \verb{@references $\{1:reference\}}: Pointers to the related literature. Usually formatted like a bibliography.
\item \verb{@seealso [$\{1:func\}()]}: Link to other related functions or urls. Usually a sentence or two, or a bulleted list.
}

Other less frequently used tags:
\itemize{
\item \verb{@backref $\{1:path\}}: Manually override the backreference that points from the \code{.Rd} file back to the source \code{.R} file. Only needed when generating code.
}
}
\seealso{
Other documentation tags: 
\code{\link{tags-rd-other}},
\code{\link{tags-rd}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag.R, R/utils-warn.R
\name{roxy_tag}
\alias{roxy_tag}
\alias{roxy_tag_parse}
\alias{roxy_tag_warning}
\alias{warn_roxy_tag}
\title{\code{roxy_tag} S3 constructor}
\usage{
roxy_tag(tag, raw, val = NULL, file = NA_character_, line = NA_character_)

roxy_tag_parse(x)

roxy_tag_warning(x, ...)

warn_roxy_tag(tag, message, parent = NULL, envir = parent.frame())
}
\arguments{
\item{tag}{Tag name. Arguments starting with \code{.} are reserved for internal
usage.}

\item{raw}{Raw tag value, a string.}

\item{val}{Parsed tag value, typically a character vector, but sometimes
a list. Usually filled in by \code{tag_parsers}}

\item{file, line}{Location of the tag}

\item{x}{A tag}

\item{...}{Additional data to be stored in the condition object.
If you supply condition fields, you should usually provide a
\code{class} argument. You may consider prefixing condition fields
with the name of your package or organisation to prevent name
collisions.}

\item{message}{Warning message}

\item{parent}{Supply \code{parent} when you rethrow an error from a
condition handler (e.g. with \code{\link[rlang:try_fetch]{try_fetch()}}).
\itemize{
\item If \code{parent} is a condition object, a \emph{chained error} is
created, which is useful when you want to enhance an error with
more details, while still retaining the original information.
\item If \code{parent} is \code{NA}, it indicates an unchained rethrow, which
is useful when you want to take ownership over an error and
rethrow it with a custom message that better fits the
surrounding context.

Technically, supplying \code{NA} lets \code{abort()} know it is called
from a condition handler. This helps it create simpler
backtraces where the condition handling context is hidden by
default.
}

For more information about error calls, see \ifelse{html}{\link[rlang:topic-error-chaining]{Including contextual information with error chains}}{\link[rlang:topic-error-chaining]{Including contextual information with error chains}}.}

\item{envir}{passed to \code{rlang::warn()} as \code{.envir}.}
}
\description{
\code{roxy_tag()} is the constructor for tag objects.
\code{roxy_tag_warning()} is superseded by \code{warn_roxy_tag()}; use to generate a
warning that includes the location of the tag.
}
\section{Methods}{

Define a method for \code{roxy_tag_parse} to support new tags. See \link{tag_parsers}
for more details.
}

\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-R6}
\alias{tags-rd-R6}
\alias{@field}
\alias{@R6method}
\title{Tags for documenting R6}
\usage{
#' @field ${1:name} ${2:description}
#' @R6method ${1:Class}$${2:method}
}
\description{
Learn the full details in \code{vignette('rd-R6')}.

Key tags:
\itemize{
\item \verb{@field $\{1:name\} $\{2:description\}}: Describe a R6 or refClass field.
\item \verb{@R6method $\{1:Class\}$$\{2:method\}}: Document an R6 method that can't be discovered by introspection, such as methods added via \verb{$set()}.
}
}
\seealso{
Other documentation tags:
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-S3}},
\code{\link{tags-rd-S4}},
\code{\link{tags-rd-S7}},
\code{\link{tags-rd-datasets}},
\code{\link{tags-rd-functions}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd.R
\name{rd_roclet}
\alias{rd_roclet}
\title{Roclet: make Rd files}
\usage{
rd_roclet()
}
\description{
This \link{roclet} automates the production of the \code{.Rd} files that
R uses to document functions, datasets, packages, classes, and more.
See \code{vignette("rd-functions")} for details.

It is run by default by \code{\link[=roxygenize]{roxygenize()}}.
}
\examples{
#' Add together two numbers
#' @param x A number.
#' @param y A number.
#' @return A number.
#' @export
#' @examples
#' add(1, 1)
#' add(10, 1)
add <- function(x, y) {
  x + y
}
}
\seealso{
\link{tags-rd-functions}, \link{tags-rd-datasets}, \link{tags-rd-S3}, \link{tags-rd-S4},
\link{tags-rd-S7}, \link{tags-rd-R6}, \link{tags-reuse},
\link{tags-index-crossref} for tags provided by this roclet.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{load_options}
\alias{load_options}
\alias{roxy_meta_get}
\title{Load roxygen2 options}
\usage{
load_options(base_path = ".")

roxy_meta_get(key = NULL, default = NULL)
}
\arguments{
\item{base_path}{Path to package.}

\item{key}{Key of the options, e.g. \code{"packages"}.}

\item{default}{Default value.}
}
\description{
Options can be stored in \code{DESCRIPTION} using \verb{Config/roxygen2/} fields,
or in \code{man/roxygen/meta.R}. Call \code{roxy_meta_get()} to access current
option values from within tag and roclet methods.

Options in \code{man/roxygen/meta.R} override those present in \code{DESCRIPTION}.
}
\section{Possible options}{

\itemize{
\item \code{roclets} \verb{<character>}: giving names of \link[=roclet]{roclets} to run. See
\code{\link[=roclet_find]{roclet_find()}} for details.
\item \code{packages} \verb{<character>}: packages to load that implement new tags.
\item \code{load} \verb{<string>}: how to load R code. See \link{load} for details.
\item \code{old_usage} \verb{<flag>}: use old style usage formatting?
\item \code{markdown} \verb{<flag>}: translate markdown syntax to Rd?
\item \code{r6} \verb{<flag>}: document R6 classes?
\item \code{current_package} \verb{<string>} (read only): name of package being documented.
\item \code{rd_family_title} \verb{<list>}: overrides for \verb{@family} titles. See the
\emph{rd-functions} vignette for details: \code{vignette("rd-functions")}
\item \code{knitr_chunk_options} \verb{<list>}: default chunk options used for knitr.
\item \code{restrict_image_formats} \verb{<flag>}: if \code{TRUE} then PDF images are only
included in the PDF manual, and SVG images are only included in the HTML
manual. (This only applies to images supplied via markdown.)
}
}

\section{How to set}{

Either set in \code{DESCRIPTION} using \verb{Config/roxygen2/} fields:

\if{html}{\out{<div class="sourceCode">}}\preformatted{Config/roxygen2/markdown: TRUE
Config/roxygen2/load: installed
}\if{html}{\out{</div>}}

Or if you need more complex options (like \code{rd_family_title} or
\code{knitr_chunk_options}), put them in \code{man/roxygen/meta.R}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{list(
  rd_family_title = list(models = "Model functions"),
  knitr_chunk_options = list(fig.width = 7)
)
}\if{html}{\out{</div>}}
}

\seealso{
Other extending:
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}},
\code{\link[=tags_list]{tags_list()}}
}
\concept{extending}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags-rd-S3}
\alias{tags-rd-S3}
\alias{@method}
\title{Tags for documenting S3}
\usage{
#' @method ${1:generic} ${2:class}
}
\description{
Learn the full details in \code{vignette('rd-S3')}.

Key tags:
\itemize{
\item \verb{@method $\{1:generic\} $\{2:class\}}: If ambiguous, clarify which generic an S3 method belongs too. Only needed in very rare cases.
}
}
\seealso{
Other documentation tags:
\code{\link{tags-index-crossref}},
\code{\link{tags-rd-R6}},
\code{\link{tags-rd-S4}},
\code{\link{tags-rd-S7}},
\code{\link{tags-rd-datasets}},
\code{\link{tags-rd-functions}},
\code{\link{tags-reuse}}
}
\concept{documentation tags}

> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{A}
\alias{A}
\title{Class A}
\description{
Class A description.
}
\details{
Class A details
}
\examples{

## ------------------------------------------------
## Method `A$meth1`
## ------------------------------------------------

## Example for meth1

## ------------------------------------------------
## Method `A$meth2`
## ------------------------------------------------

## Example for meth2
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{field1}}{A field 1.}

\item{\code{field2}}{A field 2.}

\item{\code{field3}}{A field 3.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{active1}}{A binding 1.}

\item{\code{active2}}{A binding 2.}

\item{\code{active3}}{A binding 2.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-A-meth1}{\code{A$meth1()}}
\item \href{#method-A-meth2}{\code{A$meth2()}}
\item \href{#method-A-meth3}{\code{A$meth3()}}
\item \href{#method-A-clone}{\code{A$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth1"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth1}{}}}
\subsection{Method \code{meth1()}}{
A method 1.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth1(Z)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{zzzzzzz}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{## Example for meth1
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth2"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth2}{}}}
\subsection{Method \code{meth2()}}{
Method 2 description.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth2(Z = 10, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{Overriding Z argument for meth2.}

\item{\code{...}}{Rest.}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
Method 2 details.
}

\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{## Example for meth2
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth3"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth3}{}}}
\subsection{Method \code{meth3()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth3(duplicate, missing)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{duplicate}}{This one is}

\item{\code{duplicate}}{Twice}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
Method 3 details.
}

\subsection{Returns}{
twice.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-clone"></a>}}
\if{latex}{\out{\hypertarget{method-A-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{C}
\alias{C}
\title{Class C}
\description{
Class C Description.
}
\details{
Classs C details.
}
\section{Super classes}{
\code{roxygen2::A} -> \code{roxygen2::B} -> \code{C}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{nosuchfield}}{This will warn.}

\item{\code{field5}}{C field 5.}

\item{\code{duplicatefield}}{Multiple.}

\item{\code{duplicatefield}}{times.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{nosuchfield}}{This will warn.}

\item{\code{active2}}{C binding 2.}

\item{\code{active4}}{C binding 4.}

\item{\code{active6}}{C binding 6.}

\item{\code{duplicate_binding}}{Double.}

\item{\code{duplicate_binding}}{Double double.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-C-meth2}{\code{C$meth2()}}
\item \href{#method-C-meth5}{\code{C$meth5()}}
\item \href{#method-C-undocumented_method}{\code{C$undocumented_method()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth3"><a href='../../roxygen2/html/A.html#method-A-meth3'><code>roxygen2::A$meth3()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="B" data-id="meth1"><a href='../../roxygen2/html/B.html#method-B-meth1'><code>roxygen2::B$meth1()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="B" data-id="meth4"><a href='../../roxygen2/html/B.html#method-B-meth4'><code>roxygen2::B$meth4()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-meth2"></a>}}
\if{latex}{\out{\hypertarget{method-C-meth2}{}}}
\subsection{Method \code{meth2()}}{
C method 2.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$meth2(Z = 10, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{zzzzz}

\item{\code{...}}{etc}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-meth5"></a>}}
\if{latex}{\out{\hypertarget{method-C-meth5}{}}}
\subsection{Method \code{meth5()}}{
C method 5.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$meth5()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-undocumented_method"></a>}}
\if{latex}{\out{\hypertarget{method-C-undocumented_method}{}}}
\subsection{Method \code{undocumented_method()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$undocumented_method()}\if{html}{\out{</div>}}
}

}
}

> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{B}
\alias{B}
\title{Class B}
\description{
Class B Description.
}
\details{
Class B details.
}
\section{Super class}{
\code{roxygen2::A} -> \code{B}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{field1}}{B field 1.}

\item{\code{field4}}{B field 4.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{active1}}{B binding 1.}

\item{\code{active4}}{B binding 4.}

\item{\code{active5}}{B binding 5.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-B-meth1}{\code{B$meth1()}}
\item \href{#method-B-meth4}{\code{B$meth4()}}
\item \href{#method-B-clone}{\code{B$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth2"><a href='../../roxygen2/html/A.html#method-A-meth2'><code>roxygen2::A$meth2()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth3"><a href='../../roxygen2/html/A.html#method-A-meth3'><code>roxygen2::A$meth3()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-meth1"></a>}}
\if{latex}{\out{\hypertarget{method-B-meth1}{}}}
\subsection{Method \code{meth1()}}{
B method 1.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$meth1(Z)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{Still zzzzzzzz.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-meth4"></a>}}
\if{latex}{\out{\hypertarget{method-B-meth4}{}}}
\subsection{Method \code{meth4()}}{
A method 4.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$meth4()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-clone"></a>}}
\if{latex}{\out{\hypertarget{method-B-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
